
import peg





def special_escape(s):
    return s.replace("\\n", "\\\\n").replace("\\t", "\\\\t").replace("\"", '\\\"').replace("\\r", "\\\\r")

class PegError(Exception):
    def __init__(self):
        Exception.__init__(self)

class NotError(Exception):
    def __init__(self):
        Exception.__init__(self)

class Result:
    def __init__(self, position):
        self.position = position
        self.values = []

    def getPosition(self):
        return self.position

    def nextPosition(self, amount = 1):
        self.position += amount

    def setValue(self, value):
        self.values = value

    def getLastValue(self):
        if type(self.values) is list:
            if len(self.values) > 0:
                return self.values[-1]
            else:
                return None
        return self.values
    
    def matches(self):
        return len(self.values)

    def getValues(self):
        return self.values

    def addResult(self, him):
        self.values.append(him.values)
        self.position = him.position
    
    #def extendResult(self, him):
    #    self.values.extend(him.values)
    #    self.position = him.position

class Stream:
    def __init__(self, filename = None, input = None):
        def read():
            file = open(filename, 'r')
            out = file.read()
            file.close()
            return out
        self.position = 0
        self.limit = 100
        self.furthest = 0
        self.memo = {}
        if filename != None:
            self.all = read()
        elif input != None:
            self.all = input
        else:
            raise PegError("Pass a filename or input")
        # print "Read " + str(len(self.all))

    def get(self, position, number = 1):
        if position + number > self.limit:
            # print (position + number)
            self.limit += 5000
        if position + number > len(self.all):
            return chr(0)
        # print "stream: %s" % self.all[position:position+number]
        return self.all[position:position+number]

    def get2(self, position):
        if position != self.position:
            self.file.seek(position)
        self.position = position + 1
        if position > self.limit:
            print position
            self.limit += 5000
        return self.file.read(1)

    def reportError(self):
        line = 1
        column = 1
        for i in xrange(0, self.furthest):
            if self.all[i] == '\n':
                line += 1
                column = 1
            else:
                column += 1
        context = 10
        left = self.furthest - context
        right = self.furthest + context
        if left < 0:
            left = 0
        if right > len(self.all):
            right = len(self.all)
        print "Read up till line %d, column %d" % (line, column)
        print "'%s'" % special_escape(self.all[left:right])
        print "%s^" % (' ' * (self.furthest - left))

    def update(self, rule, position, result):
        if result != None and result.getPosition() > self.furthest:
            self.furthest = result.getPosition()

        for_rule = None
        try:
            for_rule = self.memo[rule]
        except KeyError:
            self.memo[rule] = {}
            for_rule = self.memo[rule]
        
        for_position = None
        try:
            for_position = for_rule[position]
        except KeyError:
            for_rule[position] = None

        for_rule[position] = result

    def hasResult(self, rule, position):
        try:
            x = self.memo[rule][position]
            return True
        except KeyError:
            return False

    def result(self, rule, position):
        return self.memo[rule][position]



RULE_start = 0
RULE_module = 1
RULE_include = 2
RULE_more_code = 3
RULE_options = 4
RULE_option = 5
RULE_chunks = 6
RULE_error_option = 7
RULE_word = 8
RULE_rules = 9
RULE_rule = 10
RULE_pattern_line = 11
RULE_pattern = 12
RULE_raw_code = 13
RULE_code = 14
RULE_item = 15
RULE_failure = 16
RULE_line = 17
RULE_predicate = 18
RULE_utf8 = 19
RULE_ascii = 20
RULE_call_rule = 21
RULE_eof = 22
RULE_void = 23
RULE_range = 24
RULE_sub_pattern = 25
RULE_bind = 26
RULE_string = 27
RULE_modifier = 28
RULE_x_word = 29
RULE_rule_parameters = 30
RULE_value_parameters = 31
RULE_parameters_rules = 32
RULE_parameters_values = 33
RULE_word_or_dollar = 34
RULE_word_or_at = 35
RULE_word_at = 36
RULE_dollar = 37
RULE_number = 38
RULE_digit = 39
RULE_hex_number = 40
RULE_hex_digit = 41
RULE_start_symbol = 42
RULE_spaces = 43
RULE_space = 44
RULE_any_char = 45
RULE_any = 46
RULE_whitespace = 47
RULE_comment = 48
RULE_newlines_one = 49
RULE_newlines = 50




def rule_start(stream, position):
    if stream.hasResult(RULE_start, position):
        return stream.result(RULE_start, position)
    try:
        result_peg_32 = Result(position)
        result_peg_33 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_33 = rule_newlines(stream, result_peg_33.getPosition())
        if result_peg_33 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_33);
        
        result_peg_34 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_34 = rule_whitespace(stream, result_peg_34.getPosition())
        if result_peg_34 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_34);
        
        result_peg_35 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'start_symbol'
        result_peg_35 = rule_start_symbol(stream, result_peg_35.getPosition())
        if result_peg_35 == None:
            raise PegError
        start_symbol = result_peg_35.getValues()
        
        result_peg_32.addResult(result_peg_35);
        
        result_peg_36 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_36 = rule_newlines(stream, result_peg_36.getPosition())
        if result_peg_36 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_36);
        
        result_peg_37 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_37 = rule_whitespace(stream, result_peg_37.getPosition())
        if result_peg_37 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_37);
        
        result_peg_38 = Result(result_peg_32.getPosition())
        
        save_peg_39 = result_peg_38.getPosition()
        
        # print "Trying rule " + 'options'
        result_peg_38 = rule_options(stream, result_peg_38.getPosition())
        if result_peg_38 == None:
            
            result_peg_38 = Result(save_peg_39)
            result_peg_38.setValue(None)
        options = result_peg_38.getValues()
        
        result_peg_32.addResult(result_peg_38);
        
        result_peg_40 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_40 = rule_newlines(stream, result_peg_40.getPosition())
        if result_peg_40 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_40);
        
        result_peg_41 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_41 = rule_whitespace(stream, result_peg_41.getPosition())
        if result_peg_41 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_41);
        
        result_peg_42 = Result(result_peg_32.getPosition())
        
        save_peg_43 = result_peg_42.getPosition()
        
        # print "Trying rule " + 'module'
        result_peg_42 = rule_module(stream, result_peg_42.getPosition())
        if result_peg_42 == None:
            
            result_peg_42 = Result(save_peg_43)
            result_peg_42.setValue(None)
        module = result_peg_42.getValues()
        
        result_peg_32.addResult(result_peg_42);
        
        result_peg_44 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_44 = rule_newlines(stream, result_peg_44.getPosition())
        if result_peg_44 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_44);
        
        result_peg_45 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_45 = rule_whitespace(stream, result_peg_45.getPosition())
        if result_peg_45 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_45);
        
        result_peg_46 = Result(result_peg_32.getPosition())
        
        save_peg_47 = result_peg_46.getPosition()
        
        # print "Trying rule " + 'include'
        result_peg_46 = rule_include(stream, result_peg_46.getPosition())
        if result_peg_46 == None:
            
            result_peg_46 = Result(save_peg_47)
            result_peg_46.setValue(None)
        include = result_peg_46.getValues()
        
        result_peg_32.addResult(result_peg_46);
        
        result_peg_48 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_48 = rule_newlines(stream, result_peg_48.getPosition())
        if result_peg_48 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_48);
        
        result_peg_49 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_49 = rule_whitespace(stream, result_peg_49.getPosition())
        if result_peg_49 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_49);
        
        result_peg_50 = Result(result_peg_32.getPosition())
        
        save_peg_51 = result_peg_50.getPosition()
        
        # print "Trying rule " + 'more_code'
        result_peg_50 = rule_more_code(stream, result_peg_50.getPosition())
        if result_peg_50 == None:
            
            result_peg_50 = Result(save_peg_51)
            result_peg_50.setValue(None)
        code = result_peg_50.getValues()
        
        result_peg_32.addResult(result_peg_50);
        
        result_peg_52 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_52 = rule_newlines(stream, result_peg_52.getPosition())
        if result_peg_52 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_52);
        
        result_peg_53 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_53 = rule_whitespace(stream, result_peg_53.getPosition())
        if result_peg_53 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_53);
        
        result_peg_54 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'rules'
        result_peg_54 = rule_rules(stream, result_peg_54.getPosition())
        if result_peg_54 == None:
            raise PegError
        rules = result_peg_54.getValues()
        
        result_peg_32.addResult(result_peg_54);
        
        result_peg_55 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_55 = rule_newlines(stream, result_peg_55.getPosition())
        if result_peg_55 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_55);
        
        result_peg_56 = Result(result_peg_32.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_56 = rule_whitespace(stream, result_peg_56.getPosition())
        if result_peg_56 == None:
            raise PegError
        
        result_peg_32.addResult(result_peg_56);
        
        result_peg_57 = Result(result_peg_32.getPosition())
        
        if chr(0) == stream.get(result_peg_57.getPosition()):
            result_peg_57.nextPosition()
            result_peg_57.setValue(chr(0))
        else:
            raise PegError
        
        result_peg_32.addResult(result_peg_57);
        
        result_peg_58 = Result(result_peg_32.getPosition())
        
        value = None
        values = result_peg_32.getValues()
        value = peg.Peg(start_symbol, include, code, module, rules, options)
        result_peg_58.setValue(value)
        
        result_peg_32.addResult(result_peg_58);
        
        result_peg_32.setValue(result_peg_32.getLastValue())
        stream.update(RULE_start, position, result_peg_32)
        return result_peg_32
    except PegError:
        pass
    stream.update(RULE_start, position, None)
    return None


def rule_module(stream, position):
    if stream.hasResult(RULE_module, position):
        return stream.result(RULE_module, position)
    try:
        result_peg_59 = Result(position)
        result_peg_60 = Result(result_peg_59.getPosition())
        
        if 'module:' == stream.get(result_peg_60.getPosition(), 7):
            result_peg_60.nextPosition(7)
            result_peg_60.setValue('module:')
        else:
            raise PegError
        
        result_peg_59.addResult(result_peg_60);
        
        result_peg_61 = Result(result_peg_59.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_61 = rule_spaces(stream, result_peg_61.getPosition())
        if result_peg_61 == None:
            raise PegError
        
        result_peg_59.addResult(result_peg_61);
        
        result_peg_62 = Result(result_peg_59.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_62 = rule_word(stream, result_peg_62.getPosition())
        if result_peg_62 == None:
            raise PegError
        name = result_peg_62.getValues()
        
        result_peg_59.addResult(result_peg_62);
        
        result_peg_63 = Result(result_peg_59.getPosition())
        
        try:
            while True:
                result_peg_64 = Result(result_peg_63.getPosition());
                result_peg_65 = Result(result_peg_64.getPosition())
                
                if '.' == stream.get(result_peg_65.getPosition(), 1):
                    result_peg_65.nextPosition(1)
                    result_peg_65.setValue('.')
                else:
                    raise PegError
                
                result_peg_64.addResult(result_peg_65);
                
                result_peg_66 = Result(result_peg_64.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_66 = rule_word(stream, result_peg_66.getPosition())
                if result_peg_66 == None:
                    raise PegError
                
                result_peg_64.addResult(result_peg_66);
                
                result_peg_67 = Result(result_peg_64.getPosition())
                
                value = None
                values = result_peg_64.getValues()
                value = values[1]
                result_peg_67.setValue(value)
                
                result_peg_64.addResult(result_peg_67);
                
                result_peg_64.setValue(result_peg_64.getLastValue())
                result_peg_63.addResult(result_peg_64);
        except PegError:
            pass
        rest = result_peg_63.getValues()
        
        result_peg_59.addResult(result_peg_63);
        
        result_peg_68 = Result(result_peg_59.getPosition())
        
        value = None
        values = result_peg_59.getValues()
        value = [name] + rest
        result_peg_68.setValue(value)
        
        result_peg_59.addResult(result_peg_68);
        
        result_peg_59.setValue(result_peg_59.getLastValue())
        stream.update(RULE_module, position, result_peg_59)
        return result_peg_59
    except PegError:
        pass
    stream.update(RULE_module, position, None)
    return None


def rule_include(stream, position):
    if stream.hasResult(RULE_include, position):
        return stream.result(RULE_include, position)
    try:
        result_peg_69 = Result(position)
        result_peg_70 = Result(result_peg_69.getPosition())
        
        if 'include:' == stream.get(result_peg_70.getPosition(), 8):
            result_peg_70.nextPosition(8)
            result_peg_70.setValue('include:')
        else:
            raise PegError
        
        result_peg_69.addResult(result_peg_70);
        
        result_peg_71 = Result(result_peg_69.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_71 = rule_spaces(stream, result_peg_71.getPosition())
        if result_peg_71 == None:
            raise PegError
        
        result_peg_69.addResult(result_peg_71);
        
        result_peg_72 = Result(result_peg_69.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_72 = rule_code(stream, result_peg_72.getPosition())
        if result_peg_72 == None:
            raise PegError
        code = result_peg_72.getValues()
        
        result_peg_69.addResult(result_peg_72);
        
        result_peg_73 = Result(result_peg_69.getPosition())
        
        value = None
        values = result_peg_69.getValues()
        value = code.code
        result_peg_73.setValue(value)
        
        result_peg_69.addResult(result_peg_73);
        
        result_peg_69.setValue(result_peg_69.getLastValue())
        stream.update(RULE_include, position, result_peg_69)
        return result_peg_69
    except PegError:
        pass
    stream.update(RULE_include, position, None)
    return None


def rule_more_code(stream, position):
    if stream.hasResult(RULE_more_code, position):
        return stream.result(RULE_more_code, position)
    try:
        result_peg_74 = Result(position)
        result_peg_75 = Result(result_peg_74.getPosition())
        
        if 'code:' == stream.get(result_peg_75.getPosition(), 5):
            result_peg_75.nextPosition(5)
            result_peg_75.setValue('code:')
        else:
            raise PegError
        
        result_peg_74.addResult(result_peg_75);
        
        result_peg_76 = Result(result_peg_74.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_76 = rule_spaces(stream, result_peg_76.getPosition())
        if result_peg_76 == None:
            raise PegError
        
        result_peg_74.addResult(result_peg_76);
        
        result_peg_77 = Result(result_peg_74.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_77 = rule_code(stream, result_peg_77.getPosition())
        if result_peg_77 == None:
            raise PegError
        code = result_peg_77.getValues()
        
        result_peg_74.addResult(result_peg_77);
        
        result_peg_78 = Result(result_peg_74.getPosition())
        
        value = None
        values = result_peg_74.getValues()
        value = code.code
        result_peg_78.setValue(value)
        
        result_peg_74.addResult(result_peg_78);
        
        result_peg_74.setValue(result_peg_74.getLastValue())
        stream.update(RULE_more_code, position, result_peg_74)
        return result_peg_74
    except PegError:
        pass
    stream.update(RULE_more_code, position, None)
    return None


def rule_options(stream, position):
    if stream.hasResult(RULE_options, position):
        return stream.result(RULE_options, position)
    try:
        result_peg_79 = Result(position)
        result_peg_80 = Result(result_peg_79.getPosition())
        
        if 'options:' == stream.get(result_peg_80.getPosition(), 8):
            result_peg_80.nextPosition(8)
            result_peg_80.setValue('options:')
        else:
            raise PegError
        
        result_peg_79.addResult(result_peg_80);
        
        result_peg_81 = Result(result_peg_79.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_81 = rule_spaces(stream, result_peg_81.getPosition())
        if result_peg_81 == None:
            raise PegError
        
        result_peg_79.addResult(result_peg_81);
        
        result_peg_82 = Result(result_peg_79.getPosition())
        
        # print "Trying rule " + 'option'
        result_peg_82 = rule_option(stream, result_peg_82.getPosition())
        if result_peg_82 == None:
            raise PegError
        option1 = result_peg_82.getValues()
        
        result_peg_79.addResult(result_peg_82);
        
        result_peg_83 = Result(result_peg_79.getPosition())
        
        try:
            while True:
                result_peg_84 = Result(result_peg_83.getPosition());
                result_peg_85 = Result(result_peg_84.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_85 = rule_spaces(stream, result_peg_85.getPosition())
                if result_peg_85 == None:
                    raise PegError
                
                result_peg_84.addResult(result_peg_85);
                
                result_peg_86 = Result(result_peg_84.getPosition())
                
                if ',' == stream.get(result_peg_86.getPosition(), 1):
                    result_peg_86.nextPosition(1)
                    result_peg_86.setValue(',')
                else:
                    raise PegError
                
                result_peg_84.addResult(result_peg_86);
                
                result_peg_87 = Result(result_peg_84.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_87 = rule_spaces(stream, result_peg_87.getPosition())
                if result_peg_87 == None:
                    raise PegError
                
                result_peg_84.addResult(result_peg_87);
                
                result_peg_88 = Result(result_peg_84.getPosition())
                
                # print "Trying rule " + 'option'
                result_peg_88 = rule_option(stream, result_peg_88.getPosition())
                if result_peg_88 == None:
                    raise PegError
                
                result_peg_84.addResult(result_peg_88);
                
                result_peg_84.setValue(result_peg_84.getLastValue())
                result_peg_83.addResult(result_peg_84);
        except PegError:
            pass
        option_rest = result_peg_83.getValues()
        
        result_peg_79.addResult(result_peg_83);
        
        result_peg_89 = Result(result_peg_79.getPosition())
        
        value = None
        values = result_peg_79.getValues()
        value = []
        for option in ([option1] + option_rest):
            import re
            debug = re.compile("debug(\d+)")
            out = debug.match(option)
            if out != None:
                num = int(out.group(1))
                for x in xrange(1,num+1):
                    value.append('debug%d' % x)
            elif option == 'no-memo':
                value.append(option)
            else:
                value.append(option)
        result_peg_89.setValue(value)
        
        result_peg_79.addResult(result_peg_89);
        
        result_peg_79.setValue(result_peg_79.getLastValue())
        stream.update(RULE_options, position, result_peg_79)
        return result_peg_79
    except PegError:
        pass
    stream.update(RULE_options, position, None)
    return None


def rule_option(stream, position):
    if stream.hasResult(RULE_option, position):
        return stream.result(RULE_option, position)
    try:
        result_peg_90 = Result(position)
        result_peg_91 = Result(result_peg_90.getPosition())
        
        if 'debug' == stream.get(result_peg_91.getPosition(), 5):
            result_peg_91.nextPosition(5)
            result_peg_91.setValue('debug')
        else:
            raise PegError
        
        result_peg_90.addResult(result_peg_91);
        
        result_peg_92 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_92 = rule_number(stream, result_peg_92.getPosition())
        if result_peg_92 == None:
            raise PegError
        number = result_peg_92.getValues()
        
        result_peg_90.addResult(result_peg_92);
        
        result_peg_93 = Result(result_peg_90.getPosition())
        
        value = None
        values = result_peg_90.getValues()
        value = 'debug%s' % number
        result_peg_93.setValue(value)
        
        result_peg_90.addResult(result_peg_93);
        
        result_peg_90.setValue(result_peg_90.getLastValue())
        stream.update(RULE_option, position, result_peg_90)
        return result_peg_90
    except PegError:
        pass
    try:
        result_peg_94 = Result(position)
        if 'state' == stream.get(result_peg_94.getPosition(), 5):
            result_peg_94.nextPosition(5)
            result_peg_94.setValue('state')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_94)
        return result_peg_94
    except PegError:
        pass
    try:
        result_peg_95 = Result(position)
        if 'no-memo' == stream.get(result_peg_95.getPosition(), 7):
            result_peg_95.nextPosition(7)
            result_peg_95.setValue('no-memo')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_95)
        return result_peg_95
    except PegError:
        pass
    try:
        result_peg_96 = Result(position)
        # print "Trying rule " + 'error_option'
        result_peg_96 = rule_error_option(stream, result_peg_96.getPosition())
        if result_peg_96 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_96)
        return result_peg_96
    except PegError:
        pass
    try:
        result_peg_97 = Result(position)
        # print "Trying rule " + 'chunks'
        result_peg_97 = rule_chunks(stream, result_peg_97.getPosition())
        if result_peg_97 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_97)
        return result_peg_97
    except PegError:
        pass
    stream.update(RULE_option, position, None)
    return None


def rule_chunks(stream, position):
    if stream.hasResult(RULE_chunks, position):
        return stream.result(RULE_chunks, position)
    try:
        result_peg_98 = Result(position)
        result_peg_99 = Result(result_peg_98.getPosition())
        
        if 'chunks' == stream.get(result_peg_99.getPosition(), 6):
            result_peg_99.nextPosition(6)
            result_peg_99.setValue('chunks')
        else:
            raise PegError
        
        result_peg_98.addResult(result_peg_99);
        
        result_peg_100 = Result(result_peg_98.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_100 = rule_whitespace(stream, result_peg_100.getPosition())
        if result_peg_100 == None:
            raise PegError
        
        result_peg_98.addResult(result_peg_100);
        
        result_peg_101 = Result(result_peg_98.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_101 = rule_number(stream, result_peg_101.getPosition())
        if result_peg_101 == None:
            raise PegError
        number = result_peg_101.getValues()
        
        result_peg_98.addResult(result_peg_101);
        
        result_peg_102 = Result(result_peg_98.getPosition())
        
        value = None
        values = result_peg_98.getValues()
        value = 'chunks %s' % number
        result_peg_102.setValue(value)
        
        result_peg_98.addResult(result_peg_102);
        
        result_peg_98.setValue(result_peg_98.getLastValue())
        stream.update(RULE_chunks, position, result_peg_98)
        return result_peg_98
    except PegError:
        pass
    stream.update(RULE_chunks, position, None)
    return None


def rule_error_option(stream, position):
    if stream.hasResult(RULE_error_option, position):
        return stream.result(RULE_error_option, position)
    try:
        result_peg_103 = Result(position)
        result_peg_104 = Result(result_peg_103.getPosition())
        
        if 'error-length' == stream.get(result_peg_104.getPosition(), 12):
            result_peg_104.nextPosition(12)
            result_peg_104.setValue('error-length')
        else:
            raise PegError
        
        result_peg_103.addResult(result_peg_104);
        
        result_peg_105 = Result(result_peg_103.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_105 = rule_whitespace(stream, result_peg_105.getPosition())
        if result_peg_105 == None:
            raise PegError
        
        result_peg_103.addResult(result_peg_105);
        
        result_peg_106 = Result(result_peg_103.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_106 = rule_number(stream, result_peg_106.getPosition())
        if result_peg_106 == None:
            raise PegError
        number = result_peg_106.getValues()
        
        result_peg_103.addResult(result_peg_106);
        
        result_peg_107 = Result(result_peg_103.getPosition())
        
        value = None
        values = result_peg_103.getValues()
        value = 'error-length %s' % number
        result_peg_107.setValue(value)
        
        result_peg_103.addResult(result_peg_107);
        
        result_peg_103.setValue(result_peg_103.getLastValue())
        stream.update(RULE_error_option, position, result_peg_103)
        return result_peg_103
    except PegError:
        pass
    stream.update(RULE_error_option, position, None)
    return None


def rule_word(stream, position):
    if stream.hasResult(RULE_word, position):
        return stream.result(RULE_word, position)
    try:
        result_peg_108 = Result(position)
        result_peg_109 = Result(result_peg_108.getPosition())
        
        try:
            while True:
                result_peg_110 = Result(result_peg_109.getPosition());
                # print "Trying rule " + 'any_char'
                result_peg_110 = rule_any_char(stream, result_peg_110.getPosition())
                if result_peg_110 == None:
                    raise PegError
                result_peg_109.addResult(result_peg_110);
        except PegError:
            if result_peg_109.matches() == 0:
                raise PegError
                
        result_peg_108.addResult(result_peg_109);
        
        result_peg_111 = Result(result_peg_108.getPosition())
        
        value = None
        values = result_peg_108.getValues()
        # print "all start symbol values " + str(values)
        # print "values[0] " + str(values[0])
        value = ''.join(values[0]).replace('-', '__')
        # print "got word " + value
        result_peg_111.setValue(value)
        
        result_peg_108.addResult(result_peg_111);
        
        result_peg_108.setValue(result_peg_108.getLastValue())
        stream.update(RULE_word, position, result_peg_108)
        return result_peg_108
    except PegError:
        pass
    stream.update(RULE_word, position, None)
    return None


def rule_rules(stream, position):
    if stream.hasResult(RULE_rules, position):
        return stream.result(RULE_rules, position)
    try:
        result_peg_112 = Result(position)
        result_peg_113 = Result(result_peg_112.getPosition())
        
        if 'rules:' == stream.get(result_peg_113.getPosition(), 6):
            result_peg_113.nextPosition(6)
            result_peg_113.setValue('rules:')
        else:
            raise PegError
        
        result_peg_112.addResult(result_peg_113);
        
        result_peg_114 = Result(result_peg_112.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_114 = rule_whitespace(stream, result_peg_114.getPosition())
        if result_peg_114 == None:
            raise PegError
        
        result_peg_112.addResult(result_peg_114);
        
        result_peg_115 = Result(result_peg_112.getPosition())
        
        try:
            while True:
                result_peg_116 = Result(result_peg_115.getPosition());
                result_peg_117 = Result(result_peg_116.getPosition())
                
                # print "Trying rule " + 'rule'
                result_peg_117 = rule_rule(stream, result_peg_117.getPosition())
                if result_peg_117 == None:
                    raise PegError
                
                result_peg_116.addResult(result_peg_117);
                
                result_peg_118 = Result(result_peg_116.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_118 = rule_whitespace(stream, result_peg_118.getPosition())
                if result_peg_118 == None:
                    raise PegError
                
                result_peg_116.addResult(result_peg_118);
                
                result_peg_119 = Result(result_peg_116.getPosition())
                
                value = None
                values = result_peg_116.getValues()
                value = values[0]
                result_peg_119.setValue(value)
                
                result_peg_116.addResult(result_peg_119);
                
                result_peg_116.setValue(result_peg_116.getLastValue())
                result_peg_115.addResult(result_peg_116);
        except PegError:
            pass
        rules = result_peg_115.getValues()
        
        result_peg_112.addResult(result_peg_115);
        
        result_peg_120 = Result(result_peg_112.getPosition())
        
        value = None
        values = result_peg_112.getValues()
        value = rules
        result_peg_120.setValue(value)
        
        result_peg_112.addResult(result_peg_120);
        
        result_peg_112.setValue(result_peg_112.getLastValue())
        stream.update(RULE_rules, position, result_peg_112)
        return result_peg_112
    except PegError:
        pass
    stream.update(RULE_rules, position, None)
    return None


def rule_rule(stream, position):
    if stream.hasResult(RULE_rule, position):
        return stream.result(RULE_rule, position)
    try:
        result_peg_121 = Result(position)
        result_peg_122 = Result(result_peg_121.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_122 = rule_spaces(stream, result_peg_122.getPosition())
        if result_peg_122 == None:
            raise PegError
        
        result_peg_121.addResult(result_peg_122);
        
        result_peg_123 = Result(result_peg_121.getPosition())
        
        save_peg_124 = result_peg_123.getPosition()
        
        if 'inline' == stream.get(result_peg_123.getPosition(), 6):
            result_peg_123.nextPosition(6)
            result_peg_123.setValue('inline')
        else:
            
            result_peg_123 = Result(save_peg_124)
            result_peg_123.setValue(None)
        inline = result_peg_123.getValues()
        
        result_peg_121.addResult(result_peg_123);
        
        result_peg_125 = Result(result_peg_121.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_125 = rule_spaces(stream, result_peg_125.getPosition())
        if result_peg_125 == None:
            raise PegError
        
        result_peg_121.addResult(result_peg_125);
        
        result_peg_126 = Result(result_peg_121.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_126 = rule_word(stream, result_peg_126.getPosition())
        if result_peg_126 == None:
            raise PegError
        name = result_peg_126.getValues()
        
        result_peg_121.addResult(result_peg_126);
        
        result_peg_127 = Result(result_peg_121.getPosition())
        
        save_peg_128 = result_peg_127.getPosition()
        
        # print "Trying rule " + 'rule_parameters'
        result_peg_127 = rule_rule_parameters(stream, result_peg_127.getPosition())
        if result_peg_127 == None:
            
            result_peg_127 = Result(save_peg_128)
            result_peg_127.setValue(None)
        rule_parameters = result_peg_127.getValues()
        
        result_peg_121.addResult(result_peg_127);
        
        result_peg_129 = Result(result_peg_121.getPosition())
        
        save_peg_130 = result_peg_129.getPosition()
        
        # print "Trying rule " + 'value_parameters'
        result_peg_129 = rule_value_parameters(stream, result_peg_129.getPosition())
        if result_peg_129 == None:
            
            result_peg_129 = Result(save_peg_130)
            result_peg_129.setValue(None)
        parameters = result_peg_129.getValues()
        
        result_peg_121.addResult(result_peg_129);
        
        result_peg_131 = Result(result_peg_121.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_131 = rule_spaces(stream, result_peg_131.getPosition())
        if result_peg_131 == None:
            raise PegError
        
        result_peg_121.addResult(result_peg_131);
        
        result_peg_132 = Result(result_peg_121.getPosition())
        
        if '=' == stream.get(result_peg_132.getPosition(), 1):
            result_peg_132.nextPosition(1)
            result_peg_132.setValue('=')
        else:
            raise PegError
        
        result_peg_121.addResult(result_peg_132);
        
        result_peg_133 = Result(result_peg_121.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_133 = rule_spaces(stream, result_peg_133.getPosition())
        if result_peg_133 == None:
            raise PegError
        
        result_peg_121.addResult(result_peg_133);
        
        result_peg_134 = Result(result_peg_121.getPosition())
        
        # print "Trying rule " + 'pattern_line'
        result_peg_134 = rule_pattern_line(stream, result_peg_134.getPosition())
        if result_peg_134 == None:
            raise PegError
        pattern1 = result_peg_134.getValues()
        
        result_peg_121.addResult(result_peg_134);
        
        result_peg_135 = Result(result_peg_121.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_135 = rule_whitespace(stream, result_peg_135.getPosition())
        if result_peg_135 == None:
            raise PegError
        
        result_peg_121.addResult(result_peg_135);
        
        result_peg_136 = Result(result_peg_121.getPosition())
        
        try:
            while True:
                result_peg_137 = Result(result_peg_136.getPosition());
                result_peg_138 = Result(result_peg_137.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_138 = rule_spaces(stream, result_peg_138.getPosition())
                if result_peg_138 == None:
                    raise PegError
                
                result_peg_137.addResult(result_peg_138);
                
                result_peg_139 = Result(result_peg_137.getPosition())
                
                if '|' == stream.get(result_peg_139.getPosition(), 1):
                    result_peg_139.nextPosition(1)
                    result_peg_139.setValue('|')
                else:
                    raise PegError
                
                result_peg_137.addResult(result_peg_139);
                
                result_peg_140 = Result(result_peg_137.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_140 = rule_spaces(stream, result_peg_140.getPosition())
                if result_peg_140 == None:
                    raise PegError
                
                result_peg_137.addResult(result_peg_140);
                
                result_peg_141 = Result(result_peg_137.getPosition())
                
                # print "Trying rule " + 'pattern_line'
                result_peg_141 = rule_pattern_line(stream, result_peg_141.getPosition())
                if result_peg_141 == None:
                    raise PegError
                pattern = result_peg_141.getValues()
                
                result_peg_137.addResult(result_peg_141);
                
                result_peg_142 = Result(result_peg_137.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_142 = rule_whitespace(stream, result_peg_142.getPosition())
                if result_peg_142 == None:
                    raise PegError
                
                result_peg_137.addResult(result_peg_142);
                
                result_peg_143 = Result(result_peg_137.getPosition())
                
                value = None
                values = result_peg_137.getValues()
                value = pattern
                result_peg_143.setValue(value)
                
                result_peg_137.addResult(result_peg_143);
                
                result_peg_137.setValue(result_peg_137.getLastValue())
                result_peg_136.addResult(result_peg_137);
        except PegError:
            pass
        patterns = result_peg_136.getValues()
        
        result_peg_121.addResult(result_peg_136);
        
        result_peg_144 = Result(result_peg_121.getPosition())
        
        save_peg_145 = result_peg_144.getPosition()
        
        # print "Trying rule " + 'failure'
        result_peg_144 = rule_failure(stream, result_peg_144.getPosition())
        if result_peg_144 == None:
            
            result_peg_144 = Result(save_peg_145)
            result_peg_144.setValue(None)
        fail = result_peg_144.getValues()
        
        result_peg_121.addResult(result_peg_144);
        
        result_peg_146 = Result(result_peg_121.getPosition())
        
        value = None
        values = result_peg_121.getValues()
        value = peg.Rule(name, [pattern1] + patterns, inline = (inline != None), rules = rule_parameters, parameters = parameters, fail = fail)
        result_peg_146.setValue(value)
        
        result_peg_121.addResult(result_peg_146);
        
        result_peg_121.setValue(result_peg_121.getLastValue())
        stream.update(RULE_rule, position, result_peg_121)
        return result_peg_121
    except PegError:
        pass
    stream.update(RULE_rule, position, None)
    return None


def rule_pattern_line(stream, position):
    if stream.hasResult(RULE_pattern_line, position):
        return stream.result(RULE_pattern_line, position)
    try:
        result_peg_147 = Result(position)
        result_peg_148 = Result(result_peg_147.getPosition())
        
        try:
            while True:
                result_peg_149 = Result(result_peg_148.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_149 = rule_pattern(stream, result_peg_149.getPosition())
                if result_peg_149 == None:
                    raise PegError
                result_peg_148.addResult(result_peg_149);
        except PegError:
            pass
        patterns = result_peg_148.getValues()
        
        result_peg_147.addResult(result_peg_148);
        
        result_peg_150 = Result(result_peg_147.getPosition())
        
        value = None
        values = result_peg_147.getValues()
        value = peg.PatternSequence(patterns)
        #if code != None:
        #    value = code(peg.PatternSequence(patterns))
        #else:
        #    value = peg.PatternAction(peg.PatternSequence(patterns), "value = values;")
        result_peg_150.setValue(value)
        
        result_peg_147.addResult(result_peg_150);
        
        result_peg_147.setValue(result_peg_147.getLastValue())
        stream.update(RULE_pattern_line, position, result_peg_147)
        return result_peg_147
    except PegError:
        pass
    stream.update(RULE_pattern_line, position, None)
    return None


def rule_pattern(stream, position):
    if stream.hasResult(RULE_pattern, position):
        return stream.result(RULE_pattern, position)
    try:
        result_peg_151 = Result(position)
        result_peg_152 = Result(result_peg_151.getPosition())
        
        save_peg_153 = result_peg_152.getPosition()
        
        # print "Trying rule " + 'bind'
        result_peg_152 = rule_bind(stream, result_peg_152.getPosition())
        if result_peg_152 == None:
            
            result_peg_152 = Result(save_peg_153)
            result_peg_152.setValue(None)
        bind = result_peg_152.getValues()
        
        result_peg_151.addResult(result_peg_152);
        
        result_peg_154 = Result(result_peg_151.getPosition())
        
        # print "Trying rule " + 'item'
        result_peg_154 = rule_item(stream, result_peg_154.getPosition())
        if result_peg_154 == None:
            raise PegError
        item = result_peg_154.getValues()
        
        result_peg_151.addResult(result_peg_154);
        
        result_peg_155 = Result(result_peg_151.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_155 = rule_spaces(stream, result_peg_155.getPosition())
        if result_peg_155 == None:
            raise PegError
        
        result_peg_151.addResult(result_peg_155);
        
        result_peg_156 = Result(result_peg_151.getPosition())
        
        value = None
        values = result_peg_151.getValues()
        # value = peg.PatternRule(values[0])
        if bind != None:
            item = bind(item)
        value = item
        # print "Pattern is " + str(value)
        result_peg_156.setValue(value)
        
        result_peg_151.addResult(result_peg_156);
        
        result_peg_151.setValue(result_peg_151.getLastValue())
        stream.update(RULE_pattern, position, result_peg_151)
        return result_peg_151
    except PegError:
        pass
    stream.update(RULE_pattern, position, None)
    return None


def rule_raw_code(stream, position):
    if stream.hasResult(RULE_raw_code, position):
        return stream.result(RULE_raw_code, position)
    try:
        result_peg_157 = Result(position)
        result_peg_158 = Result(result_peg_157.getPosition())
        
        if '(' == stream.get(result_peg_158.getPosition(), 1):
            result_peg_158.nextPosition(1)
            result_peg_158.setValue('(')
        else:
            raise PegError
        
        result_peg_157.addResult(result_peg_158);
        
        result_peg_159 = Result(result_peg_157.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_159 = rule_spaces(stream, result_peg_159.getPosition())
        if result_peg_159 == None:
            raise PegError
        
        result_peg_157.addResult(result_peg_159);
        
        result_peg_160 = Result(result_peg_157.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_160 = rule_code(stream, result_peg_160.getPosition())
        if result_peg_160 == None:
            raise PegError
        code = result_peg_160.getValues()
        
        result_peg_157.addResult(result_peg_160);
        
        result_peg_161 = Result(result_peg_157.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_161 = rule_spaces(stream, result_peg_161.getPosition())
        if result_peg_161 == None:
            raise PegError
        
        result_peg_157.addResult(result_peg_161);
        
        result_peg_162 = Result(result_peg_157.getPosition())
        
        if ')' == stream.get(result_peg_162.getPosition(), 1):
            result_peg_162.nextPosition(1)
            result_peg_162.setValue(')')
        else:
            raise PegError
        
        result_peg_157.addResult(result_peg_162);
        
        result_peg_163 = Result(result_peg_157.getPosition())
        
        value = None
        values = result_peg_157.getValues()
        value = code.code
        result_peg_163.setValue(value)
        
        result_peg_157.addResult(result_peg_163);
        
        result_peg_157.setValue(result_peg_157.getLastValue())
        stream.update(RULE_raw_code, position, result_peg_157)
        return result_peg_157
    except PegError:
        pass
    stream.update(RULE_raw_code, position, None)
    return None


def rule_code(stream, position):
    if stream.hasResult(RULE_code, position):
        return stream.result(RULE_code, position)
    try:
        result_peg_164 = Result(position)
        result_peg_165 = Result(result_peg_164.getPosition())
        
        if '{{' == stream.get(result_peg_165.getPosition(), 2):
            result_peg_165.nextPosition(2)
            result_peg_165.setValue('{{')
        else:
            raise PegError
        
        result_peg_164.addResult(result_peg_165);
        
        result_peg_166 = Result(result_peg_164.getPosition())
        
        try:
            while True:
                result_peg_167 = Result(result_peg_166.getPosition());
                result_peg_168 = Result(result_peg_167.getPosition())
                
                result_peg_169 = Result(result_peg_168.getPosition());
                try:
                    if '}}' == stream.get(result_peg_169.getPosition(), 2):
                        result_peg_169.nextPosition(2)
                        result_peg_169.setValue('}}')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_168.setValue(None)
                        
                result_peg_167.addResult(result_peg_168);
                
                result_peg_170 = Result(result_peg_167.getPosition())
                
                temp_peg_171 = stream.get(result_peg_170.getPosition())
                if temp_peg_171 != chr(0):
                    result_peg_170.setValue(temp_peg_171)
                    result_peg_170.nextPosition()
                else:
                    raise PegError
                
                result_peg_167.addResult(result_peg_170);
                
                result_peg_172 = Result(result_peg_167.getPosition())
                
                value = None
                values = result_peg_167.getValues()
                value = values[1]
                result_peg_172.setValue(value)
                
                result_peg_167.addResult(result_peg_172);
                
                result_peg_167.setValue(result_peg_167.getLastValue())
                result_peg_166.addResult(result_peg_167);
        except PegError:
            if result_peg_166.matches() == 0:
                raise PegError
                
        result_peg_164.addResult(result_peg_166);
        
        result_peg_173 = Result(result_peg_164.getPosition())
        
        if '}}' == stream.get(result_peg_173.getPosition(), 2):
            result_peg_173.nextPosition(2)
            result_peg_173.setValue('}}')
        else:
            raise PegError
        
        result_peg_164.addResult(result_peg_173);
        
        result_peg_174 = Result(result_peg_164.getPosition())
        
        value = None
        values = result_peg_164.getValues()
        value = peg.PatternCode(''.join(values[1]))
        result_peg_174.setValue(value)
        
        result_peg_164.addResult(result_peg_174);
        
        result_peg_164.setValue(result_peg_164.getLastValue())
        stream.update(RULE_code, position, result_peg_164)
        return result_peg_164
    except PegError:
        pass
    stream.update(RULE_code, position, None)
    return None


def rule_item(stream, position):
    if stream.hasResult(RULE_item, position):
        return stream.result(RULE_item, position)
    try:
        result_peg_175 = Result(position)
        result_peg_176 = Result(result_peg_175.getPosition())
        
        save_peg_177 = result_peg_176.getPosition()
        
        if '&' == stream.get(result_peg_176.getPosition(), 1):
            result_peg_176.nextPosition(1)
            result_peg_176.setValue('&')
        else:
            
            result_peg_176 = Result(save_peg_177)
            result_peg_176.setValue(None)
        ensure = result_peg_176.getValues()
        
        result_peg_175.addResult(result_peg_176);
        
        result_peg_178 = Result(result_peg_175.getPosition())
        
        save_peg_179 = result_peg_178.getPosition()
        
        if '!' == stream.get(result_peg_178.getPosition(), 1):
            result_peg_178.nextPosition(1)
            result_peg_178.setValue('!')
        else:
            
            result_peg_178 = Result(save_peg_179)
            result_peg_178.setValue(None)
        pnot = result_peg_178.getValues()
        
        result_peg_175.addResult(result_peg_178);
        
        result_peg_180 = Result(result_peg_175.getPosition())
        
        save_peg_181 = result_peg_180.getPosition()
        
        result_peg_180 = Result(save_peg_181)
        # print "Trying rule " + 'x_word'
        result_peg_180 = rule_x_word(stream, result_peg_180.getPosition())
        if result_peg_180 == None:
            
            result_peg_180 = Result(save_peg_181)
            # print "Trying rule " + 'any'
            result_peg_180 = rule_any(stream, result_peg_180.getPosition())
            if result_peg_180 == None:
                
                result_peg_180 = Result(save_peg_181)
                # print "Trying rule " + 'eof'
                result_peg_180 = rule_eof(stream, result_peg_180.getPosition())
                if result_peg_180 == None:
                    
                    result_peg_180 = Result(save_peg_181)
                    # print "Trying rule " + 'void'
                    result_peg_180 = rule_void(stream, result_peg_180.getPosition())
                    if result_peg_180 == None:
                        
                        result_peg_180 = Result(save_peg_181)
                        # print "Trying rule " + 'range'
                        result_peg_180 = rule_range(stream, result_peg_180.getPosition())
                        if result_peg_180 == None:
                            
                            result_peg_180 = Result(save_peg_181)
                            # print "Trying rule " + 'string'
                            result_peg_180 = rule_string(stream, result_peg_180.getPosition())
                            if result_peg_180 == None:
                                
                                result_peg_180 = Result(save_peg_181)
                                # print "Trying rule " + 'line'
                                result_peg_180 = rule_line(stream, result_peg_180.getPosition())
                                if result_peg_180 == None:
                                    
                                    result_peg_180 = Result(save_peg_181)
                                    # print "Trying rule " + 'ascii'
                                    result_peg_180 = rule_ascii(stream, result_peg_180.getPosition())
                                    if result_peg_180 == None:
                                        
                                        result_peg_180 = Result(save_peg_181)
                                        # print "Trying rule " + 'utf8'
                                        result_peg_180 = rule_utf8(stream, result_peg_180.getPosition())
                                        if result_peg_180 == None:
                                            
                                            result_peg_180 = Result(save_peg_181)
                                            # print "Trying rule " + 'predicate'
                                            result_peg_180 = rule_predicate(stream, result_peg_180.getPosition())
                                            if result_peg_180 == None:
                                                
                                                result_peg_180 = Result(save_peg_181)
                                                # print "Trying rule " + 'call_rule'
                                                result_peg_180 = rule_call_rule(stream, result_peg_180.getPosition())
                                                if result_peg_180 == None:
                                                    
                                                    result_peg_180 = Result(save_peg_181)
                                                    # print "Trying rule " + 'sub_pattern'
                                                    result_peg_180 = rule_sub_pattern(stream, result_peg_180.getPosition())
                                                    if result_peg_180 == None:
                                                        
                                                        result_peg_180 = Result(save_peg_181)
                                                        # print "Trying rule " + 'code'
                                                        result_peg_180 = rule_code(stream, result_peg_180.getPosition())
                                                        if result_peg_180 == None:
                                                            raise PegError
        pattern = result_peg_180.getValues()
        
        result_peg_175.addResult(result_peg_180);
        
        result_peg_182 = Result(result_peg_175.getPosition())
        
        save_peg_183 = result_peg_182.getPosition()
        
        # print "Trying rule " + 'modifier'
        result_peg_182 = rule_modifier(stream, result_peg_182.getPosition())
        if result_peg_182 == None:
            
            result_peg_182 = Result(save_peg_183)
            result_peg_182.setValue(None)
        modifier = result_peg_182.getValues()
        
        result_peg_175.addResult(result_peg_182);
        
        result_peg_184 = Result(result_peg_175.getPosition())
        
        value = None
        values = result_peg_175.getValues()
        if modifier != None:
            pattern = modifier(pattern)
        if pnot != None:
            pattern = peg.PatternNot(pattern)
        if ensure != None:
            pattern = peg.PatternEnsure(pattern)
        value = pattern
        result_peg_184.setValue(value)
        
        result_peg_175.addResult(result_peg_184);
        
        result_peg_175.setValue(result_peg_175.getLastValue())
        stream.update(RULE_item, position, result_peg_175)
        return result_peg_175
    except PegError:
        pass
    stream.update(RULE_item, position, None)
    return None


def rule_failure(stream, position):
    if stream.hasResult(RULE_failure, position):
        return stream.result(RULE_failure, position)
    try:
        result_peg_185 = Result(position)
        result_peg_186 = Result(result_peg_185.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_186 = rule_whitespace(stream, result_peg_186.getPosition())
        if result_peg_186 == None:
            raise PegError
        
        result_peg_185.addResult(result_peg_186);
        
        result_peg_187 = Result(result_peg_185.getPosition())
        
        if '<fail>' == stream.get(result_peg_187.getPosition(), 6):
            result_peg_187.nextPosition(6)
            result_peg_187.setValue('<fail>')
        else:
            raise PegError
        
        result_peg_185.addResult(result_peg_187);
        
        result_peg_188 = Result(result_peg_185.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_188 = rule_spaces(stream, result_peg_188.getPosition())
        if result_peg_188 == None:
            raise PegError
        
        result_peg_185.addResult(result_peg_188);
        
        result_peg_189 = Result(result_peg_185.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_189 = rule_code(stream, result_peg_189.getPosition())
        if result_peg_189 == None:
            raise PegError
        code = result_peg_189.getValues()
        
        result_peg_185.addResult(result_peg_189);
        
        result_peg_190 = Result(result_peg_185.getPosition())
        
        value = None
        values = result_peg_185.getValues()
        value = code.code
        result_peg_190.setValue(value)
        
        result_peg_185.addResult(result_peg_190);
        
        result_peg_185.setValue(result_peg_185.getLastValue())
        stream.update(RULE_failure, position, result_peg_185)
        return result_peg_185
    except PegError:
        pass
    stream.update(RULE_failure, position, None)
    return None


def rule_line(stream, position):
    if stream.hasResult(RULE_line, position):
        return stream.result(RULE_line, position)
    try:
        result_peg_191 = Result(position)
        result_peg_192 = Result(result_peg_191.getPosition())
        
        if '<line>' == stream.get(result_peg_192.getPosition(), 6):
            result_peg_192.nextPosition(6)
            result_peg_192.setValue('<line>')
        else:
            raise PegError
        
        result_peg_191.addResult(result_peg_192);
        
        result_peg_193 = Result(result_peg_191.getPosition())
        
        value = None
        values = result_peg_191.getValues()
        value = peg.PatternLine()
        result_peg_193.setValue(value)
        
        result_peg_191.addResult(result_peg_193);
        
        result_peg_191.setValue(result_peg_191.getLastValue())
        stream.update(RULE_line, position, result_peg_191)
        return result_peg_191
    except PegError:
        pass
    stream.update(RULE_line, position, None)
    return None


def rule_predicate(stream, position):
    if stream.hasResult(RULE_predicate, position):
        return stream.result(RULE_predicate, position)
    try:
        result_peg_194 = Result(position)
        result_peg_195 = Result(result_peg_194.getPosition())
        
        if '<predicate' == stream.get(result_peg_195.getPosition(), 10):
            result_peg_195.nextPosition(10)
            result_peg_195.setValue('<predicate')
        else:
            raise PegError
        
        result_peg_194.addResult(result_peg_195);
        
        result_peg_196 = Result(result_peg_194.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_196 = rule_whitespace(stream, result_peg_196.getPosition())
        if result_peg_196 == None:
            raise PegError
        
        result_peg_194.addResult(result_peg_196);
        
        result_peg_197 = Result(result_peg_194.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_197 = rule_word(stream, result_peg_197.getPosition())
        if result_peg_197 == None:
            raise PegError
        variable = result_peg_197.getValues()
        
        result_peg_194.addResult(result_peg_197);
        
        result_peg_198 = Result(result_peg_194.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_198 = rule_whitespace(stream, result_peg_198.getPosition())
        if result_peg_198 == None:
            raise PegError
        
        result_peg_194.addResult(result_peg_198);
        
        result_peg_199 = Result(result_peg_194.getPosition())
        
        if '>' == stream.get(result_peg_199.getPosition(), 1):
            result_peg_199.nextPosition(1)
            result_peg_199.setValue('>')
        else:
            raise PegError
        
        result_peg_194.addResult(result_peg_199);
        
        result_peg_200 = Result(result_peg_194.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_200 = rule_whitespace(stream, result_peg_200.getPosition())
        if result_peg_200 == None:
            raise PegError
        
        result_peg_194.addResult(result_peg_200);
        
        result_peg_201 = Result(result_peg_194.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_201 = rule_code(stream, result_peg_201.getPosition())
        if result_peg_201 == None:
            raise PegError
        code = result_peg_201.getValues()
        
        result_peg_194.addResult(result_peg_201);
        
        result_peg_202 = Result(result_peg_194.getPosition())
        
        value = None
        values = result_peg_194.getValues()
        value = peg.PatternPredicate(variable, code.code)
        result_peg_202.setValue(value)
        
        result_peg_194.addResult(result_peg_202);
        
        result_peg_194.setValue(result_peg_194.getLastValue())
        stream.update(RULE_predicate, position, result_peg_194)
        return result_peg_194
    except PegError:
        pass
    stream.update(RULE_predicate, position, None)
    return None


def rule_utf8(stream, position):
    if stream.hasResult(RULE_utf8, position):
        return stream.result(RULE_utf8, position)
    try:
        result_peg_203 = Result(position)
        result_peg_204 = Result(result_peg_203.getPosition())
        
        if '<utf8' == stream.get(result_peg_204.getPosition(), 5):
            result_peg_204.nextPosition(5)
            result_peg_204.setValue('<utf8')
        else:
            raise PegError
        
        result_peg_203.addResult(result_peg_204);
        
        result_peg_205 = Result(result_peg_203.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_205 = rule_spaces(stream, result_peg_205.getPosition())
        if result_peg_205 == None:
            raise PegError
        
        result_peg_203.addResult(result_peg_205);
        
        result_peg_206 = Result(result_peg_203.getPosition())
        
        # print "Trying rule " + 'hex_number'
        result_peg_206 = rule_hex_number(stream, result_peg_206.getPosition())
        if result_peg_206 == None:
            raise PegError
        num = result_peg_206.getValues()
        
        result_peg_203.addResult(result_peg_206);
        
        result_peg_207 = Result(result_peg_203.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_207 = rule_spaces(stream, result_peg_207.getPosition())
        if result_peg_207 == None:
            raise PegError
        
        result_peg_203.addResult(result_peg_207);
        
        result_peg_208 = Result(result_peg_203.getPosition())
        
        if '>' == stream.get(result_peg_208.getPosition(), 1):
            result_peg_208.nextPosition(1)
            result_peg_208.setValue('>')
        else:
            raise PegError
        
        result_peg_203.addResult(result_peg_208);
        
        result_peg_209 = Result(result_peg_203.getPosition())
        
        value = None
        values = result_peg_203.getValues()
        value = peg.createUtf8Pattern(num)
        result_peg_209.setValue(value)
        
        result_peg_203.addResult(result_peg_209);
        
        result_peg_203.setValue(result_peg_203.getLastValue())
        stream.update(RULE_utf8, position, result_peg_203)
        return result_peg_203
    except PegError:
        pass
    stream.update(RULE_utf8, position, None)
    return None


def rule_ascii(stream, position):
    if stream.hasResult(RULE_ascii, position):
        return stream.result(RULE_ascii, position)
    try:
        result_peg_210 = Result(position)
        result_peg_211 = Result(result_peg_210.getPosition())
        
        if '<ascii' == stream.get(result_peg_211.getPosition(), 6):
            result_peg_211.nextPosition(6)
            result_peg_211.setValue('<ascii')
        else:
            raise PegError
        
        result_peg_210.addResult(result_peg_211);
        
        result_peg_212 = Result(result_peg_210.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_212 = rule_spaces(stream, result_peg_212.getPosition())
        if result_peg_212 == None:
            raise PegError
        
        result_peg_210.addResult(result_peg_212);
        
        result_peg_213 = Result(result_peg_210.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_213 = rule_number(stream, result_peg_213.getPosition())
        if result_peg_213 == None:
            raise PegError
        num = result_peg_213.getValues()
        
        result_peg_210.addResult(result_peg_213);
        
        result_peg_214 = Result(result_peg_210.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_214 = rule_spaces(stream, result_peg_214.getPosition())
        if result_peg_214 == None:
            raise PegError
        
        result_peg_210.addResult(result_peg_214);
        
        result_peg_215 = Result(result_peg_210.getPosition())
        
        if '>' == stream.get(result_peg_215.getPosition(), 1):
            result_peg_215.nextPosition(1)
            result_peg_215.setValue('>')
        else:
            raise PegError
        
        result_peg_210.addResult(result_peg_215);
        
        result_peg_216 = Result(result_peg_210.getPosition())
        
        value = None
        values = result_peg_210.getValues()
        value = peg.PatternVerbatim(int(num))
        result_peg_216.setValue(value)
        
        result_peg_210.addResult(result_peg_216);
        
        result_peg_210.setValue(result_peg_210.getLastValue())
        stream.update(RULE_ascii, position, result_peg_210)
        return result_peg_210
    except PegError:
        pass
    stream.update(RULE_ascii, position, None)
    return None


def rule_call_rule(stream, position):
    if stream.hasResult(RULE_call_rule, position):
        return stream.result(RULE_call_rule, position)
    try:
        result_peg_217 = Result(position)
        result_peg_218 = Result(result_peg_217.getPosition())
        
        if '@' == stream.get(result_peg_218.getPosition(), 1):
            result_peg_218.nextPosition(1)
            result_peg_218.setValue('@')
        else:
            raise PegError
        
        result_peg_217.addResult(result_peg_218);
        
        result_peg_219 = Result(result_peg_217.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_219 = rule_word(stream, result_peg_219.getPosition())
        if result_peg_219 == None:
            raise PegError
        name = result_peg_219.getValues()
        
        result_peg_217.addResult(result_peg_219);
        
        result_peg_220 = Result(result_peg_217.getPosition())
        
        save_peg_221 = result_peg_220.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_220 = rule_parameters_rules(stream, result_peg_220.getPosition())
        if result_peg_220 == None:
            
            result_peg_220 = Result(save_peg_221)
            result_peg_220.setValue(None)
        rule_parameters = result_peg_220.getValues()
        
        result_peg_217.addResult(result_peg_220);
        
        result_peg_222 = Result(result_peg_217.getPosition())
        
        save_peg_223 = result_peg_222.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_222 = rule_parameters_values(stream, result_peg_222.getPosition())
        if result_peg_222 == None:
            
            result_peg_222 = Result(save_peg_223)
            result_peg_222.setValue(None)
        parameters = result_peg_222.getValues()
        
        result_peg_217.addResult(result_peg_222);
        
        result_peg_224 = Result(result_peg_217.getPosition())
        
        value = None
        values = result_peg_217.getValues()
        value = peg.PatternCallRule(name, rule_parameters, parameters)
        result_peg_224.setValue(value)
        
        result_peg_217.addResult(result_peg_224);
        
        result_peg_217.setValue(result_peg_217.getLastValue())
        stream.update(RULE_call_rule, position, result_peg_217)
        return result_peg_217
    except PegError:
        pass
    stream.update(RULE_call_rule, position, None)
    return None


def rule_eof(stream, position):
    if stream.hasResult(RULE_eof, position):
        return stream.result(RULE_eof, position)
    try:
        result_peg_225 = Result(position)
        result_peg_226 = Result(result_peg_225.getPosition())
        
        if '<eof>' == stream.get(result_peg_226.getPosition(), 5):
            result_peg_226.nextPosition(5)
            result_peg_226.setValue('<eof>')
        else:
            raise PegError
        
        result_peg_225.addResult(result_peg_226);
        
        result_peg_227 = Result(result_peg_225.getPosition())
        
        value = None
        values = result_peg_225.getValues()
        value = peg.PatternEof()
        result_peg_227.setValue(value)
        
        result_peg_225.addResult(result_peg_227);
        
        result_peg_225.setValue(result_peg_225.getLastValue())
        stream.update(RULE_eof, position, result_peg_225)
        return result_peg_225
    except PegError:
        pass
    stream.update(RULE_eof, position, None)
    return None


def rule_void(stream, position):
    if stream.hasResult(RULE_void, position):
        return stream.result(RULE_void, position)
    try:
        result_peg_228 = Result(position)
        result_peg_229 = Result(result_peg_228.getPosition())
        
        if '<void>' == stream.get(result_peg_229.getPosition(), 6):
            result_peg_229.nextPosition(6)
            result_peg_229.setValue('<void>')
        else:
            raise PegError
        
        result_peg_228.addResult(result_peg_229);
        
        result_peg_230 = Result(result_peg_228.getPosition())
        
        value = None
        values = result_peg_228.getValues()
        value = peg.PatternVoid()
        result_peg_230.setValue(value)
        
        result_peg_228.addResult(result_peg_230);
        
        result_peg_228.setValue(result_peg_228.getLastValue())
        stream.update(RULE_void, position, result_peg_228)
        return result_peg_228
    except PegError:
        pass
    stream.update(RULE_void, position, None)
    return None


def rule_range(stream, position):
    if stream.hasResult(RULE_range, position):
        return stream.result(RULE_range, position)
    try:
        result_peg_231 = Result(position)
        result_peg_232 = Result(result_peg_231.getPosition())
        
        if '[' == stream.get(result_peg_232.getPosition(), 1):
            result_peg_232.nextPosition(1)
            result_peg_232.setValue('[')
        else:
            raise PegError
        
        result_peg_231.addResult(result_peg_232);
        
        result_peg_233 = Result(result_peg_231.getPosition())
        
        try:
            while True:
                result_peg_234 = Result(result_peg_233.getPosition());
                result_peg_235 = Result(result_peg_234.getPosition())
                
                result_peg_236 = Result(result_peg_235.getPosition());
                try:
                    if ']' == stream.get(result_peg_236.getPosition(), 1):
                        result_peg_236.nextPosition(1)
                        result_peg_236.setValue(']')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_235.setValue(None)
                        
                result_peg_234.addResult(result_peg_235);
                
                result_peg_237 = Result(result_peg_234.getPosition())
                
                temp_peg_238 = stream.get(result_peg_237.getPosition())
                if temp_peg_238 != chr(0):
                    result_peg_237.setValue(temp_peg_238)
                    result_peg_237.nextPosition()
                else:
                    raise PegError
                
                result_peg_234.addResult(result_peg_237);
                
                result_peg_239 = Result(result_peg_234.getPosition())
                
                value = None
                values = result_peg_234.getValues()
                value = values[1]
                result_peg_239.setValue(value)
                
                result_peg_234.addResult(result_peg_239);
                
                result_peg_234.setValue(result_peg_234.getLastValue())
                result_peg_233.addResult(result_peg_234);
        except PegError:
            pass
                
        result_peg_231.addResult(result_peg_233);
        
        result_peg_240 = Result(result_peg_231.getPosition())
        
        if ']' == stream.get(result_peg_240.getPosition(), 1):
            result_peg_240.nextPosition(1)
            result_peg_240.setValue(']')
        else:
            raise PegError
        
        result_peg_231.addResult(result_peg_240);
        
        result_peg_241 = Result(result_peg_231.getPosition())
        
        value = None
        values = result_peg_231.getValues()
        value = peg.PatternRange(''.join(values[1]))
        result_peg_241.setValue(value)
        
        result_peg_231.addResult(result_peg_241);
        
        result_peg_231.setValue(result_peg_231.getLastValue())
        stream.update(RULE_range, position, result_peg_231)
        return result_peg_231
    except PegError:
        pass
    stream.update(RULE_range, position, None)
    return None


def rule_sub_pattern(stream, position):
    if stream.hasResult(RULE_sub_pattern, position):
        return stream.result(RULE_sub_pattern, position)
    try:
        result_peg_242 = Result(position)
        result_peg_243 = Result(result_peg_242.getPosition())
        
        if '(' == stream.get(result_peg_243.getPosition(), 1):
            result_peg_243.nextPosition(1)
            result_peg_243.setValue('(')
        else:
            raise PegError
        
        result_peg_242.addResult(result_peg_243);
        
        result_peg_244 = Result(result_peg_242.getPosition())
        
        try:
            while True:
                result_peg_245 = Result(result_peg_244.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_245 = rule_pattern(stream, result_peg_245.getPosition())
                if result_peg_245 == None:
                    raise PegError
                result_peg_244.addResult(result_peg_245);
        except PegError:
            if result_peg_244.matches() == 0:
                raise PegError
                
        result_peg_242.addResult(result_peg_244);
        
        result_peg_246 = Result(result_peg_242.getPosition())
        
        if ')' == stream.get(result_peg_246.getPosition(), 1):
            result_peg_246.nextPosition(1)
            result_peg_246.setValue(')')
        else:
            raise PegError
        
        result_peg_242.addResult(result_peg_246);
        
        result_peg_247 = Result(result_peg_242.getPosition())
        
        value = None
        values = result_peg_242.getValues()
        value = peg.PatternSequence(values[1])
        result_peg_247.setValue(value)
        
        result_peg_242.addResult(result_peg_247);
        
        result_peg_242.setValue(result_peg_242.getLastValue())
        stream.update(RULE_sub_pattern, position, result_peg_242)
        return result_peg_242
    except PegError:
        pass
    stream.update(RULE_sub_pattern, position, None)
    return None


def rule_bind(stream, position):
    if stream.hasResult(RULE_bind, position):
        return stream.result(RULE_bind, position)
    try:
        result_peg_248 = Result(position)
        result_peg_249 = Result(result_peg_248.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_249 = rule_word(stream, result_peg_249.getPosition())
        if result_peg_249 == None:
            raise PegError
        name = result_peg_249.getValues()
        
        result_peg_248.addResult(result_peg_249);
        
        result_peg_250 = Result(result_peg_248.getPosition())
        
        if ':' == stream.get(result_peg_250.getPosition(), 1):
            result_peg_250.nextPosition(1)
            result_peg_250.setValue(':')
        else:
            raise PegError
        
        result_peg_248.addResult(result_peg_250);
        
        result_peg_251 = Result(result_peg_248.getPosition())
        
        value = None
        values = result_peg_248.getValues()
        value = lambda p: peg.PatternBind(name, p)
        result_peg_251.setValue(value)
        
        result_peg_248.addResult(result_peg_251);
        
        result_peg_248.setValue(result_peg_248.getLastValue())
        stream.update(RULE_bind, position, result_peg_248)
        return result_peg_248
    except PegError:
        pass
    stream.update(RULE_bind, position, None)
    return None


def rule_string(stream, position):
    if stream.hasResult(RULE_string, position):
        return stream.result(RULE_string, position)
    try:
        result_peg_252 = Result(position)
        result_peg_253 = Result(result_peg_252.getPosition())
        
        if '"' == stream.get(result_peg_253.getPosition(), 1):
            result_peg_253.nextPosition(1)
            result_peg_253.setValue('"')
        else:
            raise PegError
        
        result_peg_252.addResult(result_peg_253);
        
        result_peg_254 = Result(result_peg_252.getPosition())
        
        try:
            while True:
                result_peg_255 = Result(result_peg_254.getPosition());
                result_peg_256 = Result(result_peg_255.getPosition())
                
                result_peg_257 = Result(result_peg_256.getPosition());
                try:
                    if '"' == stream.get(result_peg_257.getPosition(), 1):
                        result_peg_257.nextPosition(1)
                        result_peg_257.setValue('"')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_256.setValue(None)
                        
                result_peg_255.addResult(result_peg_256);
                
                result_peg_258 = Result(result_peg_255.getPosition())
                
                temp_peg_259 = stream.get(result_peg_258.getPosition())
                if temp_peg_259 != chr(0):
                    result_peg_258.setValue(temp_peg_259)
                    result_peg_258.nextPosition()
                else:
                    raise PegError
                
                result_peg_255.addResult(result_peg_258);
                
                result_peg_260 = Result(result_peg_255.getPosition())
                
                value = None
                values = result_peg_255.getValues()
                value = values[1]
                result_peg_260.setValue(value)
                
                result_peg_255.addResult(result_peg_260);
                
                result_peg_255.setValue(result_peg_255.getLastValue())
                result_peg_254.addResult(result_peg_255);
        except PegError:
            pass
                
        result_peg_252.addResult(result_peg_254);
        
        result_peg_261 = Result(result_peg_252.getPosition())
        
        if '"' == stream.get(result_peg_261.getPosition(), 1):
            result_peg_261.nextPosition(1)
            result_peg_261.setValue('"')
        else:
            raise PegError
        
        result_peg_252.addResult(result_peg_261);
        
        result_peg_262 = Result(result_peg_252.getPosition())
        
        save_peg_263 = result_peg_262.getPosition()
        
        if '{case}' == stream.get(result_peg_262.getPosition(), 6):
            result_peg_262.nextPosition(6)
            result_peg_262.setValue('{case}')
        else:
            
            result_peg_262 = Result(save_peg_263)
            result_peg_262.setValue(None)
        options = result_peg_262.getValues()
        
        result_peg_252.addResult(result_peg_262);
        
        result_peg_264 = Result(result_peg_252.getPosition())
        
        value = None
        values = result_peg_252.getValues()
        value = peg.PatternVerbatim(''.join(values[1]), options)
        result_peg_264.setValue(value)
        
        result_peg_252.addResult(result_peg_264);
        
        result_peg_252.setValue(result_peg_252.getLastValue())
        stream.update(RULE_string, position, result_peg_252)
        return result_peg_252
    except PegError:
        pass
    try:
        result_peg_265 = Result(position)
        result_peg_266 = Result(result_peg_265.getPosition())
        
        if '<quote>' == stream.get(result_peg_266.getPosition(), 7):
            result_peg_266.nextPosition(7)
            result_peg_266.setValue('<quote>')
        else:
            raise PegError
        
        result_peg_265.addResult(result_peg_266);
        
        result_peg_267 = Result(result_peg_265.getPosition())
        
        value = None
        values = result_peg_265.getValues()
        value = peg.PatternVerbatim('"')
        result_peg_267.setValue(value)
        
        result_peg_265.addResult(result_peg_267);
        
        result_peg_265.setValue(result_peg_265.getLastValue())
        stream.update(RULE_string, position, result_peg_265)
        return result_peg_265
    except PegError:
        pass
    stream.update(RULE_string, position, None)
    return None


def rule_modifier(stream, position):
    if stream.hasResult(RULE_modifier, position):
        return stream.result(RULE_modifier, position)
    try:
        result_peg_268 = Result(position)
        result_peg_269 = Result(result_peg_268.getPosition())
        
        if '*' == stream.get(result_peg_269.getPosition(), 1):
            result_peg_269.nextPosition(1)
            result_peg_269.setValue('*')
        else:
            raise PegError
        
        result_peg_268.addResult(result_peg_269);
        
        result_peg_270 = Result(result_peg_268.getPosition())
        
        value = None
        values = result_peg_268.getValues()
        value = lambda p: peg.PatternRepeatMany(p)
        result_peg_270.setValue(value)
        
        result_peg_268.addResult(result_peg_270);
        
        result_peg_268.setValue(result_peg_268.getLastValue())
        stream.update(RULE_modifier, position, result_peg_268)
        return result_peg_268
    except PegError:
        pass
    try:
        result_peg_271 = Result(position)
        result_peg_272 = Result(result_peg_271.getPosition())
        
        if '?' == stream.get(result_peg_272.getPosition(), 1):
            result_peg_272.nextPosition(1)
            result_peg_272.setValue('?')
        else:
            raise PegError
        
        result_peg_271.addResult(result_peg_272);
        
        result_peg_273 = Result(result_peg_271.getPosition())
        
        value = None
        values = result_peg_271.getValues()
        value = lambda p: peg.PatternMaybe(p)
        result_peg_273.setValue(value)
        
        result_peg_271.addResult(result_peg_273);
        
        result_peg_271.setValue(result_peg_271.getLastValue())
        stream.update(RULE_modifier, position, result_peg_271)
        return result_peg_271
    except PegError:
        pass
    try:
        result_peg_274 = Result(position)
        result_peg_275 = Result(result_peg_274.getPosition())
        
        if '+' == stream.get(result_peg_275.getPosition(), 1):
            result_peg_275.nextPosition(1)
            result_peg_275.setValue('+')
        else:
            raise PegError
        
        result_peg_274.addResult(result_peg_275);
        
        result_peg_276 = Result(result_peg_274.getPosition())
        
        value = None
        values = result_peg_274.getValues()
        value = lambda p: peg.PatternRepeatOnce(p)
        result_peg_276.setValue(value)
        
        result_peg_274.addResult(result_peg_276);
        
        result_peg_274.setValue(result_peg_274.getLastValue())
        stream.update(RULE_modifier, position, result_peg_274)
        return result_peg_274
    except PegError:
        pass
    stream.update(RULE_modifier, position, None)
    return None


def rule_x_word(stream, position):
    if stream.hasResult(RULE_x_word, position):
        return stream.result(RULE_x_word, position)
    try:
        result_peg_277 = Result(position)
        result_peg_278 = Result(result_peg_277.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_278 = rule_word(stream, result_peg_278.getPosition())
        if result_peg_278 == None:
            raise PegError
        name = result_peg_278.getValues()
        
        result_peg_277.addResult(result_peg_278);
        
        result_peg_279 = Result(result_peg_277.getPosition())
        
        save_peg_280 = result_peg_279.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_279 = rule_parameters_rules(stream, result_peg_279.getPosition())
        if result_peg_279 == None:
            
            result_peg_279 = Result(save_peg_280)
            result_peg_279.setValue(None)
        rule_parameters = result_peg_279.getValues()
        
        result_peg_277.addResult(result_peg_279);
        
        result_peg_281 = Result(result_peg_277.getPosition())
        
        save_peg_282 = result_peg_281.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_281 = rule_parameters_values(stream, result_peg_281.getPosition())
        if result_peg_281 == None:
            
            result_peg_281 = Result(save_peg_282)
            result_peg_281.setValue(None)
        parameters = result_peg_281.getValues()
        
        result_peg_277.addResult(result_peg_281);
        
        result_peg_283 = Result(result_peg_277.getPosition())
        
        value = None
        values = result_peg_277.getValues()
        value = peg.PatternRule(name, rule_parameters, parameters)
        result_peg_283.setValue(value)
        
        result_peg_277.addResult(result_peg_283);
        
        result_peg_277.setValue(result_peg_277.getLastValue())
        stream.update(RULE_x_word, position, result_peg_277)
        return result_peg_277
    except PegError:
        pass
    stream.update(RULE_x_word, position, None)
    return None


def rule_rule_parameters(stream, position):
    if stream.hasResult(RULE_rule_parameters, position):
        return stream.result(RULE_rule_parameters, position)
    try:
        result_peg_284 = Result(position)
        result_peg_285 = Result(result_peg_284.getPosition())
        
        if '[' == stream.get(result_peg_285.getPosition(), 1):
            result_peg_285.nextPosition(1)
            result_peg_285.setValue('[')
        else:
            raise PegError
        
        result_peg_284.addResult(result_peg_285);
        
        result_peg_286 = Result(result_peg_284.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_286 = rule_spaces(stream, result_peg_286.getPosition())
        if result_peg_286 == None:
            raise PegError
        
        result_peg_284.addResult(result_peg_286);
        
        result_peg_287 = Result(result_peg_284.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_287 = rule_word(stream, result_peg_287.getPosition())
        if result_peg_287 == None:
            raise PegError
        param1 = result_peg_287.getValues()
        
        result_peg_284.addResult(result_peg_287);
        
        result_peg_288 = Result(result_peg_284.getPosition())
        
        try:
            while True:
                result_peg_289 = Result(result_peg_288.getPosition());
                result_peg_290 = Result(result_peg_289.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_290 = rule_spaces(stream, result_peg_290.getPosition())
                if result_peg_290 == None:
                    raise PegError
                
                result_peg_289.addResult(result_peg_290);
                
                result_peg_291 = Result(result_peg_289.getPosition())
                
                if ',' == stream.get(result_peg_291.getPosition(), 1):
                    result_peg_291.nextPosition(1)
                    result_peg_291.setValue(',')
                else:
                    raise PegError
                
                result_peg_289.addResult(result_peg_291);
                
                result_peg_292 = Result(result_peg_289.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_292 = rule_spaces(stream, result_peg_292.getPosition())
                if result_peg_292 == None:
                    raise PegError
                
                result_peg_289.addResult(result_peg_292);
                
                result_peg_293 = Result(result_peg_289.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_293 = rule_word(stream, result_peg_293.getPosition())
                if result_peg_293 == None:
                    raise PegError
                exp = result_peg_293.getValues()
                
                result_peg_289.addResult(result_peg_293);
                
                result_peg_294 = Result(result_peg_289.getPosition())
                
                value = None
                values = result_peg_289.getValues()
                value = exp
                result_peg_294.setValue(value)
                
                result_peg_289.addResult(result_peg_294);
                
                result_peg_289.setValue(result_peg_289.getLastValue())
                result_peg_288.addResult(result_peg_289);
        except PegError:
            pass
        params = result_peg_288.getValues()
        
        result_peg_284.addResult(result_peg_288);
        
        result_peg_295 = Result(result_peg_284.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_295 = rule_spaces(stream, result_peg_295.getPosition())
        if result_peg_295 == None:
            raise PegError
        
        result_peg_284.addResult(result_peg_295);
        
        result_peg_296 = Result(result_peg_284.getPosition())
        
        if ']' == stream.get(result_peg_296.getPosition(), 1):
            result_peg_296.nextPosition(1)
            result_peg_296.setValue(']')
        else:
            raise PegError
        
        result_peg_284.addResult(result_peg_296);
        
        result_peg_297 = Result(result_peg_284.getPosition())
        
        value = None
        values = result_peg_284.getValues()
        value = [param1] + params
        result_peg_297.setValue(value)
        
        result_peg_284.addResult(result_peg_297);
        
        result_peg_284.setValue(result_peg_284.getLastValue())
        stream.update(RULE_rule_parameters, position, result_peg_284)
        return result_peg_284
    except PegError:
        pass
    stream.update(RULE_rule_parameters, position, None)
    return None


def rule_value_parameters(stream, position):
    if stream.hasResult(RULE_value_parameters, position):
        return stream.result(RULE_value_parameters, position)
    try:
        result_peg_298 = Result(position)
        result_peg_299 = Result(result_peg_298.getPosition())
        
        if '(' == stream.get(result_peg_299.getPosition(), 1):
            result_peg_299.nextPosition(1)
            result_peg_299.setValue('(')
        else:
            raise PegError
        
        result_peg_298.addResult(result_peg_299);
        
        result_peg_300 = Result(result_peg_298.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_300 = rule_spaces(stream, result_peg_300.getPosition())
        if result_peg_300 == None:
            raise PegError
        
        result_peg_298.addResult(result_peg_300);
        
        result_peg_301 = Result(result_peg_298.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_301 = rule_word(stream, result_peg_301.getPosition())
        if result_peg_301 == None:
            raise PegError
        param1 = result_peg_301.getValues()
        
        result_peg_298.addResult(result_peg_301);
        
        result_peg_302 = Result(result_peg_298.getPosition())
        
        try:
            while True:
                result_peg_303 = Result(result_peg_302.getPosition());
                result_peg_304 = Result(result_peg_303.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_304 = rule_spaces(stream, result_peg_304.getPosition())
                if result_peg_304 == None:
                    raise PegError
                
                result_peg_303.addResult(result_peg_304);
                
                result_peg_305 = Result(result_peg_303.getPosition())
                
                if ',' == stream.get(result_peg_305.getPosition(), 1):
                    result_peg_305.nextPosition(1)
                    result_peg_305.setValue(',')
                else:
                    raise PegError
                
                result_peg_303.addResult(result_peg_305);
                
                result_peg_306 = Result(result_peg_303.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_306 = rule_spaces(stream, result_peg_306.getPosition())
                if result_peg_306 == None:
                    raise PegError
                
                result_peg_303.addResult(result_peg_306);
                
                result_peg_307 = Result(result_peg_303.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_307 = rule_word(stream, result_peg_307.getPosition())
                if result_peg_307 == None:
                    raise PegError
                exp = result_peg_307.getValues()
                
                result_peg_303.addResult(result_peg_307);
                
                result_peg_308 = Result(result_peg_303.getPosition())
                
                value = None
                values = result_peg_303.getValues()
                value = exp
                result_peg_308.setValue(value)
                
                result_peg_303.addResult(result_peg_308);
                
                result_peg_303.setValue(result_peg_303.getLastValue())
                result_peg_302.addResult(result_peg_303);
        except PegError:
            pass
        params = result_peg_302.getValues()
        
        result_peg_298.addResult(result_peg_302);
        
        result_peg_309 = Result(result_peg_298.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_309 = rule_spaces(stream, result_peg_309.getPosition())
        if result_peg_309 == None:
            raise PegError
        
        result_peg_298.addResult(result_peg_309);
        
        result_peg_310 = Result(result_peg_298.getPosition())
        
        if ')' == stream.get(result_peg_310.getPosition(), 1):
            result_peg_310.nextPosition(1)
            result_peg_310.setValue(')')
        else:
            raise PegError
        
        result_peg_298.addResult(result_peg_310);
        
        result_peg_311 = Result(result_peg_298.getPosition())
        
        value = None
        values = result_peg_298.getValues()
        value = [param1] + params
        result_peg_311.setValue(value)
        
        result_peg_298.addResult(result_peg_311);
        
        result_peg_298.setValue(result_peg_298.getLastValue())
        stream.update(RULE_value_parameters, position, result_peg_298)
        return result_peg_298
    except PegError:
        pass
    stream.update(RULE_value_parameters, position, None)
    return None


def rule_parameters_rules(stream, position):
    if stream.hasResult(RULE_parameters_rules, position):
        return stream.result(RULE_parameters_rules, position)
    try:
        result_peg_312 = Result(position)
        result_peg_313 = Result(result_peg_312.getPosition())
        
        if '[' == stream.get(result_peg_313.getPosition(), 1):
            result_peg_313.nextPosition(1)
            result_peg_313.setValue('[')
        else:
            raise PegError
        
        result_peg_312.addResult(result_peg_313);
        
        result_peg_314 = Result(result_peg_312.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_314 = rule_spaces(stream, result_peg_314.getPosition())
        if result_peg_314 == None:
            raise PegError
        
        result_peg_312.addResult(result_peg_314);
        
        result_peg_315 = Result(result_peg_312.getPosition())
        
        # print "Trying rule " + 'word_or_at'
        result_peg_315 = rule_word_or_at(stream, result_peg_315.getPosition())
        if result_peg_315 == None:
            raise PegError
        param1 = result_peg_315.getValues()
        
        result_peg_312.addResult(result_peg_315);
        
        result_peg_316 = Result(result_peg_312.getPosition())
        
        try:
            while True:
                result_peg_317 = Result(result_peg_316.getPosition());
                result_peg_318 = Result(result_peg_317.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_318 = rule_spaces(stream, result_peg_318.getPosition())
                if result_peg_318 == None:
                    raise PegError
                
                result_peg_317.addResult(result_peg_318);
                
                result_peg_319 = Result(result_peg_317.getPosition())
                
                if ',' == stream.get(result_peg_319.getPosition(), 1):
                    result_peg_319.nextPosition(1)
                    result_peg_319.setValue(',')
                else:
                    raise PegError
                
                result_peg_317.addResult(result_peg_319);
                
                result_peg_320 = Result(result_peg_317.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_320 = rule_spaces(stream, result_peg_320.getPosition())
                if result_peg_320 == None:
                    raise PegError
                
                result_peg_317.addResult(result_peg_320);
                
                result_peg_321 = Result(result_peg_317.getPosition())
                
                # print "Trying rule " + 'word_or_at'
                result_peg_321 = rule_word_or_at(stream, result_peg_321.getPosition())
                if result_peg_321 == None:
                    raise PegError
                exp = result_peg_321.getValues()
                
                result_peg_317.addResult(result_peg_321);
                
                result_peg_322 = Result(result_peg_317.getPosition())
                
                value = None
                values = result_peg_317.getValues()
                value = exp
                result_peg_322.setValue(value)
                
                result_peg_317.addResult(result_peg_322);
                
                result_peg_317.setValue(result_peg_317.getLastValue())
                result_peg_316.addResult(result_peg_317);
        except PegError:
            pass
        params = result_peg_316.getValues()
        
        result_peg_312.addResult(result_peg_316);
        
        result_peg_323 = Result(result_peg_312.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_323 = rule_spaces(stream, result_peg_323.getPosition())
        if result_peg_323 == None:
            raise PegError
        
        result_peg_312.addResult(result_peg_323);
        
        result_peg_324 = Result(result_peg_312.getPosition())
        
        if ']' == stream.get(result_peg_324.getPosition(), 1):
            result_peg_324.nextPosition(1)
            result_peg_324.setValue(']')
        else:
            raise PegError
        
        result_peg_312.addResult(result_peg_324);
        
        result_peg_325 = Result(result_peg_312.getPosition())
        
        value = None
        values = result_peg_312.getValues()
        value = [param1] + params
        result_peg_325.setValue(value)
        
        result_peg_312.addResult(result_peg_325);
        
        result_peg_312.setValue(result_peg_312.getLastValue())
        stream.update(RULE_parameters_rules, position, result_peg_312)
        return result_peg_312
    except PegError:
        pass
    stream.update(RULE_parameters_rules, position, None)
    return None


def rule_parameters_values(stream, position):
    if stream.hasResult(RULE_parameters_values, position):
        return stream.result(RULE_parameters_values, position)
    try:
        result_peg_326 = Result(position)
        result_peg_327 = Result(result_peg_326.getPosition())
        
        if '(' == stream.get(result_peg_327.getPosition(), 1):
            result_peg_327.nextPosition(1)
            result_peg_327.setValue('(')
        else:
            raise PegError
        
        result_peg_326.addResult(result_peg_327);
        
        result_peg_328 = Result(result_peg_326.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_328 = rule_spaces(stream, result_peg_328.getPosition())
        if result_peg_328 == None:
            raise PegError
        
        result_peg_326.addResult(result_peg_328);
        
        result_peg_329 = Result(result_peg_326.getPosition())
        
        # print "Trying rule " + 'word_or_dollar'
        result_peg_329 = rule_word_or_dollar(stream, result_peg_329.getPosition())
        if result_peg_329 == None:
            raise PegError
        param1 = result_peg_329.getValues()
        
        result_peg_326.addResult(result_peg_329);
        
        result_peg_330 = Result(result_peg_326.getPosition())
        
        try:
            while True:
                result_peg_331 = Result(result_peg_330.getPosition());
                result_peg_332 = Result(result_peg_331.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_332 = rule_spaces(stream, result_peg_332.getPosition())
                if result_peg_332 == None:
                    raise PegError
                
                result_peg_331.addResult(result_peg_332);
                
                result_peg_333 = Result(result_peg_331.getPosition())
                
                if ',' == stream.get(result_peg_333.getPosition(), 1):
                    result_peg_333.nextPosition(1)
                    result_peg_333.setValue(',')
                else:
                    raise PegError
                
                result_peg_331.addResult(result_peg_333);
                
                result_peg_334 = Result(result_peg_331.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_334 = rule_spaces(stream, result_peg_334.getPosition())
                if result_peg_334 == None:
                    raise PegError
                
                result_peg_331.addResult(result_peg_334);
                
                result_peg_335 = Result(result_peg_331.getPosition())
                
                # print "Trying rule " + 'word_or_dollar'
                result_peg_335 = rule_word_or_dollar(stream, result_peg_335.getPosition())
                if result_peg_335 == None:
                    raise PegError
                exp = result_peg_335.getValues()
                
                result_peg_331.addResult(result_peg_335);
                
                result_peg_336 = Result(result_peg_331.getPosition())
                
                value = None
                values = result_peg_331.getValues()
                value = exp
                result_peg_336.setValue(value)
                
                result_peg_331.addResult(result_peg_336);
                
                result_peg_331.setValue(result_peg_331.getLastValue())
                result_peg_330.addResult(result_peg_331);
        except PegError:
            pass
        params = result_peg_330.getValues()
        
        result_peg_326.addResult(result_peg_330);
        
        result_peg_337 = Result(result_peg_326.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_337 = rule_spaces(stream, result_peg_337.getPosition())
        if result_peg_337 == None:
            raise PegError
        
        result_peg_326.addResult(result_peg_337);
        
        result_peg_338 = Result(result_peg_326.getPosition())
        
        if ')' == stream.get(result_peg_338.getPosition(), 1):
            result_peg_338.nextPosition(1)
            result_peg_338.setValue(')')
        else:
            raise PegError
        
        result_peg_326.addResult(result_peg_338);
        
        result_peg_339 = Result(result_peg_326.getPosition())
        
        value = None
        values = result_peg_326.getValues()
        value = [param1] + params
        result_peg_339.setValue(value)
        
        result_peg_326.addResult(result_peg_339);
        
        result_peg_326.setValue(result_peg_326.getLastValue())
        stream.update(RULE_parameters_values, position, result_peg_326)
        return result_peg_326
    except PegError:
        pass
    stream.update(RULE_parameters_values, position, None)
    return None


def rule_word_or_dollar(stream, position):
    if stream.hasResult(RULE_word_or_dollar, position):
        return stream.result(RULE_word_or_dollar, position)
    try:
        result_peg_340 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_340 = rule_word(stream, result_peg_340.getPosition())
        if result_peg_340 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_340)
        return result_peg_340
    except PegError:
        pass
    try:
        result_peg_341 = Result(position)
        # print "Trying rule " + 'dollar'
        result_peg_341 = rule_dollar(stream, result_peg_341.getPosition())
        if result_peg_341 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_341)
        return result_peg_341
    except PegError:
        pass
    stream.update(RULE_word_or_dollar, position, None)
    return None


def rule_word_or_at(stream, position):
    if stream.hasResult(RULE_word_or_at, position):
        return stream.result(RULE_word_or_at, position)
    try:
        result_peg_342 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_342 = rule_word(stream, result_peg_342.getPosition())
        if result_peg_342 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_342)
        return result_peg_342
    except PegError:
        pass
    try:
        result_peg_343 = Result(position)
        # print "Trying rule " + 'word_at'
        result_peg_343 = rule_word_at(stream, result_peg_343.getPosition())
        if result_peg_343 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_343)
        return result_peg_343
    except PegError:
        pass
    stream.update(RULE_word_or_at, position, None)
    return None


def rule_word_at(stream, position):
    if stream.hasResult(RULE_word_at, position):
        return stream.result(RULE_word_at, position)
    try:
        result_peg_344 = Result(position)
        result_peg_345 = Result(result_peg_344.getPosition())
        
        if '@' == stream.get(result_peg_345.getPosition(), 1):
            result_peg_345.nextPosition(1)
            result_peg_345.setValue('@')
        else:
            raise PegError
        
        result_peg_344.addResult(result_peg_345);
        
        result_peg_346 = Result(result_peg_344.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_346 = rule_word(stream, result_peg_346.getPosition())
        if result_peg_346 == None:
            raise PegError
        word = result_peg_346.getValues()
        
        result_peg_344.addResult(result_peg_346);
        
        result_peg_347 = Result(result_peg_344.getPosition())
        
        value = None
        values = result_peg_344.getValues()
        value = '@%s' % word
        result_peg_347.setValue(value)
        
        result_peg_344.addResult(result_peg_347);
        
        result_peg_344.setValue(result_peg_344.getLastValue())
        stream.update(RULE_word_at, position, result_peg_344)
        return result_peg_344
    except PegError:
        pass
    stream.update(RULE_word_at, position, None)
    return None


def rule_dollar(stream, position):
    if stream.hasResult(RULE_dollar, position):
        return stream.result(RULE_dollar, position)
    try:
        result_peg_348 = Result(position)
        result_peg_349 = Result(result_peg_348.getPosition())
        
        if '$' == stream.get(result_peg_349.getPosition(), 1):
            result_peg_349.nextPosition(1)
            result_peg_349.setValue('$')
        else:
            raise PegError
        
        result_peg_348.addResult(result_peg_349);
        
        result_peg_350 = Result(result_peg_348.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_350 = rule_number(stream, result_peg_350.getPosition())
        if result_peg_350 == None:
            raise PegError
        number = result_peg_350.getValues()
        
        result_peg_348.addResult(result_peg_350);
        
        result_peg_351 = Result(result_peg_348.getPosition())
        
        value = None
        values = result_peg_348.getValues()
        value = "$%s" % number
        result_peg_351.setValue(value)
        
        result_peg_348.addResult(result_peg_351);
        
        result_peg_348.setValue(result_peg_348.getLastValue())
        stream.update(RULE_dollar, position, result_peg_348)
        return result_peg_348
    except PegError:
        pass
    stream.update(RULE_dollar, position, None)
    return None


def rule_number(stream, position):
    if stream.hasResult(RULE_number, position):
        return stream.result(RULE_number, position)
    try:
        result_peg_352 = Result(position)
        result_peg_353 = Result(result_peg_352.getPosition())
        
        try:
            while True:
                result_peg_354 = Result(result_peg_353.getPosition());
                # print "Trying rule " + 'digit'
                result_peg_354 = rule_digit(stream, result_peg_354.getPosition())
                if result_peg_354 == None:
                    raise PegError
                result_peg_353.addResult(result_peg_354);
        except PegError:
            if result_peg_353.matches() == 0:
                raise PegError
                
        result_peg_352.addResult(result_peg_353);
        
        result_peg_355 = Result(result_peg_352.getPosition())
        
        value = None
        values = result_peg_352.getValues()
        value = ''.join(values[0])
        result_peg_355.setValue(value)
        
        result_peg_352.addResult(result_peg_355);
        
        result_peg_352.setValue(result_peg_352.getLastValue())
        stream.update(RULE_number, position, result_peg_352)
        return result_peg_352
    except PegError:
        pass
    stream.update(RULE_number, position, None)
    return None


def rule_digit(stream, position):
    if stream.hasResult(RULE_digit, position):
        return stream.result(RULE_digit, position)
    try:
        result_peg_356 = Result(position)
        letter_peg_357 = stream.get(result_peg_356.getPosition())
        if letter_peg_357 in '0123456789':
            result_peg_356.nextPosition()
            result_peg_356.setValue(letter_peg_357)
        else:
            raise PegError
        stream.update(RULE_digit, position, result_peg_356)
        return result_peg_356
    except PegError:
        pass
    stream.update(RULE_digit, position, None)
    return None


def rule_hex_number(stream, position):
    if stream.hasResult(RULE_hex_number, position):
        return stream.result(RULE_hex_number, position)
    try:
        result_peg_358 = Result(position)
        result_peg_359 = Result(result_peg_358.getPosition())
        
        try:
            while True:
                result_peg_360 = Result(result_peg_359.getPosition());
                # print "Trying rule " + 'hex_digit'
                result_peg_360 = rule_hex_digit(stream, result_peg_360.getPosition())
                if result_peg_360 == None:
                    raise PegError
                result_peg_359.addResult(result_peg_360);
        except PegError:
            if result_peg_359.matches() == 0:
                raise PegError
                
        result_peg_358.addResult(result_peg_359);
        
        result_peg_361 = Result(result_peg_358.getPosition())
        
        value = None
        values = result_peg_358.getValues()
        value = ''.join(values[0])
        result_peg_361.setValue(value)
        
        result_peg_358.addResult(result_peg_361);
        
        result_peg_358.setValue(result_peg_358.getLastValue())
        stream.update(RULE_hex_number, position, result_peg_358)
        return result_peg_358
    except PegError:
        pass
    stream.update(RULE_hex_number, position, None)
    return None


def rule_hex_digit(stream, position):
    if stream.hasResult(RULE_hex_digit, position):
        return stream.result(RULE_hex_digit, position)
    try:
        result_peg_362 = Result(position)
        letter_peg_363 = stream.get(result_peg_362.getPosition())
        if letter_peg_363 in '0123456789abcdefABCDEF':
            result_peg_362.nextPosition()
            result_peg_362.setValue(letter_peg_363)
        else:
            raise PegError
        stream.update(RULE_hex_digit, position, result_peg_362)
        return result_peg_362
    except PegError:
        pass
    stream.update(RULE_hex_digit, position, None)
    return None


def rule_start_symbol(stream, position):
    if stream.hasResult(RULE_start_symbol, position):
        return stream.result(RULE_start_symbol, position)
    try:
        result_peg_364 = Result(position)
        result_peg_365 = Result(result_peg_364.getPosition())
        
        if 'start-symbol:' == stream.get(result_peg_365.getPosition(), 13):
            result_peg_365.nextPosition(13)
            result_peg_365.setValue('start-symbol:')
        else:
            raise PegError
        
        result_peg_364.addResult(result_peg_365);
        
        result_peg_366 = Result(result_peg_364.getPosition())
        
        try:
            while True:
                result_peg_367 = Result(result_peg_366.getPosition());
                # print "Trying rule " + 'space'
                result_peg_367 = rule_space(stream, result_peg_367.getPosition())
                if result_peg_367 == None:
                    raise PegError
                result_peg_366.addResult(result_peg_367);
        except PegError:
            pass
                
        result_peg_364.addResult(result_peg_366);
        
        result_peg_368 = Result(result_peg_364.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_368 = rule_word(stream, result_peg_368.getPosition())
        if result_peg_368 == None:
            raise PegError
        
        result_peg_364.addResult(result_peg_368);
        
        result_peg_369 = Result(result_peg_364.getPosition())
        
        value = None
        values = result_peg_364.getValues()
        value = values[2]
        result_peg_369.setValue(value)
        
        result_peg_364.addResult(result_peg_369);
        
        result_peg_364.setValue(result_peg_364.getLastValue())
        stream.update(RULE_start_symbol, position, result_peg_364)
        return result_peg_364
    except PegError:
        pass
    stream.update(RULE_start_symbol, position, None)
    return None


def rule_spaces(stream, position):
    if stream.hasResult(RULE_spaces, position):
        return stream.result(RULE_spaces, position)
    try:
        result_peg_370 = Result(position)
        result_peg_371 = Result(result_peg_370.getPosition())
        
        try:
            while True:
                result_peg_372 = Result(result_peg_371.getPosition());
                # print "Trying rule " + 'space'
                result_peg_372 = rule_space(stream, result_peg_372.getPosition())
                if result_peg_372 == None:
                    raise PegError
                result_peg_371.addResult(result_peg_372);
        except PegError:
            pass
                
        result_peg_370.addResult(result_peg_371);
        
        result_peg_370.setValue(result_peg_370.getLastValue())
        stream.update(RULE_spaces, position, result_peg_370)
        return result_peg_370
    except PegError:
        pass
    stream.update(RULE_spaces, position, None)
    return None


def rule_space(stream, position):
    if stream.hasResult(RULE_space, position):
        return stream.result(RULE_space, position)
    try:
        result_peg_373 = Result(position)
        result_peg_374 = Result(result_peg_373.getPosition())
        
        if ' ' == stream.get(result_peg_374.getPosition(), 1):
            result_peg_374.nextPosition(1)
            result_peg_374.setValue(' ')
        else:
            raise PegError
        
        result_peg_373.addResult(result_peg_374);
        
        result_peg_373.setValue(result_peg_373.getLastValue())
        stream.update(RULE_space, position, result_peg_373)
        return result_peg_373
    except PegError:
        pass
    try:
        result_peg_375 = Result(position)
        result_peg_376 = Result(result_peg_375.getPosition())
        
        if '\t' == stream.get(result_peg_376.getPosition(), 1):
            result_peg_376.nextPosition(1)
            result_peg_376.setValue('\t')
        else:
            raise PegError
        
        result_peg_375.addResult(result_peg_376);
        
        result_peg_375.setValue(result_peg_375.getLastValue())
        stream.update(RULE_space, position, result_peg_375)
        return result_peg_375
    except PegError:
        pass
    stream.update(RULE_space, position, None)
    return None


def rule_any_char(stream, position):
    if stream.hasResult(RULE_any_char, position):
        return stream.result(RULE_any_char, position)
    try:
        result_peg_377 = Result(position)
        letter_peg_378 = stream.get(result_peg_377.getPosition())
        if letter_peg_378 in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-':
            result_peg_377.nextPosition()
            result_peg_377.setValue(letter_peg_378)
        else:
            raise PegError
        stream.update(RULE_any_char, position, result_peg_377)
        return result_peg_377
    except PegError:
        pass
    stream.update(RULE_any_char, position, None)
    return None


def rule_any(stream, position):
    if stream.hasResult(RULE_any, position):
        return stream.result(RULE_any, position)
    try:
        result_peg_379 = Result(position)
        result_peg_380 = Result(result_peg_379.getPosition())
        
        if '.' == stream.get(result_peg_380.getPosition(), 1):
            result_peg_380.nextPosition(1)
            result_peg_380.setValue('.')
        else:
            raise PegError
        
        result_peg_379.addResult(result_peg_380);
        
        result_peg_381 = Result(result_peg_379.getPosition())
        
        value = None
        values = result_peg_379.getValues()
        value = peg.PatternAny()
        result_peg_381.setValue(value)
        
        result_peg_379.addResult(result_peg_381);
        
        result_peg_379.setValue(result_peg_379.getLastValue())
        stream.update(RULE_any, position, result_peg_379)
        return result_peg_379
    except PegError:
        pass
    stream.update(RULE_any, position, None)
    return None


def rule_whitespace(stream, position):
    if stream.hasResult(RULE_whitespace, position):
        return stream.result(RULE_whitespace, position)
    try:
        result_peg_382 = Result(position)
        result_peg_383 = Result(result_peg_382.getPosition())
        
        try:
            while True:
                result_peg_384 = Result(result_peg_383.getPosition());
                save_peg_385 = result_peg_384.getPosition()
                
                result_peg_384 = Result(save_peg_385)
                letter_peg_386 = stream.get(result_peg_384.getPosition())
                if letter_peg_386 in ' \t\n':
                    result_peg_384.nextPosition()
                    result_peg_384.setValue(letter_peg_386)
                else:
                    
                    result_peg_384 = Result(save_peg_385)
                    # print "Trying rule " + 'comment'
                    result_peg_384 = rule_comment(stream, result_peg_384.getPosition())
                    if result_peg_384 == None:
                        raise PegError
                result_peg_383.addResult(result_peg_384);
        except PegError:
            pass
                
        result_peg_382.addResult(result_peg_383);
        
        result_peg_382.setValue(result_peg_382.getLastValue())
        stream.update(RULE_whitespace, position, result_peg_382)
        return result_peg_382
    except PegError:
        pass
    stream.update(RULE_whitespace, position, None)
    return None


def rule_comment(stream, position):
    if stream.hasResult(RULE_comment, position):
        return stream.result(RULE_comment, position)
    try:
        result_peg_387 = Result(position)
        result_peg_388 = Result(result_peg_387.getPosition())
        
        if '#' == stream.get(result_peg_388.getPosition(), 1):
            result_peg_388.nextPosition(1)
            result_peg_388.setValue('#')
        else:
            raise PegError
        
        result_peg_387.addResult(result_peg_388);
        
        result_peg_389 = Result(result_peg_387.getPosition())
        
        try:
            while True:
                result_peg_390 = Result(result_peg_389.getPosition());
                result_peg_391 = Result(result_peg_390.getPosition())
                
                result_peg_392 = Result(result_peg_391.getPosition());
                try:
                    if '\n' == stream.get(result_peg_392.getPosition(), 1):
                        result_peg_392.nextPosition(1)
                        result_peg_392.setValue('\n')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_391.setValue(None)
                        
                result_peg_390.addResult(result_peg_391);
                
                result_peg_393 = Result(result_peg_390.getPosition())
                
                temp_peg_394 = stream.get(result_peg_393.getPosition())
                if temp_peg_394 != chr(0):
                    result_peg_393.setValue(temp_peg_394)
                    result_peg_393.nextPosition()
                else:
                    raise PegError
                
                result_peg_390.addResult(result_peg_393);
                
                result_peg_390.setValue(result_peg_390.getLastValue())
                result_peg_389.addResult(result_peg_390);
        except PegError:
            pass
                
        result_peg_387.addResult(result_peg_389);
        
        result_peg_387.setValue(result_peg_387.getLastValue())
        stream.update(RULE_comment, position, result_peg_387)
        return result_peg_387
    except PegError:
        pass
    stream.update(RULE_comment, position, None)
    return None


def rule_newlines_one(stream, position):
    if stream.hasResult(RULE_newlines_one, position):
        return stream.result(RULE_newlines_one, position)
    try:
        result_peg_395 = Result(position)
        try:
            while True:
                result_peg_396 = Result(result_peg_395.getPosition());
                if '\n' == stream.get(result_peg_396.getPosition(), 1):
                    result_peg_396.nextPosition(1)
                    result_peg_396.setValue('\n')
                else:
                    raise PegError
                result_peg_395.addResult(result_peg_396);
        except PegError:
            if result_peg_395.matches() == 0:
                raise PegError
        stream.update(RULE_newlines_one, position, result_peg_395)
        return result_peg_395
    except PegError:
        pass
    stream.update(RULE_newlines_one, position, None)
    return None


def rule_newlines(stream, position):
    if stream.hasResult(RULE_newlines, position):
        return stream.result(RULE_newlines, position)
    try:
        result_peg_397 = Result(position)
        try:
            while True:
                result_peg_398 = Result(result_peg_397.getPosition());
                if '\n' == stream.get(result_peg_398.getPosition(), 1):
                    result_peg_398.nextPosition(1)
                    result_peg_398.setValue('\n')
                else:
                    raise PegError
                result_peg_397.addResult(result_peg_398);
        except PegError:
            pass
        stream.update(RULE_newlines, position, result_peg_397)
        return result_peg_397
    except PegError:
        pass
    stream.update(RULE_newlines, position, None)
    return None


def doParse(stream):
    done = rule_start(stream, 0)
    if done == None:
        # print "Error parsing " + file
        stream.reportError()
        return None
    else:
        return done.getValues()

def parseFile(file):
    # print "Parsing " + file
    return doParse(Stream(filename = file))

def parseString(value):
    return doParse(Stream(input = value))

