
import peg





def special_escape(s):
    return s.replace("\\n", "\\\\n").replace("\\t", "\\\\t").replace("\"", '\\\"').replace("\\r", "\\\\r")

class PegError(Exception):
    def __init__(self):
        Exception.__init__(self)

class NotError(Exception):
    def __init__(self):
        Exception.__init__(self)

class Result:
    def __init__(self, position):
        self.position = position
        self.values = []

    def getPosition(self):
        return self.position

    def nextPosition(self, amount = 1):
        self.position += amount

    def setValue(self, value):
        self.values = value

    def getLastValue(self):
        if type(self.values) is list:
            if len(self.values) > 0:
                return self.values[-1]
            else:
                return None
        return self.values
    
    def matches(self):
        return len(self.values)

    def getValues(self):
        return self.values

    def addResult(self, him):
        self.values.append(him.values)
        self.position = him.position
    
    #def extendResult(self, him):
    #    self.values.extend(him.values)
    #    self.position = him.position

class Stream:
    def __init__(self, filename = None, input = None):
        def read():
            file = open(filename, 'r')
            out = file.read()
            file.close()
            return out
        self.position = 0
        self.limit = 100
        self.furthest = 0
        self.memo = {}
        if filename != None:
            self.all = read()
        elif input != None:
            self.all = input
        else:
            raise PegError("Pass a filename or input")
        # print "Read " + str(len(self.all))

    def get(self, position, number = 1):
        if position + number > self.limit:
            # print (position + number)
            self.limit += 5000
        if position + number > len(self.all):
            return chr(0)
        # print "stream: %s" % self.all[position:position+number]
        return self.all[position:position+number]

    def get2(self, position):
        if position != self.position:
            self.file.seek(position)
        self.position = position + 1
        if position > self.limit:
            print position
            self.limit += 5000
        return self.file.read(1)

    def reportError(self):
        line = 1
        column = 1
        for i in xrange(0, self.furthest):
            if self.all[i] == '\n':
                line += 1
                column = 1
            else:
                column += 1
        context = 10
        left = self.furthest - context
        right = self.furthest + context
        if left < 0:
            left = 0
        if right > len(self.all):
            right = len(self.all)
        print "Read up till line %d, column %d" % (line, column)
        print "'%s'" % special_escape(self.all[left:right])
        print "%s^" % (' ' * (self.furthest - left))

    def update(self, rule, position, result):
        if result != None and result.getPosition() > self.furthest:
            self.furthest = result.getPosition()

        for_rule = None
        try:
            for_rule = self.memo[rule]
        except KeyError:
            self.memo[rule] = {}
            for_rule = self.memo[rule]
        
        for_position = None
        try:
            for_position = for_rule[position]
        except KeyError:
            for_rule[position] = None

        for_rule[position] = result

    def hasResult(self, rule, position):
        try:
            x = self.memo[rule][position]
            return True
        except KeyError:
            return False

    def result(self, rule, position):
        return self.memo[rule][position]



RULE_start = 0
RULE_module = 1
RULE_include = 2
RULE_more_code = 3
RULE_options = 4
RULE_option = 5
RULE_chunks = 6
RULE_error_option = 7
RULE_word = 8
RULE_rules = 9
RULE_rule = 10
RULE_pattern_line = 11
RULE_pattern = 12
RULE_raw_code = 13
RULE_code = 14
RULE_item = 15
RULE_failure = 16
RULE_line = 17
RULE_predicate = 18
RULE_utf8 = 19
RULE_ascii = 20
RULE_call_rule = 21
RULE_eof = 22
RULE_void = 23
RULE_range = 24
RULE_sub_pattern = 25
RULE_bind = 26
RULE_string = 27
RULE_modifier = 28
RULE_x_word = 29
RULE_rule_parameters = 30
RULE_value_parameters = 31
RULE_parameters_rules = 32
RULE_parameters_values = 33
RULE_word_or_dollar = 34
RULE_word_or_at = 35
RULE_word_at = 36
RULE_dollar = 37
RULE_number = 38
RULE_digit = 39
RULE_hex_number = 40
RULE_hex_digit = 41
RULE_start_symbol = 42
RULE_spaces = 43
RULE_space = 44
RULE_any_char = 45
RULE_any = 46
RULE_whitespace = 47
RULE_comment = 48
RULE_newlines_one = 49
RULE_newlines = 50




def rule_start(stream, position):
    if stream.hasResult(RULE_start, position):
        return stream.result(RULE_start, position)
    try:
        result_peg_37 = Result(position)
        result_peg_38 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_38 = rule_newlines(stream, result_peg_38.getPosition())
        if result_peg_38 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_38);
        
        result_peg_39 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_39 = rule_whitespace(stream, result_peg_39.getPosition())
        if result_peg_39 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_39);
        
        result_peg_40 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'start_symbol'
        result_peg_40 = rule_start_symbol(stream, result_peg_40.getPosition())
        if result_peg_40 == None:
            raise PegError
        start_symbol = result_peg_40.getValues()
        
        result_peg_37.addResult(result_peg_40);
        
        result_peg_41 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_41 = rule_newlines(stream, result_peg_41.getPosition())
        if result_peg_41 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_41);
        
        result_peg_42 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_42 = rule_whitespace(stream, result_peg_42.getPosition())
        if result_peg_42 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_42);
        
        result_peg_43 = Result(result_peg_37.getPosition())
        
        save_peg_44 = result_peg_43.getPosition()
        
        # print "Trying rule " + 'options'
        result_peg_43 = rule_options(stream, result_peg_43.getPosition())
        if result_peg_43 == None:
            
            result_peg_43 = Result(save_peg_44)
            result_peg_43.setValue(None)
        options = result_peg_43.getValues()
        
        result_peg_37.addResult(result_peg_43);
        
        result_peg_45 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_45 = rule_newlines(stream, result_peg_45.getPosition())
        if result_peg_45 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_45);
        
        result_peg_46 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_46 = rule_whitespace(stream, result_peg_46.getPosition())
        if result_peg_46 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_46);
        
        result_peg_47 = Result(result_peg_37.getPosition())
        
        save_peg_48 = result_peg_47.getPosition()
        
        # print "Trying rule " + 'module'
        result_peg_47 = rule_module(stream, result_peg_47.getPosition())
        if result_peg_47 == None:
            
            result_peg_47 = Result(save_peg_48)
            result_peg_47.setValue(None)
        module = result_peg_47.getValues()
        
        result_peg_37.addResult(result_peg_47);
        
        result_peg_49 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_49 = rule_newlines(stream, result_peg_49.getPosition())
        if result_peg_49 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_49);
        
        result_peg_50 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_50 = rule_whitespace(stream, result_peg_50.getPosition())
        if result_peg_50 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_50);
        
        result_peg_51 = Result(result_peg_37.getPosition())
        
        save_peg_52 = result_peg_51.getPosition()
        
        # print "Trying rule " + 'include'
        result_peg_51 = rule_include(stream, result_peg_51.getPosition())
        if result_peg_51 == None:
            
            result_peg_51 = Result(save_peg_52)
            result_peg_51.setValue(None)
        include = result_peg_51.getValues()
        
        result_peg_37.addResult(result_peg_51);
        
        result_peg_53 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_53 = rule_newlines(stream, result_peg_53.getPosition())
        if result_peg_53 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_53);
        
        result_peg_54 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_54 = rule_whitespace(stream, result_peg_54.getPosition())
        if result_peg_54 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_54);
        
        result_peg_55 = Result(result_peg_37.getPosition())
        
        save_peg_56 = result_peg_55.getPosition()
        
        # print "Trying rule " + 'more_code'
        result_peg_55 = rule_more_code(stream, result_peg_55.getPosition())
        if result_peg_55 == None:
            
            result_peg_55 = Result(save_peg_56)
            result_peg_55.setValue(None)
        code = result_peg_55.getValues()
        
        result_peg_37.addResult(result_peg_55);
        
        result_peg_57 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_57 = rule_newlines(stream, result_peg_57.getPosition())
        if result_peg_57 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_57);
        
        result_peg_58 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_58 = rule_whitespace(stream, result_peg_58.getPosition())
        if result_peg_58 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_58);
        
        result_peg_59 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'rules'
        result_peg_59 = rule_rules(stream, result_peg_59.getPosition())
        if result_peg_59 == None:
            raise PegError
        rules = result_peg_59.getValues()
        
        result_peg_37.addResult(result_peg_59);
        
        result_peg_60 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_60 = rule_newlines(stream, result_peg_60.getPosition())
        if result_peg_60 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_60);
        
        result_peg_61 = Result(result_peg_37.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_61 = rule_whitespace(stream, result_peg_61.getPosition())
        if result_peg_61 == None:
            raise PegError
        
        result_peg_37.addResult(result_peg_61);
        
        result_peg_62 = Result(result_peg_37.getPosition())
        
        if chr(0) == stream.get(result_peg_62.getPosition()):
            result_peg_62.nextPosition()
            result_peg_62.setValue(chr(0))
        else:
            raise PegError
        
        result_peg_37.addResult(result_peg_62);
        
        result_peg_63 = Result(result_peg_37.getPosition())
        
        value = None
        values = result_peg_37.getValues()
        value = peg.Peg(start_symbol, include, code, module, rules, options)
        result_peg_63.setValue(value)
        
        result_peg_37.addResult(result_peg_63);
        
        result_peg_37.setValue(result_peg_37.getLastValue())
        stream.update(RULE_start, position, result_peg_37)
        return result_peg_37
    except PegError:
        pass
    stream.update(RULE_start, position, None)
    return None


def rule_module(stream, position):
    if stream.hasResult(RULE_module, position):
        return stream.result(RULE_module, position)
    try:
        result_peg_64 = Result(position)
        result_peg_65 = Result(result_peg_64.getPosition())
        
        if 'module:' == stream.get(result_peg_65.getPosition(), 7):
            result_peg_65.nextPosition(7)
            result_peg_65.setValue('module:')
        else:
            raise PegError
        
        result_peg_64.addResult(result_peg_65);
        
        result_peg_66 = Result(result_peg_64.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_66 = rule_spaces(stream, result_peg_66.getPosition())
        if result_peg_66 == None:
            raise PegError
        
        result_peg_64.addResult(result_peg_66);
        
        result_peg_67 = Result(result_peg_64.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_67 = rule_word(stream, result_peg_67.getPosition())
        if result_peg_67 == None:
            raise PegError
        name = result_peg_67.getValues()
        
        result_peg_64.addResult(result_peg_67);
        
        result_peg_68 = Result(result_peg_64.getPosition())
        
        try:
            while True:
                result_peg_69 = Result(result_peg_68.getPosition());
                result_peg_70 = Result(result_peg_69.getPosition())
                
                if '.' == stream.get(result_peg_70.getPosition(), 1):
                    result_peg_70.nextPosition(1)
                    result_peg_70.setValue('.')
                else:
                    raise PegError
                
                result_peg_69.addResult(result_peg_70);
                
                result_peg_71 = Result(result_peg_69.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_71 = rule_word(stream, result_peg_71.getPosition())
                if result_peg_71 == None:
                    raise PegError
                
                result_peg_69.addResult(result_peg_71);
                
                result_peg_72 = Result(result_peg_69.getPosition())
                
                value = None
                values = result_peg_69.getValues()
                value = values[1]
                result_peg_72.setValue(value)
                
                result_peg_69.addResult(result_peg_72);
                
                result_peg_69.setValue(result_peg_69.getLastValue())
                result_peg_68.addResult(result_peg_69);
        except PegError:
            pass
        rest = result_peg_68.getValues()
        
        result_peg_64.addResult(result_peg_68);
        
        result_peg_73 = Result(result_peg_64.getPosition())
        
        value = None
        values = result_peg_64.getValues()
        value = [name] + rest
        result_peg_73.setValue(value)
        
        result_peg_64.addResult(result_peg_73);
        
        result_peg_64.setValue(result_peg_64.getLastValue())
        stream.update(RULE_module, position, result_peg_64)
        return result_peg_64
    except PegError:
        pass
    stream.update(RULE_module, position, None)
    return None


def rule_include(stream, position):
    if stream.hasResult(RULE_include, position):
        return stream.result(RULE_include, position)
    try:
        result_peg_74 = Result(position)
        result_peg_75 = Result(result_peg_74.getPosition())
        
        if 'include:' == stream.get(result_peg_75.getPosition(), 8):
            result_peg_75.nextPosition(8)
            result_peg_75.setValue('include:')
        else:
            raise PegError
        
        result_peg_74.addResult(result_peg_75);
        
        result_peg_76 = Result(result_peg_74.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_76 = rule_spaces(stream, result_peg_76.getPosition())
        if result_peg_76 == None:
            raise PegError
        
        result_peg_74.addResult(result_peg_76);
        
        result_peg_77 = Result(result_peg_74.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_77 = rule_code(stream, result_peg_77.getPosition())
        if result_peg_77 == None:
            raise PegError
        code = result_peg_77.getValues()
        
        result_peg_74.addResult(result_peg_77);
        
        result_peg_78 = Result(result_peg_74.getPosition())
        
        value = None
        values = result_peg_74.getValues()
        value = code.code
        result_peg_78.setValue(value)
        
        result_peg_74.addResult(result_peg_78);
        
        result_peg_74.setValue(result_peg_74.getLastValue())
        stream.update(RULE_include, position, result_peg_74)
        return result_peg_74
    except PegError:
        pass
    stream.update(RULE_include, position, None)
    return None


def rule_more_code(stream, position):
    if stream.hasResult(RULE_more_code, position):
        return stream.result(RULE_more_code, position)
    try:
        result_peg_79 = Result(position)
        result_peg_80 = Result(result_peg_79.getPosition())
        
        if 'code:' == stream.get(result_peg_80.getPosition(), 5):
            result_peg_80.nextPosition(5)
            result_peg_80.setValue('code:')
        else:
            raise PegError
        
        result_peg_79.addResult(result_peg_80);
        
        result_peg_81 = Result(result_peg_79.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_81 = rule_spaces(stream, result_peg_81.getPosition())
        if result_peg_81 == None:
            raise PegError
        
        result_peg_79.addResult(result_peg_81);
        
        result_peg_82 = Result(result_peg_79.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_82 = rule_code(stream, result_peg_82.getPosition())
        if result_peg_82 == None:
            raise PegError
        code = result_peg_82.getValues()
        
        result_peg_79.addResult(result_peg_82);
        
        result_peg_83 = Result(result_peg_79.getPosition())
        
        value = None
        values = result_peg_79.getValues()
        value = code.code
        result_peg_83.setValue(value)
        
        result_peg_79.addResult(result_peg_83);
        
        result_peg_79.setValue(result_peg_79.getLastValue())
        stream.update(RULE_more_code, position, result_peg_79)
        return result_peg_79
    except PegError:
        pass
    stream.update(RULE_more_code, position, None)
    return None


def rule_options(stream, position):
    if stream.hasResult(RULE_options, position):
        return stream.result(RULE_options, position)
    try:
        result_peg_84 = Result(position)
        result_peg_85 = Result(result_peg_84.getPosition())
        
        if 'options:' == stream.get(result_peg_85.getPosition(), 8):
            result_peg_85.nextPosition(8)
            result_peg_85.setValue('options:')
        else:
            raise PegError
        
        result_peg_84.addResult(result_peg_85);
        
        result_peg_86 = Result(result_peg_84.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_86 = rule_spaces(stream, result_peg_86.getPosition())
        if result_peg_86 == None:
            raise PegError
        
        result_peg_84.addResult(result_peg_86);
        
        result_peg_87 = Result(result_peg_84.getPosition())
        
        # print "Trying rule " + 'option'
        result_peg_87 = rule_option(stream, result_peg_87.getPosition())
        if result_peg_87 == None:
            raise PegError
        option1 = result_peg_87.getValues()
        
        result_peg_84.addResult(result_peg_87);
        
        result_peg_88 = Result(result_peg_84.getPosition())
        
        try:
            while True:
                result_peg_89 = Result(result_peg_88.getPosition());
                result_peg_90 = Result(result_peg_89.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_90 = rule_spaces(stream, result_peg_90.getPosition())
                if result_peg_90 == None:
                    raise PegError
                
                result_peg_89.addResult(result_peg_90);
                
                result_peg_91 = Result(result_peg_89.getPosition())
                
                if ',' == stream.get(result_peg_91.getPosition(), 1):
                    result_peg_91.nextPosition(1)
                    result_peg_91.setValue(',')
                else:
                    raise PegError
                
                result_peg_89.addResult(result_peg_91);
                
                result_peg_92 = Result(result_peg_89.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_92 = rule_spaces(stream, result_peg_92.getPosition())
                if result_peg_92 == None:
                    raise PegError
                
                result_peg_89.addResult(result_peg_92);
                
                result_peg_93 = Result(result_peg_89.getPosition())
                
                # print "Trying rule " + 'option'
                result_peg_93 = rule_option(stream, result_peg_93.getPosition())
                if result_peg_93 == None:
                    raise PegError
                
                result_peg_89.addResult(result_peg_93);
                
                result_peg_89.setValue(result_peg_89.getLastValue())
                result_peg_88.addResult(result_peg_89);
        except PegError:
            pass
        option_rest = result_peg_88.getValues()
        
        result_peg_84.addResult(result_peg_88);
        
        result_peg_94 = Result(result_peg_84.getPosition())
        
        value = None
        values = result_peg_84.getValues()
        value = []
        for option in ([option1] + option_rest):
            import re
            debug = re.compile("debug(\d+)")
            out = debug.match(option)
            if out != None:
                num = int(out.group(1))
                for x in xrange(1,num+1):
                    value.append('debug%d' % x)
            elif option == 'no-memo':
                value.append(option)
            else:
                value.append(option)
        result_peg_94.setValue(value)
        
        result_peg_84.addResult(result_peg_94);
        
        result_peg_84.setValue(result_peg_84.getLastValue())
        stream.update(RULE_options, position, result_peg_84)
        return result_peg_84
    except PegError:
        pass
    stream.update(RULE_options, position, None)
    return None


def rule_option(stream, position):
    if stream.hasResult(RULE_option, position):
        return stream.result(RULE_option, position)
    try:
        result_peg_95 = Result(position)
        result_peg_96 = Result(result_peg_95.getPosition())
        
        if 'debug' == stream.get(result_peg_96.getPosition(), 5):
            result_peg_96.nextPosition(5)
            result_peg_96.setValue('debug')
        else:
            raise PegError
        
        result_peg_95.addResult(result_peg_96);
        
        result_peg_97 = Result(result_peg_95.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_97 = rule_number(stream, result_peg_97.getPosition())
        if result_peg_97 == None:
            raise PegError
        number = result_peg_97.getValues()
        
        result_peg_95.addResult(result_peg_97);
        
        result_peg_98 = Result(result_peg_95.getPosition())
        
        value = None
        values = result_peg_95.getValues()
        value = 'debug%s' % number
        result_peg_98.setValue(value)
        
        result_peg_95.addResult(result_peg_98);
        
        result_peg_95.setValue(result_peg_95.getLastValue())
        stream.update(RULE_option, position, result_peg_95)
        return result_peg_95
    except PegError:
        pass
    try:
        result_peg_99 = Result(position)
        if 'state' == stream.get(result_peg_99.getPosition(), 5):
            result_peg_99.nextPosition(5)
            result_peg_99.setValue('state')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_99)
        return result_peg_99
    except PegError:
        pass
    try:
        result_peg_100 = Result(position)
        if 'no-memo' == stream.get(result_peg_100.getPosition(), 7):
            result_peg_100.nextPosition(7)
            result_peg_100.setValue('no-memo')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_100)
        return result_peg_100
    except PegError:
        pass
    try:
        result_peg_101 = Result(position)
        # print "Trying rule " + 'error_option'
        result_peg_101 = rule_error_option(stream, result_peg_101.getPosition())
        if result_peg_101 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_101)
        return result_peg_101
    except PegError:
        pass
    try:
        result_peg_102 = Result(position)
        # print "Trying rule " + 'chunks'
        result_peg_102 = rule_chunks(stream, result_peg_102.getPosition())
        if result_peg_102 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_102)
        return result_peg_102
    except PegError:
        pass
    stream.update(RULE_option, position, None)
    return None


def rule_chunks(stream, position):
    if stream.hasResult(RULE_chunks, position):
        return stream.result(RULE_chunks, position)
    try:
        result_peg_103 = Result(position)
        result_peg_104 = Result(result_peg_103.getPosition())
        
        if 'chunks' == stream.get(result_peg_104.getPosition(), 6):
            result_peg_104.nextPosition(6)
            result_peg_104.setValue('chunks')
        else:
            raise PegError
        
        result_peg_103.addResult(result_peg_104);
        
        result_peg_105 = Result(result_peg_103.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_105 = rule_whitespace(stream, result_peg_105.getPosition())
        if result_peg_105 == None:
            raise PegError
        
        result_peg_103.addResult(result_peg_105);
        
        result_peg_106 = Result(result_peg_103.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_106 = rule_number(stream, result_peg_106.getPosition())
        if result_peg_106 == None:
            raise PegError
        number = result_peg_106.getValues()
        
        result_peg_103.addResult(result_peg_106);
        
        result_peg_107 = Result(result_peg_103.getPosition())
        
        value = None
        values = result_peg_103.getValues()
        value = 'chunks %s' % number
        result_peg_107.setValue(value)
        
        result_peg_103.addResult(result_peg_107);
        
        result_peg_103.setValue(result_peg_103.getLastValue())
        stream.update(RULE_chunks, position, result_peg_103)
        return result_peg_103
    except PegError:
        pass
    stream.update(RULE_chunks, position, None)
    return None


def rule_error_option(stream, position):
    if stream.hasResult(RULE_error_option, position):
        return stream.result(RULE_error_option, position)
    try:
        result_peg_108 = Result(position)
        result_peg_109 = Result(result_peg_108.getPosition())
        
        if 'error-length' == stream.get(result_peg_109.getPosition(), 12):
            result_peg_109.nextPosition(12)
            result_peg_109.setValue('error-length')
        else:
            raise PegError
        
        result_peg_108.addResult(result_peg_109);
        
        result_peg_110 = Result(result_peg_108.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_110 = rule_whitespace(stream, result_peg_110.getPosition())
        if result_peg_110 == None:
            raise PegError
        
        result_peg_108.addResult(result_peg_110);
        
        result_peg_111 = Result(result_peg_108.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_111 = rule_number(stream, result_peg_111.getPosition())
        if result_peg_111 == None:
            raise PegError
        number = result_peg_111.getValues()
        
        result_peg_108.addResult(result_peg_111);
        
        result_peg_112 = Result(result_peg_108.getPosition())
        
        value = None
        values = result_peg_108.getValues()
        value = 'error-length %s' % number
        result_peg_112.setValue(value)
        
        result_peg_108.addResult(result_peg_112);
        
        result_peg_108.setValue(result_peg_108.getLastValue())
        stream.update(RULE_error_option, position, result_peg_108)
        return result_peg_108
    except PegError:
        pass
    stream.update(RULE_error_option, position, None)
    return None


def rule_word(stream, position):
    if stream.hasResult(RULE_word, position):
        return stream.result(RULE_word, position)
    try:
        result_peg_113 = Result(position)
        result_peg_114 = Result(result_peg_113.getPosition())
        
        try:
            while True:
                result_peg_115 = Result(result_peg_114.getPosition());
                # print "Trying rule " + 'any_char'
                result_peg_115 = rule_any_char(stream, result_peg_115.getPosition())
                if result_peg_115 == None:
                    raise PegError
                result_peg_114.addResult(result_peg_115);
        except PegError:
            if result_peg_114.matches() == 0:
                raise PegError
                
        result_peg_113.addResult(result_peg_114);
        
        result_peg_116 = Result(result_peg_113.getPosition())
        
        value = None
        values = result_peg_113.getValues()
        # print "all start symbol values " + str(values)
        # print "values[0] " + str(values[0])
        value = ''.join(values[0]).replace('-', '__')
        # print "got word " + value
        result_peg_116.setValue(value)
        
        result_peg_113.addResult(result_peg_116);
        
        result_peg_113.setValue(result_peg_113.getLastValue())
        stream.update(RULE_word, position, result_peg_113)
        return result_peg_113
    except PegError:
        pass
    stream.update(RULE_word, position, None)
    return None


def rule_rules(stream, position):
    if stream.hasResult(RULE_rules, position):
        return stream.result(RULE_rules, position)
    try:
        result_peg_117 = Result(position)
        result_peg_118 = Result(result_peg_117.getPosition())
        
        if 'rules:' == stream.get(result_peg_118.getPosition(), 6):
            result_peg_118.nextPosition(6)
            result_peg_118.setValue('rules:')
        else:
            raise PegError
        
        result_peg_117.addResult(result_peg_118);
        
        result_peg_119 = Result(result_peg_117.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_119 = rule_whitespace(stream, result_peg_119.getPosition())
        if result_peg_119 == None:
            raise PegError
        
        result_peg_117.addResult(result_peg_119);
        
        result_peg_120 = Result(result_peg_117.getPosition())
        
        try:
            while True:
                result_peg_121 = Result(result_peg_120.getPosition());
                result_peg_122 = Result(result_peg_121.getPosition())
                
                # print "Trying rule " + 'rule'
                result_peg_122 = rule_rule(stream, result_peg_122.getPosition())
                if result_peg_122 == None:
                    raise PegError
                
                result_peg_121.addResult(result_peg_122);
                
                result_peg_123 = Result(result_peg_121.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_123 = rule_whitespace(stream, result_peg_123.getPosition())
                if result_peg_123 == None:
                    raise PegError
                
                result_peg_121.addResult(result_peg_123);
                
                result_peg_124 = Result(result_peg_121.getPosition())
                
                value = None
                values = result_peg_121.getValues()
                value = values[0]
                result_peg_124.setValue(value)
                
                result_peg_121.addResult(result_peg_124);
                
                result_peg_121.setValue(result_peg_121.getLastValue())
                result_peg_120.addResult(result_peg_121);
        except PegError:
            pass
        rules = result_peg_120.getValues()
        
        result_peg_117.addResult(result_peg_120);
        
        result_peg_125 = Result(result_peg_117.getPosition())
        
        value = None
        values = result_peg_117.getValues()
        value = rules
        result_peg_125.setValue(value)
        
        result_peg_117.addResult(result_peg_125);
        
        result_peg_117.setValue(result_peg_117.getLastValue())
        stream.update(RULE_rules, position, result_peg_117)
        return result_peg_117
    except PegError:
        pass
    stream.update(RULE_rules, position, None)
    return None


def rule_rule(stream, position):
    if stream.hasResult(RULE_rule, position):
        return stream.result(RULE_rule, position)
    try:
        result_peg_126 = Result(position)
        result_peg_127 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_127 = rule_spaces(stream, result_peg_127.getPosition())
        if result_peg_127 == None:
            raise PegError
        
        result_peg_126.addResult(result_peg_127);
        
        result_peg_128 = Result(result_peg_126.getPosition())
        
        save_peg_129 = result_peg_128.getPosition()
        
        if 'inline' == stream.get(result_peg_128.getPosition(), 6):
            result_peg_128.nextPosition(6)
            result_peg_128.setValue('inline')
        else:
            
            result_peg_128 = Result(save_peg_129)
            result_peg_128.setValue(None)
        inline = result_peg_128.getValues()
        
        result_peg_126.addResult(result_peg_128);
        
        result_peg_130 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_130 = rule_spaces(stream, result_peg_130.getPosition())
        if result_peg_130 == None:
            raise PegError
        
        result_peg_126.addResult(result_peg_130);
        
        result_peg_131 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_131 = rule_word(stream, result_peg_131.getPosition())
        if result_peg_131 == None:
            raise PegError
        name = result_peg_131.getValues()
        
        result_peg_126.addResult(result_peg_131);
        
        result_peg_132 = Result(result_peg_126.getPosition())
        
        save_peg_133 = result_peg_132.getPosition()
        
        # print "Trying rule " + 'rule_parameters'
        result_peg_132 = rule_rule_parameters(stream, result_peg_132.getPosition())
        if result_peg_132 == None:
            
            result_peg_132 = Result(save_peg_133)
            result_peg_132.setValue(None)
        rule_parameters = result_peg_132.getValues()
        
        result_peg_126.addResult(result_peg_132);
        
        result_peg_134 = Result(result_peg_126.getPosition())
        
        save_peg_135 = result_peg_134.getPosition()
        
        # print "Trying rule " + 'value_parameters'
        result_peg_134 = rule_value_parameters(stream, result_peg_134.getPosition())
        if result_peg_134 == None:
            
            result_peg_134 = Result(save_peg_135)
            result_peg_134.setValue(None)
        parameters = result_peg_134.getValues()
        
        result_peg_126.addResult(result_peg_134);
        
        result_peg_136 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_136 = rule_spaces(stream, result_peg_136.getPosition())
        if result_peg_136 == None:
            raise PegError
        
        result_peg_126.addResult(result_peg_136);
        
        result_peg_137 = Result(result_peg_126.getPosition())
        
        if '=' == stream.get(result_peg_137.getPosition(), 1):
            result_peg_137.nextPosition(1)
            result_peg_137.setValue('=')
        else:
            raise PegError
        
        result_peg_126.addResult(result_peg_137);
        
        result_peg_138 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_138 = rule_spaces(stream, result_peg_138.getPosition())
        if result_peg_138 == None:
            raise PegError
        
        result_peg_126.addResult(result_peg_138);
        
        result_peg_139 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'pattern_line'
        result_peg_139 = rule_pattern_line(stream, result_peg_139.getPosition())
        if result_peg_139 == None:
            raise PegError
        pattern1 = result_peg_139.getValues()
        
        result_peg_126.addResult(result_peg_139);
        
        result_peg_140 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_140 = rule_whitespace(stream, result_peg_140.getPosition())
        if result_peg_140 == None:
            raise PegError
        
        result_peg_126.addResult(result_peg_140);
        
        result_peg_141 = Result(result_peg_126.getPosition())
        
        try:
            while True:
                result_peg_142 = Result(result_peg_141.getPosition());
                result_peg_143 = Result(result_peg_142.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_143 = rule_spaces(stream, result_peg_143.getPosition())
                if result_peg_143 == None:
                    raise PegError
                
                result_peg_142.addResult(result_peg_143);
                
                result_peg_144 = Result(result_peg_142.getPosition())
                
                if '|' == stream.get(result_peg_144.getPosition(), 1):
                    result_peg_144.nextPosition(1)
                    result_peg_144.setValue('|')
                else:
                    raise PegError
                
                result_peg_142.addResult(result_peg_144);
                
                result_peg_145 = Result(result_peg_142.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_145 = rule_spaces(stream, result_peg_145.getPosition())
                if result_peg_145 == None:
                    raise PegError
                
                result_peg_142.addResult(result_peg_145);
                
                result_peg_146 = Result(result_peg_142.getPosition())
                
                # print "Trying rule " + 'pattern_line'
                result_peg_146 = rule_pattern_line(stream, result_peg_146.getPosition())
                if result_peg_146 == None:
                    raise PegError
                pattern = result_peg_146.getValues()
                
                result_peg_142.addResult(result_peg_146);
                
                result_peg_147 = Result(result_peg_142.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_147 = rule_whitespace(stream, result_peg_147.getPosition())
                if result_peg_147 == None:
                    raise PegError
                
                result_peg_142.addResult(result_peg_147);
                
                result_peg_148 = Result(result_peg_142.getPosition())
                
                value = None
                values = result_peg_142.getValues()
                value = pattern
                result_peg_148.setValue(value)
                
                result_peg_142.addResult(result_peg_148);
                
                result_peg_142.setValue(result_peg_142.getLastValue())
                result_peg_141.addResult(result_peg_142);
        except PegError:
            pass
        patterns = result_peg_141.getValues()
        
        result_peg_126.addResult(result_peg_141);
        
        result_peg_149 = Result(result_peg_126.getPosition())
        
        save_peg_150 = result_peg_149.getPosition()
        
        # print "Trying rule " + 'failure'
        result_peg_149 = rule_failure(stream, result_peg_149.getPosition())
        if result_peg_149 == None:
            
            result_peg_149 = Result(save_peg_150)
            result_peg_149.setValue(None)
        fail = result_peg_149.getValues()
        
        result_peg_126.addResult(result_peg_149);
        
        result_peg_151 = Result(result_peg_126.getPosition())
        
        value = None
        values = result_peg_126.getValues()
        value = peg.Rule(name, [pattern1] + patterns, inline = (inline != None), rules = rule_parameters, parameters = parameters, fail = fail)
        result_peg_151.setValue(value)
        
        result_peg_126.addResult(result_peg_151);
        
        result_peg_126.setValue(result_peg_126.getLastValue())
        stream.update(RULE_rule, position, result_peg_126)
        return result_peg_126
    except PegError:
        pass
    stream.update(RULE_rule, position, None)
    return None


def rule_pattern_line(stream, position):
    if stream.hasResult(RULE_pattern_line, position):
        return stream.result(RULE_pattern_line, position)
    try:
        result_peg_152 = Result(position)
        result_peg_153 = Result(result_peg_152.getPosition())
        
        try:
            while True:
                result_peg_154 = Result(result_peg_153.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_154 = rule_pattern(stream, result_peg_154.getPosition())
                if result_peg_154 == None:
                    raise PegError
                result_peg_153.addResult(result_peg_154);
        except PegError:
            pass
        patterns = result_peg_153.getValues()
        
        result_peg_152.addResult(result_peg_153);
        
        result_peg_155 = Result(result_peg_152.getPosition())
        
        value = None
        values = result_peg_152.getValues()
        value = peg.PatternSequence(patterns)
        #if code != None:
        #    value = code(peg.PatternSequence(patterns))
        #else:
        #    value = peg.PatternAction(peg.PatternSequence(patterns), "value = values;")
        result_peg_155.setValue(value)
        
        result_peg_152.addResult(result_peg_155);
        
        result_peg_152.setValue(result_peg_152.getLastValue())
        stream.update(RULE_pattern_line, position, result_peg_152)
        return result_peg_152
    except PegError:
        pass
    stream.update(RULE_pattern_line, position, None)
    return None


def rule_pattern(stream, position):
    if stream.hasResult(RULE_pattern, position):
        return stream.result(RULE_pattern, position)
    try:
        result_peg_156 = Result(position)
        result_peg_157 = Result(result_peg_156.getPosition())
        
        save_peg_158 = result_peg_157.getPosition()
        
        # print "Trying rule " + 'bind'
        result_peg_157 = rule_bind(stream, result_peg_157.getPosition())
        if result_peg_157 == None:
            
            result_peg_157 = Result(save_peg_158)
            result_peg_157.setValue(None)
        bind = result_peg_157.getValues()
        
        result_peg_156.addResult(result_peg_157);
        
        result_peg_159 = Result(result_peg_156.getPosition())
        
        # print "Trying rule " + 'item'
        result_peg_159 = rule_item(stream, result_peg_159.getPosition())
        if result_peg_159 == None:
            raise PegError
        item = result_peg_159.getValues()
        
        result_peg_156.addResult(result_peg_159);
        
        result_peg_160 = Result(result_peg_156.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_160 = rule_spaces(stream, result_peg_160.getPosition())
        if result_peg_160 == None:
            raise PegError
        
        result_peg_156.addResult(result_peg_160);
        
        result_peg_161 = Result(result_peg_156.getPosition())
        
        value = None
        values = result_peg_156.getValues()
        # value = peg.PatternRule(values[0])
        if bind != None:
            item = bind(item)
        value = item
        # print "Pattern is " + str(value)
        result_peg_161.setValue(value)
        
        result_peg_156.addResult(result_peg_161);
        
        result_peg_156.setValue(result_peg_156.getLastValue())
        stream.update(RULE_pattern, position, result_peg_156)
        return result_peg_156
    except PegError:
        pass
    stream.update(RULE_pattern, position, None)
    return None


def rule_raw_code(stream, position):
    if stream.hasResult(RULE_raw_code, position):
        return stream.result(RULE_raw_code, position)
    try:
        result_peg_162 = Result(position)
        result_peg_163 = Result(result_peg_162.getPosition())
        
        if '(' == stream.get(result_peg_163.getPosition(), 1):
            result_peg_163.nextPosition(1)
            result_peg_163.setValue('(')
        else:
            raise PegError
        
        result_peg_162.addResult(result_peg_163);
        
        result_peg_164 = Result(result_peg_162.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_164 = rule_spaces(stream, result_peg_164.getPosition())
        if result_peg_164 == None:
            raise PegError
        
        result_peg_162.addResult(result_peg_164);
        
        result_peg_165 = Result(result_peg_162.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_165 = rule_code(stream, result_peg_165.getPosition())
        if result_peg_165 == None:
            raise PegError
        code = result_peg_165.getValues()
        
        result_peg_162.addResult(result_peg_165);
        
        result_peg_166 = Result(result_peg_162.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_166 = rule_spaces(stream, result_peg_166.getPosition())
        if result_peg_166 == None:
            raise PegError
        
        result_peg_162.addResult(result_peg_166);
        
        result_peg_167 = Result(result_peg_162.getPosition())
        
        if ')' == stream.get(result_peg_167.getPosition(), 1):
            result_peg_167.nextPosition(1)
            result_peg_167.setValue(')')
        else:
            raise PegError
        
        result_peg_162.addResult(result_peg_167);
        
        result_peg_168 = Result(result_peg_162.getPosition())
        
        value = None
        values = result_peg_162.getValues()
        value = code.code
        result_peg_168.setValue(value)
        
        result_peg_162.addResult(result_peg_168);
        
        result_peg_162.setValue(result_peg_162.getLastValue())
        stream.update(RULE_raw_code, position, result_peg_162)
        return result_peg_162
    except PegError:
        pass
    stream.update(RULE_raw_code, position, None)
    return None


def rule_code(stream, position):
    if stream.hasResult(RULE_code, position):
        return stream.result(RULE_code, position)
    try:
        result_peg_169 = Result(position)
        result_peg_170 = Result(result_peg_169.getPosition())
        
        if '{{' == stream.get(result_peg_170.getPosition(), 2):
            result_peg_170.nextPosition(2)
            result_peg_170.setValue('{{')
        else:
            raise PegError
        
        result_peg_169.addResult(result_peg_170);
        
        result_peg_171 = Result(result_peg_169.getPosition())
        
        try:
            while True:
                result_peg_172 = Result(result_peg_171.getPosition());
                result_peg_173 = Result(result_peg_172.getPosition())
                
                result_peg_174 = Result(result_peg_173.getPosition());
                try:
                    if '}}' == stream.get(result_peg_174.getPosition(), 2):
                        result_peg_174.nextPosition(2)
                        result_peg_174.setValue('}}')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_173.setValue(None)
                        
                result_peg_172.addResult(result_peg_173);
                
                result_peg_175 = Result(result_peg_172.getPosition())
                
                temp_peg_176 = stream.get(result_peg_175.getPosition())
                if temp_peg_176 != chr(0):
                    result_peg_175.setValue(temp_peg_176)
                    result_peg_175.nextPosition()
                else:
                    raise PegError
                
                result_peg_172.addResult(result_peg_175);
                
                result_peg_177 = Result(result_peg_172.getPosition())
                
                value = None
                values = result_peg_172.getValues()
                value = values[1]
                result_peg_177.setValue(value)
                
                result_peg_172.addResult(result_peg_177);
                
                result_peg_172.setValue(result_peg_172.getLastValue())
                result_peg_171.addResult(result_peg_172);
        except PegError:
            if result_peg_171.matches() == 0:
                raise PegError
                
        result_peg_169.addResult(result_peg_171);
        
        result_peg_178 = Result(result_peg_169.getPosition())
        
        if '}}' == stream.get(result_peg_178.getPosition(), 2):
            result_peg_178.nextPosition(2)
            result_peg_178.setValue('}}')
        else:
            raise PegError
        
        result_peg_169.addResult(result_peg_178);
        
        result_peg_179 = Result(result_peg_169.getPosition())
        
        value = None
        values = result_peg_169.getValues()
        value = peg.PatternCode(''.join(values[1]))
        result_peg_179.setValue(value)
        
        result_peg_169.addResult(result_peg_179);
        
        result_peg_169.setValue(result_peg_169.getLastValue())
        stream.update(RULE_code, position, result_peg_169)
        return result_peg_169
    except PegError:
        pass
    stream.update(RULE_code, position, None)
    return None


def rule_item(stream, position):
    if stream.hasResult(RULE_item, position):
        return stream.result(RULE_item, position)
    try:
        result_peg_180 = Result(position)
        result_peg_181 = Result(result_peg_180.getPosition())
        
        save_peg_182 = result_peg_181.getPosition()
        
        if '&' == stream.get(result_peg_181.getPosition(), 1):
            result_peg_181.nextPosition(1)
            result_peg_181.setValue('&')
        else:
            
            result_peg_181 = Result(save_peg_182)
            result_peg_181.setValue(None)
        ensure = result_peg_181.getValues()
        
        result_peg_180.addResult(result_peg_181);
        
        result_peg_183 = Result(result_peg_180.getPosition())
        
        save_peg_184 = result_peg_183.getPosition()
        
        if '!' == stream.get(result_peg_183.getPosition(), 1):
            result_peg_183.nextPosition(1)
            result_peg_183.setValue('!')
        else:
            
            result_peg_183 = Result(save_peg_184)
            result_peg_183.setValue(None)
        pnot = result_peg_183.getValues()
        
        result_peg_180.addResult(result_peg_183);
        
        result_peg_185 = Result(result_peg_180.getPosition())
        
        save_peg_186 = result_peg_185.getPosition()
        
        result_peg_185 = Result(save_peg_186)
        # print "Trying rule " + 'x_word'
        result_peg_185 = rule_x_word(stream, result_peg_185.getPosition())
        if result_peg_185 == None:
            
            result_peg_185 = Result(save_peg_186)
            # print "Trying rule " + 'any'
            result_peg_185 = rule_any(stream, result_peg_185.getPosition())
            if result_peg_185 == None:
                
                result_peg_185 = Result(save_peg_186)
                # print "Trying rule " + 'eof'
                result_peg_185 = rule_eof(stream, result_peg_185.getPosition())
                if result_peg_185 == None:
                    
                    result_peg_185 = Result(save_peg_186)
                    # print "Trying rule " + 'void'
                    result_peg_185 = rule_void(stream, result_peg_185.getPosition())
                    if result_peg_185 == None:
                        
                        result_peg_185 = Result(save_peg_186)
                        # print "Trying rule " + 'range'
                        result_peg_185 = rule_range(stream, result_peg_185.getPosition())
                        if result_peg_185 == None:
                            
                            result_peg_185 = Result(save_peg_186)
                            # print "Trying rule " + 'string'
                            result_peg_185 = rule_string(stream, result_peg_185.getPosition())
                            if result_peg_185 == None:
                                
                                result_peg_185 = Result(save_peg_186)
                                # print "Trying rule " + 'line'
                                result_peg_185 = rule_line(stream, result_peg_185.getPosition())
                                if result_peg_185 == None:
                                    
                                    result_peg_185 = Result(save_peg_186)
                                    # print "Trying rule " + 'ascii'
                                    result_peg_185 = rule_ascii(stream, result_peg_185.getPosition())
                                    if result_peg_185 == None:
                                        
                                        result_peg_185 = Result(save_peg_186)
                                        # print "Trying rule " + 'utf8'
                                        result_peg_185 = rule_utf8(stream, result_peg_185.getPosition())
                                        if result_peg_185 == None:
                                            
                                            result_peg_185 = Result(save_peg_186)
                                            # print "Trying rule " + 'predicate'
                                            result_peg_185 = rule_predicate(stream, result_peg_185.getPosition())
                                            if result_peg_185 == None:
                                                
                                                result_peg_185 = Result(save_peg_186)
                                                # print "Trying rule " + 'call_rule'
                                                result_peg_185 = rule_call_rule(stream, result_peg_185.getPosition())
                                                if result_peg_185 == None:
                                                    
                                                    result_peg_185 = Result(save_peg_186)
                                                    # print "Trying rule " + 'sub_pattern'
                                                    result_peg_185 = rule_sub_pattern(stream, result_peg_185.getPosition())
                                                    if result_peg_185 == None:
                                                        
                                                        result_peg_185 = Result(save_peg_186)
                                                        # print "Trying rule " + 'code'
                                                        result_peg_185 = rule_code(stream, result_peg_185.getPosition())
                                                        if result_peg_185 == None:
                                                            raise PegError
        pattern = result_peg_185.getValues()
        
        result_peg_180.addResult(result_peg_185);
        
        result_peg_187 = Result(result_peg_180.getPosition())
        
        save_peg_188 = result_peg_187.getPosition()
        
        # print "Trying rule " + 'modifier'
        result_peg_187 = rule_modifier(stream, result_peg_187.getPosition())
        if result_peg_187 == None:
            
            result_peg_187 = Result(save_peg_188)
            result_peg_187.setValue(None)
        modifier = result_peg_187.getValues()
        
        result_peg_180.addResult(result_peg_187);
        
        result_peg_189 = Result(result_peg_180.getPosition())
        
        value = None
        values = result_peg_180.getValues()
        if modifier != None:
            pattern = modifier(pattern)
        if pnot != None:
            pattern = peg.PatternNot(pattern)
        if ensure != None:
            pattern = peg.PatternEnsure(pattern)
        value = pattern
        result_peg_189.setValue(value)
        
        result_peg_180.addResult(result_peg_189);
        
        result_peg_180.setValue(result_peg_180.getLastValue())
        stream.update(RULE_item, position, result_peg_180)
        return result_peg_180
    except PegError:
        pass
    stream.update(RULE_item, position, None)
    return None


def rule_failure(stream, position):
    if stream.hasResult(RULE_failure, position):
        return stream.result(RULE_failure, position)
    try:
        result_peg_190 = Result(position)
        result_peg_191 = Result(result_peg_190.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_191 = rule_whitespace(stream, result_peg_191.getPosition())
        if result_peg_191 == None:
            raise PegError
        
        result_peg_190.addResult(result_peg_191);
        
        result_peg_192 = Result(result_peg_190.getPosition())
        
        if '<fail>' == stream.get(result_peg_192.getPosition(), 6):
            result_peg_192.nextPosition(6)
            result_peg_192.setValue('<fail>')
        else:
            raise PegError
        
        result_peg_190.addResult(result_peg_192);
        
        result_peg_193 = Result(result_peg_190.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_193 = rule_spaces(stream, result_peg_193.getPosition())
        if result_peg_193 == None:
            raise PegError
        
        result_peg_190.addResult(result_peg_193);
        
        result_peg_194 = Result(result_peg_190.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_194 = rule_code(stream, result_peg_194.getPosition())
        if result_peg_194 == None:
            raise PegError
        code = result_peg_194.getValues()
        
        result_peg_190.addResult(result_peg_194);
        
        result_peg_195 = Result(result_peg_190.getPosition())
        
        value = None
        values = result_peg_190.getValues()
        value = code.code
        result_peg_195.setValue(value)
        
        result_peg_190.addResult(result_peg_195);
        
        result_peg_190.setValue(result_peg_190.getLastValue())
        stream.update(RULE_failure, position, result_peg_190)
        return result_peg_190
    except PegError:
        pass
    stream.update(RULE_failure, position, None)
    return None


def rule_line(stream, position):
    if stream.hasResult(RULE_line, position):
        return stream.result(RULE_line, position)
    try:
        result_peg_196 = Result(position)
        result_peg_197 = Result(result_peg_196.getPosition())
        
        if '<line>' == stream.get(result_peg_197.getPosition(), 6):
            result_peg_197.nextPosition(6)
            result_peg_197.setValue('<line>')
        else:
            raise PegError
        
        result_peg_196.addResult(result_peg_197);
        
        result_peg_198 = Result(result_peg_196.getPosition())
        
        value = None
        values = result_peg_196.getValues()
        value = peg.PatternLine()
        result_peg_198.setValue(value)
        
        result_peg_196.addResult(result_peg_198);
        
        result_peg_196.setValue(result_peg_196.getLastValue())
        stream.update(RULE_line, position, result_peg_196)
        return result_peg_196
    except PegError:
        pass
    stream.update(RULE_line, position, None)
    return None


def rule_predicate(stream, position):
    if stream.hasResult(RULE_predicate, position):
        return stream.result(RULE_predicate, position)
    try:
        result_peg_199 = Result(position)
        result_peg_200 = Result(result_peg_199.getPosition())
        
        if '<predicate' == stream.get(result_peg_200.getPosition(), 10):
            result_peg_200.nextPosition(10)
            result_peg_200.setValue('<predicate')
        else:
            raise PegError
        
        result_peg_199.addResult(result_peg_200);
        
        result_peg_201 = Result(result_peg_199.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_201 = rule_whitespace(stream, result_peg_201.getPosition())
        if result_peg_201 == None:
            raise PegError
        
        result_peg_199.addResult(result_peg_201);
        
        result_peg_202 = Result(result_peg_199.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_202 = rule_word(stream, result_peg_202.getPosition())
        if result_peg_202 == None:
            raise PegError
        variable = result_peg_202.getValues()
        
        result_peg_199.addResult(result_peg_202);
        
        result_peg_203 = Result(result_peg_199.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_203 = rule_whitespace(stream, result_peg_203.getPosition())
        if result_peg_203 == None:
            raise PegError
        
        result_peg_199.addResult(result_peg_203);
        
        result_peg_204 = Result(result_peg_199.getPosition())
        
        if '>' == stream.get(result_peg_204.getPosition(), 1):
            result_peg_204.nextPosition(1)
            result_peg_204.setValue('>')
        else:
            raise PegError
        
        result_peg_199.addResult(result_peg_204);
        
        result_peg_205 = Result(result_peg_199.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_205 = rule_whitespace(stream, result_peg_205.getPosition())
        if result_peg_205 == None:
            raise PegError
        
        result_peg_199.addResult(result_peg_205);
        
        result_peg_206 = Result(result_peg_199.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_206 = rule_code(stream, result_peg_206.getPosition())
        if result_peg_206 == None:
            raise PegError
        code = result_peg_206.getValues()
        
        result_peg_199.addResult(result_peg_206);
        
        result_peg_207 = Result(result_peg_199.getPosition())
        
        value = None
        values = result_peg_199.getValues()
        value = peg.PatternPredicate(variable, code.code)
        result_peg_207.setValue(value)
        
        result_peg_199.addResult(result_peg_207);
        
        result_peg_199.setValue(result_peg_199.getLastValue())
        stream.update(RULE_predicate, position, result_peg_199)
        return result_peg_199
    except PegError:
        pass
    stream.update(RULE_predicate, position, None)
    return None


def rule_utf8(stream, position):
    if stream.hasResult(RULE_utf8, position):
        return stream.result(RULE_utf8, position)
    try:
        result_peg_208 = Result(position)
        result_peg_209 = Result(result_peg_208.getPosition())
        
        if '<utf8' == stream.get(result_peg_209.getPosition(), 5):
            result_peg_209.nextPosition(5)
            result_peg_209.setValue('<utf8')
        else:
            raise PegError
        
        result_peg_208.addResult(result_peg_209);
        
        result_peg_210 = Result(result_peg_208.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_210 = rule_spaces(stream, result_peg_210.getPosition())
        if result_peg_210 == None:
            raise PegError
        
        result_peg_208.addResult(result_peg_210);
        
        result_peg_211 = Result(result_peg_208.getPosition())
        
        # print "Trying rule " + 'hex_number'
        result_peg_211 = rule_hex_number(stream, result_peg_211.getPosition())
        if result_peg_211 == None:
            raise PegError
        num = result_peg_211.getValues()
        
        result_peg_208.addResult(result_peg_211);
        
        result_peg_212 = Result(result_peg_208.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_212 = rule_spaces(stream, result_peg_212.getPosition())
        if result_peg_212 == None:
            raise PegError
        
        result_peg_208.addResult(result_peg_212);
        
        result_peg_213 = Result(result_peg_208.getPosition())
        
        if '>' == stream.get(result_peg_213.getPosition(), 1):
            result_peg_213.nextPosition(1)
            result_peg_213.setValue('>')
        else:
            raise PegError
        
        result_peg_208.addResult(result_peg_213);
        
        result_peg_214 = Result(result_peg_208.getPosition())
        
        value = None
        values = result_peg_208.getValues()
        value = peg.createUtf8Pattern(num)
        result_peg_214.setValue(value)
        
        result_peg_208.addResult(result_peg_214);
        
        result_peg_208.setValue(result_peg_208.getLastValue())
        stream.update(RULE_utf8, position, result_peg_208)
        return result_peg_208
    except PegError:
        pass
    stream.update(RULE_utf8, position, None)
    return None


def rule_ascii(stream, position):
    if stream.hasResult(RULE_ascii, position):
        return stream.result(RULE_ascii, position)
    try:
        result_peg_215 = Result(position)
        result_peg_216 = Result(result_peg_215.getPosition())
        
        if '<ascii' == stream.get(result_peg_216.getPosition(), 6):
            result_peg_216.nextPosition(6)
            result_peg_216.setValue('<ascii')
        else:
            raise PegError
        
        result_peg_215.addResult(result_peg_216);
        
        result_peg_217 = Result(result_peg_215.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_217 = rule_spaces(stream, result_peg_217.getPosition())
        if result_peg_217 == None:
            raise PegError
        
        result_peg_215.addResult(result_peg_217);
        
        result_peg_218 = Result(result_peg_215.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_218 = rule_number(stream, result_peg_218.getPosition())
        if result_peg_218 == None:
            raise PegError
        num = result_peg_218.getValues()
        
        result_peg_215.addResult(result_peg_218);
        
        result_peg_219 = Result(result_peg_215.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_219 = rule_spaces(stream, result_peg_219.getPosition())
        if result_peg_219 == None:
            raise PegError
        
        result_peg_215.addResult(result_peg_219);
        
        result_peg_220 = Result(result_peg_215.getPosition())
        
        if '>' == stream.get(result_peg_220.getPosition(), 1):
            result_peg_220.nextPosition(1)
            result_peg_220.setValue('>')
        else:
            raise PegError
        
        result_peg_215.addResult(result_peg_220);
        
        result_peg_221 = Result(result_peg_215.getPosition())
        
        value = None
        values = result_peg_215.getValues()
        value = peg.PatternVerbatim(int(num))
        result_peg_221.setValue(value)
        
        result_peg_215.addResult(result_peg_221);
        
        result_peg_215.setValue(result_peg_215.getLastValue())
        stream.update(RULE_ascii, position, result_peg_215)
        return result_peg_215
    except PegError:
        pass
    stream.update(RULE_ascii, position, None)
    return None


def rule_call_rule(stream, position):
    if stream.hasResult(RULE_call_rule, position):
        return stream.result(RULE_call_rule, position)
    try:
        result_peg_222 = Result(position)
        result_peg_223 = Result(result_peg_222.getPosition())
        
        if '@' == stream.get(result_peg_223.getPosition(), 1):
            result_peg_223.nextPosition(1)
            result_peg_223.setValue('@')
        else:
            raise PegError
        
        result_peg_222.addResult(result_peg_223);
        
        result_peg_224 = Result(result_peg_222.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_224 = rule_word(stream, result_peg_224.getPosition())
        if result_peg_224 == None:
            raise PegError
        name = result_peg_224.getValues()
        
        result_peg_222.addResult(result_peg_224);
        
        result_peg_225 = Result(result_peg_222.getPosition())
        
        save_peg_226 = result_peg_225.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_225 = rule_parameters_rules(stream, result_peg_225.getPosition())
        if result_peg_225 == None:
            
            result_peg_225 = Result(save_peg_226)
            result_peg_225.setValue(None)
        rule_parameters = result_peg_225.getValues()
        
        result_peg_222.addResult(result_peg_225);
        
        result_peg_227 = Result(result_peg_222.getPosition())
        
        save_peg_228 = result_peg_227.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_227 = rule_parameters_values(stream, result_peg_227.getPosition())
        if result_peg_227 == None:
            
            result_peg_227 = Result(save_peg_228)
            result_peg_227.setValue(None)
        parameters = result_peg_227.getValues()
        
        result_peg_222.addResult(result_peg_227);
        
        result_peg_229 = Result(result_peg_222.getPosition())
        
        value = None
        values = result_peg_222.getValues()
        value = peg.PatternCallRule(name, rule_parameters, parameters)
        result_peg_229.setValue(value)
        
        result_peg_222.addResult(result_peg_229);
        
        result_peg_222.setValue(result_peg_222.getLastValue())
        stream.update(RULE_call_rule, position, result_peg_222)
        return result_peg_222
    except PegError:
        pass
    stream.update(RULE_call_rule, position, None)
    return None


def rule_eof(stream, position):
    if stream.hasResult(RULE_eof, position):
        return stream.result(RULE_eof, position)
    try:
        result_peg_230 = Result(position)
        result_peg_231 = Result(result_peg_230.getPosition())
        
        if '<eof>' == stream.get(result_peg_231.getPosition(), 5):
            result_peg_231.nextPosition(5)
            result_peg_231.setValue('<eof>')
        else:
            raise PegError
        
        result_peg_230.addResult(result_peg_231);
        
        result_peg_232 = Result(result_peg_230.getPosition())
        
        value = None
        values = result_peg_230.getValues()
        value = peg.PatternEof()
        result_peg_232.setValue(value)
        
        result_peg_230.addResult(result_peg_232);
        
        result_peg_230.setValue(result_peg_230.getLastValue())
        stream.update(RULE_eof, position, result_peg_230)
        return result_peg_230
    except PegError:
        pass
    stream.update(RULE_eof, position, None)
    return None


def rule_void(stream, position):
    if stream.hasResult(RULE_void, position):
        return stream.result(RULE_void, position)
    try:
        result_peg_233 = Result(position)
        result_peg_234 = Result(result_peg_233.getPosition())
        
        if '<void>' == stream.get(result_peg_234.getPosition(), 6):
            result_peg_234.nextPosition(6)
            result_peg_234.setValue('<void>')
        else:
            raise PegError
        
        result_peg_233.addResult(result_peg_234);
        
        result_peg_235 = Result(result_peg_233.getPosition())
        
        value = None
        values = result_peg_233.getValues()
        value = peg.PatternVoid()
        result_peg_235.setValue(value)
        
        result_peg_233.addResult(result_peg_235);
        
        result_peg_233.setValue(result_peg_233.getLastValue())
        stream.update(RULE_void, position, result_peg_233)
        return result_peg_233
    except PegError:
        pass
    stream.update(RULE_void, position, None)
    return None


def rule_range(stream, position):
    if stream.hasResult(RULE_range, position):
        return stream.result(RULE_range, position)
    try:
        result_peg_236 = Result(position)
        result_peg_237 = Result(result_peg_236.getPosition())
        
        if '[' == stream.get(result_peg_237.getPosition(), 1):
            result_peg_237.nextPosition(1)
            result_peg_237.setValue('[')
        else:
            raise PegError
        
        result_peg_236.addResult(result_peg_237);
        
        result_peg_238 = Result(result_peg_236.getPosition())
        
        try:
            while True:
                result_peg_239 = Result(result_peg_238.getPosition());
                result_peg_240 = Result(result_peg_239.getPosition())
                
                result_peg_241 = Result(result_peg_240.getPosition());
                try:
                    if ']' == stream.get(result_peg_241.getPosition(), 1):
                        result_peg_241.nextPosition(1)
                        result_peg_241.setValue(']')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_240.setValue(None)
                        
                result_peg_239.addResult(result_peg_240);
                
                result_peg_242 = Result(result_peg_239.getPosition())
                
                temp_peg_243 = stream.get(result_peg_242.getPosition())
                if temp_peg_243 != chr(0):
                    result_peg_242.setValue(temp_peg_243)
                    result_peg_242.nextPosition()
                else:
                    raise PegError
                
                result_peg_239.addResult(result_peg_242);
                
                result_peg_244 = Result(result_peg_239.getPosition())
                
                value = None
                values = result_peg_239.getValues()
                value = values[1]
                result_peg_244.setValue(value)
                
                result_peg_239.addResult(result_peg_244);
                
                result_peg_239.setValue(result_peg_239.getLastValue())
                result_peg_238.addResult(result_peg_239);
        except PegError:
            pass
                
        result_peg_236.addResult(result_peg_238);
        
        result_peg_245 = Result(result_peg_236.getPosition())
        
        if ']' == stream.get(result_peg_245.getPosition(), 1):
            result_peg_245.nextPosition(1)
            result_peg_245.setValue(']')
        else:
            raise PegError
        
        result_peg_236.addResult(result_peg_245);
        
        result_peg_246 = Result(result_peg_236.getPosition())
        
        value = None
        values = result_peg_236.getValues()
        value = peg.PatternRange(''.join(values[1]))
        result_peg_246.setValue(value)
        
        result_peg_236.addResult(result_peg_246);
        
        result_peg_236.setValue(result_peg_236.getLastValue())
        stream.update(RULE_range, position, result_peg_236)
        return result_peg_236
    except PegError:
        pass
    stream.update(RULE_range, position, None)
    return None


def rule_sub_pattern(stream, position):
    if stream.hasResult(RULE_sub_pattern, position):
        return stream.result(RULE_sub_pattern, position)
    try:
        result_peg_247 = Result(position)
        result_peg_248 = Result(result_peg_247.getPosition())
        
        if '(' == stream.get(result_peg_248.getPosition(), 1):
            result_peg_248.nextPosition(1)
            result_peg_248.setValue('(')
        else:
            raise PegError
        
        result_peg_247.addResult(result_peg_248);
        
        result_peg_249 = Result(result_peg_247.getPosition())
        
        try:
            while True:
                result_peg_250 = Result(result_peg_249.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_250 = rule_pattern(stream, result_peg_250.getPosition())
                if result_peg_250 == None:
                    raise PegError
                result_peg_249.addResult(result_peg_250);
        except PegError:
            if result_peg_249.matches() == 0:
                raise PegError
                
        result_peg_247.addResult(result_peg_249);
        
        result_peg_251 = Result(result_peg_247.getPosition())
        
        if ')' == stream.get(result_peg_251.getPosition(), 1):
            result_peg_251.nextPosition(1)
            result_peg_251.setValue(')')
        else:
            raise PegError
        
        result_peg_247.addResult(result_peg_251);
        
        result_peg_252 = Result(result_peg_247.getPosition())
        
        value = None
        values = result_peg_247.getValues()
        value = peg.PatternSequence(values[1])
        result_peg_252.setValue(value)
        
        result_peg_247.addResult(result_peg_252);
        
        result_peg_247.setValue(result_peg_247.getLastValue())
        stream.update(RULE_sub_pattern, position, result_peg_247)
        return result_peg_247
    except PegError:
        pass
    stream.update(RULE_sub_pattern, position, None)
    return None


def rule_bind(stream, position):
    if stream.hasResult(RULE_bind, position):
        return stream.result(RULE_bind, position)
    try:
        result_peg_253 = Result(position)
        result_peg_254 = Result(result_peg_253.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_254 = rule_word(stream, result_peg_254.getPosition())
        if result_peg_254 == None:
            raise PegError
        name = result_peg_254.getValues()
        
        result_peg_253.addResult(result_peg_254);
        
        result_peg_255 = Result(result_peg_253.getPosition())
        
        if ':' == stream.get(result_peg_255.getPosition(), 1):
            result_peg_255.nextPosition(1)
            result_peg_255.setValue(':')
        else:
            raise PegError
        
        result_peg_253.addResult(result_peg_255);
        
        result_peg_256 = Result(result_peg_253.getPosition())
        
        value = None
        values = result_peg_253.getValues()
        value = lambda p: peg.PatternBind(name, p)
        result_peg_256.setValue(value)
        
        result_peg_253.addResult(result_peg_256);
        
        result_peg_253.setValue(result_peg_253.getLastValue())
        stream.update(RULE_bind, position, result_peg_253)
        return result_peg_253
    except PegError:
        pass
    stream.update(RULE_bind, position, None)
    return None


def rule_string(stream, position):
    if stream.hasResult(RULE_string, position):
        return stream.result(RULE_string, position)
    try:
        result_peg_257 = Result(position)
        result_peg_258 = Result(result_peg_257.getPosition())
        
        if '"' == stream.get(result_peg_258.getPosition(), 1):
            result_peg_258.nextPosition(1)
            result_peg_258.setValue('"')
        else:
            raise PegError
        
        result_peg_257.addResult(result_peg_258);
        
        result_peg_259 = Result(result_peg_257.getPosition())
        
        try:
            while True:
                result_peg_260 = Result(result_peg_259.getPosition());
                result_peg_261 = Result(result_peg_260.getPosition())
                
                result_peg_262 = Result(result_peg_261.getPosition());
                try:
                    if '"' == stream.get(result_peg_262.getPosition(), 1):
                        result_peg_262.nextPosition(1)
                        result_peg_262.setValue('"')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_261.setValue(None)
                        
                result_peg_260.addResult(result_peg_261);
                
                result_peg_263 = Result(result_peg_260.getPosition())
                
                temp_peg_264 = stream.get(result_peg_263.getPosition())
                if temp_peg_264 != chr(0):
                    result_peg_263.setValue(temp_peg_264)
                    result_peg_263.nextPosition()
                else:
                    raise PegError
                
                result_peg_260.addResult(result_peg_263);
                
                result_peg_265 = Result(result_peg_260.getPosition())
                
                value = None
                values = result_peg_260.getValues()
                value = values[1]
                result_peg_265.setValue(value)
                
                result_peg_260.addResult(result_peg_265);
                
                result_peg_260.setValue(result_peg_260.getLastValue())
                result_peg_259.addResult(result_peg_260);
        except PegError:
            pass
                
        result_peg_257.addResult(result_peg_259);
        
        result_peg_266 = Result(result_peg_257.getPosition())
        
        if '"' == stream.get(result_peg_266.getPosition(), 1):
            result_peg_266.nextPosition(1)
            result_peg_266.setValue('"')
        else:
            raise PegError
        
        result_peg_257.addResult(result_peg_266);
        
        result_peg_267 = Result(result_peg_257.getPosition())
        
        save_peg_268 = result_peg_267.getPosition()
        
        if '{case}' == stream.get(result_peg_267.getPosition(), 6):
            result_peg_267.nextPosition(6)
            result_peg_267.setValue('{case}')
        else:
            
            result_peg_267 = Result(save_peg_268)
            result_peg_267.setValue(None)
        options = result_peg_267.getValues()
        
        result_peg_257.addResult(result_peg_267);
        
        result_peg_269 = Result(result_peg_257.getPosition())
        
        value = None
        values = result_peg_257.getValues()
        value = peg.PatternVerbatim(''.join(values[1]), options)
        result_peg_269.setValue(value)
        
        result_peg_257.addResult(result_peg_269);
        
        result_peg_257.setValue(result_peg_257.getLastValue())
        stream.update(RULE_string, position, result_peg_257)
        return result_peg_257
    except PegError:
        pass
    try:
        result_peg_270 = Result(position)
        result_peg_271 = Result(result_peg_270.getPosition())
        
        if '<quote>' == stream.get(result_peg_271.getPosition(), 7):
            result_peg_271.nextPosition(7)
            result_peg_271.setValue('<quote>')
        else:
            raise PegError
        
        result_peg_270.addResult(result_peg_271);
        
        result_peg_272 = Result(result_peg_270.getPosition())
        
        value = None
        values = result_peg_270.getValues()
        value = peg.PatternVerbatim('"')
        result_peg_272.setValue(value)
        
        result_peg_270.addResult(result_peg_272);
        
        result_peg_270.setValue(result_peg_270.getLastValue())
        stream.update(RULE_string, position, result_peg_270)
        return result_peg_270
    except PegError:
        pass
    stream.update(RULE_string, position, None)
    return None


def rule_modifier(stream, position):
    if stream.hasResult(RULE_modifier, position):
        return stream.result(RULE_modifier, position)
    try:
        result_peg_273 = Result(position)
        result_peg_274 = Result(result_peg_273.getPosition())
        
        if '*' == stream.get(result_peg_274.getPosition(), 1):
            result_peg_274.nextPosition(1)
            result_peg_274.setValue('*')
        else:
            raise PegError
        
        result_peg_273.addResult(result_peg_274);
        
        result_peg_275 = Result(result_peg_273.getPosition())
        
        value = None
        values = result_peg_273.getValues()
        value = lambda p: peg.PatternRepeatMany(p)
        result_peg_275.setValue(value)
        
        result_peg_273.addResult(result_peg_275);
        
        result_peg_273.setValue(result_peg_273.getLastValue())
        stream.update(RULE_modifier, position, result_peg_273)
        return result_peg_273
    except PegError:
        pass
    try:
        result_peg_276 = Result(position)
        result_peg_277 = Result(result_peg_276.getPosition())
        
        if '?' == stream.get(result_peg_277.getPosition(), 1):
            result_peg_277.nextPosition(1)
            result_peg_277.setValue('?')
        else:
            raise PegError
        
        result_peg_276.addResult(result_peg_277);
        
        result_peg_278 = Result(result_peg_276.getPosition())
        
        value = None
        values = result_peg_276.getValues()
        value = lambda p: peg.PatternMaybe(p)
        result_peg_278.setValue(value)
        
        result_peg_276.addResult(result_peg_278);
        
        result_peg_276.setValue(result_peg_276.getLastValue())
        stream.update(RULE_modifier, position, result_peg_276)
        return result_peg_276
    except PegError:
        pass
    try:
        result_peg_279 = Result(position)
        result_peg_280 = Result(result_peg_279.getPosition())
        
        if '+' == stream.get(result_peg_280.getPosition(), 1):
            result_peg_280.nextPosition(1)
            result_peg_280.setValue('+')
        else:
            raise PegError
        
        result_peg_279.addResult(result_peg_280);
        
        result_peg_281 = Result(result_peg_279.getPosition())
        
        value = None
        values = result_peg_279.getValues()
        value = lambda p: peg.PatternRepeatOnce(p)
        result_peg_281.setValue(value)
        
        result_peg_279.addResult(result_peg_281);
        
        result_peg_279.setValue(result_peg_279.getLastValue())
        stream.update(RULE_modifier, position, result_peg_279)
        return result_peg_279
    except PegError:
        pass
    stream.update(RULE_modifier, position, None)
    return None


def rule_x_word(stream, position):
    if stream.hasResult(RULE_x_word, position):
        return stream.result(RULE_x_word, position)
    try:
        result_peg_282 = Result(position)
        result_peg_283 = Result(result_peg_282.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_283 = rule_word(stream, result_peg_283.getPosition())
        if result_peg_283 == None:
            raise PegError
        name = result_peg_283.getValues()
        
        result_peg_282.addResult(result_peg_283);
        
        result_peg_284 = Result(result_peg_282.getPosition())
        
        save_peg_285 = result_peg_284.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_284 = rule_parameters_rules(stream, result_peg_284.getPosition())
        if result_peg_284 == None:
            
            result_peg_284 = Result(save_peg_285)
            result_peg_284.setValue(None)
        rule_parameters = result_peg_284.getValues()
        
        result_peg_282.addResult(result_peg_284);
        
        result_peg_286 = Result(result_peg_282.getPosition())
        
        save_peg_287 = result_peg_286.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_286 = rule_parameters_values(stream, result_peg_286.getPosition())
        if result_peg_286 == None:
            
            result_peg_286 = Result(save_peg_287)
            result_peg_286.setValue(None)
        parameters = result_peg_286.getValues()
        
        result_peg_282.addResult(result_peg_286);
        
        result_peg_288 = Result(result_peg_282.getPosition())
        
        value = None
        values = result_peg_282.getValues()
        value = peg.PatternRule(name, rule_parameters, parameters)
        result_peg_288.setValue(value)
        
        result_peg_282.addResult(result_peg_288);
        
        result_peg_282.setValue(result_peg_282.getLastValue())
        stream.update(RULE_x_word, position, result_peg_282)
        return result_peg_282
    except PegError:
        pass
    stream.update(RULE_x_word, position, None)
    return None


def rule_rule_parameters(stream, position):
    if stream.hasResult(RULE_rule_parameters, position):
        return stream.result(RULE_rule_parameters, position)
    try:
        result_peg_289 = Result(position)
        result_peg_290 = Result(result_peg_289.getPosition())
        
        if '[' == stream.get(result_peg_290.getPosition(), 1):
            result_peg_290.nextPosition(1)
            result_peg_290.setValue('[')
        else:
            raise PegError
        
        result_peg_289.addResult(result_peg_290);
        
        result_peg_291 = Result(result_peg_289.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_291 = rule_spaces(stream, result_peg_291.getPosition())
        if result_peg_291 == None:
            raise PegError
        
        result_peg_289.addResult(result_peg_291);
        
        result_peg_292 = Result(result_peg_289.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_292 = rule_word(stream, result_peg_292.getPosition())
        if result_peg_292 == None:
            raise PegError
        param1 = result_peg_292.getValues()
        
        result_peg_289.addResult(result_peg_292);
        
        result_peg_293 = Result(result_peg_289.getPosition())
        
        try:
            while True:
                result_peg_294 = Result(result_peg_293.getPosition());
                result_peg_295 = Result(result_peg_294.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_295 = rule_spaces(stream, result_peg_295.getPosition())
                if result_peg_295 == None:
                    raise PegError
                
                result_peg_294.addResult(result_peg_295);
                
                result_peg_296 = Result(result_peg_294.getPosition())
                
                if ',' == stream.get(result_peg_296.getPosition(), 1):
                    result_peg_296.nextPosition(1)
                    result_peg_296.setValue(',')
                else:
                    raise PegError
                
                result_peg_294.addResult(result_peg_296);
                
                result_peg_297 = Result(result_peg_294.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_297 = rule_spaces(stream, result_peg_297.getPosition())
                if result_peg_297 == None:
                    raise PegError
                
                result_peg_294.addResult(result_peg_297);
                
                result_peg_298 = Result(result_peg_294.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_298 = rule_word(stream, result_peg_298.getPosition())
                if result_peg_298 == None:
                    raise PegError
                exp = result_peg_298.getValues()
                
                result_peg_294.addResult(result_peg_298);
                
                result_peg_299 = Result(result_peg_294.getPosition())
                
                value = None
                values = result_peg_294.getValues()
                value = exp
                result_peg_299.setValue(value)
                
                result_peg_294.addResult(result_peg_299);
                
                result_peg_294.setValue(result_peg_294.getLastValue())
                result_peg_293.addResult(result_peg_294);
        except PegError:
            pass
        params = result_peg_293.getValues()
        
        result_peg_289.addResult(result_peg_293);
        
        result_peg_300 = Result(result_peg_289.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_300 = rule_spaces(stream, result_peg_300.getPosition())
        if result_peg_300 == None:
            raise PegError
        
        result_peg_289.addResult(result_peg_300);
        
        result_peg_301 = Result(result_peg_289.getPosition())
        
        if ']' == stream.get(result_peg_301.getPosition(), 1):
            result_peg_301.nextPosition(1)
            result_peg_301.setValue(']')
        else:
            raise PegError
        
        result_peg_289.addResult(result_peg_301);
        
        result_peg_302 = Result(result_peg_289.getPosition())
        
        value = None
        values = result_peg_289.getValues()
        value = [param1] + params
        result_peg_302.setValue(value)
        
        result_peg_289.addResult(result_peg_302);
        
        result_peg_289.setValue(result_peg_289.getLastValue())
        stream.update(RULE_rule_parameters, position, result_peg_289)
        return result_peg_289
    except PegError:
        pass
    stream.update(RULE_rule_parameters, position, None)
    return None


def rule_value_parameters(stream, position):
    if stream.hasResult(RULE_value_parameters, position):
        return stream.result(RULE_value_parameters, position)
    try:
        result_peg_303 = Result(position)
        result_peg_304 = Result(result_peg_303.getPosition())
        
        if '(' == stream.get(result_peg_304.getPosition(), 1):
            result_peg_304.nextPosition(1)
            result_peg_304.setValue('(')
        else:
            raise PegError
        
        result_peg_303.addResult(result_peg_304);
        
        result_peg_305 = Result(result_peg_303.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_305 = rule_spaces(stream, result_peg_305.getPosition())
        if result_peg_305 == None:
            raise PegError
        
        result_peg_303.addResult(result_peg_305);
        
        result_peg_306 = Result(result_peg_303.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_306 = rule_word(stream, result_peg_306.getPosition())
        if result_peg_306 == None:
            raise PegError
        param1 = result_peg_306.getValues()
        
        result_peg_303.addResult(result_peg_306);
        
        result_peg_307 = Result(result_peg_303.getPosition())
        
        try:
            while True:
                result_peg_308 = Result(result_peg_307.getPosition());
                result_peg_309 = Result(result_peg_308.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_309 = rule_spaces(stream, result_peg_309.getPosition())
                if result_peg_309 == None:
                    raise PegError
                
                result_peg_308.addResult(result_peg_309);
                
                result_peg_310 = Result(result_peg_308.getPosition())
                
                if ',' == stream.get(result_peg_310.getPosition(), 1):
                    result_peg_310.nextPosition(1)
                    result_peg_310.setValue(',')
                else:
                    raise PegError
                
                result_peg_308.addResult(result_peg_310);
                
                result_peg_311 = Result(result_peg_308.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_311 = rule_spaces(stream, result_peg_311.getPosition())
                if result_peg_311 == None:
                    raise PegError
                
                result_peg_308.addResult(result_peg_311);
                
                result_peg_312 = Result(result_peg_308.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_312 = rule_word(stream, result_peg_312.getPosition())
                if result_peg_312 == None:
                    raise PegError
                exp = result_peg_312.getValues()
                
                result_peg_308.addResult(result_peg_312);
                
                result_peg_313 = Result(result_peg_308.getPosition())
                
                value = None
                values = result_peg_308.getValues()
                value = exp
                result_peg_313.setValue(value)
                
                result_peg_308.addResult(result_peg_313);
                
                result_peg_308.setValue(result_peg_308.getLastValue())
                result_peg_307.addResult(result_peg_308);
        except PegError:
            pass
        params = result_peg_307.getValues()
        
        result_peg_303.addResult(result_peg_307);
        
        result_peg_314 = Result(result_peg_303.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_314 = rule_spaces(stream, result_peg_314.getPosition())
        if result_peg_314 == None:
            raise PegError
        
        result_peg_303.addResult(result_peg_314);
        
        result_peg_315 = Result(result_peg_303.getPosition())
        
        if ')' == stream.get(result_peg_315.getPosition(), 1):
            result_peg_315.nextPosition(1)
            result_peg_315.setValue(')')
        else:
            raise PegError
        
        result_peg_303.addResult(result_peg_315);
        
        result_peg_316 = Result(result_peg_303.getPosition())
        
        value = None
        values = result_peg_303.getValues()
        value = [param1] + params
        result_peg_316.setValue(value)
        
        result_peg_303.addResult(result_peg_316);
        
        result_peg_303.setValue(result_peg_303.getLastValue())
        stream.update(RULE_value_parameters, position, result_peg_303)
        return result_peg_303
    except PegError:
        pass
    stream.update(RULE_value_parameters, position, None)
    return None


def rule_parameters_rules(stream, position):
    if stream.hasResult(RULE_parameters_rules, position):
        return stream.result(RULE_parameters_rules, position)
    try:
        result_peg_317 = Result(position)
        result_peg_318 = Result(result_peg_317.getPosition())
        
        if '[' == stream.get(result_peg_318.getPosition(), 1):
            result_peg_318.nextPosition(1)
            result_peg_318.setValue('[')
        else:
            raise PegError
        
        result_peg_317.addResult(result_peg_318);
        
        result_peg_319 = Result(result_peg_317.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_319 = rule_spaces(stream, result_peg_319.getPosition())
        if result_peg_319 == None:
            raise PegError
        
        result_peg_317.addResult(result_peg_319);
        
        result_peg_320 = Result(result_peg_317.getPosition())
        
        # print "Trying rule " + 'word_or_at'
        result_peg_320 = rule_word_or_at(stream, result_peg_320.getPosition())
        if result_peg_320 == None:
            raise PegError
        param1 = result_peg_320.getValues()
        
        result_peg_317.addResult(result_peg_320);
        
        result_peg_321 = Result(result_peg_317.getPosition())
        
        try:
            while True:
                result_peg_322 = Result(result_peg_321.getPosition());
                result_peg_323 = Result(result_peg_322.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_323 = rule_spaces(stream, result_peg_323.getPosition())
                if result_peg_323 == None:
                    raise PegError
                
                result_peg_322.addResult(result_peg_323);
                
                result_peg_324 = Result(result_peg_322.getPosition())
                
                if ',' == stream.get(result_peg_324.getPosition(), 1):
                    result_peg_324.nextPosition(1)
                    result_peg_324.setValue(',')
                else:
                    raise PegError
                
                result_peg_322.addResult(result_peg_324);
                
                result_peg_325 = Result(result_peg_322.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_325 = rule_spaces(stream, result_peg_325.getPosition())
                if result_peg_325 == None:
                    raise PegError
                
                result_peg_322.addResult(result_peg_325);
                
                result_peg_326 = Result(result_peg_322.getPosition())
                
                # print "Trying rule " + 'word_or_at'
                result_peg_326 = rule_word_or_at(stream, result_peg_326.getPosition())
                if result_peg_326 == None:
                    raise PegError
                exp = result_peg_326.getValues()
                
                result_peg_322.addResult(result_peg_326);
                
                result_peg_327 = Result(result_peg_322.getPosition())
                
                value = None
                values = result_peg_322.getValues()
                value = exp
                result_peg_327.setValue(value)
                
                result_peg_322.addResult(result_peg_327);
                
                result_peg_322.setValue(result_peg_322.getLastValue())
                result_peg_321.addResult(result_peg_322);
        except PegError:
            pass
        params = result_peg_321.getValues()
        
        result_peg_317.addResult(result_peg_321);
        
        result_peg_328 = Result(result_peg_317.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_328 = rule_spaces(stream, result_peg_328.getPosition())
        if result_peg_328 == None:
            raise PegError
        
        result_peg_317.addResult(result_peg_328);
        
        result_peg_329 = Result(result_peg_317.getPosition())
        
        if ']' == stream.get(result_peg_329.getPosition(), 1):
            result_peg_329.nextPosition(1)
            result_peg_329.setValue(']')
        else:
            raise PegError
        
        result_peg_317.addResult(result_peg_329);
        
        result_peg_330 = Result(result_peg_317.getPosition())
        
        value = None
        values = result_peg_317.getValues()
        value = [param1] + params
        result_peg_330.setValue(value)
        
        result_peg_317.addResult(result_peg_330);
        
        result_peg_317.setValue(result_peg_317.getLastValue())
        stream.update(RULE_parameters_rules, position, result_peg_317)
        return result_peg_317
    except PegError:
        pass
    stream.update(RULE_parameters_rules, position, None)
    return None


def rule_parameters_values(stream, position):
    if stream.hasResult(RULE_parameters_values, position):
        return stream.result(RULE_parameters_values, position)
    try:
        result_peg_331 = Result(position)
        result_peg_332 = Result(result_peg_331.getPosition())
        
        if '(' == stream.get(result_peg_332.getPosition(), 1):
            result_peg_332.nextPosition(1)
            result_peg_332.setValue('(')
        else:
            raise PegError
        
        result_peg_331.addResult(result_peg_332);
        
        result_peg_333 = Result(result_peg_331.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_333 = rule_spaces(stream, result_peg_333.getPosition())
        if result_peg_333 == None:
            raise PegError
        
        result_peg_331.addResult(result_peg_333);
        
        result_peg_334 = Result(result_peg_331.getPosition())
        
        # print "Trying rule " + 'word_or_dollar'
        result_peg_334 = rule_word_or_dollar(stream, result_peg_334.getPosition())
        if result_peg_334 == None:
            raise PegError
        param1 = result_peg_334.getValues()
        
        result_peg_331.addResult(result_peg_334);
        
        result_peg_335 = Result(result_peg_331.getPosition())
        
        try:
            while True:
                result_peg_336 = Result(result_peg_335.getPosition());
                result_peg_337 = Result(result_peg_336.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_337 = rule_spaces(stream, result_peg_337.getPosition())
                if result_peg_337 == None:
                    raise PegError
                
                result_peg_336.addResult(result_peg_337);
                
                result_peg_338 = Result(result_peg_336.getPosition())
                
                if ',' == stream.get(result_peg_338.getPosition(), 1):
                    result_peg_338.nextPosition(1)
                    result_peg_338.setValue(',')
                else:
                    raise PegError
                
                result_peg_336.addResult(result_peg_338);
                
                result_peg_339 = Result(result_peg_336.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_339 = rule_spaces(stream, result_peg_339.getPosition())
                if result_peg_339 == None:
                    raise PegError
                
                result_peg_336.addResult(result_peg_339);
                
                result_peg_340 = Result(result_peg_336.getPosition())
                
                # print "Trying rule " + 'word_or_dollar'
                result_peg_340 = rule_word_or_dollar(stream, result_peg_340.getPosition())
                if result_peg_340 == None:
                    raise PegError
                exp = result_peg_340.getValues()
                
                result_peg_336.addResult(result_peg_340);
                
                result_peg_341 = Result(result_peg_336.getPosition())
                
                value = None
                values = result_peg_336.getValues()
                value = exp
                result_peg_341.setValue(value)
                
                result_peg_336.addResult(result_peg_341);
                
                result_peg_336.setValue(result_peg_336.getLastValue())
                result_peg_335.addResult(result_peg_336);
        except PegError:
            pass
        params = result_peg_335.getValues()
        
        result_peg_331.addResult(result_peg_335);
        
        result_peg_342 = Result(result_peg_331.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_342 = rule_spaces(stream, result_peg_342.getPosition())
        if result_peg_342 == None:
            raise PegError
        
        result_peg_331.addResult(result_peg_342);
        
        result_peg_343 = Result(result_peg_331.getPosition())
        
        if ')' == stream.get(result_peg_343.getPosition(), 1):
            result_peg_343.nextPosition(1)
            result_peg_343.setValue(')')
        else:
            raise PegError
        
        result_peg_331.addResult(result_peg_343);
        
        result_peg_344 = Result(result_peg_331.getPosition())
        
        value = None
        values = result_peg_331.getValues()
        value = [param1] + params
        result_peg_344.setValue(value)
        
        result_peg_331.addResult(result_peg_344);
        
        result_peg_331.setValue(result_peg_331.getLastValue())
        stream.update(RULE_parameters_values, position, result_peg_331)
        return result_peg_331
    except PegError:
        pass
    stream.update(RULE_parameters_values, position, None)
    return None


def rule_word_or_dollar(stream, position):
    if stream.hasResult(RULE_word_or_dollar, position):
        return stream.result(RULE_word_or_dollar, position)
    try:
        result_peg_345 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_345 = rule_word(stream, result_peg_345.getPosition())
        if result_peg_345 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_345)
        return result_peg_345
    except PegError:
        pass
    try:
        result_peg_346 = Result(position)
        # print "Trying rule " + 'dollar'
        result_peg_346 = rule_dollar(stream, result_peg_346.getPosition())
        if result_peg_346 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_346)
        return result_peg_346
    except PegError:
        pass
    stream.update(RULE_word_or_dollar, position, None)
    return None


def rule_word_or_at(stream, position):
    if stream.hasResult(RULE_word_or_at, position):
        return stream.result(RULE_word_or_at, position)
    try:
        result_peg_347 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_347 = rule_word(stream, result_peg_347.getPosition())
        if result_peg_347 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_347)
        return result_peg_347
    except PegError:
        pass
    try:
        result_peg_348 = Result(position)
        # print "Trying rule " + 'word_at'
        result_peg_348 = rule_word_at(stream, result_peg_348.getPosition())
        if result_peg_348 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_348)
        return result_peg_348
    except PegError:
        pass
    stream.update(RULE_word_or_at, position, None)
    return None


def rule_word_at(stream, position):
    if stream.hasResult(RULE_word_at, position):
        return stream.result(RULE_word_at, position)
    try:
        result_peg_349 = Result(position)
        result_peg_350 = Result(result_peg_349.getPosition())
        
        if '@' == stream.get(result_peg_350.getPosition(), 1):
            result_peg_350.nextPosition(1)
            result_peg_350.setValue('@')
        else:
            raise PegError
        
        result_peg_349.addResult(result_peg_350);
        
        result_peg_351 = Result(result_peg_349.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_351 = rule_word(stream, result_peg_351.getPosition())
        if result_peg_351 == None:
            raise PegError
        word = result_peg_351.getValues()
        
        result_peg_349.addResult(result_peg_351);
        
        result_peg_352 = Result(result_peg_349.getPosition())
        
        value = None
        values = result_peg_349.getValues()
        value = '@%s' % word
        result_peg_352.setValue(value)
        
        result_peg_349.addResult(result_peg_352);
        
        result_peg_349.setValue(result_peg_349.getLastValue())
        stream.update(RULE_word_at, position, result_peg_349)
        return result_peg_349
    except PegError:
        pass
    stream.update(RULE_word_at, position, None)
    return None


def rule_dollar(stream, position):
    if stream.hasResult(RULE_dollar, position):
        return stream.result(RULE_dollar, position)
    try:
        result_peg_353 = Result(position)
        result_peg_354 = Result(result_peg_353.getPosition())
        
        if '$' == stream.get(result_peg_354.getPosition(), 1):
            result_peg_354.nextPosition(1)
            result_peg_354.setValue('$')
        else:
            raise PegError
        
        result_peg_353.addResult(result_peg_354);
        
        result_peg_355 = Result(result_peg_353.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_355 = rule_number(stream, result_peg_355.getPosition())
        if result_peg_355 == None:
            raise PegError
        number = result_peg_355.getValues()
        
        result_peg_353.addResult(result_peg_355);
        
        result_peg_356 = Result(result_peg_353.getPosition())
        
        value = None
        values = result_peg_353.getValues()
        value = "$%s" % number
        result_peg_356.setValue(value)
        
        result_peg_353.addResult(result_peg_356);
        
        result_peg_353.setValue(result_peg_353.getLastValue())
        stream.update(RULE_dollar, position, result_peg_353)
        return result_peg_353
    except PegError:
        pass
    stream.update(RULE_dollar, position, None)
    return None


def rule_number(stream, position):
    if stream.hasResult(RULE_number, position):
        return stream.result(RULE_number, position)
    try:
        result_peg_357 = Result(position)
        result_peg_358 = Result(result_peg_357.getPosition())
        
        try:
            while True:
                result_peg_359 = Result(result_peg_358.getPosition());
                # print "Trying rule " + 'digit'
                result_peg_359 = rule_digit(stream, result_peg_359.getPosition())
                if result_peg_359 == None:
                    raise PegError
                result_peg_358.addResult(result_peg_359);
        except PegError:
            if result_peg_358.matches() == 0:
                raise PegError
                
        result_peg_357.addResult(result_peg_358);
        
        result_peg_360 = Result(result_peg_357.getPosition())
        
        value = None
        values = result_peg_357.getValues()
        value = ''.join(values[0])
        result_peg_360.setValue(value)
        
        result_peg_357.addResult(result_peg_360);
        
        result_peg_357.setValue(result_peg_357.getLastValue())
        stream.update(RULE_number, position, result_peg_357)
        return result_peg_357
    except PegError:
        pass
    stream.update(RULE_number, position, None)
    return None


def rule_digit(stream, position):
    if stream.hasResult(RULE_digit, position):
        return stream.result(RULE_digit, position)
    try:
        result_peg_361 = Result(position)
        letter_peg_362 = stream.get(result_peg_361.getPosition())
        if letter_peg_362 in '0123456789':
            result_peg_361.nextPosition()
            result_peg_361.setValue(letter_peg_362)
        else:
            raise PegError
        stream.update(RULE_digit, position, result_peg_361)
        return result_peg_361
    except PegError:
        pass
    stream.update(RULE_digit, position, None)
    return None


def rule_hex_number(stream, position):
    if stream.hasResult(RULE_hex_number, position):
        return stream.result(RULE_hex_number, position)
    try:
        result_peg_363 = Result(position)
        result_peg_364 = Result(result_peg_363.getPosition())
        
        try:
            while True:
                result_peg_365 = Result(result_peg_364.getPosition());
                # print "Trying rule " + 'hex_digit'
                result_peg_365 = rule_hex_digit(stream, result_peg_365.getPosition())
                if result_peg_365 == None:
                    raise PegError
                result_peg_364.addResult(result_peg_365);
        except PegError:
            if result_peg_364.matches() == 0:
                raise PegError
                
        result_peg_363.addResult(result_peg_364);
        
        result_peg_366 = Result(result_peg_363.getPosition())
        
        value = None
        values = result_peg_363.getValues()
        value = ''.join(values[0])
        result_peg_366.setValue(value)
        
        result_peg_363.addResult(result_peg_366);
        
        result_peg_363.setValue(result_peg_363.getLastValue())
        stream.update(RULE_hex_number, position, result_peg_363)
        return result_peg_363
    except PegError:
        pass
    stream.update(RULE_hex_number, position, None)
    return None


def rule_hex_digit(stream, position):
    if stream.hasResult(RULE_hex_digit, position):
        return stream.result(RULE_hex_digit, position)
    try:
        result_peg_367 = Result(position)
        letter_peg_368 = stream.get(result_peg_367.getPosition())
        if letter_peg_368 in '0123456789abcdefABCDEF':
            result_peg_367.nextPosition()
            result_peg_367.setValue(letter_peg_368)
        else:
            raise PegError
        stream.update(RULE_hex_digit, position, result_peg_367)
        return result_peg_367
    except PegError:
        pass
    stream.update(RULE_hex_digit, position, None)
    return None


def rule_start_symbol(stream, position):
    if stream.hasResult(RULE_start_symbol, position):
        return stream.result(RULE_start_symbol, position)
    try:
        result_peg_369 = Result(position)
        result_peg_370 = Result(result_peg_369.getPosition())
        
        if 'start-symbol:' == stream.get(result_peg_370.getPosition(), 13):
            result_peg_370.nextPosition(13)
            result_peg_370.setValue('start-symbol:')
        else:
            raise PegError
        
        result_peg_369.addResult(result_peg_370);
        
        result_peg_371 = Result(result_peg_369.getPosition())
        
        try:
            while True:
                result_peg_372 = Result(result_peg_371.getPosition());
                # print "Trying rule " + 'space'
                result_peg_372 = rule_space(stream, result_peg_372.getPosition())
                if result_peg_372 == None:
                    raise PegError
                result_peg_371.addResult(result_peg_372);
        except PegError:
            pass
                
        result_peg_369.addResult(result_peg_371);
        
        result_peg_373 = Result(result_peg_369.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_373 = rule_word(stream, result_peg_373.getPosition())
        if result_peg_373 == None:
            raise PegError
        
        result_peg_369.addResult(result_peg_373);
        
        result_peg_374 = Result(result_peg_369.getPosition())
        
        value = None
        values = result_peg_369.getValues()
        value = values[2]
        result_peg_374.setValue(value)
        
        result_peg_369.addResult(result_peg_374);
        
        result_peg_369.setValue(result_peg_369.getLastValue())
        stream.update(RULE_start_symbol, position, result_peg_369)
        return result_peg_369
    except PegError:
        pass
    stream.update(RULE_start_symbol, position, None)
    return None


def rule_spaces(stream, position):
    if stream.hasResult(RULE_spaces, position):
        return stream.result(RULE_spaces, position)
    try:
        result_peg_375 = Result(position)
        result_peg_376 = Result(result_peg_375.getPosition())
        
        try:
            while True:
                result_peg_377 = Result(result_peg_376.getPosition());
                # print "Trying rule " + 'space'
                result_peg_377 = rule_space(stream, result_peg_377.getPosition())
                if result_peg_377 == None:
                    raise PegError
                result_peg_376.addResult(result_peg_377);
        except PegError:
            pass
                
        result_peg_375.addResult(result_peg_376);
        
        result_peg_375.setValue(result_peg_375.getLastValue())
        stream.update(RULE_spaces, position, result_peg_375)
        return result_peg_375
    except PegError:
        pass
    stream.update(RULE_spaces, position, None)
    return None


def rule_space(stream, position):
    if stream.hasResult(RULE_space, position):
        return stream.result(RULE_space, position)
    try:
        result_peg_378 = Result(position)
        result_peg_379 = Result(result_peg_378.getPosition())
        
        if ' ' == stream.get(result_peg_379.getPosition(), 1):
            result_peg_379.nextPosition(1)
            result_peg_379.setValue(' ')
        else:
            raise PegError
        
        result_peg_378.addResult(result_peg_379);
        
        result_peg_378.setValue(result_peg_378.getLastValue())
        stream.update(RULE_space, position, result_peg_378)
        return result_peg_378
    except PegError:
        pass
    try:
        result_peg_380 = Result(position)
        result_peg_381 = Result(result_peg_380.getPosition())
        
        if '\t' == stream.get(result_peg_381.getPosition(), 1):
            result_peg_381.nextPosition(1)
            result_peg_381.setValue('\t')
        else:
            raise PegError
        
        result_peg_380.addResult(result_peg_381);
        
        result_peg_380.setValue(result_peg_380.getLastValue())
        stream.update(RULE_space, position, result_peg_380)
        return result_peg_380
    except PegError:
        pass
    stream.update(RULE_space, position, None)
    return None


def rule_any_char(stream, position):
    if stream.hasResult(RULE_any_char, position):
        return stream.result(RULE_any_char, position)
    try:
        result_peg_382 = Result(position)
        letter_peg_383 = stream.get(result_peg_382.getPosition())
        if letter_peg_383 in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-':
            result_peg_382.nextPosition()
            result_peg_382.setValue(letter_peg_383)
        else:
            raise PegError
        stream.update(RULE_any_char, position, result_peg_382)
        return result_peg_382
    except PegError:
        pass
    stream.update(RULE_any_char, position, None)
    return None


def rule_any(stream, position):
    if stream.hasResult(RULE_any, position):
        return stream.result(RULE_any, position)
    try:
        result_peg_384 = Result(position)
        result_peg_385 = Result(result_peg_384.getPosition())
        
        if '.' == stream.get(result_peg_385.getPosition(), 1):
            result_peg_385.nextPosition(1)
            result_peg_385.setValue('.')
        else:
            raise PegError
        
        result_peg_384.addResult(result_peg_385);
        
        result_peg_386 = Result(result_peg_384.getPosition())
        
        value = None
        values = result_peg_384.getValues()
        value = peg.PatternAny()
        result_peg_386.setValue(value)
        
        result_peg_384.addResult(result_peg_386);
        
        result_peg_384.setValue(result_peg_384.getLastValue())
        stream.update(RULE_any, position, result_peg_384)
        return result_peg_384
    except PegError:
        pass
    stream.update(RULE_any, position, None)
    return None


def rule_whitespace(stream, position):
    if stream.hasResult(RULE_whitespace, position):
        return stream.result(RULE_whitespace, position)
    try:
        result_peg_387 = Result(position)
        result_peg_388 = Result(result_peg_387.getPosition())
        
        try:
            while True:
                result_peg_389 = Result(result_peg_388.getPosition());
                save_peg_390 = result_peg_389.getPosition()
                
                result_peg_389 = Result(save_peg_390)
                letter_peg_391 = stream.get(result_peg_389.getPosition())
                if letter_peg_391 in ' \t\n':
                    result_peg_389.nextPosition()
                    result_peg_389.setValue(letter_peg_391)
                else:
                    
                    result_peg_389 = Result(save_peg_390)
                    # print "Trying rule " + 'comment'
                    result_peg_389 = rule_comment(stream, result_peg_389.getPosition())
                    if result_peg_389 == None:
                        raise PegError
                result_peg_388.addResult(result_peg_389);
        except PegError:
            pass
                
        result_peg_387.addResult(result_peg_388);
        
        result_peg_387.setValue(result_peg_387.getLastValue())
        stream.update(RULE_whitespace, position, result_peg_387)
        return result_peg_387
    except PegError:
        pass
    stream.update(RULE_whitespace, position, None)
    return None


def rule_comment(stream, position):
    if stream.hasResult(RULE_comment, position):
        return stream.result(RULE_comment, position)
    try:
        result_peg_392 = Result(position)
        result_peg_393 = Result(result_peg_392.getPosition())
        
        if '#' == stream.get(result_peg_393.getPosition(), 1):
            result_peg_393.nextPosition(1)
            result_peg_393.setValue('#')
        else:
            raise PegError
        
        result_peg_392.addResult(result_peg_393);
        
        result_peg_394 = Result(result_peg_392.getPosition())
        
        try:
            while True:
                result_peg_395 = Result(result_peg_394.getPosition());
                result_peg_396 = Result(result_peg_395.getPosition())
                
                result_peg_397 = Result(result_peg_396.getPosition());
                try:
                    if '\n' == stream.get(result_peg_397.getPosition(), 1):
                        result_peg_397.nextPosition(1)
                        result_peg_397.setValue('\n')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_396.setValue(None)
                        
                result_peg_395.addResult(result_peg_396);
                
                result_peg_398 = Result(result_peg_395.getPosition())
                
                temp_peg_399 = stream.get(result_peg_398.getPosition())
                if temp_peg_399 != chr(0):
                    result_peg_398.setValue(temp_peg_399)
                    result_peg_398.nextPosition()
                else:
                    raise PegError
                
                result_peg_395.addResult(result_peg_398);
                
                result_peg_395.setValue(result_peg_395.getLastValue())
                result_peg_394.addResult(result_peg_395);
        except PegError:
            pass
                
        result_peg_392.addResult(result_peg_394);
        
        result_peg_392.setValue(result_peg_392.getLastValue())
        stream.update(RULE_comment, position, result_peg_392)
        return result_peg_392
    except PegError:
        pass
    stream.update(RULE_comment, position, None)
    return None


def rule_newlines_one(stream, position):
    if stream.hasResult(RULE_newlines_one, position):
        return stream.result(RULE_newlines_one, position)
    try:
        result_peg_400 = Result(position)
        try:
            while True:
                result_peg_401 = Result(result_peg_400.getPosition());
                if '\n' == stream.get(result_peg_401.getPosition(), 1):
                    result_peg_401.nextPosition(1)
                    result_peg_401.setValue('\n')
                else:
                    raise PegError
                result_peg_400.addResult(result_peg_401);
        except PegError:
            if result_peg_400.matches() == 0:
                raise PegError
        stream.update(RULE_newlines_one, position, result_peg_400)
        return result_peg_400
    except PegError:
        pass
    stream.update(RULE_newlines_one, position, None)
    return None


def rule_newlines(stream, position):
    if stream.hasResult(RULE_newlines, position):
        return stream.result(RULE_newlines, position)
    try:
        result_peg_402 = Result(position)
        try:
            while True:
                result_peg_403 = Result(result_peg_402.getPosition());
                if '\n' == stream.get(result_peg_403.getPosition(), 1):
                    result_peg_403.nextPosition(1)
                    result_peg_403.setValue('\n')
                else:
                    raise PegError
                result_peg_402.addResult(result_peg_403);
        except PegError:
            pass
        stream.update(RULE_newlines, position, result_peg_402)
        return result_peg_402
    except PegError:
        pass
    stream.update(RULE_newlines, position, None)
    return None


def doParse(stream):
    done = rule_start(stream, 0)
    if done == None:
        # print "Error parsing " + file
        stream.reportError()
        return None
    else:
        return done.getValues()

def parseFile(file):
    # print "Parsing " + file
    return doParse(Stream(filename = file))

def parseString(value):
    return doParse(Stream(input = value))

