
import peg





def special_escape(s):
    return s.replace("\\n", "\\\\n").replace("\\t", "\\\\t").replace("\"", '\\\"').replace("\\r", "\\\\r")

class PegError(Exception):
    def __init__(self):
        Exception.__init__(self)

class NotError(Exception):
    def __init__(self):
        Exception.__init__(self)

class Result:
    def __init__(self, position):
        self.position = position
        self.values = []

    def getPosition(self):
        return self.position

    def nextPosition(self, amount = 1):
        self.position += amount

    def setValue(self, value):
        self.values = value

    def getLastValue(self):
        if type(self.values) is list:
            if len(self.values) > 0:
                return self.values[-1]
            else:
                return None
        return self.values
    
    def matches(self):
        return len(self.values)

    def getValues(self):
        return self.values

    def addResult(self, him):
        self.values.append(him.values)
        self.position = him.position
    
    #def extendResult(self, him):
    #    self.values.extend(him.values)
    #    self.position = him.position

class Stream:
    def __init__(self, filename = None, input = None):
        def read():
            file = open(filename, 'r')
            out = file.read()
            file.close()
            return out
        self.position = 0
        self.limit = 100
        self.furthest = 0
        self.memo = {}
        if filename != None:
            self.all = read()
        elif input != None:
            self.all = input
        else:
            raise PegError("Pass a filename or input")
        # print "Read " + str(len(self.all))

    def get(self, position, number = 1):
        if position + number > self.limit:
            # print (position + number)
            self.limit += 5000
        if position + number > len(self.all):
            return chr(0)
        # print "stream: %s" % self.all[position:position+number]
        return self.all[position:position+number]

    def get2(self, position):
        if position != self.position:
            self.file.seek(position)
        self.position = position + 1
        if position > self.limit:
            print position
            self.limit += 5000
        return self.file.read(1)

    def reportError(self):
        line = 1
        column = 1
        for i in xrange(0, self.furthest):
            if self.all[i] == '\n':
                line += 1
                column = 1
            else:
                column += 1
        context = 10
        left = self.furthest - context
        right = self.furthest + context
        if left < 0:
            left = 0
        if right > len(self.all):
            right = len(self.all)
        print "Read up till line %d, column %d" % (line, column)
        print "'%s'" % special_escape(self.all[left:right])
        print "%s^" % (' ' * (self.furthest - left))

    def update(self, rule, position, result):
        if result != None and result.getPosition() > self.furthest:
            self.furthest = result.getPosition()

        for_rule = None
        try:
            for_rule = self.memo[rule]
        except KeyError:
            self.memo[rule] = {}
            for_rule = self.memo[rule]
        
        for_position = None
        try:
            for_position = for_rule[position]
        except KeyError:
            for_rule[position] = None

        for_rule[position] = result

    def hasResult(self, rule, position):
        try:
            x = self.memo[rule][position]
            return True
        except KeyError:
            return False

    def result(self, rule, position):
        return self.memo[rule][position]



RULE_start = 0
RULE_module = 1
RULE_include = 2
RULE_more_code = 3
RULE_options = 4
RULE_option = 5
RULE_chunks = 6
RULE_error_option = 7
RULE_word = 8
RULE_rules = 9
RULE_rule = 10
RULE_pattern_line = 11
RULE_pattern = 12
RULE_raw_code = 13
RULE_code = 14
RULE_item = 15
RULE_failure = 16
RULE_line = 17
RULE_predicate = 18
RULE_utf8 = 19
RULE_ascii = 20
RULE_call_rule = 21
RULE_eof = 22
RULE_void = 23
RULE_range = 24
RULE_sub_pattern = 25
RULE_bind = 26
RULE_string = 27
RULE_modifier = 28
RULE_x_word = 29
RULE_rule_parameters = 30
RULE_value_parameters = 31
RULE_parameters_rules = 32
RULE_parameters_values = 33
RULE_word_or_dollar = 34
RULE_word_or_at = 35
RULE_word_at = 36
RULE_dollar = 37
RULE_number = 38
RULE_digit = 39
RULE_hex_number = 40
RULE_hex_digit = 41
RULE_start_symbol = 42
RULE_spaces = 43
RULE_space = 44
RULE_any_char = 45
RULE_any = 46
RULE_whitespace = 47
RULE_comment = 48
RULE_newlines_one = 49
RULE_newlines = 50




def rule_start(stream, position):
    if stream.hasResult(RULE_start, position):
        return stream.result(RULE_start, position)
    try:
        result_peg_1 = Result(position)
        result_peg_2 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_2 = rule_newlines(stream, result_peg_2.getPosition())
        if result_peg_2 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_2);
        
        result_peg_3 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_3 = rule_whitespace(stream, result_peg_3.getPosition())
        if result_peg_3 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_3);
        
        result_peg_4 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'start_symbol'
        result_peg_4 = rule_start_symbol(stream, result_peg_4.getPosition())
        if result_peg_4 == None:
            raise PegError
        start_symbol = result_peg_4.getValues()
        
        result_peg_1.addResult(result_peg_4);
        
        result_peg_5 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_5 = rule_newlines(stream, result_peg_5.getPosition())
        if result_peg_5 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_5);
        
        result_peg_6 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_6 = rule_whitespace(stream, result_peg_6.getPosition())
        if result_peg_6 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_6);
        
        result_peg_7 = Result(result_peg_1.getPosition())
        
        save_peg_8 = result_peg_7.getPosition()
        
        # print "Trying rule " + 'options'
        result_peg_7 = rule_options(stream, result_peg_7.getPosition())
        if result_peg_7 == None:
            
            result_peg_7 = Result(save_peg_8)
            result_peg_7.setValue(None)
        options = result_peg_7.getValues()
        
        result_peg_1.addResult(result_peg_7);
        
        result_peg_9 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_9 = rule_newlines(stream, result_peg_9.getPosition())
        if result_peg_9 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_9);
        
        result_peg_10 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_10 = rule_whitespace(stream, result_peg_10.getPosition())
        if result_peg_10 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_10);
        
        result_peg_11 = Result(result_peg_1.getPosition())
        
        save_peg_12 = result_peg_11.getPosition()
        
        # print "Trying rule " + 'module'
        result_peg_11 = rule_module(stream, result_peg_11.getPosition())
        if result_peg_11 == None:
            
            result_peg_11 = Result(save_peg_12)
            result_peg_11.setValue(None)
        module = result_peg_11.getValues()
        
        result_peg_1.addResult(result_peg_11);
        
        result_peg_13 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_13 = rule_newlines(stream, result_peg_13.getPosition())
        if result_peg_13 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_13);
        
        result_peg_14 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_14 = rule_whitespace(stream, result_peg_14.getPosition())
        if result_peg_14 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_14);
        
        result_peg_15 = Result(result_peg_1.getPosition())
        
        save_peg_16 = result_peg_15.getPosition()
        
        # print "Trying rule " + 'include'
        result_peg_15 = rule_include(stream, result_peg_15.getPosition())
        if result_peg_15 == None:
            
            result_peg_15 = Result(save_peg_16)
            result_peg_15.setValue(None)
        include = result_peg_15.getValues()
        
        result_peg_1.addResult(result_peg_15);
        
        result_peg_17 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_17 = rule_newlines(stream, result_peg_17.getPosition())
        if result_peg_17 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_17);
        
        result_peg_18 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_18 = rule_whitespace(stream, result_peg_18.getPosition())
        if result_peg_18 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_18);
        
        result_peg_19 = Result(result_peg_1.getPosition())
        
        save_peg_20 = result_peg_19.getPosition()
        
        # print "Trying rule " + 'more_code'
        result_peg_19 = rule_more_code(stream, result_peg_19.getPosition())
        if result_peg_19 == None:
            
            result_peg_19 = Result(save_peg_20)
            result_peg_19.setValue(None)
        code = result_peg_19.getValues()
        
        result_peg_1.addResult(result_peg_19);
        
        result_peg_21 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_21 = rule_newlines(stream, result_peg_21.getPosition())
        if result_peg_21 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_21);
        
        result_peg_22 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_22 = rule_whitespace(stream, result_peg_22.getPosition())
        if result_peg_22 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_22);
        
        result_peg_23 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'rules'
        result_peg_23 = rule_rules(stream, result_peg_23.getPosition())
        if result_peg_23 == None:
            raise PegError
        rules = result_peg_23.getValues()
        
        result_peg_1.addResult(result_peg_23);
        
        result_peg_24 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_24 = rule_newlines(stream, result_peg_24.getPosition())
        if result_peg_24 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_24);
        
        result_peg_25 = Result(result_peg_1.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_25 = rule_whitespace(stream, result_peg_25.getPosition())
        if result_peg_25 == None:
            raise PegError
        
        result_peg_1.addResult(result_peg_25);
        
        result_peg_26 = Result(result_peg_1.getPosition())
        
        if chr(0) == stream.get(result_peg_26.getPosition()):
            result_peg_26.nextPosition()
            result_peg_26.setValue(chr(0))
        else:
            raise PegError
        
        result_peg_1.addResult(result_peg_26);
        
        result_peg_27 = Result(result_peg_1.getPosition())
        
        value = None
        values = result_peg_1.getValues()
        value = peg.Peg(start_symbol, include, code, module, rules, options)
        result_peg_27.setValue(value)
        
        result_peg_1.addResult(result_peg_27);
        
        result_peg_1.setValue(result_peg_1.getLastValue())
        stream.update(RULE_start, position, result_peg_1)
        return result_peg_1
    except PegError:
        pass
    stream.update(RULE_start, position, None)
    return None


def rule_module(stream, position):
    if stream.hasResult(RULE_module, position):
        return stream.result(RULE_module, position)
    try:
        result_peg_28 = Result(position)
        result_peg_29 = Result(result_peg_28.getPosition())
        
        if 'module:' == stream.get(result_peg_29.getPosition(), 7):
            result_peg_29.nextPosition(7)
            result_peg_29.setValue('module:')
        else:
            raise PegError
        
        result_peg_28.addResult(result_peg_29);
        
        result_peg_30 = Result(result_peg_28.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_30 = rule_spaces(stream, result_peg_30.getPosition())
        if result_peg_30 == None:
            raise PegError
        
        result_peg_28.addResult(result_peg_30);
        
        result_peg_31 = Result(result_peg_28.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_31 = rule_word(stream, result_peg_31.getPosition())
        if result_peg_31 == None:
            raise PegError
        name = result_peg_31.getValues()
        
        result_peg_28.addResult(result_peg_31);
        
        result_peg_32 = Result(result_peg_28.getPosition())
        
        try:
            while True:
                result_peg_33 = Result(result_peg_32.getPosition());
                result_peg_34 = Result(result_peg_33.getPosition())
                
                if '.' == stream.get(result_peg_34.getPosition(), 1):
                    result_peg_34.nextPosition(1)
                    result_peg_34.setValue('.')
                else:
                    raise PegError
                
                result_peg_33.addResult(result_peg_34);
                
                result_peg_35 = Result(result_peg_33.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_35 = rule_word(stream, result_peg_35.getPosition())
                if result_peg_35 == None:
                    raise PegError
                
                result_peg_33.addResult(result_peg_35);
                
                result_peg_36 = Result(result_peg_33.getPosition())
                
                value = None
                values = result_peg_33.getValues()
                value = values[1]
                result_peg_36.setValue(value)
                
                result_peg_33.addResult(result_peg_36);
                
                result_peg_33.setValue(result_peg_33.getLastValue())
                result_peg_32.addResult(result_peg_33);
        except PegError:
            pass
        rest = result_peg_32.getValues()
        
        result_peg_28.addResult(result_peg_32);
        
        result_peg_37 = Result(result_peg_28.getPosition())
        
        value = None
        values = result_peg_28.getValues()
        value = [name] + rest
        result_peg_37.setValue(value)
        
        result_peg_28.addResult(result_peg_37);
        
        result_peg_28.setValue(result_peg_28.getLastValue())
        stream.update(RULE_module, position, result_peg_28)
        return result_peg_28
    except PegError:
        pass
    stream.update(RULE_module, position, None)
    return None


def rule_include(stream, position):
    if stream.hasResult(RULE_include, position):
        return stream.result(RULE_include, position)
    try:
        result_peg_38 = Result(position)
        result_peg_39 = Result(result_peg_38.getPosition())
        
        if 'include:' == stream.get(result_peg_39.getPosition(), 8):
            result_peg_39.nextPosition(8)
            result_peg_39.setValue('include:')
        else:
            raise PegError
        
        result_peg_38.addResult(result_peg_39);
        
        result_peg_40 = Result(result_peg_38.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_40 = rule_spaces(stream, result_peg_40.getPosition())
        if result_peg_40 == None:
            raise PegError
        
        result_peg_38.addResult(result_peg_40);
        
        result_peg_41 = Result(result_peg_38.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_41 = rule_code(stream, result_peg_41.getPosition())
        if result_peg_41 == None:
            raise PegError
        code = result_peg_41.getValues()
        
        result_peg_38.addResult(result_peg_41);
        
        result_peg_42 = Result(result_peg_38.getPosition())
        
        value = None
        values = result_peg_38.getValues()
        value = code.code
        result_peg_42.setValue(value)
        
        result_peg_38.addResult(result_peg_42);
        
        result_peg_38.setValue(result_peg_38.getLastValue())
        stream.update(RULE_include, position, result_peg_38)
        return result_peg_38
    except PegError:
        pass
    stream.update(RULE_include, position, None)
    return None


def rule_more_code(stream, position):
    if stream.hasResult(RULE_more_code, position):
        return stream.result(RULE_more_code, position)
    try:
        result_peg_43 = Result(position)
        result_peg_44 = Result(result_peg_43.getPosition())
        
        if 'code:' == stream.get(result_peg_44.getPosition(), 5):
            result_peg_44.nextPosition(5)
            result_peg_44.setValue('code:')
        else:
            raise PegError
        
        result_peg_43.addResult(result_peg_44);
        
        result_peg_45 = Result(result_peg_43.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_45 = rule_spaces(stream, result_peg_45.getPosition())
        if result_peg_45 == None:
            raise PegError
        
        result_peg_43.addResult(result_peg_45);
        
        result_peg_46 = Result(result_peg_43.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_46 = rule_code(stream, result_peg_46.getPosition())
        if result_peg_46 == None:
            raise PegError
        code = result_peg_46.getValues()
        
        result_peg_43.addResult(result_peg_46);
        
        result_peg_47 = Result(result_peg_43.getPosition())
        
        value = None
        values = result_peg_43.getValues()
        value = code.code
        result_peg_47.setValue(value)
        
        result_peg_43.addResult(result_peg_47);
        
        result_peg_43.setValue(result_peg_43.getLastValue())
        stream.update(RULE_more_code, position, result_peg_43)
        return result_peg_43
    except PegError:
        pass
    stream.update(RULE_more_code, position, None)
    return None


def rule_options(stream, position):
    if stream.hasResult(RULE_options, position):
        return stream.result(RULE_options, position)
    try:
        result_peg_48 = Result(position)
        result_peg_49 = Result(result_peg_48.getPosition())
        
        if 'options:' == stream.get(result_peg_49.getPosition(), 8):
            result_peg_49.nextPosition(8)
            result_peg_49.setValue('options:')
        else:
            raise PegError
        
        result_peg_48.addResult(result_peg_49);
        
        result_peg_50 = Result(result_peg_48.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_50 = rule_spaces(stream, result_peg_50.getPosition())
        if result_peg_50 == None:
            raise PegError
        
        result_peg_48.addResult(result_peg_50);
        
        result_peg_51 = Result(result_peg_48.getPosition())
        
        # print "Trying rule " + 'option'
        result_peg_51 = rule_option(stream, result_peg_51.getPosition())
        if result_peg_51 == None:
            raise PegError
        option1 = result_peg_51.getValues()
        
        result_peg_48.addResult(result_peg_51);
        
        result_peg_52 = Result(result_peg_48.getPosition())
        
        try:
            while True:
                result_peg_53 = Result(result_peg_52.getPosition());
                result_peg_54 = Result(result_peg_53.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_54 = rule_spaces(stream, result_peg_54.getPosition())
                if result_peg_54 == None:
                    raise PegError
                
                result_peg_53.addResult(result_peg_54);
                
                result_peg_55 = Result(result_peg_53.getPosition())
                
                if ',' == stream.get(result_peg_55.getPosition(), 1):
                    result_peg_55.nextPosition(1)
                    result_peg_55.setValue(',')
                else:
                    raise PegError
                
                result_peg_53.addResult(result_peg_55);
                
                result_peg_56 = Result(result_peg_53.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_56 = rule_spaces(stream, result_peg_56.getPosition())
                if result_peg_56 == None:
                    raise PegError
                
                result_peg_53.addResult(result_peg_56);
                
                result_peg_57 = Result(result_peg_53.getPosition())
                
                # print "Trying rule " + 'option'
                result_peg_57 = rule_option(stream, result_peg_57.getPosition())
                if result_peg_57 == None:
                    raise PegError
                
                result_peg_53.addResult(result_peg_57);
                
                result_peg_53.setValue(result_peg_53.getLastValue())
                result_peg_52.addResult(result_peg_53);
        except PegError:
            pass
        option_rest = result_peg_52.getValues()
        
        result_peg_48.addResult(result_peg_52);
        
        result_peg_58 = Result(result_peg_48.getPosition())
        
        value = None
        values = result_peg_48.getValues()
        value = []
        for option in ([option1] + option_rest):
            import re
            debug = re.compile("debug(\d+)")
            out = debug.match(option)
            if out != None:
                num = int(out.group(1))
                for x in xrange(1,num+1):
                    value.append('debug%d' % x)
            elif option == 'no-memo':
                value.append(option)
            else:
                value.append(option)
        result_peg_58.setValue(value)
        
        result_peg_48.addResult(result_peg_58);
        
        result_peg_48.setValue(result_peg_48.getLastValue())
        stream.update(RULE_options, position, result_peg_48)
        return result_peg_48
    except PegError:
        pass
    stream.update(RULE_options, position, None)
    return None


def rule_option(stream, position):
    if stream.hasResult(RULE_option, position):
        return stream.result(RULE_option, position)
    try:
        result_peg_59 = Result(position)
        result_peg_60 = Result(result_peg_59.getPosition())
        
        if 'debug' == stream.get(result_peg_60.getPosition(), 5):
            result_peg_60.nextPosition(5)
            result_peg_60.setValue('debug')
        else:
            raise PegError
        
        result_peg_59.addResult(result_peg_60);
        
        result_peg_61 = Result(result_peg_59.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_61 = rule_number(stream, result_peg_61.getPosition())
        if result_peg_61 == None:
            raise PegError
        number = result_peg_61.getValues()
        
        result_peg_59.addResult(result_peg_61);
        
        result_peg_62 = Result(result_peg_59.getPosition())
        
        value = None
        values = result_peg_59.getValues()
        value = 'debug%s' % number
        result_peg_62.setValue(value)
        
        result_peg_59.addResult(result_peg_62);
        
        result_peg_59.setValue(result_peg_59.getLastValue())
        stream.update(RULE_option, position, result_peg_59)
        return result_peg_59
    except PegError:
        pass
    try:
        result_peg_63 = Result(position)
        if 'state' == stream.get(result_peg_63.getPosition(), 5):
            result_peg_63.nextPosition(5)
            result_peg_63.setValue('state')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_63)
        return result_peg_63
    except PegError:
        pass
    try:
        result_peg_64 = Result(position)
        if 'no-memo' == stream.get(result_peg_64.getPosition(), 7):
            result_peg_64.nextPosition(7)
            result_peg_64.setValue('no-memo')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_64)
        return result_peg_64
    except PegError:
        pass
    try:
        result_peg_65 = Result(position)
        # print "Trying rule " + 'error_option'
        result_peg_65 = rule_error_option(stream, result_peg_65.getPosition())
        if result_peg_65 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_65)
        return result_peg_65
    except PegError:
        pass
    try:
        result_peg_66 = Result(position)
        # print "Trying rule " + 'chunks'
        result_peg_66 = rule_chunks(stream, result_peg_66.getPosition())
        if result_peg_66 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_66)
        return result_peg_66
    except PegError:
        pass
    stream.update(RULE_option, position, None)
    return None


def rule_chunks(stream, position):
    if stream.hasResult(RULE_chunks, position):
        return stream.result(RULE_chunks, position)
    try:
        result_peg_67 = Result(position)
        result_peg_68 = Result(result_peg_67.getPosition())
        
        if 'chunks' == stream.get(result_peg_68.getPosition(), 6):
            result_peg_68.nextPosition(6)
            result_peg_68.setValue('chunks')
        else:
            raise PegError
        
        result_peg_67.addResult(result_peg_68);
        
        result_peg_69 = Result(result_peg_67.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_69 = rule_whitespace(stream, result_peg_69.getPosition())
        if result_peg_69 == None:
            raise PegError
        
        result_peg_67.addResult(result_peg_69);
        
        result_peg_70 = Result(result_peg_67.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_70 = rule_number(stream, result_peg_70.getPosition())
        if result_peg_70 == None:
            raise PegError
        number = result_peg_70.getValues()
        
        result_peg_67.addResult(result_peg_70);
        
        result_peg_71 = Result(result_peg_67.getPosition())
        
        value = None
        values = result_peg_67.getValues()
        value = 'chunks %s' % number
        result_peg_71.setValue(value)
        
        result_peg_67.addResult(result_peg_71);
        
        result_peg_67.setValue(result_peg_67.getLastValue())
        stream.update(RULE_chunks, position, result_peg_67)
        return result_peg_67
    except PegError:
        pass
    stream.update(RULE_chunks, position, None)
    return None


def rule_error_option(stream, position):
    if stream.hasResult(RULE_error_option, position):
        return stream.result(RULE_error_option, position)
    try:
        result_peg_72 = Result(position)
        result_peg_73 = Result(result_peg_72.getPosition())
        
        if 'error-length' == stream.get(result_peg_73.getPosition(), 12):
            result_peg_73.nextPosition(12)
            result_peg_73.setValue('error-length')
        else:
            raise PegError
        
        result_peg_72.addResult(result_peg_73);
        
        result_peg_74 = Result(result_peg_72.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_74 = rule_whitespace(stream, result_peg_74.getPosition())
        if result_peg_74 == None:
            raise PegError
        
        result_peg_72.addResult(result_peg_74);
        
        result_peg_75 = Result(result_peg_72.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_75 = rule_number(stream, result_peg_75.getPosition())
        if result_peg_75 == None:
            raise PegError
        number = result_peg_75.getValues()
        
        result_peg_72.addResult(result_peg_75);
        
        result_peg_76 = Result(result_peg_72.getPosition())
        
        value = None
        values = result_peg_72.getValues()
        value = 'error-length %s' % number
        result_peg_76.setValue(value)
        
        result_peg_72.addResult(result_peg_76);
        
        result_peg_72.setValue(result_peg_72.getLastValue())
        stream.update(RULE_error_option, position, result_peg_72)
        return result_peg_72
    except PegError:
        pass
    stream.update(RULE_error_option, position, None)
    return None


def rule_word(stream, position):
    if stream.hasResult(RULE_word, position):
        return stream.result(RULE_word, position)
    try:
        result_peg_77 = Result(position)
        result_peg_78 = Result(result_peg_77.getPosition())
        
        try:
            while True:
                result_peg_79 = Result(result_peg_78.getPosition());
                # print "Trying rule " + 'any_char'
                result_peg_79 = rule_any_char(stream, result_peg_79.getPosition())
                if result_peg_79 == None:
                    raise PegError
                result_peg_78.addResult(result_peg_79);
        except PegError:
            if result_peg_78.matches() == 0:
                raise PegError
                
        result_peg_77.addResult(result_peg_78);
        
        result_peg_80 = Result(result_peg_77.getPosition())
        
        value = None
        values = result_peg_77.getValues()
        # print "all start symbol values " + str(values)
        # print "values[0] " + str(values[0])
        value = ''.join(values[0]).replace('-', '__')
        # print "got word " + value
        result_peg_80.setValue(value)
        
        result_peg_77.addResult(result_peg_80);
        
        result_peg_77.setValue(result_peg_77.getLastValue())
        stream.update(RULE_word, position, result_peg_77)
        return result_peg_77
    except PegError:
        pass
    stream.update(RULE_word, position, None)
    return None


def rule_rules(stream, position):
    if stream.hasResult(RULE_rules, position):
        return stream.result(RULE_rules, position)
    try:
        result_peg_81 = Result(position)
        result_peg_82 = Result(result_peg_81.getPosition())
        
        if 'rules:' == stream.get(result_peg_82.getPosition(), 6):
            result_peg_82.nextPosition(6)
            result_peg_82.setValue('rules:')
        else:
            raise PegError
        
        result_peg_81.addResult(result_peg_82);
        
        result_peg_83 = Result(result_peg_81.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_83 = rule_whitespace(stream, result_peg_83.getPosition())
        if result_peg_83 == None:
            raise PegError
        
        result_peg_81.addResult(result_peg_83);
        
        result_peg_84 = Result(result_peg_81.getPosition())
        
        try:
            while True:
                result_peg_85 = Result(result_peg_84.getPosition());
                result_peg_86 = Result(result_peg_85.getPosition())
                
                # print "Trying rule " + 'rule'
                result_peg_86 = rule_rule(stream, result_peg_86.getPosition())
                if result_peg_86 == None:
                    raise PegError
                
                result_peg_85.addResult(result_peg_86);
                
                result_peg_87 = Result(result_peg_85.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_87 = rule_whitespace(stream, result_peg_87.getPosition())
                if result_peg_87 == None:
                    raise PegError
                
                result_peg_85.addResult(result_peg_87);
                
                result_peg_88 = Result(result_peg_85.getPosition())
                
                value = None
                values = result_peg_85.getValues()
                value = values[0]
                result_peg_88.setValue(value)
                
                result_peg_85.addResult(result_peg_88);
                
                result_peg_85.setValue(result_peg_85.getLastValue())
                result_peg_84.addResult(result_peg_85);
        except PegError:
            pass
        rules = result_peg_84.getValues()
        
        result_peg_81.addResult(result_peg_84);
        
        result_peg_89 = Result(result_peg_81.getPosition())
        
        value = None
        values = result_peg_81.getValues()
        value = rules
        result_peg_89.setValue(value)
        
        result_peg_81.addResult(result_peg_89);
        
        result_peg_81.setValue(result_peg_81.getLastValue())
        stream.update(RULE_rules, position, result_peg_81)
        return result_peg_81
    except PegError:
        pass
    stream.update(RULE_rules, position, None)
    return None


def rule_rule(stream, position):
    if stream.hasResult(RULE_rule, position):
        return stream.result(RULE_rule, position)
    try:
        result_peg_90 = Result(position)
        result_peg_91 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_91 = rule_spaces(stream, result_peg_91.getPosition())
        if result_peg_91 == None:
            raise PegError
        
        result_peg_90.addResult(result_peg_91);
        
        result_peg_92 = Result(result_peg_90.getPosition())
        
        save_peg_93 = result_peg_92.getPosition()
        
        if 'inline' == stream.get(result_peg_92.getPosition(), 6):
            result_peg_92.nextPosition(6)
            result_peg_92.setValue('inline')
        else:
            
            result_peg_92 = Result(save_peg_93)
            result_peg_92.setValue(None)
        inline = result_peg_92.getValues()
        
        result_peg_90.addResult(result_peg_92);
        
        result_peg_94 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_94 = rule_spaces(stream, result_peg_94.getPosition())
        if result_peg_94 == None:
            raise PegError
        
        result_peg_90.addResult(result_peg_94);
        
        result_peg_95 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_95 = rule_word(stream, result_peg_95.getPosition())
        if result_peg_95 == None:
            raise PegError
        name = result_peg_95.getValues()
        
        result_peg_90.addResult(result_peg_95);
        
        result_peg_96 = Result(result_peg_90.getPosition())
        
        save_peg_97 = result_peg_96.getPosition()
        
        # print "Trying rule " + 'rule_parameters'
        result_peg_96 = rule_rule_parameters(stream, result_peg_96.getPosition())
        if result_peg_96 == None:
            
            result_peg_96 = Result(save_peg_97)
            result_peg_96.setValue(None)
        rule_parameters = result_peg_96.getValues()
        
        result_peg_90.addResult(result_peg_96);
        
        result_peg_98 = Result(result_peg_90.getPosition())
        
        save_peg_99 = result_peg_98.getPosition()
        
        # print "Trying rule " + 'value_parameters'
        result_peg_98 = rule_value_parameters(stream, result_peg_98.getPosition())
        if result_peg_98 == None:
            
            result_peg_98 = Result(save_peg_99)
            result_peg_98.setValue(None)
        parameters = result_peg_98.getValues()
        
        result_peg_90.addResult(result_peg_98);
        
        result_peg_100 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_100 = rule_spaces(stream, result_peg_100.getPosition())
        if result_peg_100 == None:
            raise PegError
        
        result_peg_90.addResult(result_peg_100);
        
        result_peg_101 = Result(result_peg_90.getPosition())
        
        if '=' == stream.get(result_peg_101.getPosition(), 1):
            result_peg_101.nextPosition(1)
            result_peg_101.setValue('=')
        else:
            raise PegError
        
        result_peg_90.addResult(result_peg_101);
        
        result_peg_102 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_102 = rule_spaces(stream, result_peg_102.getPosition())
        if result_peg_102 == None:
            raise PegError
        
        result_peg_90.addResult(result_peg_102);
        
        result_peg_103 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'pattern_line'
        result_peg_103 = rule_pattern_line(stream, result_peg_103.getPosition())
        if result_peg_103 == None:
            raise PegError
        pattern1 = result_peg_103.getValues()
        
        result_peg_90.addResult(result_peg_103);
        
        result_peg_104 = Result(result_peg_90.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_104 = rule_whitespace(stream, result_peg_104.getPosition())
        if result_peg_104 == None:
            raise PegError
        
        result_peg_90.addResult(result_peg_104);
        
        result_peg_105 = Result(result_peg_90.getPosition())
        
        try:
            while True:
                result_peg_106 = Result(result_peg_105.getPosition());
                result_peg_107 = Result(result_peg_106.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_107 = rule_spaces(stream, result_peg_107.getPosition())
                if result_peg_107 == None:
                    raise PegError
                
                result_peg_106.addResult(result_peg_107);
                
                result_peg_108 = Result(result_peg_106.getPosition())
                
                if '|' == stream.get(result_peg_108.getPosition(), 1):
                    result_peg_108.nextPosition(1)
                    result_peg_108.setValue('|')
                else:
                    raise PegError
                
                result_peg_106.addResult(result_peg_108);
                
                result_peg_109 = Result(result_peg_106.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_109 = rule_spaces(stream, result_peg_109.getPosition())
                if result_peg_109 == None:
                    raise PegError
                
                result_peg_106.addResult(result_peg_109);
                
                result_peg_110 = Result(result_peg_106.getPosition())
                
                # print "Trying rule " + 'pattern_line'
                result_peg_110 = rule_pattern_line(stream, result_peg_110.getPosition())
                if result_peg_110 == None:
                    raise PegError
                pattern = result_peg_110.getValues()
                
                result_peg_106.addResult(result_peg_110);
                
                result_peg_111 = Result(result_peg_106.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_111 = rule_whitespace(stream, result_peg_111.getPosition())
                if result_peg_111 == None:
                    raise PegError
                
                result_peg_106.addResult(result_peg_111);
                
                result_peg_112 = Result(result_peg_106.getPosition())
                
                value = None
                values = result_peg_106.getValues()
                value = pattern
                result_peg_112.setValue(value)
                
                result_peg_106.addResult(result_peg_112);
                
                result_peg_106.setValue(result_peg_106.getLastValue())
                result_peg_105.addResult(result_peg_106);
        except PegError:
            pass
        patterns = result_peg_105.getValues()
        
        result_peg_90.addResult(result_peg_105);
        
        result_peg_113 = Result(result_peg_90.getPosition())
        
        save_peg_114 = result_peg_113.getPosition()
        
        # print "Trying rule " + 'failure'
        result_peg_113 = rule_failure(stream, result_peg_113.getPosition())
        if result_peg_113 == None:
            
            result_peg_113 = Result(save_peg_114)
            result_peg_113.setValue(None)
        fail = result_peg_113.getValues()
        
        result_peg_90.addResult(result_peg_113);
        
        result_peg_115 = Result(result_peg_90.getPosition())
        
        value = None
        values = result_peg_90.getValues()
        value = peg.Rule(name, [pattern1] + patterns, inline = (inline != None), rules = rule_parameters, parameters = parameters, fail = fail)
        result_peg_115.setValue(value)
        
        result_peg_90.addResult(result_peg_115);
        
        result_peg_90.setValue(result_peg_90.getLastValue())
        stream.update(RULE_rule, position, result_peg_90)
        return result_peg_90
    except PegError:
        pass
    stream.update(RULE_rule, position, None)
    return None


def rule_pattern_line(stream, position):
    if stream.hasResult(RULE_pattern_line, position):
        return stream.result(RULE_pattern_line, position)
    try:
        result_peg_116 = Result(position)
        result_peg_117 = Result(result_peg_116.getPosition())
        
        try:
            while True:
                result_peg_118 = Result(result_peg_117.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_118 = rule_pattern(stream, result_peg_118.getPosition())
                if result_peg_118 == None:
                    raise PegError
                result_peg_117.addResult(result_peg_118);
        except PegError:
            pass
        patterns = result_peg_117.getValues()
        
        result_peg_116.addResult(result_peg_117);
        
        result_peg_119 = Result(result_peg_116.getPosition())
        
        value = None
        values = result_peg_116.getValues()
        value = peg.PatternSequence(patterns)
        #if code != None:
        #    value = code(peg.PatternSequence(patterns))
        #else:
        #    value = peg.PatternAction(peg.PatternSequence(patterns), "value = values;")
        result_peg_119.setValue(value)
        
        result_peg_116.addResult(result_peg_119);
        
        result_peg_116.setValue(result_peg_116.getLastValue())
        stream.update(RULE_pattern_line, position, result_peg_116)
        return result_peg_116
    except PegError:
        pass
    stream.update(RULE_pattern_line, position, None)
    return None


def rule_pattern(stream, position):
    if stream.hasResult(RULE_pattern, position):
        return stream.result(RULE_pattern, position)
    try:
        result_peg_120 = Result(position)
        result_peg_121 = Result(result_peg_120.getPosition())
        
        save_peg_122 = result_peg_121.getPosition()
        
        # print "Trying rule " + 'bind'
        result_peg_121 = rule_bind(stream, result_peg_121.getPosition())
        if result_peg_121 == None:
            
            result_peg_121 = Result(save_peg_122)
            result_peg_121.setValue(None)
        bind = result_peg_121.getValues()
        
        result_peg_120.addResult(result_peg_121);
        
        result_peg_123 = Result(result_peg_120.getPosition())
        
        # print "Trying rule " + 'item'
        result_peg_123 = rule_item(stream, result_peg_123.getPosition())
        if result_peg_123 == None:
            raise PegError
        item = result_peg_123.getValues()
        
        result_peg_120.addResult(result_peg_123);
        
        result_peg_124 = Result(result_peg_120.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_124 = rule_spaces(stream, result_peg_124.getPosition())
        if result_peg_124 == None:
            raise PegError
        
        result_peg_120.addResult(result_peg_124);
        
        result_peg_125 = Result(result_peg_120.getPosition())
        
        value = None
        values = result_peg_120.getValues()
        # value = peg.PatternRule(values[0])
        if bind != None:
            item = bind(item)
        value = item
        # print "Pattern is " + str(value)
        result_peg_125.setValue(value)
        
        result_peg_120.addResult(result_peg_125);
        
        result_peg_120.setValue(result_peg_120.getLastValue())
        stream.update(RULE_pattern, position, result_peg_120)
        return result_peg_120
    except PegError:
        pass
    stream.update(RULE_pattern, position, None)
    return None


def rule_raw_code(stream, position):
    if stream.hasResult(RULE_raw_code, position):
        return stream.result(RULE_raw_code, position)
    try:
        result_peg_126 = Result(position)
        result_peg_127 = Result(result_peg_126.getPosition())
        
        if '(' == stream.get(result_peg_127.getPosition(), 1):
            result_peg_127.nextPosition(1)
            result_peg_127.setValue('(')
        else:
            raise PegError
        
        result_peg_126.addResult(result_peg_127);
        
        result_peg_128 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_128 = rule_spaces(stream, result_peg_128.getPosition())
        if result_peg_128 == None:
            raise PegError
        
        result_peg_126.addResult(result_peg_128);
        
        result_peg_129 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_129 = rule_code(stream, result_peg_129.getPosition())
        if result_peg_129 == None:
            raise PegError
        code = result_peg_129.getValues()
        
        result_peg_126.addResult(result_peg_129);
        
        result_peg_130 = Result(result_peg_126.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_130 = rule_spaces(stream, result_peg_130.getPosition())
        if result_peg_130 == None:
            raise PegError
        
        result_peg_126.addResult(result_peg_130);
        
        result_peg_131 = Result(result_peg_126.getPosition())
        
        if ')' == stream.get(result_peg_131.getPosition(), 1):
            result_peg_131.nextPosition(1)
            result_peg_131.setValue(')')
        else:
            raise PegError
        
        result_peg_126.addResult(result_peg_131);
        
        result_peg_132 = Result(result_peg_126.getPosition())
        
        value = None
        values = result_peg_126.getValues()
        value = code.code
        result_peg_132.setValue(value)
        
        result_peg_126.addResult(result_peg_132);
        
        result_peg_126.setValue(result_peg_126.getLastValue())
        stream.update(RULE_raw_code, position, result_peg_126)
        return result_peg_126
    except PegError:
        pass
    stream.update(RULE_raw_code, position, None)
    return None


def rule_code(stream, position):
    if stream.hasResult(RULE_code, position):
        return stream.result(RULE_code, position)
    try:
        result_peg_133 = Result(position)
        result_peg_134 = Result(result_peg_133.getPosition())
        
        if '{{' == stream.get(result_peg_134.getPosition(), 2):
            result_peg_134.nextPosition(2)
            result_peg_134.setValue('{{')
        else:
            raise PegError
        
        result_peg_133.addResult(result_peg_134);
        
        result_peg_135 = Result(result_peg_133.getPosition())
        
        try:
            while True:
                result_peg_136 = Result(result_peg_135.getPosition());
                result_peg_137 = Result(result_peg_136.getPosition())
                
                result_peg_138 = Result(result_peg_137.getPosition());
                try:
                    if '}}' == stream.get(result_peg_138.getPosition(), 2):
                        result_peg_138.nextPosition(2)
                        result_peg_138.setValue('}}')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_137.setValue(None)
                        
                result_peg_136.addResult(result_peg_137);
                
                result_peg_139 = Result(result_peg_136.getPosition())
                
                temp_peg_140 = stream.get(result_peg_139.getPosition())
                if temp_peg_140 != chr(0):
                    result_peg_139.setValue(temp_peg_140)
                    result_peg_139.nextPosition()
                else:
                    raise PegError
                
                result_peg_136.addResult(result_peg_139);
                
                result_peg_141 = Result(result_peg_136.getPosition())
                
                value = None
                values = result_peg_136.getValues()
                value = values[1]
                result_peg_141.setValue(value)
                
                result_peg_136.addResult(result_peg_141);
                
                result_peg_136.setValue(result_peg_136.getLastValue())
                result_peg_135.addResult(result_peg_136);
        except PegError:
            if result_peg_135.matches() == 0:
                raise PegError
                
        result_peg_133.addResult(result_peg_135);
        
        result_peg_142 = Result(result_peg_133.getPosition())
        
        if '}}' == stream.get(result_peg_142.getPosition(), 2):
            result_peg_142.nextPosition(2)
            result_peg_142.setValue('}}')
        else:
            raise PegError
        
        result_peg_133.addResult(result_peg_142);
        
        result_peg_143 = Result(result_peg_133.getPosition())
        
        value = None
        values = result_peg_133.getValues()
        value = peg.PatternCode(''.join(values[1]))
        result_peg_143.setValue(value)
        
        result_peg_133.addResult(result_peg_143);
        
        result_peg_133.setValue(result_peg_133.getLastValue())
        stream.update(RULE_code, position, result_peg_133)
        return result_peg_133
    except PegError:
        pass
    stream.update(RULE_code, position, None)
    return None


def rule_item(stream, position):
    if stream.hasResult(RULE_item, position):
        return stream.result(RULE_item, position)
    try:
        result_peg_144 = Result(position)
        result_peg_145 = Result(result_peg_144.getPosition())
        
        save_peg_146 = result_peg_145.getPosition()
        
        if '&' == stream.get(result_peg_145.getPosition(), 1):
            result_peg_145.nextPosition(1)
            result_peg_145.setValue('&')
        else:
            
            result_peg_145 = Result(save_peg_146)
            result_peg_145.setValue(None)
        ensure = result_peg_145.getValues()
        
        result_peg_144.addResult(result_peg_145);
        
        result_peg_147 = Result(result_peg_144.getPosition())
        
        save_peg_148 = result_peg_147.getPosition()
        
        if '!' == stream.get(result_peg_147.getPosition(), 1):
            result_peg_147.nextPosition(1)
            result_peg_147.setValue('!')
        else:
            
            result_peg_147 = Result(save_peg_148)
            result_peg_147.setValue(None)
        pnot = result_peg_147.getValues()
        
        result_peg_144.addResult(result_peg_147);
        
        result_peg_149 = Result(result_peg_144.getPosition())
        
        save_peg_150 = result_peg_149.getPosition()
        
        result_peg_149 = Result(save_peg_150)
        # print "Trying rule " + 'x_word'
        result_peg_149 = rule_x_word(stream, result_peg_149.getPosition())
        if result_peg_149 == None:
            
            result_peg_149 = Result(save_peg_150)
            # print "Trying rule " + 'any'
            result_peg_149 = rule_any(stream, result_peg_149.getPosition())
            if result_peg_149 == None:
                
                result_peg_149 = Result(save_peg_150)
                # print "Trying rule " + 'eof'
                result_peg_149 = rule_eof(stream, result_peg_149.getPosition())
                if result_peg_149 == None:
                    
                    result_peg_149 = Result(save_peg_150)
                    # print "Trying rule " + 'void'
                    result_peg_149 = rule_void(stream, result_peg_149.getPosition())
                    if result_peg_149 == None:
                        
                        result_peg_149 = Result(save_peg_150)
                        # print "Trying rule " + 'range'
                        result_peg_149 = rule_range(stream, result_peg_149.getPosition())
                        if result_peg_149 == None:
                            
                            result_peg_149 = Result(save_peg_150)
                            # print "Trying rule " + 'string'
                            result_peg_149 = rule_string(stream, result_peg_149.getPosition())
                            if result_peg_149 == None:
                                
                                result_peg_149 = Result(save_peg_150)
                                # print "Trying rule " + 'line'
                                result_peg_149 = rule_line(stream, result_peg_149.getPosition())
                                if result_peg_149 == None:
                                    
                                    result_peg_149 = Result(save_peg_150)
                                    # print "Trying rule " + 'ascii'
                                    result_peg_149 = rule_ascii(stream, result_peg_149.getPosition())
                                    if result_peg_149 == None:
                                        
                                        result_peg_149 = Result(save_peg_150)
                                        # print "Trying rule " + 'utf8'
                                        result_peg_149 = rule_utf8(stream, result_peg_149.getPosition())
                                        if result_peg_149 == None:
                                            
                                            result_peg_149 = Result(save_peg_150)
                                            # print "Trying rule " + 'predicate'
                                            result_peg_149 = rule_predicate(stream, result_peg_149.getPosition())
                                            if result_peg_149 == None:
                                                
                                                result_peg_149 = Result(save_peg_150)
                                                # print "Trying rule " + 'call_rule'
                                                result_peg_149 = rule_call_rule(stream, result_peg_149.getPosition())
                                                if result_peg_149 == None:
                                                    
                                                    result_peg_149 = Result(save_peg_150)
                                                    # print "Trying rule " + 'sub_pattern'
                                                    result_peg_149 = rule_sub_pattern(stream, result_peg_149.getPosition())
                                                    if result_peg_149 == None:
                                                        
                                                        result_peg_149 = Result(save_peg_150)
                                                        # print "Trying rule " + 'code'
                                                        result_peg_149 = rule_code(stream, result_peg_149.getPosition())
                                                        if result_peg_149 == None:
                                                            raise PegError
        pattern = result_peg_149.getValues()
        
        result_peg_144.addResult(result_peg_149);
        
        result_peg_151 = Result(result_peg_144.getPosition())
        
        save_peg_152 = result_peg_151.getPosition()
        
        # print "Trying rule " + 'modifier'
        result_peg_151 = rule_modifier(stream, result_peg_151.getPosition())
        if result_peg_151 == None:
            
            result_peg_151 = Result(save_peg_152)
            result_peg_151.setValue(None)
        modifier = result_peg_151.getValues()
        
        result_peg_144.addResult(result_peg_151);
        
        result_peg_153 = Result(result_peg_144.getPosition())
        
        value = None
        values = result_peg_144.getValues()
        if modifier != None:
            pattern = modifier(pattern)
        if pnot != None:
            pattern = peg.PatternNot(pattern)
        if ensure != None:
            pattern = peg.PatternEnsure(pattern)
        value = pattern
        result_peg_153.setValue(value)
        
        result_peg_144.addResult(result_peg_153);
        
        result_peg_144.setValue(result_peg_144.getLastValue())
        stream.update(RULE_item, position, result_peg_144)
        return result_peg_144
    except PegError:
        pass
    stream.update(RULE_item, position, None)
    return None


def rule_failure(stream, position):
    if stream.hasResult(RULE_failure, position):
        return stream.result(RULE_failure, position)
    try:
        result_peg_154 = Result(position)
        result_peg_155 = Result(result_peg_154.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_155 = rule_whitespace(stream, result_peg_155.getPosition())
        if result_peg_155 == None:
            raise PegError
        
        result_peg_154.addResult(result_peg_155);
        
        result_peg_156 = Result(result_peg_154.getPosition())
        
        if '<fail>' == stream.get(result_peg_156.getPosition(), 6):
            result_peg_156.nextPosition(6)
            result_peg_156.setValue('<fail>')
        else:
            raise PegError
        
        result_peg_154.addResult(result_peg_156);
        
        result_peg_157 = Result(result_peg_154.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_157 = rule_spaces(stream, result_peg_157.getPosition())
        if result_peg_157 == None:
            raise PegError
        
        result_peg_154.addResult(result_peg_157);
        
        result_peg_158 = Result(result_peg_154.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_158 = rule_code(stream, result_peg_158.getPosition())
        if result_peg_158 == None:
            raise PegError
        code = result_peg_158.getValues()
        
        result_peg_154.addResult(result_peg_158);
        
        result_peg_159 = Result(result_peg_154.getPosition())
        
        value = None
        values = result_peg_154.getValues()
        value = code.code
        result_peg_159.setValue(value)
        
        result_peg_154.addResult(result_peg_159);
        
        result_peg_154.setValue(result_peg_154.getLastValue())
        stream.update(RULE_failure, position, result_peg_154)
        return result_peg_154
    except PegError:
        pass
    stream.update(RULE_failure, position, None)
    return None


def rule_line(stream, position):
    if stream.hasResult(RULE_line, position):
        return stream.result(RULE_line, position)
    try:
        result_peg_160 = Result(position)
        result_peg_161 = Result(result_peg_160.getPosition())
        
        if '<line>' == stream.get(result_peg_161.getPosition(), 6):
            result_peg_161.nextPosition(6)
            result_peg_161.setValue('<line>')
        else:
            raise PegError
        
        result_peg_160.addResult(result_peg_161);
        
        result_peg_162 = Result(result_peg_160.getPosition())
        
        value = None
        values = result_peg_160.getValues()
        value = peg.PatternLine()
        result_peg_162.setValue(value)
        
        result_peg_160.addResult(result_peg_162);
        
        result_peg_160.setValue(result_peg_160.getLastValue())
        stream.update(RULE_line, position, result_peg_160)
        return result_peg_160
    except PegError:
        pass
    stream.update(RULE_line, position, None)
    return None


def rule_predicate(stream, position):
    if stream.hasResult(RULE_predicate, position):
        return stream.result(RULE_predicate, position)
    try:
        result_peg_163 = Result(position)
        result_peg_164 = Result(result_peg_163.getPosition())
        
        if '<predicate' == stream.get(result_peg_164.getPosition(), 10):
            result_peg_164.nextPosition(10)
            result_peg_164.setValue('<predicate')
        else:
            raise PegError
        
        result_peg_163.addResult(result_peg_164);
        
        result_peg_165 = Result(result_peg_163.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_165 = rule_whitespace(stream, result_peg_165.getPosition())
        if result_peg_165 == None:
            raise PegError
        
        result_peg_163.addResult(result_peg_165);
        
        result_peg_166 = Result(result_peg_163.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_166 = rule_word(stream, result_peg_166.getPosition())
        if result_peg_166 == None:
            raise PegError
        variable = result_peg_166.getValues()
        
        result_peg_163.addResult(result_peg_166);
        
        result_peg_167 = Result(result_peg_163.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_167 = rule_whitespace(stream, result_peg_167.getPosition())
        if result_peg_167 == None:
            raise PegError
        
        result_peg_163.addResult(result_peg_167);
        
        result_peg_168 = Result(result_peg_163.getPosition())
        
        if '>' == stream.get(result_peg_168.getPosition(), 1):
            result_peg_168.nextPosition(1)
            result_peg_168.setValue('>')
        else:
            raise PegError
        
        result_peg_163.addResult(result_peg_168);
        
        result_peg_169 = Result(result_peg_163.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_169 = rule_whitespace(stream, result_peg_169.getPosition())
        if result_peg_169 == None:
            raise PegError
        
        result_peg_163.addResult(result_peg_169);
        
        result_peg_170 = Result(result_peg_163.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_170 = rule_code(stream, result_peg_170.getPosition())
        if result_peg_170 == None:
            raise PegError
        code = result_peg_170.getValues()
        
        result_peg_163.addResult(result_peg_170);
        
        result_peg_171 = Result(result_peg_163.getPosition())
        
        value = None
        values = result_peg_163.getValues()
        value = peg.PatternPredicate(variable, code.code)
        result_peg_171.setValue(value)
        
        result_peg_163.addResult(result_peg_171);
        
        result_peg_163.setValue(result_peg_163.getLastValue())
        stream.update(RULE_predicate, position, result_peg_163)
        return result_peg_163
    except PegError:
        pass
    stream.update(RULE_predicate, position, None)
    return None


def rule_utf8(stream, position):
    if stream.hasResult(RULE_utf8, position):
        return stream.result(RULE_utf8, position)
    try:
        result_peg_172 = Result(position)
        result_peg_173 = Result(result_peg_172.getPosition())
        
        if '<utf8' == stream.get(result_peg_173.getPosition(), 5):
            result_peg_173.nextPosition(5)
            result_peg_173.setValue('<utf8')
        else:
            raise PegError
        
        result_peg_172.addResult(result_peg_173);
        
        result_peg_174 = Result(result_peg_172.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_174 = rule_spaces(stream, result_peg_174.getPosition())
        if result_peg_174 == None:
            raise PegError
        
        result_peg_172.addResult(result_peg_174);
        
        result_peg_175 = Result(result_peg_172.getPosition())
        
        # print "Trying rule " + 'hex_number'
        result_peg_175 = rule_hex_number(stream, result_peg_175.getPosition())
        if result_peg_175 == None:
            raise PegError
        num = result_peg_175.getValues()
        
        result_peg_172.addResult(result_peg_175);
        
        result_peg_176 = Result(result_peg_172.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_176 = rule_spaces(stream, result_peg_176.getPosition())
        if result_peg_176 == None:
            raise PegError
        
        result_peg_172.addResult(result_peg_176);
        
        result_peg_177 = Result(result_peg_172.getPosition())
        
        if '>' == stream.get(result_peg_177.getPosition(), 1):
            result_peg_177.nextPosition(1)
            result_peg_177.setValue('>')
        else:
            raise PegError
        
        result_peg_172.addResult(result_peg_177);
        
        result_peg_178 = Result(result_peg_172.getPosition())
        
        value = None
        values = result_peg_172.getValues()
        value = peg.createUtf8Pattern(num)
        result_peg_178.setValue(value)
        
        result_peg_172.addResult(result_peg_178);
        
        result_peg_172.setValue(result_peg_172.getLastValue())
        stream.update(RULE_utf8, position, result_peg_172)
        return result_peg_172
    except PegError:
        pass
    stream.update(RULE_utf8, position, None)
    return None


def rule_ascii(stream, position):
    if stream.hasResult(RULE_ascii, position):
        return stream.result(RULE_ascii, position)
    try:
        result_peg_179 = Result(position)
        result_peg_180 = Result(result_peg_179.getPosition())
        
        if '<ascii' == stream.get(result_peg_180.getPosition(), 6):
            result_peg_180.nextPosition(6)
            result_peg_180.setValue('<ascii')
        else:
            raise PegError
        
        result_peg_179.addResult(result_peg_180);
        
        result_peg_181 = Result(result_peg_179.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_181 = rule_spaces(stream, result_peg_181.getPosition())
        if result_peg_181 == None:
            raise PegError
        
        result_peg_179.addResult(result_peg_181);
        
        result_peg_182 = Result(result_peg_179.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_182 = rule_number(stream, result_peg_182.getPosition())
        if result_peg_182 == None:
            raise PegError
        num = result_peg_182.getValues()
        
        result_peg_179.addResult(result_peg_182);
        
        result_peg_183 = Result(result_peg_179.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_183 = rule_spaces(stream, result_peg_183.getPosition())
        if result_peg_183 == None:
            raise PegError
        
        result_peg_179.addResult(result_peg_183);
        
        result_peg_184 = Result(result_peg_179.getPosition())
        
        if '>' == stream.get(result_peg_184.getPosition(), 1):
            result_peg_184.nextPosition(1)
            result_peg_184.setValue('>')
        else:
            raise PegError
        
        result_peg_179.addResult(result_peg_184);
        
        result_peg_185 = Result(result_peg_179.getPosition())
        
        value = None
        values = result_peg_179.getValues()
        value = peg.PatternVerbatim(int(num))
        result_peg_185.setValue(value)
        
        result_peg_179.addResult(result_peg_185);
        
        result_peg_179.setValue(result_peg_179.getLastValue())
        stream.update(RULE_ascii, position, result_peg_179)
        return result_peg_179
    except PegError:
        pass
    stream.update(RULE_ascii, position, None)
    return None


def rule_call_rule(stream, position):
    if stream.hasResult(RULE_call_rule, position):
        return stream.result(RULE_call_rule, position)
    try:
        result_peg_186 = Result(position)
        result_peg_187 = Result(result_peg_186.getPosition())
        
        if '@' == stream.get(result_peg_187.getPosition(), 1):
            result_peg_187.nextPosition(1)
            result_peg_187.setValue('@')
        else:
            raise PegError
        
        result_peg_186.addResult(result_peg_187);
        
        result_peg_188 = Result(result_peg_186.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_188 = rule_word(stream, result_peg_188.getPosition())
        if result_peg_188 == None:
            raise PegError
        name = result_peg_188.getValues()
        
        result_peg_186.addResult(result_peg_188);
        
        result_peg_189 = Result(result_peg_186.getPosition())
        
        save_peg_190 = result_peg_189.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_189 = rule_parameters_rules(stream, result_peg_189.getPosition())
        if result_peg_189 == None:
            
            result_peg_189 = Result(save_peg_190)
            result_peg_189.setValue(None)
        rule_parameters = result_peg_189.getValues()
        
        result_peg_186.addResult(result_peg_189);
        
        result_peg_191 = Result(result_peg_186.getPosition())
        
        save_peg_192 = result_peg_191.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_191 = rule_parameters_values(stream, result_peg_191.getPosition())
        if result_peg_191 == None:
            
            result_peg_191 = Result(save_peg_192)
            result_peg_191.setValue(None)
        parameters = result_peg_191.getValues()
        
        result_peg_186.addResult(result_peg_191);
        
        result_peg_193 = Result(result_peg_186.getPosition())
        
        value = None
        values = result_peg_186.getValues()
        value = peg.PatternCallRule(name, rule_parameters, parameters)
        result_peg_193.setValue(value)
        
        result_peg_186.addResult(result_peg_193);
        
        result_peg_186.setValue(result_peg_186.getLastValue())
        stream.update(RULE_call_rule, position, result_peg_186)
        return result_peg_186
    except PegError:
        pass
    stream.update(RULE_call_rule, position, None)
    return None


def rule_eof(stream, position):
    if stream.hasResult(RULE_eof, position):
        return stream.result(RULE_eof, position)
    try:
        result_peg_194 = Result(position)
        result_peg_195 = Result(result_peg_194.getPosition())
        
        if '<eof>' == stream.get(result_peg_195.getPosition(), 5):
            result_peg_195.nextPosition(5)
            result_peg_195.setValue('<eof>')
        else:
            raise PegError
        
        result_peg_194.addResult(result_peg_195);
        
        result_peg_196 = Result(result_peg_194.getPosition())
        
        value = None
        values = result_peg_194.getValues()
        value = peg.PatternEof()
        result_peg_196.setValue(value)
        
        result_peg_194.addResult(result_peg_196);
        
        result_peg_194.setValue(result_peg_194.getLastValue())
        stream.update(RULE_eof, position, result_peg_194)
        return result_peg_194
    except PegError:
        pass
    stream.update(RULE_eof, position, None)
    return None


def rule_void(stream, position):
    if stream.hasResult(RULE_void, position):
        return stream.result(RULE_void, position)
    try:
        result_peg_197 = Result(position)
        result_peg_198 = Result(result_peg_197.getPosition())
        
        if '<void>' == stream.get(result_peg_198.getPosition(), 6):
            result_peg_198.nextPosition(6)
            result_peg_198.setValue('<void>')
        else:
            raise PegError
        
        result_peg_197.addResult(result_peg_198);
        
        result_peg_199 = Result(result_peg_197.getPosition())
        
        value = None
        values = result_peg_197.getValues()
        value = peg.PatternVoid()
        result_peg_199.setValue(value)
        
        result_peg_197.addResult(result_peg_199);
        
        result_peg_197.setValue(result_peg_197.getLastValue())
        stream.update(RULE_void, position, result_peg_197)
        return result_peg_197
    except PegError:
        pass
    stream.update(RULE_void, position, None)
    return None


def rule_range(stream, position):
    if stream.hasResult(RULE_range, position):
        return stream.result(RULE_range, position)
    try:
        result_peg_200 = Result(position)
        result_peg_201 = Result(result_peg_200.getPosition())
        
        if '[' == stream.get(result_peg_201.getPosition(), 1):
            result_peg_201.nextPosition(1)
            result_peg_201.setValue('[')
        else:
            raise PegError
        
        result_peg_200.addResult(result_peg_201);
        
        result_peg_202 = Result(result_peg_200.getPosition())
        
        try:
            while True:
                result_peg_203 = Result(result_peg_202.getPosition());
                result_peg_204 = Result(result_peg_203.getPosition())
                
                result_peg_205 = Result(result_peg_204.getPosition());
                try:
                    if ']' == stream.get(result_peg_205.getPosition(), 1):
                        result_peg_205.nextPosition(1)
                        result_peg_205.setValue(']')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_204.setValue(None)
                        
                result_peg_203.addResult(result_peg_204);
                
                result_peg_206 = Result(result_peg_203.getPosition())
                
                temp_peg_207 = stream.get(result_peg_206.getPosition())
                if temp_peg_207 != chr(0):
                    result_peg_206.setValue(temp_peg_207)
                    result_peg_206.nextPosition()
                else:
                    raise PegError
                
                result_peg_203.addResult(result_peg_206);
                
                result_peg_208 = Result(result_peg_203.getPosition())
                
                value = None
                values = result_peg_203.getValues()
                value = values[1]
                result_peg_208.setValue(value)
                
                result_peg_203.addResult(result_peg_208);
                
                result_peg_203.setValue(result_peg_203.getLastValue())
                result_peg_202.addResult(result_peg_203);
        except PegError:
            pass
                
        result_peg_200.addResult(result_peg_202);
        
        result_peg_209 = Result(result_peg_200.getPosition())
        
        if ']' == stream.get(result_peg_209.getPosition(), 1):
            result_peg_209.nextPosition(1)
            result_peg_209.setValue(']')
        else:
            raise PegError
        
        result_peg_200.addResult(result_peg_209);
        
        result_peg_210 = Result(result_peg_200.getPosition())
        
        value = None
        values = result_peg_200.getValues()
        value = peg.PatternRange(''.join(values[1]))
        result_peg_210.setValue(value)
        
        result_peg_200.addResult(result_peg_210);
        
        result_peg_200.setValue(result_peg_200.getLastValue())
        stream.update(RULE_range, position, result_peg_200)
        return result_peg_200
    except PegError:
        pass
    stream.update(RULE_range, position, None)
    return None


def rule_sub_pattern(stream, position):
    if stream.hasResult(RULE_sub_pattern, position):
        return stream.result(RULE_sub_pattern, position)
    try:
        result_peg_211 = Result(position)
        result_peg_212 = Result(result_peg_211.getPosition())
        
        if '(' == stream.get(result_peg_212.getPosition(), 1):
            result_peg_212.nextPosition(1)
            result_peg_212.setValue('(')
        else:
            raise PegError
        
        result_peg_211.addResult(result_peg_212);
        
        result_peg_213 = Result(result_peg_211.getPosition())
        
        try:
            while True:
                result_peg_214 = Result(result_peg_213.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_214 = rule_pattern(stream, result_peg_214.getPosition())
                if result_peg_214 == None:
                    raise PegError
                result_peg_213.addResult(result_peg_214);
        except PegError:
            if result_peg_213.matches() == 0:
                raise PegError
                
        result_peg_211.addResult(result_peg_213);
        
        result_peg_215 = Result(result_peg_211.getPosition())
        
        if ')' == stream.get(result_peg_215.getPosition(), 1):
            result_peg_215.nextPosition(1)
            result_peg_215.setValue(')')
        else:
            raise PegError
        
        result_peg_211.addResult(result_peg_215);
        
        result_peg_216 = Result(result_peg_211.getPosition())
        
        value = None
        values = result_peg_211.getValues()
        value = peg.PatternSequence(values[1])
        result_peg_216.setValue(value)
        
        result_peg_211.addResult(result_peg_216);
        
        result_peg_211.setValue(result_peg_211.getLastValue())
        stream.update(RULE_sub_pattern, position, result_peg_211)
        return result_peg_211
    except PegError:
        pass
    stream.update(RULE_sub_pattern, position, None)
    return None


def rule_bind(stream, position):
    if stream.hasResult(RULE_bind, position):
        return stream.result(RULE_bind, position)
    try:
        result_peg_217 = Result(position)
        result_peg_218 = Result(result_peg_217.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_218 = rule_word(stream, result_peg_218.getPosition())
        if result_peg_218 == None:
            raise PegError
        name = result_peg_218.getValues()
        
        result_peg_217.addResult(result_peg_218);
        
        result_peg_219 = Result(result_peg_217.getPosition())
        
        if ':' == stream.get(result_peg_219.getPosition(), 1):
            result_peg_219.nextPosition(1)
            result_peg_219.setValue(':')
        else:
            raise PegError
        
        result_peg_217.addResult(result_peg_219);
        
        result_peg_220 = Result(result_peg_217.getPosition())
        
        value = None
        values = result_peg_217.getValues()
        value = lambda p: peg.PatternBind(name, p)
        result_peg_220.setValue(value)
        
        result_peg_217.addResult(result_peg_220);
        
        result_peg_217.setValue(result_peg_217.getLastValue())
        stream.update(RULE_bind, position, result_peg_217)
        return result_peg_217
    except PegError:
        pass
    stream.update(RULE_bind, position, None)
    return None


def rule_string(stream, position):
    if stream.hasResult(RULE_string, position):
        return stream.result(RULE_string, position)
    try:
        result_peg_221 = Result(position)
        result_peg_222 = Result(result_peg_221.getPosition())
        
        if '"' == stream.get(result_peg_222.getPosition(), 1):
            result_peg_222.nextPosition(1)
            result_peg_222.setValue('"')
        else:
            raise PegError
        
        result_peg_221.addResult(result_peg_222);
        
        result_peg_223 = Result(result_peg_221.getPosition())
        
        try:
            while True:
                result_peg_224 = Result(result_peg_223.getPosition());
                result_peg_225 = Result(result_peg_224.getPosition())
                
                result_peg_226 = Result(result_peg_225.getPosition());
                try:
                    if '"' == stream.get(result_peg_226.getPosition(), 1):
                        result_peg_226.nextPosition(1)
                        result_peg_226.setValue('"')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_225.setValue(None)
                        
                result_peg_224.addResult(result_peg_225);
                
                result_peg_227 = Result(result_peg_224.getPosition())
                
                temp_peg_228 = stream.get(result_peg_227.getPosition())
                if temp_peg_228 != chr(0):
                    result_peg_227.setValue(temp_peg_228)
                    result_peg_227.nextPosition()
                else:
                    raise PegError
                
                result_peg_224.addResult(result_peg_227);
                
                result_peg_229 = Result(result_peg_224.getPosition())
                
                value = None
                values = result_peg_224.getValues()
                value = values[1]
                result_peg_229.setValue(value)
                
                result_peg_224.addResult(result_peg_229);
                
                result_peg_224.setValue(result_peg_224.getLastValue())
                result_peg_223.addResult(result_peg_224);
        except PegError:
            pass
                
        result_peg_221.addResult(result_peg_223);
        
        result_peg_230 = Result(result_peg_221.getPosition())
        
        if '"' == stream.get(result_peg_230.getPosition(), 1):
            result_peg_230.nextPosition(1)
            result_peg_230.setValue('"')
        else:
            raise PegError
        
        result_peg_221.addResult(result_peg_230);
        
        result_peg_231 = Result(result_peg_221.getPosition())
        
        save_peg_232 = result_peg_231.getPosition()
        
        if '{case}' == stream.get(result_peg_231.getPosition(), 6):
            result_peg_231.nextPosition(6)
            result_peg_231.setValue('{case}')
        else:
            
            result_peg_231 = Result(save_peg_232)
            result_peg_231.setValue(None)
        options = result_peg_231.getValues()
        
        result_peg_221.addResult(result_peg_231);
        
        result_peg_233 = Result(result_peg_221.getPosition())
        
        value = None
        values = result_peg_221.getValues()
        value = peg.PatternVerbatim(''.join(values[1]), options)
        result_peg_233.setValue(value)
        
        result_peg_221.addResult(result_peg_233);
        
        result_peg_221.setValue(result_peg_221.getLastValue())
        stream.update(RULE_string, position, result_peg_221)
        return result_peg_221
    except PegError:
        pass
    try:
        result_peg_234 = Result(position)
        result_peg_235 = Result(result_peg_234.getPosition())
        
        if '<quote>' == stream.get(result_peg_235.getPosition(), 7):
            result_peg_235.nextPosition(7)
            result_peg_235.setValue('<quote>')
        else:
            raise PegError
        
        result_peg_234.addResult(result_peg_235);
        
        result_peg_236 = Result(result_peg_234.getPosition())
        
        value = None
        values = result_peg_234.getValues()
        value = peg.PatternVerbatim('"')
        result_peg_236.setValue(value)
        
        result_peg_234.addResult(result_peg_236);
        
        result_peg_234.setValue(result_peg_234.getLastValue())
        stream.update(RULE_string, position, result_peg_234)
        return result_peg_234
    except PegError:
        pass
    stream.update(RULE_string, position, None)
    return None


def rule_modifier(stream, position):
    if stream.hasResult(RULE_modifier, position):
        return stream.result(RULE_modifier, position)
    try:
        result_peg_237 = Result(position)
        result_peg_238 = Result(result_peg_237.getPosition())
        
        if '*' == stream.get(result_peg_238.getPosition(), 1):
            result_peg_238.nextPosition(1)
            result_peg_238.setValue('*')
        else:
            raise PegError
        
        result_peg_237.addResult(result_peg_238);
        
        result_peg_239 = Result(result_peg_237.getPosition())
        
        value = None
        values = result_peg_237.getValues()
        value = lambda p: peg.PatternRepeatMany(p)
        result_peg_239.setValue(value)
        
        result_peg_237.addResult(result_peg_239);
        
        result_peg_237.setValue(result_peg_237.getLastValue())
        stream.update(RULE_modifier, position, result_peg_237)
        return result_peg_237
    except PegError:
        pass
    try:
        result_peg_240 = Result(position)
        result_peg_241 = Result(result_peg_240.getPosition())
        
        if '?' == stream.get(result_peg_241.getPosition(), 1):
            result_peg_241.nextPosition(1)
            result_peg_241.setValue('?')
        else:
            raise PegError
        
        result_peg_240.addResult(result_peg_241);
        
        result_peg_242 = Result(result_peg_240.getPosition())
        
        value = None
        values = result_peg_240.getValues()
        value = lambda p: peg.PatternMaybe(p)
        result_peg_242.setValue(value)
        
        result_peg_240.addResult(result_peg_242);
        
        result_peg_240.setValue(result_peg_240.getLastValue())
        stream.update(RULE_modifier, position, result_peg_240)
        return result_peg_240
    except PegError:
        pass
    try:
        result_peg_243 = Result(position)
        result_peg_244 = Result(result_peg_243.getPosition())
        
        if '+' == stream.get(result_peg_244.getPosition(), 1):
            result_peg_244.nextPosition(1)
            result_peg_244.setValue('+')
        else:
            raise PegError
        
        result_peg_243.addResult(result_peg_244);
        
        result_peg_245 = Result(result_peg_243.getPosition())
        
        value = None
        values = result_peg_243.getValues()
        value = lambda p: peg.PatternRepeatOnce(p)
        result_peg_245.setValue(value)
        
        result_peg_243.addResult(result_peg_245);
        
        result_peg_243.setValue(result_peg_243.getLastValue())
        stream.update(RULE_modifier, position, result_peg_243)
        return result_peg_243
    except PegError:
        pass
    stream.update(RULE_modifier, position, None)
    return None


def rule_x_word(stream, position):
    if stream.hasResult(RULE_x_word, position):
        return stream.result(RULE_x_word, position)
    try:
        result_peg_246 = Result(position)
        result_peg_247 = Result(result_peg_246.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_247 = rule_word(stream, result_peg_247.getPosition())
        if result_peg_247 == None:
            raise PegError
        name = result_peg_247.getValues()
        
        result_peg_246.addResult(result_peg_247);
        
        result_peg_248 = Result(result_peg_246.getPosition())
        
        save_peg_249 = result_peg_248.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_248 = rule_parameters_rules(stream, result_peg_248.getPosition())
        if result_peg_248 == None:
            
            result_peg_248 = Result(save_peg_249)
            result_peg_248.setValue(None)
        rule_parameters = result_peg_248.getValues()
        
        result_peg_246.addResult(result_peg_248);
        
        result_peg_250 = Result(result_peg_246.getPosition())
        
        save_peg_251 = result_peg_250.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_250 = rule_parameters_values(stream, result_peg_250.getPosition())
        if result_peg_250 == None:
            
            result_peg_250 = Result(save_peg_251)
            result_peg_250.setValue(None)
        parameters = result_peg_250.getValues()
        
        result_peg_246.addResult(result_peg_250);
        
        result_peg_252 = Result(result_peg_246.getPosition())
        
        value = None
        values = result_peg_246.getValues()
        value = peg.PatternRule(name, rule_parameters, parameters)
        result_peg_252.setValue(value)
        
        result_peg_246.addResult(result_peg_252);
        
        result_peg_246.setValue(result_peg_246.getLastValue())
        stream.update(RULE_x_word, position, result_peg_246)
        return result_peg_246
    except PegError:
        pass
    stream.update(RULE_x_word, position, None)
    return None


def rule_rule_parameters(stream, position):
    if stream.hasResult(RULE_rule_parameters, position):
        return stream.result(RULE_rule_parameters, position)
    try:
        result_peg_253 = Result(position)
        result_peg_254 = Result(result_peg_253.getPosition())
        
        if '[' == stream.get(result_peg_254.getPosition(), 1):
            result_peg_254.nextPosition(1)
            result_peg_254.setValue('[')
        else:
            raise PegError
        
        result_peg_253.addResult(result_peg_254);
        
        result_peg_255 = Result(result_peg_253.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_255 = rule_spaces(stream, result_peg_255.getPosition())
        if result_peg_255 == None:
            raise PegError
        
        result_peg_253.addResult(result_peg_255);
        
        result_peg_256 = Result(result_peg_253.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_256 = rule_word(stream, result_peg_256.getPosition())
        if result_peg_256 == None:
            raise PegError
        param1 = result_peg_256.getValues()
        
        result_peg_253.addResult(result_peg_256);
        
        result_peg_257 = Result(result_peg_253.getPosition())
        
        try:
            while True:
                result_peg_258 = Result(result_peg_257.getPosition());
                result_peg_259 = Result(result_peg_258.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_259 = rule_spaces(stream, result_peg_259.getPosition())
                if result_peg_259 == None:
                    raise PegError
                
                result_peg_258.addResult(result_peg_259);
                
                result_peg_260 = Result(result_peg_258.getPosition())
                
                if ',' == stream.get(result_peg_260.getPosition(), 1):
                    result_peg_260.nextPosition(1)
                    result_peg_260.setValue(',')
                else:
                    raise PegError
                
                result_peg_258.addResult(result_peg_260);
                
                result_peg_261 = Result(result_peg_258.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_261 = rule_spaces(stream, result_peg_261.getPosition())
                if result_peg_261 == None:
                    raise PegError
                
                result_peg_258.addResult(result_peg_261);
                
                result_peg_262 = Result(result_peg_258.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_262 = rule_word(stream, result_peg_262.getPosition())
                if result_peg_262 == None:
                    raise PegError
                exp = result_peg_262.getValues()
                
                result_peg_258.addResult(result_peg_262);
                
                result_peg_263 = Result(result_peg_258.getPosition())
                
                value = None
                values = result_peg_258.getValues()
                value = exp
                result_peg_263.setValue(value)
                
                result_peg_258.addResult(result_peg_263);
                
                result_peg_258.setValue(result_peg_258.getLastValue())
                result_peg_257.addResult(result_peg_258);
        except PegError:
            pass
        params = result_peg_257.getValues()
        
        result_peg_253.addResult(result_peg_257);
        
        result_peg_264 = Result(result_peg_253.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_264 = rule_spaces(stream, result_peg_264.getPosition())
        if result_peg_264 == None:
            raise PegError
        
        result_peg_253.addResult(result_peg_264);
        
        result_peg_265 = Result(result_peg_253.getPosition())
        
        if ']' == stream.get(result_peg_265.getPosition(), 1):
            result_peg_265.nextPosition(1)
            result_peg_265.setValue(']')
        else:
            raise PegError
        
        result_peg_253.addResult(result_peg_265);
        
        result_peg_266 = Result(result_peg_253.getPosition())
        
        value = None
        values = result_peg_253.getValues()
        value = [param1] + params
        result_peg_266.setValue(value)
        
        result_peg_253.addResult(result_peg_266);
        
        result_peg_253.setValue(result_peg_253.getLastValue())
        stream.update(RULE_rule_parameters, position, result_peg_253)
        return result_peg_253
    except PegError:
        pass
    stream.update(RULE_rule_parameters, position, None)
    return None


def rule_value_parameters(stream, position):
    if stream.hasResult(RULE_value_parameters, position):
        return stream.result(RULE_value_parameters, position)
    try:
        result_peg_267 = Result(position)
        result_peg_268 = Result(result_peg_267.getPosition())
        
        if '(' == stream.get(result_peg_268.getPosition(), 1):
            result_peg_268.nextPosition(1)
            result_peg_268.setValue('(')
        else:
            raise PegError
        
        result_peg_267.addResult(result_peg_268);
        
        result_peg_269 = Result(result_peg_267.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_269 = rule_spaces(stream, result_peg_269.getPosition())
        if result_peg_269 == None:
            raise PegError
        
        result_peg_267.addResult(result_peg_269);
        
        result_peg_270 = Result(result_peg_267.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_270 = rule_word(stream, result_peg_270.getPosition())
        if result_peg_270 == None:
            raise PegError
        param1 = result_peg_270.getValues()
        
        result_peg_267.addResult(result_peg_270);
        
        result_peg_271 = Result(result_peg_267.getPosition())
        
        try:
            while True:
                result_peg_272 = Result(result_peg_271.getPosition());
                result_peg_273 = Result(result_peg_272.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_273 = rule_spaces(stream, result_peg_273.getPosition())
                if result_peg_273 == None:
                    raise PegError
                
                result_peg_272.addResult(result_peg_273);
                
                result_peg_274 = Result(result_peg_272.getPosition())
                
                if ',' == stream.get(result_peg_274.getPosition(), 1):
                    result_peg_274.nextPosition(1)
                    result_peg_274.setValue(',')
                else:
                    raise PegError
                
                result_peg_272.addResult(result_peg_274);
                
                result_peg_275 = Result(result_peg_272.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_275 = rule_spaces(stream, result_peg_275.getPosition())
                if result_peg_275 == None:
                    raise PegError
                
                result_peg_272.addResult(result_peg_275);
                
                result_peg_276 = Result(result_peg_272.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_276 = rule_word(stream, result_peg_276.getPosition())
                if result_peg_276 == None:
                    raise PegError
                exp = result_peg_276.getValues()
                
                result_peg_272.addResult(result_peg_276);
                
                result_peg_277 = Result(result_peg_272.getPosition())
                
                value = None
                values = result_peg_272.getValues()
                value = exp
                result_peg_277.setValue(value)
                
                result_peg_272.addResult(result_peg_277);
                
                result_peg_272.setValue(result_peg_272.getLastValue())
                result_peg_271.addResult(result_peg_272);
        except PegError:
            pass
        params = result_peg_271.getValues()
        
        result_peg_267.addResult(result_peg_271);
        
        result_peg_278 = Result(result_peg_267.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_278 = rule_spaces(stream, result_peg_278.getPosition())
        if result_peg_278 == None:
            raise PegError
        
        result_peg_267.addResult(result_peg_278);
        
        result_peg_279 = Result(result_peg_267.getPosition())
        
        if ')' == stream.get(result_peg_279.getPosition(), 1):
            result_peg_279.nextPosition(1)
            result_peg_279.setValue(')')
        else:
            raise PegError
        
        result_peg_267.addResult(result_peg_279);
        
        result_peg_280 = Result(result_peg_267.getPosition())
        
        value = None
        values = result_peg_267.getValues()
        value = [param1] + params
        result_peg_280.setValue(value)
        
        result_peg_267.addResult(result_peg_280);
        
        result_peg_267.setValue(result_peg_267.getLastValue())
        stream.update(RULE_value_parameters, position, result_peg_267)
        return result_peg_267
    except PegError:
        pass
    stream.update(RULE_value_parameters, position, None)
    return None


def rule_parameters_rules(stream, position):
    if stream.hasResult(RULE_parameters_rules, position):
        return stream.result(RULE_parameters_rules, position)
    try:
        result_peg_281 = Result(position)
        result_peg_282 = Result(result_peg_281.getPosition())
        
        if '[' == stream.get(result_peg_282.getPosition(), 1):
            result_peg_282.nextPosition(1)
            result_peg_282.setValue('[')
        else:
            raise PegError
        
        result_peg_281.addResult(result_peg_282);
        
        result_peg_283 = Result(result_peg_281.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_283 = rule_spaces(stream, result_peg_283.getPosition())
        if result_peg_283 == None:
            raise PegError
        
        result_peg_281.addResult(result_peg_283);
        
        result_peg_284 = Result(result_peg_281.getPosition())
        
        # print "Trying rule " + 'word_or_at'
        result_peg_284 = rule_word_or_at(stream, result_peg_284.getPosition())
        if result_peg_284 == None:
            raise PegError
        param1 = result_peg_284.getValues()
        
        result_peg_281.addResult(result_peg_284);
        
        result_peg_285 = Result(result_peg_281.getPosition())
        
        try:
            while True:
                result_peg_286 = Result(result_peg_285.getPosition());
                result_peg_287 = Result(result_peg_286.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_287 = rule_spaces(stream, result_peg_287.getPosition())
                if result_peg_287 == None:
                    raise PegError
                
                result_peg_286.addResult(result_peg_287);
                
                result_peg_288 = Result(result_peg_286.getPosition())
                
                if ',' == stream.get(result_peg_288.getPosition(), 1):
                    result_peg_288.nextPosition(1)
                    result_peg_288.setValue(',')
                else:
                    raise PegError
                
                result_peg_286.addResult(result_peg_288);
                
                result_peg_289 = Result(result_peg_286.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_289 = rule_spaces(stream, result_peg_289.getPosition())
                if result_peg_289 == None:
                    raise PegError
                
                result_peg_286.addResult(result_peg_289);
                
                result_peg_290 = Result(result_peg_286.getPosition())
                
                # print "Trying rule " + 'word_or_at'
                result_peg_290 = rule_word_or_at(stream, result_peg_290.getPosition())
                if result_peg_290 == None:
                    raise PegError
                exp = result_peg_290.getValues()
                
                result_peg_286.addResult(result_peg_290);
                
                result_peg_291 = Result(result_peg_286.getPosition())
                
                value = None
                values = result_peg_286.getValues()
                value = exp
                result_peg_291.setValue(value)
                
                result_peg_286.addResult(result_peg_291);
                
                result_peg_286.setValue(result_peg_286.getLastValue())
                result_peg_285.addResult(result_peg_286);
        except PegError:
            pass
        params = result_peg_285.getValues()
        
        result_peg_281.addResult(result_peg_285);
        
        result_peg_292 = Result(result_peg_281.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_292 = rule_spaces(stream, result_peg_292.getPosition())
        if result_peg_292 == None:
            raise PegError
        
        result_peg_281.addResult(result_peg_292);
        
        result_peg_293 = Result(result_peg_281.getPosition())
        
        if ']' == stream.get(result_peg_293.getPosition(), 1):
            result_peg_293.nextPosition(1)
            result_peg_293.setValue(']')
        else:
            raise PegError
        
        result_peg_281.addResult(result_peg_293);
        
        result_peg_294 = Result(result_peg_281.getPosition())
        
        value = None
        values = result_peg_281.getValues()
        value = [param1] + params
        result_peg_294.setValue(value)
        
        result_peg_281.addResult(result_peg_294);
        
        result_peg_281.setValue(result_peg_281.getLastValue())
        stream.update(RULE_parameters_rules, position, result_peg_281)
        return result_peg_281
    except PegError:
        pass
    stream.update(RULE_parameters_rules, position, None)
    return None


def rule_parameters_values(stream, position):
    if stream.hasResult(RULE_parameters_values, position):
        return stream.result(RULE_parameters_values, position)
    try:
        result_peg_295 = Result(position)
        result_peg_296 = Result(result_peg_295.getPosition())
        
        if '(' == stream.get(result_peg_296.getPosition(), 1):
            result_peg_296.nextPosition(1)
            result_peg_296.setValue('(')
        else:
            raise PegError
        
        result_peg_295.addResult(result_peg_296);
        
        result_peg_297 = Result(result_peg_295.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_297 = rule_spaces(stream, result_peg_297.getPosition())
        if result_peg_297 == None:
            raise PegError
        
        result_peg_295.addResult(result_peg_297);
        
        result_peg_298 = Result(result_peg_295.getPosition())
        
        # print "Trying rule " + 'word_or_dollar'
        result_peg_298 = rule_word_or_dollar(stream, result_peg_298.getPosition())
        if result_peg_298 == None:
            raise PegError
        param1 = result_peg_298.getValues()
        
        result_peg_295.addResult(result_peg_298);
        
        result_peg_299 = Result(result_peg_295.getPosition())
        
        try:
            while True:
                result_peg_300 = Result(result_peg_299.getPosition());
                result_peg_301 = Result(result_peg_300.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_301 = rule_spaces(stream, result_peg_301.getPosition())
                if result_peg_301 == None:
                    raise PegError
                
                result_peg_300.addResult(result_peg_301);
                
                result_peg_302 = Result(result_peg_300.getPosition())
                
                if ',' == stream.get(result_peg_302.getPosition(), 1):
                    result_peg_302.nextPosition(1)
                    result_peg_302.setValue(',')
                else:
                    raise PegError
                
                result_peg_300.addResult(result_peg_302);
                
                result_peg_303 = Result(result_peg_300.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_303 = rule_spaces(stream, result_peg_303.getPosition())
                if result_peg_303 == None:
                    raise PegError
                
                result_peg_300.addResult(result_peg_303);
                
                result_peg_304 = Result(result_peg_300.getPosition())
                
                # print "Trying rule " + 'word_or_dollar'
                result_peg_304 = rule_word_or_dollar(stream, result_peg_304.getPosition())
                if result_peg_304 == None:
                    raise PegError
                exp = result_peg_304.getValues()
                
                result_peg_300.addResult(result_peg_304);
                
                result_peg_305 = Result(result_peg_300.getPosition())
                
                value = None
                values = result_peg_300.getValues()
                value = exp
                result_peg_305.setValue(value)
                
                result_peg_300.addResult(result_peg_305);
                
                result_peg_300.setValue(result_peg_300.getLastValue())
                result_peg_299.addResult(result_peg_300);
        except PegError:
            pass
        params = result_peg_299.getValues()
        
        result_peg_295.addResult(result_peg_299);
        
        result_peg_306 = Result(result_peg_295.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_306 = rule_spaces(stream, result_peg_306.getPosition())
        if result_peg_306 == None:
            raise PegError
        
        result_peg_295.addResult(result_peg_306);
        
        result_peg_307 = Result(result_peg_295.getPosition())
        
        if ')' == stream.get(result_peg_307.getPosition(), 1):
            result_peg_307.nextPosition(1)
            result_peg_307.setValue(')')
        else:
            raise PegError
        
        result_peg_295.addResult(result_peg_307);
        
        result_peg_308 = Result(result_peg_295.getPosition())
        
        value = None
        values = result_peg_295.getValues()
        value = [param1] + params
        result_peg_308.setValue(value)
        
        result_peg_295.addResult(result_peg_308);
        
        result_peg_295.setValue(result_peg_295.getLastValue())
        stream.update(RULE_parameters_values, position, result_peg_295)
        return result_peg_295
    except PegError:
        pass
    stream.update(RULE_parameters_values, position, None)
    return None


def rule_word_or_dollar(stream, position):
    if stream.hasResult(RULE_word_or_dollar, position):
        return stream.result(RULE_word_or_dollar, position)
    try:
        result_peg_309 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_309 = rule_word(stream, result_peg_309.getPosition())
        if result_peg_309 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_309)
        return result_peg_309
    except PegError:
        pass
    try:
        result_peg_310 = Result(position)
        # print "Trying rule " + 'dollar'
        result_peg_310 = rule_dollar(stream, result_peg_310.getPosition())
        if result_peg_310 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_310)
        return result_peg_310
    except PegError:
        pass
    stream.update(RULE_word_or_dollar, position, None)
    return None


def rule_word_or_at(stream, position):
    if stream.hasResult(RULE_word_or_at, position):
        return stream.result(RULE_word_or_at, position)
    try:
        result_peg_311 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_311 = rule_word(stream, result_peg_311.getPosition())
        if result_peg_311 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_311)
        return result_peg_311
    except PegError:
        pass
    try:
        result_peg_312 = Result(position)
        # print "Trying rule " + 'word_at'
        result_peg_312 = rule_word_at(stream, result_peg_312.getPosition())
        if result_peg_312 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_312)
        return result_peg_312
    except PegError:
        pass
    stream.update(RULE_word_or_at, position, None)
    return None


def rule_word_at(stream, position):
    if stream.hasResult(RULE_word_at, position):
        return stream.result(RULE_word_at, position)
    try:
        result_peg_313 = Result(position)
        result_peg_314 = Result(result_peg_313.getPosition())
        
        if '@' == stream.get(result_peg_314.getPosition(), 1):
            result_peg_314.nextPosition(1)
            result_peg_314.setValue('@')
        else:
            raise PegError
        
        result_peg_313.addResult(result_peg_314);
        
        result_peg_315 = Result(result_peg_313.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_315 = rule_word(stream, result_peg_315.getPosition())
        if result_peg_315 == None:
            raise PegError
        word = result_peg_315.getValues()
        
        result_peg_313.addResult(result_peg_315);
        
        result_peg_316 = Result(result_peg_313.getPosition())
        
        value = None
        values = result_peg_313.getValues()
        value = '@%s' % word
        result_peg_316.setValue(value)
        
        result_peg_313.addResult(result_peg_316);
        
        result_peg_313.setValue(result_peg_313.getLastValue())
        stream.update(RULE_word_at, position, result_peg_313)
        return result_peg_313
    except PegError:
        pass
    stream.update(RULE_word_at, position, None)
    return None


def rule_dollar(stream, position):
    if stream.hasResult(RULE_dollar, position):
        return stream.result(RULE_dollar, position)
    try:
        result_peg_317 = Result(position)
        result_peg_318 = Result(result_peg_317.getPosition())
        
        if '$' == stream.get(result_peg_318.getPosition(), 1):
            result_peg_318.nextPosition(1)
            result_peg_318.setValue('$')
        else:
            raise PegError
        
        result_peg_317.addResult(result_peg_318);
        
        result_peg_319 = Result(result_peg_317.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_319 = rule_number(stream, result_peg_319.getPosition())
        if result_peg_319 == None:
            raise PegError
        number = result_peg_319.getValues()
        
        result_peg_317.addResult(result_peg_319);
        
        result_peg_320 = Result(result_peg_317.getPosition())
        
        value = None
        values = result_peg_317.getValues()
        value = "$%s" % number
        result_peg_320.setValue(value)
        
        result_peg_317.addResult(result_peg_320);
        
        result_peg_317.setValue(result_peg_317.getLastValue())
        stream.update(RULE_dollar, position, result_peg_317)
        return result_peg_317
    except PegError:
        pass
    stream.update(RULE_dollar, position, None)
    return None


def rule_number(stream, position):
    if stream.hasResult(RULE_number, position):
        return stream.result(RULE_number, position)
    try:
        result_peg_321 = Result(position)
        result_peg_322 = Result(result_peg_321.getPosition())
        
        try:
            while True:
                result_peg_323 = Result(result_peg_322.getPosition());
                # print "Trying rule " + 'digit'
                result_peg_323 = rule_digit(stream, result_peg_323.getPosition())
                if result_peg_323 == None:
                    raise PegError
                result_peg_322.addResult(result_peg_323);
        except PegError:
            if result_peg_322.matches() == 0:
                raise PegError
                
        result_peg_321.addResult(result_peg_322);
        
        result_peg_324 = Result(result_peg_321.getPosition())
        
        value = None
        values = result_peg_321.getValues()
        value = ''.join(values[0])
        result_peg_324.setValue(value)
        
        result_peg_321.addResult(result_peg_324);
        
        result_peg_321.setValue(result_peg_321.getLastValue())
        stream.update(RULE_number, position, result_peg_321)
        return result_peg_321
    except PegError:
        pass
    stream.update(RULE_number, position, None)
    return None


def rule_digit(stream, position):
    if stream.hasResult(RULE_digit, position):
        return stream.result(RULE_digit, position)
    try:
        result_peg_325 = Result(position)
        letter_peg_326 = stream.get(result_peg_325.getPosition())
        if letter_peg_326 in '0123456789':
            result_peg_325.nextPosition()
            result_peg_325.setValue(letter_peg_326)
        else:
            raise PegError
        stream.update(RULE_digit, position, result_peg_325)
        return result_peg_325
    except PegError:
        pass
    stream.update(RULE_digit, position, None)
    return None


def rule_hex_number(stream, position):
    if stream.hasResult(RULE_hex_number, position):
        return stream.result(RULE_hex_number, position)
    try:
        result_peg_327 = Result(position)
        result_peg_328 = Result(result_peg_327.getPosition())
        
        try:
            while True:
                result_peg_329 = Result(result_peg_328.getPosition());
                # print "Trying rule " + 'hex_digit'
                result_peg_329 = rule_hex_digit(stream, result_peg_329.getPosition())
                if result_peg_329 == None:
                    raise PegError
                result_peg_328.addResult(result_peg_329);
        except PegError:
            if result_peg_328.matches() == 0:
                raise PegError
                
        result_peg_327.addResult(result_peg_328);
        
        result_peg_330 = Result(result_peg_327.getPosition())
        
        value = None
        values = result_peg_327.getValues()
        value = ''.join(values[0])
        result_peg_330.setValue(value)
        
        result_peg_327.addResult(result_peg_330);
        
        result_peg_327.setValue(result_peg_327.getLastValue())
        stream.update(RULE_hex_number, position, result_peg_327)
        return result_peg_327
    except PegError:
        pass
    stream.update(RULE_hex_number, position, None)
    return None


def rule_hex_digit(stream, position):
    if stream.hasResult(RULE_hex_digit, position):
        return stream.result(RULE_hex_digit, position)
    try:
        result_peg_331 = Result(position)
        letter_peg_332 = stream.get(result_peg_331.getPosition())
        if letter_peg_332 in '0123456789abcdefABCDEF':
            result_peg_331.nextPosition()
            result_peg_331.setValue(letter_peg_332)
        else:
            raise PegError
        stream.update(RULE_hex_digit, position, result_peg_331)
        return result_peg_331
    except PegError:
        pass
    stream.update(RULE_hex_digit, position, None)
    return None


def rule_start_symbol(stream, position):
    if stream.hasResult(RULE_start_symbol, position):
        return stream.result(RULE_start_symbol, position)
    try:
        result_peg_333 = Result(position)
        result_peg_334 = Result(result_peg_333.getPosition())
        
        if 'start-symbol:' == stream.get(result_peg_334.getPosition(), 13):
            result_peg_334.nextPosition(13)
            result_peg_334.setValue('start-symbol:')
        else:
            raise PegError
        
        result_peg_333.addResult(result_peg_334);
        
        result_peg_335 = Result(result_peg_333.getPosition())
        
        try:
            while True:
                result_peg_336 = Result(result_peg_335.getPosition());
                # print "Trying rule " + 'space'
                result_peg_336 = rule_space(stream, result_peg_336.getPosition())
                if result_peg_336 == None:
                    raise PegError
                result_peg_335.addResult(result_peg_336);
        except PegError:
            pass
                
        result_peg_333.addResult(result_peg_335);
        
        result_peg_337 = Result(result_peg_333.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_337 = rule_word(stream, result_peg_337.getPosition())
        if result_peg_337 == None:
            raise PegError
        
        result_peg_333.addResult(result_peg_337);
        
        result_peg_338 = Result(result_peg_333.getPosition())
        
        value = None
        values = result_peg_333.getValues()
        value = values[2]
        result_peg_338.setValue(value)
        
        result_peg_333.addResult(result_peg_338);
        
        result_peg_333.setValue(result_peg_333.getLastValue())
        stream.update(RULE_start_symbol, position, result_peg_333)
        return result_peg_333
    except PegError:
        pass
    stream.update(RULE_start_symbol, position, None)
    return None


def rule_spaces(stream, position):
    if stream.hasResult(RULE_spaces, position):
        return stream.result(RULE_spaces, position)
    try:
        result_peg_339 = Result(position)
        result_peg_340 = Result(result_peg_339.getPosition())
        
        try:
            while True:
                result_peg_341 = Result(result_peg_340.getPosition());
                # print "Trying rule " + 'space'
                result_peg_341 = rule_space(stream, result_peg_341.getPosition())
                if result_peg_341 == None:
                    raise PegError
                result_peg_340.addResult(result_peg_341);
        except PegError:
            pass
                
        result_peg_339.addResult(result_peg_340);
        
        result_peg_339.setValue(result_peg_339.getLastValue())
        stream.update(RULE_spaces, position, result_peg_339)
        return result_peg_339
    except PegError:
        pass
    stream.update(RULE_spaces, position, None)
    return None


def rule_space(stream, position):
    if stream.hasResult(RULE_space, position):
        return stream.result(RULE_space, position)
    try:
        result_peg_342 = Result(position)
        result_peg_343 = Result(result_peg_342.getPosition())
        
        if ' ' == stream.get(result_peg_343.getPosition(), 1):
            result_peg_343.nextPosition(1)
            result_peg_343.setValue(' ')
        else:
            raise PegError
        
        result_peg_342.addResult(result_peg_343);
        
        result_peg_342.setValue(result_peg_342.getLastValue())
        stream.update(RULE_space, position, result_peg_342)
        return result_peg_342
    except PegError:
        pass
    try:
        result_peg_344 = Result(position)
        result_peg_345 = Result(result_peg_344.getPosition())
        
        if '\t' == stream.get(result_peg_345.getPosition(), 1):
            result_peg_345.nextPosition(1)
            result_peg_345.setValue('\t')
        else:
            raise PegError
        
        result_peg_344.addResult(result_peg_345);
        
        result_peg_344.setValue(result_peg_344.getLastValue())
        stream.update(RULE_space, position, result_peg_344)
        return result_peg_344
    except PegError:
        pass
    stream.update(RULE_space, position, None)
    return None


def rule_any_char(stream, position):
    if stream.hasResult(RULE_any_char, position):
        return stream.result(RULE_any_char, position)
    try:
        result_peg_346 = Result(position)
        letter_peg_347 = stream.get(result_peg_346.getPosition())
        if letter_peg_347 in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-':
            result_peg_346.nextPosition()
            result_peg_346.setValue(letter_peg_347)
        else:
            raise PegError
        stream.update(RULE_any_char, position, result_peg_346)
        return result_peg_346
    except PegError:
        pass
    stream.update(RULE_any_char, position, None)
    return None


def rule_any(stream, position):
    if stream.hasResult(RULE_any, position):
        return stream.result(RULE_any, position)
    try:
        result_peg_348 = Result(position)
        result_peg_349 = Result(result_peg_348.getPosition())
        
        if '.' == stream.get(result_peg_349.getPosition(), 1):
            result_peg_349.nextPosition(1)
            result_peg_349.setValue('.')
        else:
            raise PegError
        
        result_peg_348.addResult(result_peg_349);
        
        result_peg_350 = Result(result_peg_348.getPosition())
        
        value = None
        values = result_peg_348.getValues()
        value = peg.PatternAny()
        result_peg_350.setValue(value)
        
        result_peg_348.addResult(result_peg_350);
        
        result_peg_348.setValue(result_peg_348.getLastValue())
        stream.update(RULE_any, position, result_peg_348)
        return result_peg_348
    except PegError:
        pass
    stream.update(RULE_any, position, None)
    return None


def rule_whitespace(stream, position):
    if stream.hasResult(RULE_whitespace, position):
        return stream.result(RULE_whitespace, position)
    try:
        result_peg_351 = Result(position)
        result_peg_352 = Result(result_peg_351.getPosition())
        
        try:
            while True:
                result_peg_353 = Result(result_peg_352.getPosition());
                save_peg_354 = result_peg_353.getPosition()
                
                result_peg_353 = Result(save_peg_354)
                letter_peg_355 = stream.get(result_peg_353.getPosition())
                if letter_peg_355 in ' \t\n':
                    result_peg_353.nextPosition()
                    result_peg_353.setValue(letter_peg_355)
                else:
                    
                    result_peg_353 = Result(save_peg_354)
                    # print "Trying rule " + 'comment'
                    result_peg_353 = rule_comment(stream, result_peg_353.getPosition())
                    if result_peg_353 == None:
                        raise PegError
                result_peg_352.addResult(result_peg_353);
        except PegError:
            pass
                
        result_peg_351.addResult(result_peg_352);
        
        result_peg_351.setValue(result_peg_351.getLastValue())
        stream.update(RULE_whitespace, position, result_peg_351)
        return result_peg_351
    except PegError:
        pass
    stream.update(RULE_whitespace, position, None)
    return None


def rule_comment(stream, position):
    if stream.hasResult(RULE_comment, position):
        return stream.result(RULE_comment, position)
    try:
        result_peg_356 = Result(position)
        result_peg_357 = Result(result_peg_356.getPosition())
        
        if '#' == stream.get(result_peg_357.getPosition(), 1):
            result_peg_357.nextPosition(1)
            result_peg_357.setValue('#')
        else:
            raise PegError
        
        result_peg_356.addResult(result_peg_357);
        
        result_peg_358 = Result(result_peg_356.getPosition())
        
        try:
            while True:
                result_peg_359 = Result(result_peg_358.getPosition());
                result_peg_360 = Result(result_peg_359.getPosition())
                
                result_peg_361 = Result(result_peg_360.getPosition());
                try:
                    if '\n' == stream.get(result_peg_361.getPosition(), 1):
                        result_peg_361.nextPosition(1)
                        result_peg_361.setValue('\n')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_360.setValue(None)
                        
                result_peg_359.addResult(result_peg_360);
                
                result_peg_362 = Result(result_peg_359.getPosition())
                
                temp_peg_363 = stream.get(result_peg_362.getPosition())
                if temp_peg_363 != chr(0):
                    result_peg_362.setValue(temp_peg_363)
                    result_peg_362.nextPosition()
                else:
                    raise PegError
                
                result_peg_359.addResult(result_peg_362);
                
                result_peg_359.setValue(result_peg_359.getLastValue())
                result_peg_358.addResult(result_peg_359);
        except PegError:
            pass
                
        result_peg_356.addResult(result_peg_358);
        
        result_peg_356.setValue(result_peg_356.getLastValue())
        stream.update(RULE_comment, position, result_peg_356)
        return result_peg_356
    except PegError:
        pass
    stream.update(RULE_comment, position, None)
    return None


def rule_newlines_one(stream, position):
    if stream.hasResult(RULE_newlines_one, position):
        return stream.result(RULE_newlines_one, position)
    try:
        result_peg_364 = Result(position)
        try:
            while True:
                result_peg_365 = Result(result_peg_364.getPosition());
                if '\n' == stream.get(result_peg_365.getPosition(), 1):
                    result_peg_365.nextPosition(1)
                    result_peg_365.setValue('\n')
                else:
                    raise PegError
                result_peg_364.addResult(result_peg_365);
        except PegError:
            if result_peg_364.matches() == 0:
                raise PegError
        stream.update(RULE_newlines_one, position, result_peg_364)
        return result_peg_364
    except PegError:
        pass
    stream.update(RULE_newlines_one, position, None)
    return None


def rule_newlines(stream, position):
    if stream.hasResult(RULE_newlines, position):
        return stream.result(RULE_newlines, position)
    try:
        result_peg_366 = Result(position)
        try:
            while True:
                result_peg_367 = Result(result_peg_366.getPosition());
                if '\n' == stream.get(result_peg_367.getPosition(), 1):
                    result_peg_367.nextPosition(1)
                    result_peg_367.setValue('\n')
                else:
                    raise PegError
                result_peg_366.addResult(result_peg_367);
        except PegError:
            pass
        stream.update(RULE_newlines, position, result_peg_366)
        return result_peg_366
    except PegError:
        pass
    stream.update(RULE_newlines, position, None)
    return None


def doParse(stream):
    done = rule_start(stream, 0)
    if done == None:
        # print "Error parsing " + file
        stream.reportError()
        return None
    else:
        return done.getValues()

def parseFile(file):
    # print "Parsing " + file
    return doParse(Stream(filename = file))

def parseString(value):
    return doParse(Stream(input = value))

