
import peg





def special_escape(s):
    return s.replace("\\n", "\\\\n").replace("\\t", "\\\\t").replace("\"", '\\\"').replace("\\r", "\\\\r")

class PegError(Exception):
    def __init__(self):
        Exception.__init__(self)

class NotError(Exception):
    def __init__(self):
        Exception.__init__(self)

class Result:
    def __init__(self, position):
        self.position = position
        self.values = []

    def getPosition(self):
        return self.position

    def nextPosition(self, amount = 1):
        self.position += amount

    def setValue(self, value):
        self.values = value

    def getLastValue(self):
        if type(self.values) is list:
            if len(self.values) > 0:
                return self.values[-1]
            else:
                return None
        return self.values
    
    def matches(self):
        return len(self.values)

    def getValues(self):
        return self.values

    def addResult(self, him):
        self.values.append(him.values)
        self.position = him.position
    
    #def extendResult(self, him):
    #    self.values.extend(him.values)
    #    self.position = him.position

class Stream:
    def __init__(self, filename = None, input = None):
        def read():
            file = open(filename, 'r')
            out = file.read()
            file.close()
            return out
        self.position = 0
        self.limit = 100
        self.furthest = 0
        self.memo = {}
        if filename != None:
            self.all = read()
        elif input != None:
            self.all = input
        else:
            raise PegError("Pass a filename or input")
        # print "Read " + str(len(self.all))

    def get(self, position, number = 1):
        if position + number > self.limit:
            # print (position + number)
            self.limit += 5000
        if position + number > len(self.all):
            return chr(0)
        # print "stream: %s" % self.all[position:position+number]
        return self.all[position:position+number]

    def get2(self, position):
        if position != self.position:
            self.file.seek(position)
        self.position = position + 1
        if position > self.limit:
            print position
            self.limit += 5000
        return self.file.read(1)

    def reportError(self):
        line = 1
        column = 1
        for i in xrange(0, self.furthest):
            if self.all[i] == '\n':
                line += 1
                column = 1
            else:
                column += 1
        context = 10
        left = self.furthest - context
        right = self.furthest + context
        if left < 0:
            left = 0
        if right > len(self.all):
            right = len(self.all)
        print "Read up till line %d, column %d" % (line, column)
        print "'%s'" % special_escape(self.all[left:right])
        print "%s^" % (' ' * (self.furthest - left))

    def update(self, rule, position, result):
        if result != None and result.getPosition() > self.furthest:
            self.furthest = result.getPosition()

        for_rule = None
        try:
            for_rule = self.memo[rule]
        except KeyError:
            self.memo[rule] = {}
            for_rule = self.memo[rule]
        
        for_position = None
        try:
            for_position = for_rule[position]
        except KeyError:
            for_rule[position] = None

        for_rule[position] = result

    def hasResult(self, rule, position):
        try:
            x = self.memo[rule][position]
            return True
        except KeyError:
            return False

    def result(self, rule, position):
        return self.memo[rule][position]



RULE_start = 0
RULE_module = 1
RULE_include = 2
RULE_more_code = 3
RULE_options = 4
RULE_option = 5
RULE_chunks = 6
RULE_error_option = 7
RULE_word = 8
RULE_rules = 9
RULE_rule = 10
RULE_pattern_line = 11
RULE_pattern = 12
RULE_raw_code = 13
RULE_code = 14
RULE_item = 15
RULE_failure = 16
RULE_line = 17
RULE_predicate = 18
RULE_utf8 = 19
RULE_ascii = 20
RULE_call_rule = 21
RULE_eof = 22
RULE_void = 23
RULE_range = 24
RULE_sub_pattern = 25
RULE_bind = 26
RULE_string = 27
RULE_modifier = 28
RULE_x_word = 29
RULE_rule_parameters = 30
RULE_value_parameters = 31
RULE_parameters_rules = 32
RULE_parameters_values = 33
RULE_word_or_dollar = 34
RULE_word_or_at = 35
RULE_word_at = 36
RULE_dollar = 37
RULE_number = 38
RULE_digit = 39
RULE_hex_number = 40
RULE_hex_digit = 41
RULE_start_symbol = 42
RULE_spaces = 43
RULE_space = 44
RULE_any_char = 45
RULE_any = 46
RULE_whitespace = 47
RULE_comment = 48
RULE_newlines_one = 49
RULE_newlines = 50




def rule_start(stream, position):
    if stream.hasResult(RULE_start, position):
        return stream.result(RULE_start, position)
    try:
        result_peg_11 = Result(position)
        result_peg_12 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_12 = rule_newlines(stream, result_peg_12.getPosition())
        if result_peg_12 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_12);
        
        result_peg_13 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_13 = rule_whitespace(stream, result_peg_13.getPosition())
        if result_peg_13 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_13);
        
        result_peg_14 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'start_symbol'
        result_peg_14 = rule_start_symbol(stream, result_peg_14.getPosition())
        if result_peg_14 == None:
            raise PegError
        start_symbol = result_peg_14.getValues()
        
        result_peg_11.addResult(result_peg_14);
        
        result_peg_15 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_15 = rule_newlines(stream, result_peg_15.getPosition())
        if result_peg_15 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_15);
        
        result_peg_16 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_16 = rule_whitespace(stream, result_peg_16.getPosition())
        if result_peg_16 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_16);
        
        result_peg_17 = Result(result_peg_11.getPosition())
        
        save_peg_18 = result_peg_17.getPosition()
        
        # print "Trying rule " + 'options'
        result_peg_17 = rule_options(stream, result_peg_17.getPosition())
        if result_peg_17 == None:
            
            result_peg_17 = Result(save_peg_18)
            result_peg_17.setValue(None)
        options = result_peg_17.getValues()
        
        result_peg_11.addResult(result_peg_17);
        
        result_peg_19 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_19 = rule_newlines(stream, result_peg_19.getPosition())
        if result_peg_19 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_19);
        
        result_peg_20 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_20 = rule_whitespace(stream, result_peg_20.getPosition())
        if result_peg_20 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_20);
        
        result_peg_21 = Result(result_peg_11.getPosition())
        
        save_peg_22 = result_peg_21.getPosition()
        
        # print "Trying rule " + 'module'
        result_peg_21 = rule_module(stream, result_peg_21.getPosition())
        if result_peg_21 == None:
            
            result_peg_21 = Result(save_peg_22)
            result_peg_21.setValue(None)
        module = result_peg_21.getValues()
        
        result_peg_11.addResult(result_peg_21);
        
        result_peg_23 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_23 = rule_newlines(stream, result_peg_23.getPosition())
        if result_peg_23 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_23);
        
        result_peg_24 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_24 = rule_whitespace(stream, result_peg_24.getPosition())
        if result_peg_24 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_24);
        
        result_peg_25 = Result(result_peg_11.getPosition())
        
        save_peg_26 = result_peg_25.getPosition()
        
        # print "Trying rule " + 'include'
        result_peg_25 = rule_include(stream, result_peg_25.getPosition())
        if result_peg_25 == None:
            
            result_peg_25 = Result(save_peg_26)
            result_peg_25.setValue(None)
        include = result_peg_25.getValues()
        
        result_peg_11.addResult(result_peg_25);
        
        result_peg_27 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_27 = rule_newlines(stream, result_peg_27.getPosition())
        if result_peg_27 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_27);
        
        result_peg_28 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_28 = rule_whitespace(stream, result_peg_28.getPosition())
        if result_peg_28 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_28);
        
        result_peg_29 = Result(result_peg_11.getPosition())
        
        save_peg_30 = result_peg_29.getPosition()
        
        # print "Trying rule " + 'more_code'
        result_peg_29 = rule_more_code(stream, result_peg_29.getPosition())
        if result_peg_29 == None:
            
            result_peg_29 = Result(save_peg_30)
            result_peg_29.setValue(None)
        code = result_peg_29.getValues()
        
        result_peg_11.addResult(result_peg_29);
        
        result_peg_31 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_31 = rule_newlines(stream, result_peg_31.getPosition())
        if result_peg_31 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_31);
        
        result_peg_32 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_32 = rule_whitespace(stream, result_peg_32.getPosition())
        if result_peg_32 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_32);
        
        result_peg_33 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'rules'
        result_peg_33 = rule_rules(stream, result_peg_33.getPosition())
        if result_peg_33 == None:
            raise PegError
        rules = result_peg_33.getValues()
        
        result_peg_11.addResult(result_peg_33);
        
        result_peg_34 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'newlines'
        result_peg_34 = rule_newlines(stream, result_peg_34.getPosition())
        if result_peg_34 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_34);
        
        result_peg_35 = Result(result_peg_11.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_35 = rule_whitespace(stream, result_peg_35.getPosition())
        if result_peg_35 == None:
            raise PegError
        
        result_peg_11.addResult(result_peg_35);
        
        result_peg_36 = Result(result_peg_11.getPosition())
        
        if chr(0) == stream.get(result_peg_36.getPosition()):
            result_peg_36.nextPosition()
            result_peg_36.setValue(chr(0))
        else:
            raise PegError
        
        result_peg_11.addResult(result_peg_36);
        
        result_peg_37 = Result(result_peg_11.getPosition())
        
        value = None
        values = result_peg_11.getValues()
        value = peg.Peg(start_symbol, include, code, module, rules, options)
        result_peg_37.setValue(value)
        
        result_peg_11.addResult(result_peg_37);
        
        result_peg_11.setValue(result_peg_11.getLastValue())
        stream.update(RULE_start, position, result_peg_11)
        return result_peg_11
    except PegError:
        pass
    stream.update(RULE_start, position, None)
    return None


def rule_module(stream, position):
    if stream.hasResult(RULE_module, position):
        return stream.result(RULE_module, position)
    try:
        result_peg_38 = Result(position)
        result_peg_39 = Result(result_peg_38.getPosition())
        
        if 'module:' == stream.get(result_peg_39.getPosition(), 7):
            result_peg_39.nextPosition(7)
            result_peg_39.setValue('module:')
        else:
            raise PegError
        
        result_peg_38.addResult(result_peg_39);
        
        result_peg_40 = Result(result_peg_38.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_40 = rule_spaces(stream, result_peg_40.getPosition())
        if result_peg_40 == None:
            raise PegError
        
        result_peg_38.addResult(result_peg_40);
        
        result_peg_41 = Result(result_peg_38.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_41 = rule_word(stream, result_peg_41.getPosition())
        if result_peg_41 == None:
            raise PegError
        name = result_peg_41.getValues()
        
        result_peg_38.addResult(result_peg_41);
        
        result_peg_42 = Result(result_peg_38.getPosition())
        
        try:
            while True:
                result_peg_43 = Result(result_peg_42.getPosition());
                result_peg_44 = Result(result_peg_43.getPosition())
                
                if '.' == stream.get(result_peg_44.getPosition(), 1):
                    result_peg_44.nextPosition(1)
                    result_peg_44.setValue('.')
                else:
                    raise PegError
                
                result_peg_43.addResult(result_peg_44);
                
                result_peg_45 = Result(result_peg_43.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_45 = rule_word(stream, result_peg_45.getPosition())
                if result_peg_45 == None:
                    raise PegError
                
                result_peg_43.addResult(result_peg_45);
                
                result_peg_46 = Result(result_peg_43.getPosition())
                
                value = None
                values = result_peg_43.getValues()
                value = values[1]
                result_peg_46.setValue(value)
                
                result_peg_43.addResult(result_peg_46);
                
                result_peg_43.setValue(result_peg_43.getLastValue())
                result_peg_42.addResult(result_peg_43);
        except PegError:
            pass
        rest = result_peg_42.getValues()
        
        result_peg_38.addResult(result_peg_42);
        
        result_peg_47 = Result(result_peg_38.getPosition())
        
        value = None
        values = result_peg_38.getValues()
        value = [name] + rest
        result_peg_47.setValue(value)
        
        result_peg_38.addResult(result_peg_47);
        
        result_peg_38.setValue(result_peg_38.getLastValue())
        stream.update(RULE_module, position, result_peg_38)
        return result_peg_38
    except PegError:
        pass
    stream.update(RULE_module, position, None)
    return None


def rule_include(stream, position):
    if stream.hasResult(RULE_include, position):
        return stream.result(RULE_include, position)
    try:
        result_peg_48 = Result(position)
        result_peg_49 = Result(result_peg_48.getPosition())
        
        if 'include:' == stream.get(result_peg_49.getPosition(), 8):
            result_peg_49.nextPosition(8)
            result_peg_49.setValue('include:')
        else:
            raise PegError
        
        result_peg_48.addResult(result_peg_49);
        
        result_peg_50 = Result(result_peg_48.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_50 = rule_spaces(stream, result_peg_50.getPosition())
        if result_peg_50 == None:
            raise PegError
        
        result_peg_48.addResult(result_peg_50);
        
        result_peg_51 = Result(result_peg_48.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_51 = rule_code(stream, result_peg_51.getPosition())
        if result_peg_51 == None:
            raise PegError
        code = result_peg_51.getValues()
        
        result_peg_48.addResult(result_peg_51);
        
        result_peg_52 = Result(result_peg_48.getPosition())
        
        value = None
        values = result_peg_48.getValues()
        value = code.code
        result_peg_52.setValue(value)
        
        result_peg_48.addResult(result_peg_52);
        
        result_peg_48.setValue(result_peg_48.getLastValue())
        stream.update(RULE_include, position, result_peg_48)
        return result_peg_48
    except PegError:
        pass
    stream.update(RULE_include, position, None)
    return None


def rule_more_code(stream, position):
    if stream.hasResult(RULE_more_code, position):
        return stream.result(RULE_more_code, position)
    try:
        result_peg_53 = Result(position)
        result_peg_54 = Result(result_peg_53.getPosition())
        
        if 'code:' == stream.get(result_peg_54.getPosition(), 5):
            result_peg_54.nextPosition(5)
            result_peg_54.setValue('code:')
        else:
            raise PegError
        
        result_peg_53.addResult(result_peg_54);
        
        result_peg_55 = Result(result_peg_53.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_55 = rule_spaces(stream, result_peg_55.getPosition())
        if result_peg_55 == None:
            raise PegError
        
        result_peg_53.addResult(result_peg_55);
        
        result_peg_56 = Result(result_peg_53.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_56 = rule_code(stream, result_peg_56.getPosition())
        if result_peg_56 == None:
            raise PegError
        code = result_peg_56.getValues()
        
        result_peg_53.addResult(result_peg_56);
        
        result_peg_57 = Result(result_peg_53.getPosition())
        
        value = None
        values = result_peg_53.getValues()
        value = code.code
        result_peg_57.setValue(value)
        
        result_peg_53.addResult(result_peg_57);
        
        result_peg_53.setValue(result_peg_53.getLastValue())
        stream.update(RULE_more_code, position, result_peg_53)
        return result_peg_53
    except PegError:
        pass
    stream.update(RULE_more_code, position, None)
    return None


def rule_options(stream, position):
    if stream.hasResult(RULE_options, position):
        return stream.result(RULE_options, position)
    try:
        result_peg_58 = Result(position)
        result_peg_59 = Result(result_peg_58.getPosition())
        
        if 'options:' == stream.get(result_peg_59.getPosition(), 8):
            result_peg_59.nextPosition(8)
            result_peg_59.setValue('options:')
        else:
            raise PegError
        
        result_peg_58.addResult(result_peg_59);
        
        result_peg_60 = Result(result_peg_58.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_60 = rule_spaces(stream, result_peg_60.getPosition())
        if result_peg_60 == None:
            raise PegError
        
        result_peg_58.addResult(result_peg_60);
        
        result_peg_61 = Result(result_peg_58.getPosition())
        
        # print "Trying rule " + 'option'
        result_peg_61 = rule_option(stream, result_peg_61.getPosition())
        if result_peg_61 == None:
            raise PegError
        option1 = result_peg_61.getValues()
        
        result_peg_58.addResult(result_peg_61);
        
        result_peg_62 = Result(result_peg_58.getPosition())
        
        try:
            while True:
                result_peg_63 = Result(result_peg_62.getPosition());
                result_peg_64 = Result(result_peg_63.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_64 = rule_spaces(stream, result_peg_64.getPosition())
                if result_peg_64 == None:
                    raise PegError
                
                result_peg_63.addResult(result_peg_64);
                
                result_peg_65 = Result(result_peg_63.getPosition())
                
                if ',' == stream.get(result_peg_65.getPosition(), 1):
                    result_peg_65.nextPosition(1)
                    result_peg_65.setValue(',')
                else:
                    raise PegError
                
                result_peg_63.addResult(result_peg_65);
                
                result_peg_66 = Result(result_peg_63.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_66 = rule_spaces(stream, result_peg_66.getPosition())
                if result_peg_66 == None:
                    raise PegError
                
                result_peg_63.addResult(result_peg_66);
                
                result_peg_67 = Result(result_peg_63.getPosition())
                
                # print "Trying rule " + 'option'
                result_peg_67 = rule_option(stream, result_peg_67.getPosition())
                if result_peg_67 == None:
                    raise PegError
                
                result_peg_63.addResult(result_peg_67);
                
                result_peg_63.setValue(result_peg_63.getLastValue())
                result_peg_62.addResult(result_peg_63);
        except PegError:
            pass
        option_rest = result_peg_62.getValues()
        
        result_peg_58.addResult(result_peg_62);
        
        result_peg_68 = Result(result_peg_58.getPosition())
        
        value = None
        values = result_peg_58.getValues()
        value = []
        for option in ([option1] + option_rest):
            import re
            debug = re.compile("debug(\d+)")
            out = debug.match(option)
            if out != None:
                num = int(out.group(1))
                for x in xrange(1,num+1):
                    value.append('debug%d' % x)
            elif option == 'no-memo':
                value.append(option)
            else:
                value.append(option)
        result_peg_68.setValue(value)
        
        result_peg_58.addResult(result_peg_68);
        
        result_peg_58.setValue(result_peg_58.getLastValue())
        stream.update(RULE_options, position, result_peg_58)
        return result_peg_58
    except PegError:
        pass
    stream.update(RULE_options, position, None)
    return None


def rule_option(stream, position):
    if stream.hasResult(RULE_option, position):
        return stream.result(RULE_option, position)
    try:
        result_peg_69 = Result(position)
        result_peg_70 = Result(result_peg_69.getPosition())
        
        if 'debug' == stream.get(result_peg_70.getPosition(), 5):
            result_peg_70.nextPosition(5)
            result_peg_70.setValue('debug')
        else:
            raise PegError
        
        result_peg_69.addResult(result_peg_70);
        
        result_peg_71 = Result(result_peg_69.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_71 = rule_number(stream, result_peg_71.getPosition())
        if result_peg_71 == None:
            raise PegError
        number = result_peg_71.getValues()
        
        result_peg_69.addResult(result_peg_71);
        
        result_peg_72 = Result(result_peg_69.getPosition())
        
        value = None
        values = result_peg_69.getValues()
        value = 'debug%s' % number
        result_peg_72.setValue(value)
        
        result_peg_69.addResult(result_peg_72);
        
        result_peg_69.setValue(result_peg_69.getLastValue())
        stream.update(RULE_option, position, result_peg_69)
        return result_peg_69
    except PegError:
        pass
    try:
        result_peg_73 = Result(position)
        if 'state' == stream.get(result_peg_73.getPosition(), 5):
            result_peg_73.nextPosition(5)
            result_peg_73.setValue('state')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_73)
        return result_peg_73
    except PegError:
        pass
    try:
        result_peg_74 = Result(position)
        if 'no-memo' == stream.get(result_peg_74.getPosition(), 7):
            result_peg_74.nextPosition(7)
            result_peg_74.setValue('no-memo')
        else:
            raise PegError
        stream.update(RULE_option, position, result_peg_74)
        return result_peg_74
    except PegError:
        pass
    try:
        result_peg_75 = Result(position)
        # print "Trying rule " + 'error_option'
        result_peg_75 = rule_error_option(stream, result_peg_75.getPosition())
        if result_peg_75 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_75)
        return result_peg_75
    except PegError:
        pass
    try:
        result_peg_76 = Result(position)
        # print "Trying rule " + 'chunks'
        result_peg_76 = rule_chunks(stream, result_peg_76.getPosition())
        if result_peg_76 == None:
            raise PegError
        stream.update(RULE_option, position, result_peg_76)
        return result_peg_76
    except PegError:
        pass
    stream.update(RULE_option, position, None)
    return None


def rule_chunks(stream, position):
    if stream.hasResult(RULE_chunks, position):
        return stream.result(RULE_chunks, position)
    try:
        result_peg_77 = Result(position)
        result_peg_78 = Result(result_peg_77.getPosition())
        
        if 'chunks' == stream.get(result_peg_78.getPosition(), 6):
            result_peg_78.nextPosition(6)
            result_peg_78.setValue('chunks')
        else:
            raise PegError
        
        result_peg_77.addResult(result_peg_78);
        
        result_peg_79 = Result(result_peg_77.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_79 = rule_whitespace(stream, result_peg_79.getPosition())
        if result_peg_79 == None:
            raise PegError
        
        result_peg_77.addResult(result_peg_79);
        
        result_peg_80 = Result(result_peg_77.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_80 = rule_number(stream, result_peg_80.getPosition())
        if result_peg_80 == None:
            raise PegError
        number = result_peg_80.getValues()
        
        result_peg_77.addResult(result_peg_80);
        
        result_peg_81 = Result(result_peg_77.getPosition())
        
        value = None
        values = result_peg_77.getValues()
        value = 'chunks %s' % number
        result_peg_81.setValue(value)
        
        result_peg_77.addResult(result_peg_81);
        
        result_peg_77.setValue(result_peg_77.getLastValue())
        stream.update(RULE_chunks, position, result_peg_77)
        return result_peg_77
    except PegError:
        pass
    stream.update(RULE_chunks, position, None)
    return None


def rule_error_option(stream, position):
    if stream.hasResult(RULE_error_option, position):
        return stream.result(RULE_error_option, position)
    try:
        result_peg_82 = Result(position)
        result_peg_83 = Result(result_peg_82.getPosition())
        
        if 'error-length' == stream.get(result_peg_83.getPosition(), 12):
            result_peg_83.nextPosition(12)
            result_peg_83.setValue('error-length')
        else:
            raise PegError
        
        result_peg_82.addResult(result_peg_83);
        
        result_peg_84 = Result(result_peg_82.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_84 = rule_whitespace(stream, result_peg_84.getPosition())
        if result_peg_84 == None:
            raise PegError
        
        result_peg_82.addResult(result_peg_84);
        
        result_peg_85 = Result(result_peg_82.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_85 = rule_number(stream, result_peg_85.getPosition())
        if result_peg_85 == None:
            raise PegError
        number = result_peg_85.getValues()
        
        result_peg_82.addResult(result_peg_85);
        
        result_peg_86 = Result(result_peg_82.getPosition())
        
        value = None
        values = result_peg_82.getValues()
        value = 'error-length %s' % number
        result_peg_86.setValue(value)
        
        result_peg_82.addResult(result_peg_86);
        
        result_peg_82.setValue(result_peg_82.getLastValue())
        stream.update(RULE_error_option, position, result_peg_82)
        return result_peg_82
    except PegError:
        pass
    stream.update(RULE_error_option, position, None)
    return None


def rule_word(stream, position):
    if stream.hasResult(RULE_word, position):
        return stream.result(RULE_word, position)
    try:
        result_peg_87 = Result(position)
        result_peg_88 = Result(result_peg_87.getPosition())
        
        try:
            while True:
                result_peg_89 = Result(result_peg_88.getPosition());
                # print "Trying rule " + 'any_char'
                result_peg_89 = rule_any_char(stream, result_peg_89.getPosition())
                if result_peg_89 == None:
                    raise PegError
                result_peg_88.addResult(result_peg_89);
        except PegError:
            if result_peg_88.matches() == 0:
                raise PegError
                
        result_peg_87.addResult(result_peg_88);
        
        result_peg_90 = Result(result_peg_87.getPosition())
        
        value = None
        values = result_peg_87.getValues()
        # print "all start symbol values " + str(values)
        # print "values[0] " + str(values[0])
        value = ''.join(values[0]).replace('-', '__')
        # print "got word " + value
        result_peg_90.setValue(value)
        
        result_peg_87.addResult(result_peg_90);
        
        result_peg_87.setValue(result_peg_87.getLastValue())
        stream.update(RULE_word, position, result_peg_87)
        return result_peg_87
    except PegError:
        pass
    stream.update(RULE_word, position, None)
    return None


def rule_rules(stream, position):
    if stream.hasResult(RULE_rules, position):
        return stream.result(RULE_rules, position)
    try:
        result_peg_91 = Result(position)
        result_peg_92 = Result(result_peg_91.getPosition())
        
        if 'rules:' == stream.get(result_peg_92.getPosition(), 6):
            result_peg_92.nextPosition(6)
            result_peg_92.setValue('rules:')
        else:
            raise PegError
        
        result_peg_91.addResult(result_peg_92);
        
        result_peg_93 = Result(result_peg_91.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_93 = rule_whitespace(stream, result_peg_93.getPosition())
        if result_peg_93 == None:
            raise PegError
        
        result_peg_91.addResult(result_peg_93);
        
        result_peg_94 = Result(result_peg_91.getPosition())
        
        try:
            while True:
                result_peg_95 = Result(result_peg_94.getPosition());
                result_peg_96 = Result(result_peg_95.getPosition())
                
                # print "Trying rule " + 'rule'
                result_peg_96 = rule_rule(stream, result_peg_96.getPosition())
                if result_peg_96 == None:
                    raise PegError
                
                result_peg_95.addResult(result_peg_96);
                
                result_peg_97 = Result(result_peg_95.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_97 = rule_whitespace(stream, result_peg_97.getPosition())
                if result_peg_97 == None:
                    raise PegError
                
                result_peg_95.addResult(result_peg_97);
                
                result_peg_98 = Result(result_peg_95.getPosition())
                
                value = None
                values = result_peg_95.getValues()
                value = values[0]
                result_peg_98.setValue(value)
                
                result_peg_95.addResult(result_peg_98);
                
                result_peg_95.setValue(result_peg_95.getLastValue())
                result_peg_94.addResult(result_peg_95);
        except PegError:
            pass
        rules = result_peg_94.getValues()
        
        result_peg_91.addResult(result_peg_94);
        
        result_peg_99 = Result(result_peg_91.getPosition())
        
        value = None
        values = result_peg_91.getValues()
        value = rules
        result_peg_99.setValue(value)
        
        result_peg_91.addResult(result_peg_99);
        
        result_peg_91.setValue(result_peg_91.getLastValue())
        stream.update(RULE_rules, position, result_peg_91)
        return result_peg_91
    except PegError:
        pass
    stream.update(RULE_rules, position, None)
    return None


def rule_rule(stream, position):
    if stream.hasResult(RULE_rule, position):
        return stream.result(RULE_rule, position)
    try:
        result_peg_100 = Result(position)
        result_peg_101 = Result(result_peg_100.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_101 = rule_spaces(stream, result_peg_101.getPosition())
        if result_peg_101 == None:
            raise PegError
        
        result_peg_100.addResult(result_peg_101);
        
        result_peg_102 = Result(result_peg_100.getPosition())
        
        save_peg_103 = result_peg_102.getPosition()
        
        if 'inline' == stream.get(result_peg_102.getPosition(), 6):
            result_peg_102.nextPosition(6)
            result_peg_102.setValue('inline')
        else:
            
            result_peg_102 = Result(save_peg_103)
            result_peg_102.setValue(None)
        inline = result_peg_102.getValues()
        
        result_peg_100.addResult(result_peg_102);
        
        result_peg_104 = Result(result_peg_100.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_104 = rule_spaces(stream, result_peg_104.getPosition())
        if result_peg_104 == None:
            raise PegError
        
        result_peg_100.addResult(result_peg_104);
        
        result_peg_105 = Result(result_peg_100.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_105 = rule_word(stream, result_peg_105.getPosition())
        if result_peg_105 == None:
            raise PegError
        name = result_peg_105.getValues()
        
        result_peg_100.addResult(result_peg_105);
        
        result_peg_106 = Result(result_peg_100.getPosition())
        
        save_peg_107 = result_peg_106.getPosition()
        
        # print "Trying rule " + 'rule_parameters'
        result_peg_106 = rule_rule_parameters(stream, result_peg_106.getPosition())
        if result_peg_106 == None:
            
            result_peg_106 = Result(save_peg_107)
            result_peg_106.setValue(None)
        rule_parameters = result_peg_106.getValues()
        
        result_peg_100.addResult(result_peg_106);
        
        result_peg_108 = Result(result_peg_100.getPosition())
        
        save_peg_109 = result_peg_108.getPosition()
        
        # print "Trying rule " + 'value_parameters'
        result_peg_108 = rule_value_parameters(stream, result_peg_108.getPosition())
        if result_peg_108 == None:
            
            result_peg_108 = Result(save_peg_109)
            result_peg_108.setValue(None)
        parameters = result_peg_108.getValues()
        
        result_peg_100.addResult(result_peg_108);
        
        result_peg_110 = Result(result_peg_100.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_110 = rule_spaces(stream, result_peg_110.getPosition())
        if result_peg_110 == None:
            raise PegError
        
        result_peg_100.addResult(result_peg_110);
        
        result_peg_111 = Result(result_peg_100.getPosition())
        
        if '=' == stream.get(result_peg_111.getPosition(), 1):
            result_peg_111.nextPosition(1)
            result_peg_111.setValue('=')
        else:
            raise PegError
        
        result_peg_100.addResult(result_peg_111);
        
        result_peg_112 = Result(result_peg_100.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_112 = rule_spaces(stream, result_peg_112.getPosition())
        if result_peg_112 == None:
            raise PegError
        
        result_peg_100.addResult(result_peg_112);
        
        result_peg_113 = Result(result_peg_100.getPosition())
        
        # print "Trying rule " + 'pattern_line'
        result_peg_113 = rule_pattern_line(stream, result_peg_113.getPosition())
        if result_peg_113 == None:
            raise PegError
        pattern1 = result_peg_113.getValues()
        
        result_peg_100.addResult(result_peg_113);
        
        result_peg_114 = Result(result_peg_100.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_114 = rule_whitespace(stream, result_peg_114.getPosition())
        if result_peg_114 == None:
            raise PegError
        
        result_peg_100.addResult(result_peg_114);
        
        result_peg_115 = Result(result_peg_100.getPosition())
        
        try:
            while True:
                result_peg_116 = Result(result_peg_115.getPosition());
                result_peg_117 = Result(result_peg_116.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_117 = rule_spaces(stream, result_peg_117.getPosition())
                if result_peg_117 == None:
                    raise PegError
                
                result_peg_116.addResult(result_peg_117);
                
                result_peg_118 = Result(result_peg_116.getPosition())
                
                if '|' == stream.get(result_peg_118.getPosition(), 1):
                    result_peg_118.nextPosition(1)
                    result_peg_118.setValue('|')
                else:
                    raise PegError
                
                result_peg_116.addResult(result_peg_118);
                
                result_peg_119 = Result(result_peg_116.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_119 = rule_spaces(stream, result_peg_119.getPosition())
                if result_peg_119 == None:
                    raise PegError
                
                result_peg_116.addResult(result_peg_119);
                
                result_peg_120 = Result(result_peg_116.getPosition())
                
                # print "Trying rule " + 'pattern_line'
                result_peg_120 = rule_pattern_line(stream, result_peg_120.getPosition())
                if result_peg_120 == None:
                    raise PegError
                pattern = result_peg_120.getValues()
                
                result_peg_116.addResult(result_peg_120);
                
                result_peg_121 = Result(result_peg_116.getPosition())
                
                # print "Trying rule " + 'whitespace'
                result_peg_121 = rule_whitespace(stream, result_peg_121.getPosition())
                if result_peg_121 == None:
                    raise PegError
                
                result_peg_116.addResult(result_peg_121);
                
                result_peg_122 = Result(result_peg_116.getPosition())
                
                value = None
                values = result_peg_116.getValues()
                value = pattern
                result_peg_122.setValue(value)
                
                result_peg_116.addResult(result_peg_122);
                
                result_peg_116.setValue(result_peg_116.getLastValue())
                result_peg_115.addResult(result_peg_116);
        except PegError:
            pass
        patterns = result_peg_115.getValues()
        
        result_peg_100.addResult(result_peg_115);
        
        result_peg_123 = Result(result_peg_100.getPosition())
        
        save_peg_124 = result_peg_123.getPosition()
        
        # print "Trying rule " + 'failure'
        result_peg_123 = rule_failure(stream, result_peg_123.getPosition())
        if result_peg_123 == None:
            
            result_peg_123 = Result(save_peg_124)
            result_peg_123.setValue(None)
        fail = result_peg_123.getValues()
        
        result_peg_100.addResult(result_peg_123);
        
        result_peg_125 = Result(result_peg_100.getPosition())
        
        value = None
        values = result_peg_100.getValues()
        value = peg.Rule(name, [pattern1] + patterns, inline = (inline != None), rules = rule_parameters, parameters = parameters, fail = fail)
        result_peg_125.setValue(value)
        
        result_peg_100.addResult(result_peg_125);
        
        result_peg_100.setValue(result_peg_100.getLastValue())
        stream.update(RULE_rule, position, result_peg_100)
        return result_peg_100
    except PegError:
        pass
    stream.update(RULE_rule, position, None)
    return None


def rule_pattern_line(stream, position):
    if stream.hasResult(RULE_pattern_line, position):
        return stream.result(RULE_pattern_line, position)
    try:
        result_peg_126 = Result(position)
        result_peg_127 = Result(result_peg_126.getPosition())
        
        try:
            while True:
                result_peg_128 = Result(result_peg_127.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_128 = rule_pattern(stream, result_peg_128.getPosition())
                if result_peg_128 == None:
                    raise PegError
                result_peg_127.addResult(result_peg_128);
        except PegError:
            pass
        patterns = result_peg_127.getValues()
        
        result_peg_126.addResult(result_peg_127);
        
        result_peg_129 = Result(result_peg_126.getPosition())
        
        value = None
        values = result_peg_126.getValues()
        value = peg.PatternSequence(patterns)
        #if code != None:
        #    value = code(peg.PatternSequence(patterns))
        #else:
        #    value = peg.PatternAction(peg.PatternSequence(patterns), "value = values;")
        result_peg_129.setValue(value)
        
        result_peg_126.addResult(result_peg_129);
        
        result_peg_126.setValue(result_peg_126.getLastValue())
        stream.update(RULE_pattern_line, position, result_peg_126)
        return result_peg_126
    except PegError:
        pass
    stream.update(RULE_pattern_line, position, None)
    return None


def rule_pattern(stream, position):
    if stream.hasResult(RULE_pattern, position):
        return stream.result(RULE_pattern, position)
    try:
        result_peg_130 = Result(position)
        result_peg_131 = Result(result_peg_130.getPosition())
        
        save_peg_132 = result_peg_131.getPosition()
        
        # print "Trying rule " + 'bind'
        result_peg_131 = rule_bind(stream, result_peg_131.getPosition())
        if result_peg_131 == None:
            
            result_peg_131 = Result(save_peg_132)
            result_peg_131.setValue(None)
        bind = result_peg_131.getValues()
        
        result_peg_130.addResult(result_peg_131);
        
        result_peg_133 = Result(result_peg_130.getPosition())
        
        # print "Trying rule " + 'item'
        result_peg_133 = rule_item(stream, result_peg_133.getPosition())
        if result_peg_133 == None:
            raise PegError
        item = result_peg_133.getValues()
        
        result_peg_130.addResult(result_peg_133);
        
        result_peg_134 = Result(result_peg_130.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_134 = rule_spaces(stream, result_peg_134.getPosition())
        if result_peg_134 == None:
            raise PegError
        
        result_peg_130.addResult(result_peg_134);
        
        result_peg_135 = Result(result_peg_130.getPosition())
        
        value = None
        values = result_peg_130.getValues()
        # value = peg.PatternRule(values[0])
        if bind != None:
            item = bind(item)
        value = item
        # print "Pattern is " + str(value)
        result_peg_135.setValue(value)
        
        result_peg_130.addResult(result_peg_135);
        
        result_peg_130.setValue(result_peg_130.getLastValue())
        stream.update(RULE_pattern, position, result_peg_130)
        return result_peg_130
    except PegError:
        pass
    stream.update(RULE_pattern, position, None)
    return None


def rule_raw_code(stream, position):
    if stream.hasResult(RULE_raw_code, position):
        return stream.result(RULE_raw_code, position)
    try:
        result_peg_136 = Result(position)
        result_peg_137 = Result(result_peg_136.getPosition())
        
        if '(' == stream.get(result_peg_137.getPosition(), 1):
            result_peg_137.nextPosition(1)
            result_peg_137.setValue('(')
        else:
            raise PegError
        
        result_peg_136.addResult(result_peg_137);
        
        result_peg_138 = Result(result_peg_136.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_138 = rule_spaces(stream, result_peg_138.getPosition())
        if result_peg_138 == None:
            raise PegError
        
        result_peg_136.addResult(result_peg_138);
        
        result_peg_139 = Result(result_peg_136.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_139 = rule_code(stream, result_peg_139.getPosition())
        if result_peg_139 == None:
            raise PegError
        code = result_peg_139.getValues()
        
        result_peg_136.addResult(result_peg_139);
        
        result_peg_140 = Result(result_peg_136.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_140 = rule_spaces(stream, result_peg_140.getPosition())
        if result_peg_140 == None:
            raise PegError
        
        result_peg_136.addResult(result_peg_140);
        
        result_peg_141 = Result(result_peg_136.getPosition())
        
        if ')' == stream.get(result_peg_141.getPosition(), 1):
            result_peg_141.nextPosition(1)
            result_peg_141.setValue(')')
        else:
            raise PegError
        
        result_peg_136.addResult(result_peg_141);
        
        result_peg_142 = Result(result_peg_136.getPosition())
        
        value = None
        values = result_peg_136.getValues()
        value = code.code
        result_peg_142.setValue(value)
        
        result_peg_136.addResult(result_peg_142);
        
        result_peg_136.setValue(result_peg_136.getLastValue())
        stream.update(RULE_raw_code, position, result_peg_136)
        return result_peg_136
    except PegError:
        pass
    stream.update(RULE_raw_code, position, None)
    return None


def rule_code(stream, position):
    if stream.hasResult(RULE_code, position):
        return stream.result(RULE_code, position)
    try:
        result_peg_143 = Result(position)
        result_peg_144 = Result(result_peg_143.getPosition())
        
        if '{{' == stream.get(result_peg_144.getPosition(), 2):
            result_peg_144.nextPosition(2)
            result_peg_144.setValue('{{')
        else:
            raise PegError
        
        result_peg_143.addResult(result_peg_144);
        
        result_peg_145 = Result(result_peg_143.getPosition())
        
        try:
            while True:
                result_peg_146 = Result(result_peg_145.getPosition());
                result_peg_147 = Result(result_peg_146.getPosition())
                
                result_peg_148 = Result(result_peg_147.getPosition());
                try:
                    if '}}' == stream.get(result_peg_148.getPosition(), 2):
                        result_peg_148.nextPosition(2)
                        result_peg_148.setValue('}}')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_147.setValue(None)
                        
                result_peg_146.addResult(result_peg_147);
                
                result_peg_149 = Result(result_peg_146.getPosition())
                
                temp_peg_150 = stream.get(result_peg_149.getPosition())
                if temp_peg_150 != chr(0):
                    result_peg_149.setValue(temp_peg_150)
                    result_peg_149.nextPosition()
                else:
                    raise PegError
                
                result_peg_146.addResult(result_peg_149);
                
                result_peg_151 = Result(result_peg_146.getPosition())
                
                value = None
                values = result_peg_146.getValues()
                value = values[1]
                result_peg_151.setValue(value)
                
                result_peg_146.addResult(result_peg_151);
                
                result_peg_146.setValue(result_peg_146.getLastValue())
                result_peg_145.addResult(result_peg_146);
        except PegError:
            if result_peg_145.matches() == 0:
                raise PegError
                
        result_peg_143.addResult(result_peg_145);
        
        result_peg_152 = Result(result_peg_143.getPosition())
        
        if '}}' == stream.get(result_peg_152.getPosition(), 2):
            result_peg_152.nextPosition(2)
            result_peg_152.setValue('}}')
        else:
            raise PegError
        
        result_peg_143.addResult(result_peg_152);
        
        result_peg_153 = Result(result_peg_143.getPosition())
        
        value = None
        values = result_peg_143.getValues()
        value = peg.PatternCode(''.join(values[1]))
        result_peg_153.setValue(value)
        
        result_peg_143.addResult(result_peg_153);
        
        result_peg_143.setValue(result_peg_143.getLastValue())
        stream.update(RULE_code, position, result_peg_143)
        return result_peg_143
    except PegError:
        pass
    stream.update(RULE_code, position, None)
    return None


def rule_item(stream, position):
    if stream.hasResult(RULE_item, position):
        return stream.result(RULE_item, position)
    try:
        result_peg_154 = Result(position)
        result_peg_155 = Result(result_peg_154.getPosition())
        
        save_peg_156 = result_peg_155.getPosition()
        
        if '&' == stream.get(result_peg_155.getPosition(), 1):
            result_peg_155.nextPosition(1)
            result_peg_155.setValue('&')
        else:
            
            result_peg_155 = Result(save_peg_156)
            result_peg_155.setValue(None)
        ensure = result_peg_155.getValues()
        
        result_peg_154.addResult(result_peg_155);
        
        result_peg_157 = Result(result_peg_154.getPosition())
        
        save_peg_158 = result_peg_157.getPosition()
        
        if '!' == stream.get(result_peg_157.getPosition(), 1):
            result_peg_157.nextPosition(1)
            result_peg_157.setValue('!')
        else:
            
            result_peg_157 = Result(save_peg_158)
            result_peg_157.setValue(None)
        pnot = result_peg_157.getValues()
        
        result_peg_154.addResult(result_peg_157);
        
        result_peg_159 = Result(result_peg_154.getPosition())
        
        save_peg_160 = result_peg_159.getPosition()
        
        result_peg_159 = Result(save_peg_160)
        # print "Trying rule " + 'x_word'
        result_peg_159 = rule_x_word(stream, result_peg_159.getPosition())
        if result_peg_159 == None:
            
            result_peg_159 = Result(save_peg_160)
            # print "Trying rule " + 'any'
            result_peg_159 = rule_any(stream, result_peg_159.getPosition())
            if result_peg_159 == None:
                
                result_peg_159 = Result(save_peg_160)
                # print "Trying rule " + 'eof'
                result_peg_159 = rule_eof(stream, result_peg_159.getPosition())
                if result_peg_159 == None:
                    
                    result_peg_159 = Result(save_peg_160)
                    # print "Trying rule " + 'void'
                    result_peg_159 = rule_void(stream, result_peg_159.getPosition())
                    if result_peg_159 == None:
                        
                        result_peg_159 = Result(save_peg_160)
                        # print "Trying rule " + 'range'
                        result_peg_159 = rule_range(stream, result_peg_159.getPosition())
                        if result_peg_159 == None:
                            
                            result_peg_159 = Result(save_peg_160)
                            # print "Trying rule " + 'string'
                            result_peg_159 = rule_string(stream, result_peg_159.getPosition())
                            if result_peg_159 == None:
                                
                                result_peg_159 = Result(save_peg_160)
                                # print "Trying rule " + 'line'
                                result_peg_159 = rule_line(stream, result_peg_159.getPosition())
                                if result_peg_159 == None:
                                    
                                    result_peg_159 = Result(save_peg_160)
                                    # print "Trying rule " + 'ascii'
                                    result_peg_159 = rule_ascii(stream, result_peg_159.getPosition())
                                    if result_peg_159 == None:
                                        
                                        result_peg_159 = Result(save_peg_160)
                                        # print "Trying rule " + 'utf8'
                                        result_peg_159 = rule_utf8(stream, result_peg_159.getPosition())
                                        if result_peg_159 == None:
                                            
                                            result_peg_159 = Result(save_peg_160)
                                            # print "Trying rule " + 'predicate'
                                            result_peg_159 = rule_predicate(stream, result_peg_159.getPosition())
                                            if result_peg_159 == None:
                                                
                                                result_peg_159 = Result(save_peg_160)
                                                # print "Trying rule " + 'call_rule'
                                                result_peg_159 = rule_call_rule(stream, result_peg_159.getPosition())
                                                if result_peg_159 == None:
                                                    
                                                    result_peg_159 = Result(save_peg_160)
                                                    # print "Trying rule " + 'sub_pattern'
                                                    result_peg_159 = rule_sub_pattern(stream, result_peg_159.getPosition())
                                                    if result_peg_159 == None:
                                                        
                                                        result_peg_159 = Result(save_peg_160)
                                                        # print "Trying rule " + 'code'
                                                        result_peg_159 = rule_code(stream, result_peg_159.getPosition())
                                                        if result_peg_159 == None:
                                                            raise PegError
        pattern = result_peg_159.getValues()
        
        result_peg_154.addResult(result_peg_159);
        
        result_peg_161 = Result(result_peg_154.getPosition())
        
        save_peg_162 = result_peg_161.getPosition()
        
        # print "Trying rule " + 'modifier'
        result_peg_161 = rule_modifier(stream, result_peg_161.getPosition())
        if result_peg_161 == None:
            
            result_peg_161 = Result(save_peg_162)
            result_peg_161.setValue(None)
        modifier = result_peg_161.getValues()
        
        result_peg_154.addResult(result_peg_161);
        
        result_peg_163 = Result(result_peg_154.getPosition())
        
        value = None
        values = result_peg_154.getValues()
        if modifier != None:
            pattern = modifier(pattern)
        if pnot != None:
            pattern = peg.PatternNot(pattern)
        if ensure != None:
            pattern = peg.PatternEnsure(pattern)
        value = pattern
        result_peg_163.setValue(value)
        
        result_peg_154.addResult(result_peg_163);
        
        result_peg_154.setValue(result_peg_154.getLastValue())
        stream.update(RULE_item, position, result_peg_154)
        return result_peg_154
    except PegError:
        pass
    stream.update(RULE_item, position, None)
    return None


def rule_failure(stream, position):
    if stream.hasResult(RULE_failure, position):
        return stream.result(RULE_failure, position)
    try:
        result_peg_164 = Result(position)
        result_peg_165 = Result(result_peg_164.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_165 = rule_whitespace(stream, result_peg_165.getPosition())
        if result_peg_165 == None:
            raise PegError
        
        result_peg_164.addResult(result_peg_165);
        
        result_peg_166 = Result(result_peg_164.getPosition())
        
        if '<fail>' == stream.get(result_peg_166.getPosition(), 6):
            result_peg_166.nextPosition(6)
            result_peg_166.setValue('<fail>')
        else:
            raise PegError
        
        result_peg_164.addResult(result_peg_166);
        
        result_peg_167 = Result(result_peg_164.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_167 = rule_spaces(stream, result_peg_167.getPosition())
        if result_peg_167 == None:
            raise PegError
        
        result_peg_164.addResult(result_peg_167);
        
        result_peg_168 = Result(result_peg_164.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_168 = rule_code(stream, result_peg_168.getPosition())
        if result_peg_168 == None:
            raise PegError
        code = result_peg_168.getValues()
        
        result_peg_164.addResult(result_peg_168);
        
        result_peg_169 = Result(result_peg_164.getPosition())
        
        value = None
        values = result_peg_164.getValues()
        value = code.code
        result_peg_169.setValue(value)
        
        result_peg_164.addResult(result_peg_169);
        
        result_peg_164.setValue(result_peg_164.getLastValue())
        stream.update(RULE_failure, position, result_peg_164)
        return result_peg_164
    except PegError:
        pass
    stream.update(RULE_failure, position, None)
    return None


def rule_line(stream, position):
    if stream.hasResult(RULE_line, position):
        return stream.result(RULE_line, position)
    try:
        result_peg_170 = Result(position)
        result_peg_171 = Result(result_peg_170.getPosition())
        
        if '<line>' == stream.get(result_peg_171.getPosition(), 6):
            result_peg_171.nextPosition(6)
            result_peg_171.setValue('<line>')
        else:
            raise PegError
        
        result_peg_170.addResult(result_peg_171);
        
        result_peg_172 = Result(result_peg_170.getPosition())
        
        value = None
        values = result_peg_170.getValues()
        value = peg.PatternLine()
        result_peg_172.setValue(value)
        
        result_peg_170.addResult(result_peg_172);
        
        result_peg_170.setValue(result_peg_170.getLastValue())
        stream.update(RULE_line, position, result_peg_170)
        return result_peg_170
    except PegError:
        pass
    stream.update(RULE_line, position, None)
    return None


def rule_predicate(stream, position):
    if stream.hasResult(RULE_predicate, position):
        return stream.result(RULE_predicate, position)
    try:
        result_peg_173 = Result(position)
        result_peg_174 = Result(result_peg_173.getPosition())
        
        if '<predicate' == stream.get(result_peg_174.getPosition(), 10):
            result_peg_174.nextPosition(10)
            result_peg_174.setValue('<predicate')
        else:
            raise PegError
        
        result_peg_173.addResult(result_peg_174);
        
        result_peg_175 = Result(result_peg_173.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_175 = rule_whitespace(stream, result_peg_175.getPosition())
        if result_peg_175 == None:
            raise PegError
        
        result_peg_173.addResult(result_peg_175);
        
        result_peg_176 = Result(result_peg_173.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_176 = rule_word(stream, result_peg_176.getPosition())
        if result_peg_176 == None:
            raise PegError
        variable = result_peg_176.getValues()
        
        result_peg_173.addResult(result_peg_176);
        
        result_peg_177 = Result(result_peg_173.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_177 = rule_whitespace(stream, result_peg_177.getPosition())
        if result_peg_177 == None:
            raise PegError
        
        result_peg_173.addResult(result_peg_177);
        
        result_peg_178 = Result(result_peg_173.getPosition())
        
        if '>' == stream.get(result_peg_178.getPosition(), 1):
            result_peg_178.nextPosition(1)
            result_peg_178.setValue('>')
        else:
            raise PegError
        
        result_peg_173.addResult(result_peg_178);
        
        result_peg_179 = Result(result_peg_173.getPosition())
        
        # print "Trying rule " + 'whitespace'
        result_peg_179 = rule_whitespace(stream, result_peg_179.getPosition())
        if result_peg_179 == None:
            raise PegError
        
        result_peg_173.addResult(result_peg_179);
        
        result_peg_180 = Result(result_peg_173.getPosition())
        
        # print "Trying rule " + 'code'
        result_peg_180 = rule_code(stream, result_peg_180.getPosition())
        if result_peg_180 == None:
            raise PegError
        code = result_peg_180.getValues()
        
        result_peg_173.addResult(result_peg_180);
        
        result_peg_181 = Result(result_peg_173.getPosition())
        
        value = None
        values = result_peg_173.getValues()
        value = peg.PatternPredicate(variable, code.code)
        result_peg_181.setValue(value)
        
        result_peg_173.addResult(result_peg_181);
        
        result_peg_173.setValue(result_peg_173.getLastValue())
        stream.update(RULE_predicate, position, result_peg_173)
        return result_peg_173
    except PegError:
        pass
    stream.update(RULE_predicate, position, None)
    return None


def rule_utf8(stream, position):
    if stream.hasResult(RULE_utf8, position):
        return stream.result(RULE_utf8, position)
    try:
        result_peg_182 = Result(position)
        result_peg_183 = Result(result_peg_182.getPosition())
        
        if '<utf8' == stream.get(result_peg_183.getPosition(), 5):
            result_peg_183.nextPosition(5)
            result_peg_183.setValue('<utf8')
        else:
            raise PegError
        
        result_peg_182.addResult(result_peg_183);
        
        result_peg_184 = Result(result_peg_182.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_184 = rule_spaces(stream, result_peg_184.getPosition())
        if result_peg_184 == None:
            raise PegError
        
        result_peg_182.addResult(result_peg_184);
        
        result_peg_185 = Result(result_peg_182.getPosition())
        
        # print "Trying rule " + 'hex_number'
        result_peg_185 = rule_hex_number(stream, result_peg_185.getPosition())
        if result_peg_185 == None:
            raise PegError
        num = result_peg_185.getValues()
        
        result_peg_182.addResult(result_peg_185);
        
        result_peg_186 = Result(result_peg_182.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_186 = rule_spaces(stream, result_peg_186.getPosition())
        if result_peg_186 == None:
            raise PegError
        
        result_peg_182.addResult(result_peg_186);
        
        result_peg_187 = Result(result_peg_182.getPosition())
        
        if '>' == stream.get(result_peg_187.getPosition(), 1):
            result_peg_187.nextPosition(1)
            result_peg_187.setValue('>')
        else:
            raise PegError
        
        result_peg_182.addResult(result_peg_187);
        
        result_peg_188 = Result(result_peg_182.getPosition())
        
        value = None
        values = result_peg_182.getValues()
        value = peg.createUtf8Pattern(num)
        result_peg_188.setValue(value)
        
        result_peg_182.addResult(result_peg_188);
        
        result_peg_182.setValue(result_peg_182.getLastValue())
        stream.update(RULE_utf8, position, result_peg_182)
        return result_peg_182
    except PegError:
        pass
    stream.update(RULE_utf8, position, None)
    return None


def rule_ascii(stream, position):
    if stream.hasResult(RULE_ascii, position):
        return stream.result(RULE_ascii, position)
    try:
        result_peg_189 = Result(position)
        result_peg_190 = Result(result_peg_189.getPosition())
        
        if '<ascii' == stream.get(result_peg_190.getPosition(), 6):
            result_peg_190.nextPosition(6)
            result_peg_190.setValue('<ascii')
        else:
            raise PegError
        
        result_peg_189.addResult(result_peg_190);
        
        result_peg_191 = Result(result_peg_189.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_191 = rule_spaces(stream, result_peg_191.getPosition())
        if result_peg_191 == None:
            raise PegError
        
        result_peg_189.addResult(result_peg_191);
        
        result_peg_192 = Result(result_peg_189.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_192 = rule_number(stream, result_peg_192.getPosition())
        if result_peg_192 == None:
            raise PegError
        num = result_peg_192.getValues()
        
        result_peg_189.addResult(result_peg_192);
        
        result_peg_193 = Result(result_peg_189.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_193 = rule_spaces(stream, result_peg_193.getPosition())
        if result_peg_193 == None:
            raise PegError
        
        result_peg_189.addResult(result_peg_193);
        
        result_peg_194 = Result(result_peg_189.getPosition())
        
        if '>' == stream.get(result_peg_194.getPosition(), 1):
            result_peg_194.nextPosition(1)
            result_peg_194.setValue('>')
        else:
            raise PegError
        
        result_peg_189.addResult(result_peg_194);
        
        result_peg_195 = Result(result_peg_189.getPosition())
        
        value = None
        values = result_peg_189.getValues()
        value = peg.PatternVerbatim(int(num))
        result_peg_195.setValue(value)
        
        result_peg_189.addResult(result_peg_195);
        
        result_peg_189.setValue(result_peg_189.getLastValue())
        stream.update(RULE_ascii, position, result_peg_189)
        return result_peg_189
    except PegError:
        pass
    stream.update(RULE_ascii, position, None)
    return None


def rule_call_rule(stream, position):
    if stream.hasResult(RULE_call_rule, position):
        return stream.result(RULE_call_rule, position)
    try:
        result_peg_196 = Result(position)
        result_peg_197 = Result(result_peg_196.getPosition())
        
        if '@' == stream.get(result_peg_197.getPosition(), 1):
            result_peg_197.nextPosition(1)
            result_peg_197.setValue('@')
        else:
            raise PegError
        
        result_peg_196.addResult(result_peg_197);
        
        result_peg_198 = Result(result_peg_196.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_198 = rule_word(stream, result_peg_198.getPosition())
        if result_peg_198 == None:
            raise PegError
        name = result_peg_198.getValues()
        
        result_peg_196.addResult(result_peg_198);
        
        result_peg_199 = Result(result_peg_196.getPosition())
        
        save_peg_200 = result_peg_199.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_199 = rule_parameters_rules(stream, result_peg_199.getPosition())
        if result_peg_199 == None:
            
            result_peg_199 = Result(save_peg_200)
            result_peg_199.setValue(None)
        rule_parameters = result_peg_199.getValues()
        
        result_peg_196.addResult(result_peg_199);
        
        result_peg_201 = Result(result_peg_196.getPosition())
        
        save_peg_202 = result_peg_201.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_201 = rule_parameters_values(stream, result_peg_201.getPosition())
        if result_peg_201 == None:
            
            result_peg_201 = Result(save_peg_202)
            result_peg_201.setValue(None)
        parameters = result_peg_201.getValues()
        
        result_peg_196.addResult(result_peg_201);
        
        result_peg_203 = Result(result_peg_196.getPosition())
        
        value = None
        values = result_peg_196.getValues()
        value = peg.PatternCallRule(name, rule_parameters, parameters)
        result_peg_203.setValue(value)
        
        result_peg_196.addResult(result_peg_203);
        
        result_peg_196.setValue(result_peg_196.getLastValue())
        stream.update(RULE_call_rule, position, result_peg_196)
        return result_peg_196
    except PegError:
        pass
    stream.update(RULE_call_rule, position, None)
    return None


def rule_eof(stream, position):
    if stream.hasResult(RULE_eof, position):
        return stream.result(RULE_eof, position)
    try:
        result_peg_204 = Result(position)
        result_peg_205 = Result(result_peg_204.getPosition())
        
        if '<eof>' == stream.get(result_peg_205.getPosition(), 5):
            result_peg_205.nextPosition(5)
            result_peg_205.setValue('<eof>')
        else:
            raise PegError
        
        result_peg_204.addResult(result_peg_205);
        
        result_peg_206 = Result(result_peg_204.getPosition())
        
        value = None
        values = result_peg_204.getValues()
        value = peg.PatternEof()
        result_peg_206.setValue(value)
        
        result_peg_204.addResult(result_peg_206);
        
        result_peg_204.setValue(result_peg_204.getLastValue())
        stream.update(RULE_eof, position, result_peg_204)
        return result_peg_204
    except PegError:
        pass
    stream.update(RULE_eof, position, None)
    return None


def rule_void(stream, position):
    if stream.hasResult(RULE_void, position):
        return stream.result(RULE_void, position)
    try:
        result_peg_207 = Result(position)
        result_peg_208 = Result(result_peg_207.getPosition())
        
        if '<void>' == stream.get(result_peg_208.getPosition(), 6):
            result_peg_208.nextPosition(6)
            result_peg_208.setValue('<void>')
        else:
            raise PegError
        
        result_peg_207.addResult(result_peg_208);
        
        result_peg_209 = Result(result_peg_207.getPosition())
        
        value = None
        values = result_peg_207.getValues()
        value = peg.PatternVoid()
        result_peg_209.setValue(value)
        
        result_peg_207.addResult(result_peg_209);
        
        result_peg_207.setValue(result_peg_207.getLastValue())
        stream.update(RULE_void, position, result_peg_207)
        return result_peg_207
    except PegError:
        pass
    stream.update(RULE_void, position, None)
    return None


def rule_range(stream, position):
    if stream.hasResult(RULE_range, position):
        return stream.result(RULE_range, position)
    try:
        result_peg_210 = Result(position)
        result_peg_211 = Result(result_peg_210.getPosition())
        
        if '[' == stream.get(result_peg_211.getPosition(), 1):
            result_peg_211.nextPosition(1)
            result_peg_211.setValue('[')
        else:
            raise PegError
        
        result_peg_210.addResult(result_peg_211);
        
        result_peg_212 = Result(result_peg_210.getPosition())
        
        try:
            while True:
                result_peg_213 = Result(result_peg_212.getPosition());
                result_peg_214 = Result(result_peg_213.getPosition())
                
                result_peg_215 = Result(result_peg_214.getPosition());
                try:
                    if ']' == stream.get(result_peg_215.getPosition(), 1):
                        result_peg_215.nextPosition(1)
                        result_peg_215.setValue(']')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_214.setValue(None)
                        
                result_peg_213.addResult(result_peg_214);
                
                result_peg_216 = Result(result_peg_213.getPosition())
                
                temp_peg_217 = stream.get(result_peg_216.getPosition())
                if temp_peg_217 != chr(0):
                    result_peg_216.setValue(temp_peg_217)
                    result_peg_216.nextPosition()
                else:
                    raise PegError
                
                result_peg_213.addResult(result_peg_216);
                
                result_peg_218 = Result(result_peg_213.getPosition())
                
                value = None
                values = result_peg_213.getValues()
                value = values[1]
                result_peg_218.setValue(value)
                
                result_peg_213.addResult(result_peg_218);
                
                result_peg_213.setValue(result_peg_213.getLastValue())
                result_peg_212.addResult(result_peg_213);
        except PegError:
            pass
                
        result_peg_210.addResult(result_peg_212);
        
        result_peg_219 = Result(result_peg_210.getPosition())
        
        if ']' == stream.get(result_peg_219.getPosition(), 1):
            result_peg_219.nextPosition(1)
            result_peg_219.setValue(']')
        else:
            raise PegError
        
        result_peg_210.addResult(result_peg_219);
        
        result_peg_220 = Result(result_peg_210.getPosition())
        
        value = None
        values = result_peg_210.getValues()
        value = peg.PatternRange(''.join(values[1]))
        result_peg_220.setValue(value)
        
        result_peg_210.addResult(result_peg_220);
        
        result_peg_210.setValue(result_peg_210.getLastValue())
        stream.update(RULE_range, position, result_peg_210)
        return result_peg_210
    except PegError:
        pass
    stream.update(RULE_range, position, None)
    return None


def rule_sub_pattern(stream, position):
    if stream.hasResult(RULE_sub_pattern, position):
        return stream.result(RULE_sub_pattern, position)
    try:
        result_peg_221 = Result(position)
        result_peg_222 = Result(result_peg_221.getPosition())
        
        if '(' == stream.get(result_peg_222.getPosition(), 1):
            result_peg_222.nextPosition(1)
            result_peg_222.setValue('(')
        else:
            raise PegError
        
        result_peg_221.addResult(result_peg_222);
        
        result_peg_223 = Result(result_peg_221.getPosition())
        
        try:
            while True:
                result_peg_224 = Result(result_peg_223.getPosition());
                # print "Trying rule " + 'pattern'
                result_peg_224 = rule_pattern(stream, result_peg_224.getPosition())
                if result_peg_224 == None:
                    raise PegError
                result_peg_223.addResult(result_peg_224);
        except PegError:
            if result_peg_223.matches() == 0:
                raise PegError
                
        result_peg_221.addResult(result_peg_223);
        
        result_peg_225 = Result(result_peg_221.getPosition())
        
        if ')' == stream.get(result_peg_225.getPosition(), 1):
            result_peg_225.nextPosition(1)
            result_peg_225.setValue(')')
        else:
            raise PegError
        
        result_peg_221.addResult(result_peg_225);
        
        result_peg_226 = Result(result_peg_221.getPosition())
        
        value = None
        values = result_peg_221.getValues()
        value = peg.PatternSequence(values[1])
        result_peg_226.setValue(value)
        
        result_peg_221.addResult(result_peg_226);
        
        result_peg_221.setValue(result_peg_221.getLastValue())
        stream.update(RULE_sub_pattern, position, result_peg_221)
        return result_peg_221
    except PegError:
        pass
    stream.update(RULE_sub_pattern, position, None)
    return None


def rule_bind(stream, position):
    if stream.hasResult(RULE_bind, position):
        return stream.result(RULE_bind, position)
    try:
        result_peg_227 = Result(position)
        result_peg_228 = Result(result_peg_227.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_228 = rule_word(stream, result_peg_228.getPosition())
        if result_peg_228 == None:
            raise PegError
        name = result_peg_228.getValues()
        
        result_peg_227.addResult(result_peg_228);
        
        result_peg_229 = Result(result_peg_227.getPosition())
        
        if ':' == stream.get(result_peg_229.getPosition(), 1):
            result_peg_229.nextPosition(1)
            result_peg_229.setValue(':')
        else:
            raise PegError
        
        result_peg_227.addResult(result_peg_229);
        
        result_peg_230 = Result(result_peg_227.getPosition())
        
        value = None
        values = result_peg_227.getValues()
        value = lambda p: peg.PatternBind(name, p)
        result_peg_230.setValue(value)
        
        result_peg_227.addResult(result_peg_230);
        
        result_peg_227.setValue(result_peg_227.getLastValue())
        stream.update(RULE_bind, position, result_peg_227)
        return result_peg_227
    except PegError:
        pass
    stream.update(RULE_bind, position, None)
    return None


def rule_string(stream, position):
    if stream.hasResult(RULE_string, position):
        return stream.result(RULE_string, position)
    try:
        result_peg_231 = Result(position)
        result_peg_232 = Result(result_peg_231.getPosition())
        
        if '"' == stream.get(result_peg_232.getPosition(), 1):
            result_peg_232.nextPosition(1)
            result_peg_232.setValue('"')
        else:
            raise PegError
        
        result_peg_231.addResult(result_peg_232);
        
        result_peg_233 = Result(result_peg_231.getPosition())
        
        try:
            while True:
                result_peg_234 = Result(result_peg_233.getPosition());
                result_peg_235 = Result(result_peg_234.getPosition())
                
                result_peg_236 = Result(result_peg_235.getPosition());
                try:
                    if '"' == stream.get(result_peg_236.getPosition(), 1):
                        result_peg_236.nextPosition(1)
                        result_peg_236.setValue('"')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_235.setValue(None)
                        
                result_peg_234.addResult(result_peg_235);
                
                result_peg_237 = Result(result_peg_234.getPosition())
                
                temp_peg_238 = stream.get(result_peg_237.getPosition())
                if temp_peg_238 != chr(0):
                    result_peg_237.setValue(temp_peg_238)
                    result_peg_237.nextPosition()
                else:
                    raise PegError
                
                result_peg_234.addResult(result_peg_237);
                
                result_peg_239 = Result(result_peg_234.getPosition())
                
                value = None
                values = result_peg_234.getValues()
                value = values[1]
                result_peg_239.setValue(value)
                
                result_peg_234.addResult(result_peg_239);
                
                result_peg_234.setValue(result_peg_234.getLastValue())
                result_peg_233.addResult(result_peg_234);
        except PegError:
            pass
                
        result_peg_231.addResult(result_peg_233);
        
        result_peg_240 = Result(result_peg_231.getPosition())
        
        if '"' == stream.get(result_peg_240.getPosition(), 1):
            result_peg_240.nextPosition(1)
            result_peg_240.setValue('"')
        else:
            raise PegError
        
        result_peg_231.addResult(result_peg_240);
        
        result_peg_241 = Result(result_peg_231.getPosition())
        
        save_peg_242 = result_peg_241.getPosition()
        
        if '{case}' == stream.get(result_peg_241.getPosition(), 6):
            result_peg_241.nextPosition(6)
            result_peg_241.setValue('{case}')
        else:
            
            result_peg_241 = Result(save_peg_242)
            result_peg_241.setValue(None)
        options = result_peg_241.getValues()
        
        result_peg_231.addResult(result_peg_241);
        
        result_peg_243 = Result(result_peg_231.getPosition())
        
        value = None
        values = result_peg_231.getValues()
        value = peg.PatternVerbatim(''.join(values[1]), options)
        result_peg_243.setValue(value)
        
        result_peg_231.addResult(result_peg_243);
        
        result_peg_231.setValue(result_peg_231.getLastValue())
        stream.update(RULE_string, position, result_peg_231)
        return result_peg_231
    except PegError:
        pass
    try:
        result_peg_244 = Result(position)
        result_peg_245 = Result(result_peg_244.getPosition())
        
        if '<quote>' == stream.get(result_peg_245.getPosition(), 7):
            result_peg_245.nextPosition(7)
            result_peg_245.setValue('<quote>')
        else:
            raise PegError
        
        result_peg_244.addResult(result_peg_245);
        
        result_peg_246 = Result(result_peg_244.getPosition())
        
        value = None
        values = result_peg_244.getValues()
        value = peg.PatternVerbatim('"')
        result_peg_246.setValue(value)
        
        result_peg_244.addResult(result_peg_246);
        
        result_peg_244.setValue(result_peg_244.getLastValue())
        stream.update(RULE_string, position, result_peg_244)
        return result_peg_244
    except PegError:
        pass
    stream.update(RULE_string, position, None)
    return None


def rule_modifier(stream, position):
    if stream.hasResult(RULE_modifier, position):
        return stream.result(RULE_modifier, position)
    try:
        result_peg_247 = Result(position)
        result_peg_248 = Result(result_peg_247.getPosition())
        
        if '*' == stream.get(result_peg_248.getPosition(), 1):
            result_peg_248.nextPosition(1)
            result_peg_248.setValue('*')
        else:
            raise PegError
        
        result_peg_247.addResult(result_peg_248);
        
        result_peg_249 = Result(result_peg_247.getPosition())
        
        value = None
        values = result_peg_247.getValues()
        value = lambda p: peg.PatternRepeatMany(p)
        result_peg_249.setValue(value)
        
        result_peg_247.addResult(result_peg_249);
        
        result_peg_247.setValue(result_peg_247.getLastValue())
        stream.update(RULE_modifier, position, result_peg_247)
        return result_peg_247
    except PegError:
        pass
    try:
        result_peg_250 = Result(position)
        result_peg_251 = Result(result_peg_250.getPosition())
        
        if '?' == stream.get(result_peg_251.getPosition(), 1):
            result_peg_251.nextPosition(1)
            result_peg_251.setValue('?')
        else:
            raise PegError
        
        result_peg_250.addResult(result_peg_251);
        
        result_peg_252 = Result(result_peg_250.getPosition())
        
        value = None
        values = result_peg_250.getValues()
        value = lambda p: peg.PatternMaybe(p)
        result_peg_252.setValue(value)
        
        result_peg_250.addResult(result_peg_252);
        
        result_peg_250.setValue(result_peg_250.getLastValue())
        stream.update(RULE_modifier, position, result_peg_250)
        return result_peg_250
    except PegError:
        pass
    try:
        result_peg_253 = Result(position)
        result_peg_254 = Result(result_peg_253.getPosition())
        
        if '+' == stream.get(result_peg_254.getPosition(), 1):
            result_peg_254.nextPosition(1)
            result_peg_254.setValue('+')
        else:
            raise PegError
        
        result_peg_253.addResult(result_peg_254);
        
        result_peg_255 = Result(result_peg_253.getPosition())
        
        value = None
        values = result_peg_253.getValues()
        value = lambda p: peg.PatternRepeatOnce(p)
        result_peg_255.setValue(value)
        
        result_peg_253.addResult(result_peg_255);
        
        result_peg_253.setValue(result_peg_253.getLastValue())
        stream.update(RULE_modifier, position, result_peg_253)
        return result_peg_253
    except PegError:
        pass
    stream.update(RULE_modifier, position, None)
    return None


def rule_x_word(stream, position):
    if stream.hasResult(RULE_x_word, position):
        return stream.result(RULE_x_word, position)
    try:
        result_peg_256 = Result(position)
        result_peg_257 = Result(result_peg_256.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_257 = rule_word(stream, result_peg_257.getPosition())
        if result_peg_257 == None:
            raise PegError
        name = result_peg_257.getValues()
        
        result_peg_256.addResult(result_peg_257);
        
        result_peg_258 = Result(result_peg_256.getPosition())
        
        save_peg_259 = result_peg_258.getPosition()
        
        # print "Trying rule " + 'parameters_rules'
        result_peg_258 = rule_parameters_rules(stream, result_peg_258.getPosition())
        if result_peg_258 == None:
            
            result_peg_258 = Result(save_peg_259)
            result_peg_258.setValue(None)
        rule_parameters = result_peg_258.getValues()
        
        result_peg_256.addResult(result_peg_258);
        
        result_peg_260 = Result(result_peg_256.getPosition())
        
        save_peg_261 = result_peg_260.getPosition()
        
        # print "Trying rule " + 'parameters_values'
        result_peg_260 = rule_parameters_values(stream, result_peg_260.getPosition())
        if result_peg_260 == None:
            
            result_peg_260 = Result(save_peg_261)
            result_peg_260.setValue(None)
        parameters = result_peg_260.getValues()
        
        result_peg_256.addResult(result_peg_260);
        
        result_peg_262 = Result(result_peg_256.getPosition())
        
        value = None
        values = result_peg_256.getValues()
        value = peg.PatternRule(name, rule_parameters, parameters)
        result_peg_262.setValue(value)
        
        result_peg_256.addResult(result_peg_262);
        
        result_peg_256.setValue(result_peg_256.getLastValue())
        stream.update(RULE_x_word, position, result_peg_256)
        return result_peg_256
    except PegError:
        pass
    stream.update(RULE_x_word, position, None)
    return None


def rule_rule_parameters(stream, position):
    if stream.hasResult(RULE_rule_parameters, position):
        return stream.result(RULE_rule_parameters, position)
    try:
        result_peg_263 = Result(position)
        result_peg_264 = Result(result_peg_263.getPosition())
        
        if '[' == stream.get(result_peg_264.getPosition(), 1):
            result_peg_264.nextPosition(1)
            result_peg_264.setValue('[')
        else:
            raise PegError
        
        result_peg_263.addResult(result_peg_264);
        
        result_peg_265 = Result(result_peg_263.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_265 = rule_spaces(stream, result_peg_265.getPosition())
        if result_peg_265 == None:
            raise PegError
        
        result_peg_263.addResult(result_peg_265);
        
        result_peg_266 = Result(result_peg_263.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_266 = rule_word(stream, result_peg_266.getPosition())
        if result_peg_266 == None:
            raise PegError
        param1 = result_peg_266.getValues()
        
        result_peg_263.addResult(result_peg_266);
        
        result_peg_267 = Result(result_peg_263.getPosition())
        
        try:
            while True:
                result_peg_268 = Result(result_peg_267.getPosition());
                result_peg_269 = Result(result_peg_268.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_269 = rule_spaces(stream, result_peg_269.getPosition())
                if result_peg_269 == None:
                    raise PegError
                
                result_peg_268.addResult(result_peg_269);
                
                result_peg_270 = Result(result_peg_268.getPosition())
                
                if ',' == stream.get(result_peg_270.getPosition(), 1):
                    result_peg_270.nextPosition(1)
                    result_peg_270.setValue(',')
                else:
                    raise PegError
                
                result_peg_268.addResult(result_peg_270);
                
                result_peg_271 = Result(result_peg_268.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_271 = rule_spaces(stream, result_peg_271.getPosition())
                if result_peg_271 == None:
                    raise PegError
                
                result_peg_268.addResult(result_peg_271);
                
                result_peg_272 = Result(result_peg_268.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_272 = rule_word(stream, result_peg_272.getPosition())
                if result_peg_272 == None:
                    raise PegError
                exp = result_peg_272.getValues()
                
                result_peg_268.addResult(result_peg_272);
                
                result_peg_273 = Result(result_peg_268.getPosition())
                
                value = None
                values = result_peg_268.getValues()
                value = exp
                result_peg_273.setValue(value)
                
                result_peg_268.addResult(result_peg_273);
                
                result_peg_268.setValue(result_peg_268.getLastValue())
                result_peg_267.addResult(result_peg_268);
        except PegError:
            pass
        params = result_peg_267.getValues()
        
        result_peg_263.addResult(result_peg_267);
        
        result_peg_274 = Result(result_peg_263.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_274 = rule_spaces(stream, result_peg_274.getPosition())
        if result_peg_274 == None:
            raise PegError
        
        result_peg_263.addResult(result_peg_274);
        
        result_peg_275 = Result(result_peg_263.getPosition())
        
        if ']' == stream.get(result_peg_275.getPosition(), 1):
            result_peg_275.nextPosition(1)
            result_peg_275.setValue(']')
        else:
            raise PegError
        
        result_peg_263.addResult(result_peg_275);
        
        result_peg_276 = Result(result_peg_263.getPosition())
        
        value = None
        values = result_peg_263.getValues()
        value = [param1] + params
        result_peg_276.setValue(value)
        
        result_peg_263.addResult(result_peg_276);
        
        result_peg_263.setValue(result_peg_263.getLastValue())
        stream.update(RULE_rule_parameters, position, result_peg_263)
        return result_peg_263
    except PegError:
        pass
    stream.update(RULE_rule_parameters, position, None)
    return None


def rule_value_parameters(stream, position):
    if stream.hasResult(RULE_value_parameters, position):
        return stream.result(RULE_value_parameters, position)
    try:
        result_peg_277 = Result(position)
        result_peg_278 = Result(result_peg_277.getPosition())
        
        if '(' == stream.get(result_peg_278.getPosition(), 1):
            result_peg_278.nextPosition(1)
            result_peg_278.setValue('(')
        else:
            raise PegError
        
        result_peg_277.addResult(result_peg_278);
        
        result_peg_279 = Result(result_peg_277.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_279 = rule_spaces(stream, result_peg_279.getPosition())
        if result_peg_279 == None:
            raise PegError
        
        result_peg_277.addResult(result_peg_279);
        
        result_peg_280 = Result(result_peg_277.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_280 = rule_word(stream, result_peg_280.getPosition())
        if result_peg_280 == None:
            raise PegError
        param1 = result_peg_280.getValues()
        
        result_peg_277.addResult(result_peg_280);
        
        result_peg_281 = Result(result_peg_277.getPosition())
        
        try:
            while True:
                result_peg_282 = Result(result_peg_281.getPosition());
                result_peg_283 = Result(result_peg_282.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_283 = rule_spaces(stream, result_peg_283.getPosition())
                if result_peg_283 == None:
                    raise PegError
                
                result_peg_282.addResult(result_peg_283);
                
                result_peg_284 = Result(result_peg_282.getPosition())
                
                if ',' == stream.get(result_peg_284.getPosition(), 1):
                    result_peg_284.nextPosition(1)
                    result_peg_284.setValue(',')
                else:
                    raise PegError
                
                result_peg_282.addResult(result_peg_284);
                
                result_peg_285 = Result(result_peg_282.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_285 = rule_spaces(stream, result_peg_285.getPosition())
                if result_peg_285 == None:
                    raise PegError
                
                result_peg_282.addResult(result_peg_285);
                
                result_peg_286 = Result(result_peg_282.getPosition())
                
                # print "Trying rule " + 'word'
                result_peg_286 = rule_word(stream, result_peg_286.getPosition())
                if result_peg_286 == None:
                    raise PegError
                exp = result_peg_286.getValues()
                
                result_peg_282.addResult(result_peg_286);
                
                result_peg_287 = Result(result_peg_282.getPosition())
                
                value = None
                values = result_peg_282.getValues()
                value = exp
                result_peg_287.setValue(value)
                
                result_peg_282.addResult(result_peg_287);
                
                result_peg_282.setValue(result_peg_282.getLastValue())
                result_peg_281.addResult(result_peg_282);
        except PegError:
            pass
        params = result_peg_281.getValues()
        
        result_peg_277.addResult(result_peg_281);
        
        result_peg_288 = Result(result_peg_277.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_288 = rule_spaces(stream, result_peg_288.getPosition())
        if result_peg_288 == None:
            raise PegError
        
        result_peg_277.addResult(result_peg_288);
        
        result_peg_289 = Result(result_peg_277.getPosition())
        
        if ')' == stream.get(result_peg_289.getPosition(), 1):
            result_peg_289.nextPosition(1)
            result_peg_289.setValue(')')
        else:
            raise PegError
        
        result_peg_277.addResult(result_peg_289);
        
        result_peg_290 = Result(result_peg_277.getPosition())
        
        value = None
        values = result_peg_277.getValues()
        value = [param1] + params
        result_peg_290.setValue(value)
        
        result_peg_277.addResult(result_peg_290);
        
        result_peg_277.setValue(result_peg_277.getLastValue())
        stream.update(RULE_value_parameters, position, result_peg_277)
        return result_peg_277
    except PegError:
        pass
    stream.update(RULE_value_parameters, position, None)
    return None


def rule_parameters_rules(stream, position):
    if stream.hasResult(RULE_parameters_rules, position):
        return stream.result(RULE_parameters_rules, position)
    try:
        result_peg_291 = Result(position)
        result_peg_292 = Result(result_peg_291.getPosition())
        
        if '[' == stream.get(result_peg_292.getPosition(), 1):
            result_peg_292.nextPosition(1)
            result_peg_292.setValue('[')
        else:
            raise PegError
        
        result_peg_291.addResult(result_peg_292);
        
        result_peg_293 = Result(result_peg_291.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_293 = rule_spaces(stream, result_peg_293.getPosition())
        if result_peg_293 == None:
            raise PegError
        
        result_peg_291.addResult(result_peg_293);
        
        result_peg_294 = Result(result_peg_291.getPosition())
        
        # print "Trying rule " + 'word_or_at'
        result_peg_294 = rule_word_or_at(stream, result_peg_294.getPosition())
        if result_peg_294 == None:
            raise PegError
        param1 = result_peg_294.getValues()
        
        result_peg_291.addResult(result_peg_294);
        
        result_peg_295 = Result(result_peg_291.getPosition())
        
        try:
            while True:
                result_peg_296 = Result(result_peg_295.getPosition());
                result_peg_297 = Result(result_peg_296.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_297 = rule_spaces(stream, result_peg_297.getPosition())
                if result_peg_297 == None:
                    raise PegError
                
                result_peg_296.addResult(result_peg_297);
                
                result_peg_298 = Result(result_peg_296.getPosition())
                
                if ',' == stream.get(result_peg_298.getPosition(), 1):
                    result_peg_298.nextPosition(1)
                    result_peg_298.setValue(',')
                else:
                    raise PegError
                
                result_peg_296.addResult(result_peg_298);
                
                result_peg_299 = Result(result_peg_296.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_299 = rule_spaces(stream, result_peg_299.getPosition())
                if result_peg_299 == None:
                    raise PegError
                
                result_peg_296.addResult(result_peg_299);
                
                result_peg_300 = Result(result_peg_296.getPosition())
                
                # print "Trying rule " + 'word_or_at'
                result_peg_300 = rule_word_or_at(stream, result_peg_300.getPosition())
                if result_peg_300 == None:
                    raise PegError
                exp = result_peg_300.getValues()
                
                result_peg_296.addResult(result_peg_300);
                
                result_peg_301 = Result(result_peg_296.getPosition())
                
                value = None
                values = result_peg_296.getValues()
                value = exp
                result_peg_301.setValue(value)
                
                result_peg_296.addResult(result_peg_301);
                
                result_peg_296.setValue(result_peg_296.getLastValue())
                result_peg_295.addResult(result_peg_296);
        except PegError:
            pass
        params = result_peg_295.getValues()
        
        result_peg_291.addResult(result_peg_295);
        
        result_peg_302 = Result(result_peg_291.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_302 = rule_spaces(stream, result_peg_302.getPosition())
        if result_peg_302 == None:
            raise PegError
        
        result_peg_291.addResult(result_peg_302);
        
        result_peg_303 = Result(result_peg_291.getPosition())
        
        if ']' == stream.get(result_peg_303.getPosition(), 1):
            result_peg_303.nextPosition(1)
            result_peg_303.setValue(']')
        else:
            raise PegError
        
        result_peg_291.addResult(result_peg_303);
        
        result_peg_304 = Result(result_peg_291.getPosition())
        
        value = None
        values = result_peg_291.getValues()
        value = [param1] + params
        result_peg_304.setValue(value)
        
        result_peg_291.addResult(result_peg_304);
        
        result_peg_291.setValue(result_peg_291.getLastValue())
        stream.update(RULE_parameters_rules, position, result_peg_291)
        return result_peg_291
    except PegError:
        pass
    stream.update(RULE_parameters_rules, position, None)
    return None


def rule_parameters_values(stream, position):
    if stream.hasResult(RULE_parameters_values, position):
        return stream.result(RULE_parameters_values, position)
    try:
        result_peg_305 = Result(position)
        result_peg_306 = Result(result_peg_305.getPosition())
        
        if '(' == stream.get(result_peg_306.getPosition(), 1):
            result_peg_306.nextPosition(1)
            result_peg_306.setValue('(')
        else:
            raise PegError
        
        result_peg_305.addResult(result_peg_306);
        
        result_peg_307 = Result(result_peg_305.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_307 = rule_spaces(stream, result_peg_307.getPosition())
        if result_peg_307 == None:
            raise PegError
        
        result_peg_305.addResult(result_peg_307);
        
        result_peg_308 = Result(result_peg_305.getPosition())
        
        # print "Trying rule " + 'word_or_dollar'
        result_peg_308 = rule_word_or_dollar(stream, result_peg_308.getPosition())
        if result_peg_308 == None:
            raise PegError
        param1 = result_peg_308.getValues()
        
        result_peg_305.addResult(result_peg_308);
        
        result_peg_309 = Result(result_peg_305.getPosition())
        
        try:
            while True:
                result_peg_310 = Result(result_peg_309.getPosition());
                result_peg_311 = Result(result_peg_310.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_311 = rule_spaces(stream, result_peg_311.getPosition())
                if result_peg_311 == None:
                    raise PegError
                
                result_peg_310.addResult(result_peg_311);
                
                result_peg_312 = Result(result_peg_310.getPosition())
                
                if ',' == stream.get(result_peg_312.getPosition(), 1):
                    result_peg_312.nextPosition(1)
                    result_peg_312.setValue(',')
                else:
                    raise PegError
                
                result_peg_310.addResult(result_peg_312);
                
                result_peg_313 = Result(result_peg_310.getPosition())
                
                # print "Trying rule " + 'spaces'
                result_peg_313 = rule_spaces(stream, result_peg_313.getPosition())
                if result_peg_313 == None:
                    raise PegError
                
                result_peg_310.addResult(result_peg_313);
                
                result_peg_314 = Result(result_peg_310.getPosition())
                
                # print "Trying rule " + 'word_or_dollar'
                result_peg_314 = rule_word_or_dollar(stream, result_peg_314.getPosition())
                if result_peg_314 == None:
                    raise PegError
                exp = result_peg_314.getValues()
                
                result_peg_310.addResult(result_peg_314);
                
                result_peg_315 = Result(result_peg_310.getPosition())
                
                value = None
                values = result_peg_310.getValues()
                value = exp
                result_peg_315.setValue(value)
                
                result_peg_310.addResult(result_peg_315);
                
                result_peg_310.setValue(result_peg_310.getLastValue())
                result_peg_309.addResult(result_peg_310);
        except PegError:
            pass
        params = result_peg_309.getValues()
        
        result_peg_305.addResult(result_peg_309);
        
        result_peg_316 = Result(result_peg_305.getPosition())
        
        # print "Trying rule " + 'spaces'
        result_peg_316 = rule_spaces(stream, result_peg_316.getPosition())
        if result_peg_316 == None:
            raise PegError
        
        result_peg_305.addResult(result_peg_316);
        
        result_peg_317 = Result(result_peg_305.getPosition())
        
        if ')' == stream.get(result_peg_317.getPosition(), 1):
            result_peg_317.nextPosition(1)
            result_peg_317.setValue(')')
        else:
            raise PegError
        
        result_peg_305.addResult(result_peg_317);
        
        result_peg_318 = Result(result_peg_305.getPosition())
        
        value = None
        values = result_peg_305.getValues()
        value = [param1] + params
        result_peg_318.setValue(value)
        
        result_peg_305.addResult(result_peg_318);
        
        result_peg_305.setValue(result_peg_305.getLastValue())
        stream.update(RULE_parameters_values, position, result_peg_305)
        return result_peg_305
    except PegError:
        pass
    stream.update(RULE_parameters_values, position, None)
    return None


def rule_word_or_dollar(stream, position):
    if stream.hasResult(RULE_word_or_dollar, position):
        return stream.result(RULE_word_or_dollar, position)
    try:
        result_peg_319 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_319 = rule_word(stream, result_peg_319.getPosition())
        if result_peg_319 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_319)
        return result_peg_319
    except PegError:
        pass
    try:
        result_peg_320 = Result(position)
        # print "Trying rule " + 'dollar'
        result_peg_320 = rule_dollar(stream, result_peg_320.getPosition())
        if result_peg_320 == None:
            raise PegError
        stream.update(RULE_word_or_dollar, position, result_peg_320)
        return result_peg_320
    except PegError:
        pass
    stream.update(RULE_word_or_dollar, position, None)
    return None


def rule_word_or_at(stream, position):
    if stream.hasResult(RULE_word_or_at, position):
        return stream.result(RULE_word_or_at, position)
    try:
        result_peg_321 = Result(position)
        # print "Trying rule " + 'word'
        result_peg_321 = rule_word(stream, result_peg_321.getPosition())
        if result_peg_321 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_321)
        return result_peg_321
    except PegError:
        pass
    try:
        result_peg_322 = Result(position)
        # print "Trying rule " + 'word_at'
        result_peg_322 = rule_word_at(stream, result_peg_322.getPosition())
        if result_peg_322 == None:
            raise PegError
        stream.update(RULE_word_or_at, position, result_peg_322)
        return result_peg_322
    except PegError:
        pass
    stream.update(RULE_word_or_at, position, None)
    return None


def rule_word_at(stream, position):
    if stream.hasResult(RULE_word_at, position):
        return stream.result(RULE_word_at, position)
    try:
        result_peg_323 = Result(position)
        result_peg_324 = Result(result_peg_323.getPosition())
        
        if '@' == stream.get(result_peg_324.getPosition(), 1):
            result_peg_324.nextPosition(1)
            result_peg_324.setValue('@')
        else:
            raise PegError
        
        result_peg_323.addResult(result_peg_324);
        
        result_peg_325 = Result(result_peg_323.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_325 = rule_word(stream, result_peg_325.getPosition())
        if result_peg_325 == None:
            raise PegError
        word = result_peg_325.getValues()
        
        result_peg_323.addResult(result_peg_325);
        
        result_peg_326 = Result(result_peg_323.getPosition())
        
        value = None
        values = result_peg_323.getValues()
        value = '@%s' % word
        result_peg_326.setValue(value)
        
        result_peg_323.addResult(result_peg_326);
        
        result_peg_323.setValue(result_peg_323.getLastValue())
        stream.update(RULE_word_at, position, result_peg_323)
        return result_peg_323
    except PegError:
        pass
    stream.update(RULE_word_at, position, None)
    return None


def rule_dollar(stream, position):
    if stream.hasResult(RULE_dollar, position):
        return stream.result(RULE_dollar, position)
    try:
        result_peg_327 = Result(position)
        result_peg_328 = Result(result_peg_327.getPosition())
        
        if '$' == stream.get(result_peg_328.getPosition(), 1):
            result_peg_328.nextPosition(1)
            result_peg_328.setValue('$')
        else:
            raise PegError
        
        result_peg_327.addResult(result_peg_328);
        
        result_peg_329 = Result(result_peg_327.getPosition())
        
        # print "Trying rule " + 'number'
        result_peg_329 = rule_number(stream, result_peg_329.getPosition())
        if result_peg_329 == None:
            raise PegError
        number = result_peg_329.getValues()
        
        result_peg_327.addResult(result_peg_329);
        
        result_peg_330 = Result(result_peg_327.getPosition())
        
        value = None
        values = result_peg_327.getValues()
        value = "$%s" % number
        result_peg_330.setValue(value)
        
        result_peg_327.addResult(result_peg_330);
        
        result_peg_327.setValue(result_peg_327.getLastValue())
        stream.update(RULE_dollar, position, result_peg_327)
        return result_peg_327
    except PegError:
        pass
    stream.update(RULE_dollar, position, None)
    return None


def rule_number(stream, position):
    if stream.hasResult(RULE_number, position):
        return stream.result(RULE_number, position)
    try:
        result_peg_331 = Result(position)
        result_peg_332 = Result(result_peg_331.getPosition())
        
        try:
            while True:
                result_peg_333 = Result(result_peg_332.getPosition());
                # print "Trying rule " + 'digit'
                result_peg_333 = rule_digit(stream, result_peg_333.getPosition())
                if result_peg_333 == None:
                    raise PegError
                result_peg_332.addResult(result_peg_333);
        except PegError:
            if result_peg_332.matches() == 0:
                raise PegError
                
        result_peg_331.addResult(result_peg_332);
        
        result_peg_334 = Result(result_peg_331.getPosition())
        
        value = None
        values = result_peg_331.getValues()
        value = ''.join(values[0])
        result_peg_334.setValue(value)
        
        result_peg_331.addResult(result_peg_334);
        
        result_peg_331.setValue(result_peg_331.getLastValue())
        stream.update(RULE_number, position, result_peg_331)
        return result_peg_331
    except PegError:
        pass
    stream.update(RULE_number, position, None)
    return None


def rule_digit(stream, position):
    if stream.hasResult(RULE_digit, position):
        return stream.result(RULE_digit, position)
    try:
        result_peg_335 = Result(position)
        letter_peg_336 = stream.get(result_peg_335.getPosition())
        if letter_peg_336 in '0123456789':
            result_peg_335.nextPosition()
            result_peg_335.setValue(letter_peg_336)
        else:
            raise PegError
        stream.update(RULE_digit, position, result_peg_335)
        return result_peg_335
    except PegError:
        pass
    stream.update(RULE_digit, position, None)
    return None


def rule_hex_number(stream, position):
    if stream.hasResult(RULE_hex_number, position):
        return stream.result(RULE_hex_number, position)
    try:
        result_peg_337 = Result(position)
        result_peg_338 = Result(result_peg_337.getPosition())
        
        try:
            while True:
                result_peg_339 = Result(result_peg_338.getPosition());
                # print "Trying rule " + 'hex_digit'
                result_peg_339 = rule_hex_digit(stream, result_peg_339.getPosition())
                if result_peg_339 == None:
                    raise PegError
                result_peg_338.addResult(result_peg_339);
        except PegError:
            if result_peg_338.matches() == 0:
                raise PegError
                
        result_peg_337.addResult(result_peg_338);
        
        result_peg_340 = Result(result_peg_337.getPosition())
        
        value = None
        values = result_peg_337.getValues()
        value = ''.join(values[0])
        result_peg_340.setValue(value)
        
        result_peg_337.addResult(result_peg_340);
        
        result_peg_337.setValue(result_peg_337.getLastValue())
        stream.update(RULE_hex_number, position, result_peg_337)
        return result_peg_337
    except PegError:
        pass
    stream.update(RULE_hex_number, position, None)
    return None


def rule_hex_digit(stream, position):
    if stream.hasResult(RULE_hex_digit, position):
        return stream.result(RULE_hex_digit, position)
    try:
        result_peg_341 = Result(position)
        letter_peg_342 = stream.get(result_peg_341.getPosition())
        if letter_peg_342 in '0123456789abcdefABCDEF':
            result_peg_341.nextPosition()
            result_peg_341.setValue(letter_peg_342)
        else:
            raise PegError
        stream.update(RULE_hex_digit, position, result_peg_341)
        return result_peg_341
    except PegError:
        pass
    stream.update(RULE_hex_digit, position, None)
    return None


def rule_start_symbol(stream, position):
    if stream.hasResult(RULE_start_symbol, position):
        return stream.result(RULE_start_symbol, position)
    try:
        result_peg_343 = Result(position)
        result_peg_344 = Result(result_peg_343.getPosition())
        
        if 'start-symbol:' == stream.get(result_peg_344.getPosition(), 13):
            result_peg_344.nextPosition(13)
            result_peg_344.setValue('start-symbol:')
        else:
            raise PegError
        
        result_peg_343.addResult(result_peg_344);
        
        result_peg_345 = Result(result_peg_343.getPosition())
        
        try:
            while True:
                result_peg_346 = Result(result_peg_345.getPosition());
                # print "Trying rule " + 'space'
                result_peg_346 = rule_space(stream, result_peg_346.getPosition())
                if result_peg_346 == None:
                    raise PegError
                result_peg_345.addResult(result_peg_346);
        except PegError:
            pass
                
        result_peg_343.addResult(result_peg_345);
        
        result_peg_347 = Result(result_peg_343.getPosition())
        
        # print "Trying rule " + 'word'
        result_peg_347 = rule_word(stream, result_peg_347.getPosition())
        if result_peg_347 == None:
            raise PegError
        
        result_peg_343.addResult(result_peg_347);
        
        result_peg_348 = Result(result_peg_343.getPosition())
        
        value = None
        values = result_peg_343.getValues()
        value = values[2]
        result_peg_348.setValue(value)
        
        result_peg_343.addResult(result_peg_348);
        
        result_peg_343.setValue(result_peg_343.getLastValue())
        stream.update(RULE_start_symbol, position, result_peg_343)
        return result_peg_343
    except PegError:
        pass
    stream.update(RULE_start_symbol, position, None)
    return None


def rule_spaces(stream, position):
    if stream.hasResult(RULE_spaces, position):
        return stream.result(RULE_spaces, position)
    try:
        result_peg_349 = Result(position)
        result_peg_350 = Result(result_peg_349.getPosition())
        
        try:
            while True:
                result_peg_351 = Result(result_peg_350.getPosition());
                # print "Trying rule " + 'space'
                result_peg_351 = rule_space(stream, result_peg_351.getPosition())
                if result_peg_351 == None:
                    raise PegError
                result_peg_350.addResult(result_peg_351);
        except PegError:
            pass
                
        result_peg_349.addResult(result_peg_350);
        
        result_peg_349.setValue(result_peg_349.getLastValue())
        stream.update(RULE_spaces, position, result_peg_349)
        return result_peg_349
    except PegError:
        pass
    stream.update(RULE_spaces, position, None)
    return None


def rule_space(stream, position):
    if stream.hasResult(RULE_space, position):
        return stream.result(RULE_space, position)
    try:
        result_peg_352 = Result(position)
        result_peg_353 = Result(result_peg_352.getPosition())
        
        if ' ' == stream.get(result_peg_353.getPosition(), 1):
            result_peg_353.nextPosition(1)
            result_peg_353.setValue(' ')
        else:
            raise PegError
        
        result_peg_352.addResult(result_peg_353);
        
        result_peg_352.setValue(result_peg_352.getLastValue())
        stream.update(RULE_space, position, result_peg_352)
        return result_peg_352
    except PegError:
        pass
    try:
        result_peg_354 = Result(position)
        result_peg_355 = Result(result_peg_354.getPosition())
        
        if '\t' == stream.get(result_peg_355.getPosition(), 1):
            result_peg_355.nextPosition(1)
            result_peg_355.setValue('\t')
        else:
            raise PegError
        
        result_peg_354.addResult(result_peg_355);
        
        result_peg_354.setValue(result_peg_354.getLastValue())
        stream.update(RULE_space, position, result_peg_354)
        return result_peg_354
    except PegError:
        pass
    stream.update(RULE_space, position, None)
    return None


def rule_any_char(stream, position):
    if stream.hasResult(RULE_any_char, position):
        return stream.result(RULE_any_char, position)
    try:
        result_peg_356 = Result(position)
        letter_peg_357 = stream.get(result_peg_356.getPosition())
        if letter_peg_357 in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_-':
            result_peg_356.nextPosition()
            result_peg_356.setValue(letter_peg_357)
        else:
            raise PegError
        stream.update(RULE_any_char, position, result_peg_356)
        return result_peg_356
    except PegError:
        pass
    stream.update(RULE_any_char, position, None)
    return None


def rule_any(stream, position):
    if stream.hasResult(RULE_any, position):
        return stream.result(RULE_any, position)
    try:
        result_peg_358 = Result(position)
        result_peg_359 = Result(result_peg_358.getPosition())
        
        if '.' == stream.get(result_peg_359.getPosition(), 1):
            result_peg_359.nextPosition(1)
            result_peg_359.setValue('.')
        else:
            raise PegError
        
        result_peg_358.addResult(result_peg_359);
        
        result_peg_360 = Result(result_peg_358.getPosition())
        
        value = None
        values = result_peg_358.getValues()
        value = peg.PatternAny()
        result_peg_360.setValue(value)
        
        result_peg_358.addResult(result_peg_360);
        
        result_peg_358.setValue(result_peg_358.getLastValue())
        stream.update(RULE_any, position, result_peg_358)
        return result_peg_358
    except PegError:
        pass
    stream.update(RULE_any, position, None)
    return None


def rule_whitespace(stream, position):
    if stream.hasResult(RULE_whitespace, position):
        return stream.result(RULE_whitespace, position)
    try:
        result_peg_361 = Result(position)
        result_peg_362 = Result(result_peg_361.getPosition())
        
        try:
            while True:
                result_peg_363 = Result(result_peg_362.getPosition());
                save_peg_364 = result_peg_363.getPosition()
                
                result_peg_363 = Result(save_peg_364)
                letter_peg_365 = stream.get(result_peg_363.getPosition())
                if letter_peg_365 in ' \t\n':
                    result_peg_363.nextPosition()
                    result_peg_363.setValue(letter_peg_365)
                else:
                    
                    result_peg_363 = Result(save_peg_364)
                    # print "Trying rule " + 'comment'
                    result_peg_363 = rule_comment(stream, result_peg_363.getPosition())
                    if result_peg_363 == None:
                        raise PegError
                result_peg_362.addResult(result_peg_363);
        except PegError:
            pass
                
        result_peg_361.addResult(result_peg_362);
        
        result_peg_361.setValue(result_peg_361.getLastValue())
        stream.update(RULE_whitespace, position, result_peg_361)
        return result_peg_361
    except PegError:
        pass
    stream.update(RULE_whitespace, position, None)
    return None


def rule_comment(stream, position):
    if stream.hasResult(RULE_comment, position):
        return stream.result(RULE_comment, position)
    try:
        result_peg_366 = Result(position)
        result_peg_367 = Result(result_peg_366.getPosition())
        
        if '#' == stream.get(result_peg_367.getPosition(), 1):
            result_peg_367.nextPosition(1)
            result_peg_367.setValue('#')
        else:
            raise PegError
        
        result_peg_366.addResult(result_peg_367);
        
        result_peg_368 = Result(result_peg_366.getPosition())
        
        try:
            while True:
                result_peg_369 = Result(result_peg_368.getPosition());
                result_peg_370 = Result(result_peg_369.getPosition())
                
                result_peg_371 = Result(result_peg_370.getPosition());
                try:
                    if '\n' == stream.get(result_peg_371.getPosition(), 1):
                        result_peg_371.nextPosition(1)
                        result_peg_371.setValue('\n')
                    else:
                        raise NotError
                    raise PegError
                except NotError:
                    result_peg_370.setValue(None)
                        
                result_peg_369.addResult(result_peg_370);
                
                result_peg_372 = Result(result_peg_369.getPosition())
                
                temp_peg_373 = stream.get(result_peg_372.getPosition())
                if temp_peg_373 != chr(0):
                    result_peg_372.setValue(temp_peg_373)
                    result_peg_372.nextPosition()
                else:
                    raise PegError
                
                result_peg_369.addResult(result_peg_372);
                
                result_peg_369.setValue(result_peg_369.getLastValue())
                result_peg_368.addResult(result_peg_369);
        except PegError:
            pass
                
        result_peg_366.addResult(result_peg_368);
        
        result_peg_366.setValue(result_peg_366.getLastValue())
        stream.update(RULE_comment, position, result_peg_366)
        return result_peg_366
    except PegError:
        pass
    stream.update(RULE_comment, position, None)
    return None


def rule_newlines_one(stream, position):
    if stream.hasResult(RULE_newlines_one, position):
        return stream.result(RULE_newlines_one, position)
    try:
        result_peg_374 = Result(position)
        try:
            while True:
                result_peg_375 = Result(result_peg_374.getPosition());
                if '\n' == stream.get(result_peg_375.getPosition(), 1):
                    result_peg_375.nextPosition(1)
                    result_peg_375.setValue('\n')
                else:
                    raise PegError
                result_peg_374.addResult(result_peg_375);
        except PegError:
            if result_peg_374.matches() == 0:
                raise PegError
        stream.update(RULE_newlines_one, position, result_peg_374)
        return result_peg_374
    except PegError:
        pass
    stream.update(RULE_newlines_one, position, None)
    return None


def rule_newlines(stream, position):
    if stream.hasResult(RULE_newlines, position):
        return stream.result(RULE_newlines, position)
    try:
        result_peg_376 = Result(position)
        try:
            while True:
                result_peg_377 = Result(result_peg_376.getPosition());
                if '\n' == stream.get(result_peg_377.getPosition(), 1):
                    result_peg_377.nextPosition(1)
                    result_peg_377.setValue('\n')
                else:
                    raise PegError
                result_peg_376.addResult(result_peg_377);
        except PegError:
            pass
        stream.update(RULE_newlines, position, result_peg_376)
        return result_peg_376
    except PegError:
        pass
    stream.update(RULE_newlines, position, None)
    return None


def doParse(stream):
    done = rule_start(stream, 0)
    if done == None:
        # print "Error parsing " + file
        stream.reportError()
        return None
    else:
        return done.getValues()

def parseFile(file):
    # print "Parsing " + file
    return doParse(Stream(filename = file))

def parseString(value):
    return doParse(Stream(input = value))

